@layer rdg{@layer Defaults,
    FocusSink,
    CheckboxInput,
    CheckboxIcon,
    CheckboxLabel,
    Cell,
    HeaderCell,
    SummaryCell,
    EditCell,
    Row,
    HeaderRow,
    SummaryRow,
    GroupedRow,
    Root}@layer rdg.MeasuringCell{.mlln6zg7-0-0-beta-57{contain:strict;grid-row:1;visibility:hidden}}@layer rdg.Cell{.cj343x07-0-0-beta-57{align-content:center;background-color:inherit;border-block-end:var(--rdg-border-width) solid var(--rdg-border-color);border-inline-end:var(--rdg-border-width) solid var(--rdg-border-color);grid-row-start:var(--rdg-grid-row-start);outline:none;overflow:clip;padding-block:0;padding-inline:8px;position:relative;text-overflow:ellipsis;white-space:nowrap;&[aria-selected=true]{outline:var(--rdg-selection-width) solid var(--rdg-selection-color);outline-offset:calc(var(--rdg-selection-width)*-1)}}}@layer rdg.Cell{.csofj7r7-0-0-beta-57{position:sticky;z-index:1;&:nth-last-child(1 of &){box-shadow:var(--rdg-cell-frozen-box-shadow)}}}@layer rdg.DragHandle{.ch2wcw87-0-0-beta-57{--rdg-drag-handle-size:8px;background-color:var(--rdg-selection-color);block-size:var(--rdg-drag-handle-size);cursor:move;inline-size:var(--rdg-drag-handle-size);place-self:end;z-index:0;&:hover{--rdg-drag-handle-size:16px;background-color:var(--rdg-background-color);border:2px solid var(--rdg-selection-color)}}}@layer rdg.DragHandle{.c1wvphzh7-0-0-beta-57{position:sticky;z-index:1}}@layer rdg.CheckboxInput{.c1bn88vv7-0-0-beta-57{block-size:20px;display:block;inline-size:20px;margin:auto;&:focus-visible{outline:2px solid var(--rdg-checkbox-focus-color);outline-offset:-3px}&:enabled{cursor:pointer}}}@layer rdg.GroupCellContent{.g1s9ylgp7-0-0-beta-57{outline:none}}@layer rdg.GroupCellCaret{.cz54e4y7-0-0-beta-57{margin-inline-start:4px;stroke:currentColor;stroke-width:1.5px;fill:#0000;vertical-align:middle;>path{transition:d .1s}}}@layer rdg.SortableHeaderCell{.h44jtk67-0-0-beta-57{display:flex}}@layer rdg.SortableHeaderCellName{.hcgkhxz7-0-0-beta-57{flex-grow:1;overflow:clip;text-overflow:ellipsis}}@layer rdg.Cell{.c6ra8a37-0-0-beta-57{background-color:#ccf}}@layer rdg.EditCell{.cis5rrm7-0-0-beta-57{padding:0}}@layer rdg.HeaderCell{.c6l2wv17-0-0-beta-57{cursor:pointer}.c1kqdw7y7-0-0-beta-57{touch-action:none}.r1y6ywlx7-0-0-beta-57{cursor:col-resize;inline-size:10px;inset-block-end:0;inset-block-start:0;inset-inline-end:0;position:absolute}.c1bezg5o7-0-0-beta-57,.c1vc96037-0-0-beta-57{background-color:var(--rdg-header-draggable-background-color)}.d8rwc9w7-0-0-beta-57{border-radius:4px;outline:2px solid #008cff;outline-offset:-2px;width:fit-content}}@layer rdg.HeaderCell{}@layer rdg.HeaderCell{}@layer rdg.HeaderCell{}@layer rdg.HeaderCell{}@layer rdg.HeaderCell{}@layer rdg.Row{.r1upfr807-0-0-beta-57{background-color:var(--rdg-background-color);display:contents;&:hover{background-color:var(--rdg-row-hover-background-color)}&[aria-selected=true]{background-color:var(--rdg-row-selected-background-color);&:hover{background-color:var(--rdg-row-selected-hover-background-color)}}}}@layer rdg.FocusSink{.r190mhd37-0-0-beta-57{outline:2px solid var(--rdg-selection-color);outline-offset:-2px}}@layer rdg.FocusSink{.r139qu9m7-0-0-beta-57{&:before{block-size:100%;border-inline-start:2px solid var(--rdg-selection-color);content:"";display:inline-block;inset-inline-start:0;position:sticky}}}@layer rdg.HeaderRow{.h10tskcx7-0-0-beta-57{background-color:var(--rdg-header-background-color);display:contents;font-weight:700;&>.cj343x07-0-0-beta-57{position:sticky;z-index:2}&>.csofj7r7-0-0-beta-57{z-index:3}}}@layer rdg.SortIcon{.a3ejtar7-0-0-beta-57{fill:currentColor;>path{transition:d .1s}}}@layer rdg.Defaults{.rnvodz57-0-0-beta-57{*,:after,:before{box-sizing:inherit}}}@layer rdg.Root{.rnvodz57-0-0-beta-57{--rdg-color:#000;--rdg-border-color:#ddd;--rdg-summary-border-color:#aaa;--rdg-background-color:#fff;--rdg-header-background-color:#f9f9f9;--rdg-header-draggable-background-color:#e7e7e7;--rdg-row-hover-background-color:#f5f5f5;--rdg-row-selected-background-color:#dbecfa;--rdg-row-selected-hover-background-color:#c9e3f8;--rdg-checkbox-focus-color:#61b8ff;--rdg-selection-width:2px;--rdg-selection-color:#67afe9;--rdg-font-size:14px;--rdg-cell-frozen-box-shadow:2px 0 5px -2px #8888884d;--rdg-border-width:1px;--rdg-summary-border-width:calc(var(--rdg-border-width)*2);accent-color:light-dark(#005194,#94cfff);background-color:var(--rdg-background-color);block-size:350px;border:1px solid var(--rdg-border-color);box-sizing:border-box;color:var(--rdg-color);color-scheme:var(--rdg-color-scheme,light dark);contain:content;content-visibility:auto;display:grid;font-size:var(--rdg-font-size);overflow:auto;&:dir(rtl){--rdg-cell-frozen-box-shadow:-2px 0 5px -2px #8888884d}&:before{content:"";grid-column:1/-1;grid-row:1/-1}&.rdg-dark{--rdg-color-scheme:dark;--rdg-color:#ddd;--rdg-border-color:#444;--rdg-summary-border-color:#555;--rdg-background-color:#212121;--rdg-header-background-color:#1b1b1b;--rdg-header-draggable-background-color:#2d2d2d;--rdg-row-hover-background-color:#171717;--rdg-row-selected-background-color:#1a73bc;--rdg-row-selected-hover-background-color:#1768ab;--rdg-checkbox-focus-color:#c7e6ff}&.rdg-light{--rdg-color-scheme:light}>:nth-last-child(1 of .rdg-top-summary-row){>.cj343x07-0-0-beta-57{border-block-end:var(--rdg-summary-border-width) solid var(--rdg-summary-border-color)}}>:nth-child(1 of .rdg-bottom-summary-row){>.cj343x07-0-0-beta-57{border-block-start:var(--rdg-summary-border-width) solid var(--rdg-summary-border-color)}}}@media(prefers-color-scheme:dark){.rnvodz57-0-0-beta-57{&:not(.rdg-light){--rdg-color:#ddd;--rdg-border-color:#444;--rdg-summary-border-color:#555;--rdg-background-color:#212121;--rdg-header-background-color:#1b1b1b;--rdg-header-draggable-background-color:#2d2d2d;--rdg-row-hover-background-color:#171717;--rdg-row-selected-background-color:#1a73bc;--rdg-row-selected-hover-background-color:#1768ab;--rdg-checkbox-focus-color:#c7e6ff}}}}@layer rdg.Root{.vlqv91k7-0-0-beta-57{-webkit-user-select:none;user-select:none;& .r1upfr807-0-0-beta-57{cursor:move}}}@layer rdg.FocusSink{.f1lsfrzw7-0-0-beta-57{grid-column:1/-1;pointer-events:none;z-index:1}.f1cte0lg7-0-0-beta-57{z-index:3}}@layer rdg.FocusSink{}@layer rdg.SummaryCell{.s8wc6fl7-0-0-beta-57{inset-block-end:var(--rdg-summary-row-bottom);inset-block-start:var(--rdg-summary-row-top)}}@layer rdg.SummaryRow{.skuhp557-0-0-beta-57{>.cj343x07-0-0-beta-57{position:sticky}}}@layer rdg.SummaryRow{.tf8l5ub7-0-0-beta-57{>.cj343x07-0-0-beta-57{z-index:2}>.csofj7r7-0-0-beta-57{z-index:3}}}@layer rdg.GroupedRow{.g1yxluv37-0-0-beta-57{&:not([aria-selected=true]){background-color:var(--rdg-header-background-color)}>.cj343x07-0-0-beta-57:not(:last-child,.csofj7r7-0-0-beta-57),>:nth-last-child(n+2 of .csofj7r7-0-0-beta-57){border-inline-end:none}}}@layer rdg.TextEditor{.t7vyx3i7-0-0-beta-57{appearance:none;background-color:var(--rdg-background-color);block-size:100%;border:2px solid #ccc;box-sizing:border-box;color:var(--rdg-color);font-family:inherit;font-size:var(--rdg-font-size);inline-size:100%;padding-block:0;padding-inline:6px;vertical-align:top;&:focus{border-color:var(--rdg-selection-color);outline:none}&::placeholder{color:#999;opacity:1}}}