"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[8638],{83560(a,t,_){_.d(t,{X:()=>u});var p=_(96540),o=_(36490);const u=({message:r,when:e=!0})=>{const s=o.Ny.getHistory();return(0,p.useEffect)(()=>{if(!e)return;const b=s.block(r);return()=>{b()}},[e,r,s]),null}},75103(a,t,_){a.exports=_.p+"static/img/dashboard_library_1.93b7e752.jpg"},7728(a,t,_){a.exports=_.p+"static/img/dashboard_library_2.e540bb1d.jpg"},52621(a,t,_){a.exports=_.p+"static/img/dashboard_library_3.8f41d293.jpg"},36422(a,t,_){a.exports=_.p+"static/img/dashboard_library_4.0a1e811b.jpg"},26531(a,t,_){a.exports=_.p+"static/img/dashboard_library_5.7f4bc831.jpg"},96820(a,t,_){a.exports=_.p+"static/img/dashboard_library_6.cf5e2209.jpg"},12326(a,t,_){a.exports=_.p+"static/img/grafana_text_logo_dark.70a1e5ee.svg"},71542(a,t,_){a.exports=_.p+"static/img/grafana_text_logo_light.467e047c.svg"}}]);

//# sourceMappingURL=DashboardScenePage.f5eefb27c46e9b0005de.js.map