"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[5354],{12057(u,o,t){t.r(o),t.d(o,{TestStuffPage:()=>i,default:()=>l});var n=t(14553),c=t(46907),s=t(45861),_=t(66404),d=t(41654),E=t(13791),f=t(64762);const i=()=>{const e={id:"test-page",text:"Test page",icon:"dashboard",subTitle:"FOR TESTING!",url:"sandbox/test"},a=(0,f._2)();return(0,n.jsxs)(E.Y,{navModel:{node:e,main:e},children:[(0,n.jsx)(g,{extensionPointId:"grafana/sandbox/testing"}),(0,n.jsx)(_.E,{variant:"h5",children:"Application notifications (toasts) testing"}),(0,n.jsxs)(d.B,{children:[(0,n.jsx)(s.$n,{onClick:()=>a.success("Success toast","some more text goes here"),variant:"primary",children:"Success"}),(0,n.jsx)(s.$n,{onClick:()=>a.warning("Warning toast","some more text goes here","bogus-trace-99999"),variant:"secondary",children:"Warning"}),(0,n.jsx)(s.$n,{onClick:()=>a.error("Error toast","some more text goes here","bogus-trace-fdsfdfsfds"),variant:"destructive",children:"Error"})]})]})};function g({extensionPointId:e}){const{links:a}=(0,c.U)({extensionPointId:e});return a.length===0?null:(0,n.jsx)("div",{children:a.map((r,P)=>(0,n.jsx)(s.z9,{href:r.path,title:r.description,children:r.title},r.id))})}const l=i}}]);

//# sourceMappingURL=TestStuffPage-react19.dee562c945317fa489a8.js.map