"use strict";(globalThis.webpackChunkmssql=globalThis.webpackChunkmssql||[]).push([[685],{6685(e){e.exports=JSON.parse('{"grafana-sql":{"components":{"confirm-modal":{"builder-mode":"Režim nástroje pro tvorbu nezobrazuje změny provedené v kódu. Nástroj pro tvorbu dotazů zobrazí poslední změny provedené v režimu nástroje pro tvorbu.","cancel":"Zrušit","clipboard":"Chcete zkopírovat kód do schránky?","copy-code-and-switch":"Kopírovat kód a přepnout","discard-code-and-switch":"Zahodit kód a přepnout","warning":"Varování"},"connection-limits":{"auto-max-idle":"Auto. max. nečinný","content-auto-max-idle":"","content-max-idle":"Maximální počet připojení v nečinném připojovacím souboru. Pokud je <i>maximální počet otevřených připojení</i> větší než 0, ale menší než <i>maximální počet nečinných připojení</i>, pak bude <i>maximální počet nečinných připojení</i> snížen tak, aby odpovídal limitu <i>maximálního počtu otevřených připojení</i>. Pokud je nastaveno na 0, nejsou zachována žádná nečinná připojení.","content-max-lifetime":"Maximální doba v sekundách, po kterou může být připojení znovu použito. Pokud je nastaveno na 0, připojení jsou znovu používána opakovaně.","content-max-open":"","max-idle":"Max. nečinný","max-lifetime":"Max. životnost","max-open":"Max. otevřený","title-connection-limits":"Omezení spojení"},"dataset-selector":{"aria-label-dataset-selector":"Volič datové sady"},"error-boundary":{"fall-back":{"error":"Chyba"}},"get-custom-operators":{"custom-operators":{"label":{"macros":"Makra"}}},"make-render-column":{"render-column":{"aria-label-group-by":"Seřadit podle","title-remove-group-by-column":"Odebrat skupinu podle sloupce"}},"order-by-row":{"aria-label-order-by":"Řadit podle","label-limit":"Limit","label-offset":"Odchylka","label-order-by":"Řadit podle"},"preview":{"label-element":{"preview":"Náhled","tooltip-copy-to-clipboard":"Kopírovat do schránky"}},"query-header":{"content-invalid-query":"","editor-modes":{"label-builder":"Nástroj pro tvorbu","label-code":"Kód"},"label-dataset":"Soubor dat","label-filter":"Filtrovat","label-format":"Formát","label-group":"Skupina","label-order":"Objednávka","label-preview":"Náhled","label-table":"Tabulka","placeholder-select-format":"Zvolit formát","run-query":"Spustit dotaz"},"query-meta":{"variables":{"textField":"Textové pole","valueField":"Pole hodnoty"}},"query-toolbox":{"content-hit-ctrlcmdreturn-to-run-query":"Stiskněte CTRL/CMD + Return pro spuštění dotazu","tooltip-collapse":"Sbalit editor","tooltip-expand":"Rozbalit editor","tooltip-format-query":"Formátovat dotaz"},"query-validator":{"query-will-process":"<0></0> Tento dotaz zpracuje <strong>{{bytes}}</strong> při spuštění.","validating-query":"Ověřování dotazu…"},"raw-editor":{"render-placeholder":{"editing-in-expanded-code-editor":"Úpravy v rozšířeném editoru kódu"},"title-query-num":"Dotaz {{queryNum}}"},"select-column":{"label-column":"Sloupec"},"select-custom-function-parameters":{"aria-label-parameter":"Parametr {{index}} pro sloupec {{columnIndex}}","render-parameters":{"params":{"title-remove-parameter":"Odebrat parametr"}},"title-add-parameter":"Přidat parametr"},"select-row":{"aggregate-options":{"options":{"label":{"aggregations":"Agregace","macros":"Makra"}}},"label":{"time":"čas","value":"hodnota"},"label-alias":"Přezdívka","label-data-operations":"Datové operace","title-add-column":"Přidat sloupec","title-remove-column":"Odebrat sloupec"},"settings":{"aria-label-conjunction":"Konjunkce","aria-label-field":"Pole","aria-label-operator":"Operátor","title-button-filter":"{{ buttonLabel }} filtr"},"sql-query-editor-lazy":{"text-loading-editor":"Načítání editoru"},"table-selector":{"aria-label-table-selector":"Volič tabulky","placeholder-loading":"Načítání tabulek","placeholder-select-table":"Vyberte tabulku"},"tlssecrets-config":{"content-tlsssl-client-certificate":"Pro ověření pomocí klientského certifikátu TLS/SSL zadejte klientský certifikát zde.","content-tlsssl-client-key":"Pro ověření pomocí klientského certifikátu TLS/SSL zadejte klíč zde.","content-tlsssl-root-certificate":"Pokud vybraný režim TLS/SSL vyžaduje kořenový certifikát serveru, zadejte ho zde","tlsssl-client-certificate":"Certifikát klienta TLS/SSL","tlsssl-client-key":"Klíč klienta TLS/SSL","tlsssl-root-certificate":"Kořenový certifikát TLS/SSL"},"visual-editor":{"label-filter-by-column-value":"Filtrovat podle hodnoty sloupce","label-group-by-column":"Seřadit podle sloupce"},"widgets":{"aria-label-macros-value-selector":"Volič hodnoty maker"}},"utils":{"get-columns-width-indices":{"label-selected-columns":"Vybrané sloupce"}}}}')}}]);