// Copyright (c) 2017 Uber Technologies, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import type * as React from 'react';

import type TNil from '../../types/TNil';

import type DraggableManager from './DraggableManager';
import type EUpdateTypes from './EUpdateTypes';

export type DraggableBounds = {
  clientXLeft: number;
  maxValue?: number;
  minValue?: number;
  width: number;
};

export type DraggingUpdate = {
  event: React.MouseEvent<unknown> | MouseEvent;
  manager: DraggableManager;
  tag: string | TNil;
  type: EUpdateTypes;
  value: number;
  x: number;
};
