"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[4564],{34564(m,s,r){r.r(s),r.d(s,{RudderstackBackend:()=>o});var d=r(33749),i=r(78646),c=r(25240);class o{constructor(e){this.options=e,this.supportedEvents=[i.on.Pageview,i.on.Interaction,i.on.ExperimentView],this.addEvent=a=>{window.rudderanalytics&&((0,d.nB)(a)&&window.rudderanalytics.page?.(),(0,d.fX)(a)&&window.rudderanalytics.track?.(a.payload.interactionName,a.payload.properties),(0,d.p6)(a)&&window.rudderanalytics.track?.("experiment_viewed",{experiment_id:a.payload.experimentId,experiment_group:a.payload.experimentGroup,experiment_variant:a.payload.experimentVariant}))},this.flush=()=>{};const f=e.sdkUrl||"https://cdn.rudderlabs.com/v1/rudder-analytics.min.js";(0,c.k0)(f);const u=window.rudderanalytics=[],l=["load","page","track","identify","alias","group","ready","reset","getAnonymousId","setAnonymousId"];for(let a=0;a<l.length;a++){const n=l[a];u[n]=function(t){return function(){u.push([t].concat(Array.prototype.slice.call(arguments)))}}(n)}if(window.rudderanalytics?.load?.(e.writeKey,e.dataPlaneUrl,{configUrl:e.configUrl,destSDKBaseURL:e.integrationsUrl}),e.user){const{identifier:a,intercomIdentifier:n}=e.user.analytics,t={};n&&(t.Intercom={user_hash:n}),window.rudderanalytics?.identify?.(a,{email:e.user.email,orgId:e.user.orgId,language:e.user.language,version:e.buildInfo.version,edition:e.buildInfo.edition},t)}}}}}]);

//# sourceMappingURL=4564.eb5af646fb65fe0e423f.js.map