"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[1825],{61825(f,d,n){n.r(d),n.d(d,{RudderstackBackend:()=>o});var r=n(33749),t=n(78646),l=n(25240);class o{constructor(a){this.options=a,this.supportedEvents=[t.on.Pageview,t.on.Interaction,t.on.ExperimentView],this.addEvent=e=>{window.rudderanalytics&&((0,r.nB)(e)&&window.rudderanalytics.page?.(),(0,r.fX)(e)&&window.rudderanalytics.track?.(e.payload.interactionName,e.payload.properties),(0,r.p6)(e)&&window.rudderanalytics.track?.("experiment_viewed",{experiment_id:e.payload.experimentId,experiment_group:e.payload.experimentGroup,experiment_variant:e.payload.experimentVariant}))},this.flush=()=>{};const u=a.sdkUrl||"https://cdn.rudderlabs.com/v3/modern/rsa.min.js";(0,l.k0)(u);const i=[];if(["setDefaultInstanceKey","load","ready","page","track","identify","alias","group","reset","setAnonymousId","startSession","endSession","consent","addCustomIntegration"].forEach(e=>{Object.assign(i,{[e]:function(){i.push([e].concat(Array.prototype.slice.call(arguments)))}})}),Object.assign(window,{rudderanalytics:i}),window.rudderanalytics?.load?.(a.writeKey,a.dataPlaneUrl,{configUrl:a.configUrl,destSDKBaseURL:a.integrationsUrl,storage:{encryption:{version:"legacy"},migrate:!1}}),a.user){const{identifier:e,intercomIdentifier:s}=a.user.analytics,c={};s&&(c.Intercom={user_hash:s}),window.rudderanalytics?.identify?.(e,{email:a.user.email,orgId:a.user.orgId,language:a.user.language,version:a.buildInfo.version,edition:a.buildInfo.edition},c)}window.rudderanalytics?.page?.()}}}}]);

//# sourceMappingURL=1825-react19.09a65c7fedba9719a45a.js.map