"use strict";(globalThis.webpackChunkmssql=globalThis.webpackChunkmssql||[]).push([[880],{880(e){e.exports=JSON.parse('{"grafana-sql":{"components":{"confirm-modal":{"builder-mode":"В режиме конструктора не отображаются изменения, внесенные в код. Конструктор запросов отобразит последние изменения, внесенные вами в режиме конструктора.","cancel":"Отмена","clipboard":"Копировать код в буфер обмена?","copy-code-and-switch":"Копировать код и переключиться","discard-code-and-switch":"Отменить код и переключиться","warning":"Предупреждение"},"connection-limits":{"auto-max-idle":"Автоматический максимум неактивных подключений","content-auto-max-idle":"","content-max-idle":"Максимальное количество подключений в пуле неактивных подключений. Если значение параметра <i>Макс. кол-во открытых подключений</i> больше 0, но меньше значения <i>Макс. кол-во неактивных подключений</i>, то значение <i>Макс. кол-во неактивных подключений</i> уменьшается в соответствии с предельным значением параметра <i>Макс. кол-во открытых подключений</i>. Если установлено значение 0, неактивные подключения не сохраняются.","content-max-lifetime":"Максимальное время (в секундах), в течение которого подключение может быть использовано повторно. Если установлено значение 0, подключения всегда используются повторно.","content-max-open":"","max-idle":"Макс. кол-во неактивных подключений","max-lifetime":"Макс. время жизни","max-open":"Макс. кол-во открытых подключений","title-connection-limits":"Ограничения на кол-во подключений"},"dataset-selector":{"aria-label-dataset-selector":"Селектор наборов данных"},"error-boundary":{"fall-back":{"error":"Ошибка"}},"get-custom-operators":{"custom-operators":{"label":{"macros":"Макрос"}}},"make-render-column":{"render-column":{"aria-label-group-by":"Группировать по","title-remove-group-by-column":"Удалите группу по столбцу"}},"order-by-row":{"aria-label-order-by":"Упорядочить по","label-limit":"Предельное значение","label-offset":"Смещение","label-order-by":"Упорядочить по"},"preview":{"label-element":{"preview":"Предварительный просмотр","tooltip-copy-to-clipboard":"Копировать в буфер обмена"}},"query-header":{"content-invalid-query":"","editor-modes":{"label-builder":"Конструктор","label-code":"Код"},"label-dataset":"Набор данных","label-filter":"Фильтр","label-format":"Формат","label-group":"Группа","label-order":"Порядок","label-preview":"Предварительный просмотр","label-table":"Таблица","placeholder-select-format":"Выбрать формат","run-query":"Выполнить запрос"},"query-meta":{"variables":{"textField":"Текстовое поле","valueField":"Поле значения"}},"query-toolbox":{"content-hit-ctrlcmdreturn-to-run-query":"Нажмите CTRL/CMD+Return, чтобы выполнить запрос","tooltip-collapse":"Свернуть редактор","tooltip-expand":"Развернуть редактор","tooltip-format-query":"Форматировать запрос"},"query-validator":{"query-will-process":"<0></0> При выполнении запрос обработает <strong>{{bytes}}</strong>.","validating-query":"Проверка запроса..."},"raw-editor":{"render-placeholder":{"editing-in-expanded-code-editor":"Редактирование в расширенном редакторе кода"},"title-query-num":"Запрос {{queryNum}}"},"select-column":{"label-column":"Столбец"},"select-custom-function-parameters":{"aria-label-parameter":"Параметр {{index}} для столбца {{columnIndex}}","render-parameters":{"params":{"title-remove-parameter":"Удалить параметр"}},"title-add-parameter":"Добавить параметр"},"select-row":{"aggregate-options":{"options":{"label":{"aggregations":"Агрегации","macros":"Макрос"}}},"label":{"time":"время","value":"значение"},"label-alias":"Псевдоним","label-data-operations":"Операции с данными","title-add-column":"Добавить столбец","title-remove-column":"Удалить столбец"},"settings":{"aria-label-conjunction":"Соединение","aria-label-field":"Поле","aria-label-operator":"Оператор","title-button-filter":"Фильтр {{ buttonLabel }}"},"sql-query-editor-lazy":{"text-loading-editor":"Загрузка редактора"},"table-selector":{"aria-label-table-selector":"Селектор таблиц","placeholder-loading":"Загрузка таблиц","placeholder-select-table":"Выбрать таблицу"},"tlssecrets-config":{"content-tlsssl-client-certificate":"Чтобы выполнить аутентификацию с помощью сертификата клиента TLS/SSL, укажите сертификат клиента здесь.","content-tlsssl-client-key":"Чтобы выполнить аутентификацию с помощью сертификата клиента TLS/SSL, укажите ключ здесь.","content-tlsssl-root-certificate":"Если выбранный режим TLS/SSL требует корневого сертификата сервера, укажите его здесь.","tlsssl-client-certificate":"Сертификат клиента TLS/SSL","tlsssl-client-key":"Ключ клиента TLS/SSL","tlsssl-root-certificate":"Корневой сертификат TLS/SSL"},"visual-editor":{"label-filter-by-column-value":"Фильтр по значению столбца","label-group-by-column":"Группировать по столбцу"},"widgets":{"aria-label-macros-value-selector":"Селектор значений макросов"}},"utils":{"get-columns-width-indices":{"label-selected-columns":"Выбранные столбцы"}}}}')}}]);