"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[7967],{10348(u,i,s){s.d(i,{SuggestionsDrawerButton:()=>c});var n=s(74848),t=s(22803),e=s(92745),l=s(45861),_=s(66404),r=s(41654),g=s(63142);const c=({suggestions:a,handleOpenDrawer:d})=>{const o=(0,g.of)(E);return(0,n.jsx)("div",{className:o.buttonWrapper,"data-testid":"suggestions-badge",children:(0,n.jsx)(l.$n,{variant:"secondary",fill:"outline",size:"sm",onClick:d,icon:"list-ol",children:(0,n.jsxs)(r.B,{direction:"row",gap:1,alignItems:"center",children:[(0,n.jsx)(e.x6,{i18nKey:"sql-expressions.suggestions",children:"Suggestions"}),(0,n.jsx)("span",{className:o.countBadge,children:(0,n.jsx)(_.E,{variant:"bodySmall",weight:"bold",children:a.length})})]})})})},E=a=>({countBadge:(0,t.css)({color:a.colors.primary.text,fontWeight:"bold"}),buttonWrapper:(0,t.css)({position:"relative",display:"inline-block"})})}}]);

//# sourceMappingURL=7967.384d483304c52f315993.js.map