"use strict";(globalThis.webpackChunkmssql=globalThis.webpackChunkmssql||[]).push([[30],{30(e){e.exports=JSON.parse('{"grafana-sql":{"components":{"confirm-modal":{"builder-mode":"Tryb konstruktora nie wyświetla zmian wprowadzonych w kodzie. Konstruktor zapytań wyświetla ostatnie zmiany wprowadzone w trybie konstruktora.","cancel":"Anuluj","clipboard":"Czy chcesz skopiować kod do schowka?","copy-code-and-switch":"Skopiuj kod i przełącz","discard-code-and-switch":"Odrzuć kod i przełącz","warning":"Ostrzeżenie"},"connection-limits":{"auto-max-idle":"Automatyczny maksymalny czas bezczynności","content-auto-max-idle":"","content-max-idle":"Maksymalna liczba połączeń w puli bezczynnych połączeń. Jeśli <i>maksymalna liczba otwartych połączeń</i> jest większa niż 0, ale mniejsza niż <i>maksymalna liczba bezczynnych połączeń</i>, to <i>maksymalna liczba bezczynnych połączeń</i> zostanie zmniejszona, aby dopasować ją do limitu <i>maksymalnej liczby otwartych połączeń</i>. Jeśli ustawiono wartość 0, nie są zachowywane żadne bezczynne połączenia.","content-max-lifetime":"Maksymalny czas w sekundach, przez który połączenie może być ponownie użyte. Jeśli ustawiono wartość 0, połączenia są ponownie wykorzystywane bez daty zakończenia.","content-max-open":"","max-idle":"Maks. czas bezczynności","max-lifetime":"Maks. czas istnienia","max-open":"Maks. czas otwarcia","title-connection-limits":"Limity połączeń"},"dataset-selector":{"aria-label-dataset-selector":"Selektor zbioru danych"},"error-boundary":{"fall-back":{"error":"Błąd"}},"get-custom-operators":{"custom-operators":{"label":{"macros":"Makra"}}},"make-render-column":{"render-column":{"aria-label-group-by":"Grupuj według","title-remove-group-by-column":"Usuń grupę według kolumny"}},"order-by-row":{"aria-label-order-by":"Kolejność według","label-limit":"Limit","label-offset":"Przesunięcie","label-order-by":"Kolejność według"},"preview":{"label-element":{"preview":"Podgląd","tooltip-copy-to-clipboard":"Kopiuj do schowka"}},"query-header":{"content-invalid-query":"","editor-modes":{"label-builder":"Konstruktor","label-code":"Kod"},"label-dataset":"Zbiór danych","label-filter":"Filtr","label-format":"Format","label-group":"Grupa","label-order":"Kolejność","label-preview":"Podgląd","label-table":"Tabela","placeholder-select-format":"Wybierz format","run-query":"Uruchom zapytanie"},"query-meta":{"variables":{"textField":"Pole tekstowe","valueField":"Pole wartości"}},"query-toolbox":{"content-hit-ctrlcmdreturn-to-run-query":"Naciśnij CTRL/CMD + Return, aby uruchomić zapytanie","tooltip-collapse":"Zwiń edytor","tooltip-expand":"Rozwiń edytor","tooltip-format-query":"Formatuj zapytanie"},"query-validator":{"query-will-process":"<0></0> To zapytanie przetworzy <strong>{{bytes}}</strong> po uruchomieniu.","validating-query":"Sprawdzanie poprawności zapytania…"},"raw-editor":{"render-placeholder":{"editing-in-expanded-code-editor":"Edycja w rozszerzonym edytorze kodu"},"title-query-num":"Zapytanie {{queryNum}}"},"select-column":{"label-column":"Kolumna"},"select-custom-function-parameters":{"aria-label-parameter":"Parametr {{index}} dotyczący kolumny {{columnIndex}}","render-parameters":{"params":{"title-remove-parameter":"Usuń parametr"}},"title-add-parameter":"Dodaj parametr"},"select-row":{"aggregate-options":{"options":{"label":{"aggregations":"Agregacje","macros":"Makra"}}},"label":{"time":"czas","value":"wartość"},"label-alias":"Alias","label-data-operations":"Operacje na danych","title-add-column":"Dodaj kolumnę","title-remove-column":"Usuń kolumnę"},"settings":{"aria-label-conjunction":"Koniunkcja","aria-label-field":"Pole","aria-label-operator":"Operator","title-button-filter":"Filtr: {{ buttonLabel }}"},"sql-query-editor-lazy":{"text-loading-editor":"Ładowanie edytora…"},"table-selector":{"aria-label-table-selector":"Selektor tabel","placeholder-loading":"Ładowanie tabel","placeholder-select-table":"Wybierz tabelę"},"tlssecrets-config":{"content-tlsssl-client-certificate":"Aby uwierzytelnić się za pomocą certyfikatu klienta TLS/SSL, podaj tutaj certyfikat klienta.","content-tlsssl-client-key":"Aby uwierzytelnić się za pomocą certyfikatu klienta TLS/SSL, podaj tutaj klucz.","content-tlsssl-root-certificate":"Jeśli wybrany tryb TLS/SSL wymaga certyfikatu głównego serwera, podaj go tutaj","tlsssl-client-certificate":"Certyfikat klienta TLS/SSL","tlsssl-client-key":"Klucz TLS/SSL klienta","tlsssl-root-certificate":"Certyfikat główny TLS/SSL"},"visual-editor":{"label-filter-by-column-value":"Filtruj według wartości kolumny","label-group-by-column":"Grupuj wg kolumny"},"widgets":{"aria-label-macros-value-selector":"Selektor wartości makr"}},"utils":{"get-columns-width-indices":{"label-selected-columns":"Wybrane kolumny"}}}}')}}]);