"use strict";(globalThis.webpackChunkmssql=globalThis.webpackChunkmssql||[]).push([[852],{4852(e){e.exports=JSON.parse('{"grafana-sql":{"components":{"confirm-modal":{"builder-mode":"Az építő mód nem jeleníti meg a kódban végzett módosításokat. A lekérdezésépítő megjeleníti az utolsó módosításokat, amelyeket az építő módban végzett.","cancel":"Mégse","clipboard":"Szeretné a kódot a vágólapra másolni?","copy-code-and-switch":"Kód másolása és váltás","discard-code-and-switch":"Kód elvetése és váltás","warning":"Figyelmeztetés"},"connection-limits":{"auto-max-idle":"Inaktív automatikus maximális száma","content-auto-max-idle":"","content-max-idle":"Az inaktív kapcsolatok készletében lévő kapcsolatok maximális száma. Ha a <i>Megnyitott kapcsolatok maximális száma</i> nagyobb, mint 0, de kisebb, mint az <i>Inaktív kapcsolatok maximális száma</i>, akkor az <i>Inaktív kapcsolatok maximális száma</i> csökken, hogy megfeleljen a <i>Megnyitott kapcsolatok maximális száma</i> határértékének. Ha 0-ra van állítva, akkor egyetlen inaktív kapcsolat sem marad meg.","content-max-lifetime":"A csatlakozás újrafelhasználásának maximális időtartama másodpercben. Ha 0-ra van állítva, a kapcsolatok újrafelhasználása végtelen.","content-max-open":"","max-idle":"Max. inaktív","max-lifetime":"Max. élettartam","max-open":"Max. nyitott","title-connection-limits":"Kapcsolati korlátozások"},"dataset-selector":{"aria-label-dataset-selector":"Adatkészlet-választó"},"error-boundary":{"fall-back":{"error":"Hiba"}},"get-custom-operators":{"custom-operators":{"label":{"macros":"Makrók"}}},"make-render-column":{"render-column":{"aria-label-group-by":"Csoportosítási szempont","title-remove-group-by-column":"Csoport eltávolítása oszlop szerint"}},"order-by-row":{"aria-label-order-by":"Rendezési szempont","label-limit":"Korlát","label-offset":"Eltolás","label-order-by":"Rendezési szempont"},"preview":{"label-element":{"preview":"Előnézet","tooltip-copy-to-clipboard":"Másolás vágólapra"}},"query-header":{"content-invalid-query":"","editor-modes":{"label-builder":"Építő","label-code":"Kód"},"label-dataset":"Adatkészlet","label-filter":"Szűrő","label-format":"Formátum","label-group":"Csoport","label-order":"Sorrend","label-preview":"Előnézet","label-table":"Táblázat","placeholder-select-format":"Formátum kiválasztása","run-query":"Lekérdezés"},"query-meta":{"variables":{"textField":"Szöveges mező","valueField":"Értékmező"}},"query-toolbox":{"content-hit-ctrlcmdreturn-to-run-query":"A lekérdezés futtatásához nyomja le a CTRL/CMD+Enter billentyűkombinációt","tooltip-collapse":"Szerkesztő összecsukása","tooltip-expand":"Szerkesztő kibontása","tooltip-format-query":"Lekérdezés formázása"},"query-validator":{"query-will-process":"<0></0> Ez a lekérdezés <strong>{{bytes}}</strong> feldolgozását végzi futtatáskor.","validating-query":"Lekérdezés érvényesítése…"},"raw-editor":{"render-placeholder":{"editing-in-expanded-code-editor":"Szerkesztés kibontott kódszerkesztőben"},"title-query-num":"{{queryNum}}. lekérdezés"},"select-column":{"label-column":"Oszlop"},"select-custom-function-parameters":{"aria-label-parameter":"{{columnIndex}}. oszlop {{index}}. paramétere","render-parameters":{"params":{"title-remove-parameter":"Paraméter eltávolítása"}},"title-add-parameter":"Paraméter hozzáadása"},"select-row":{"aggregate-options":{"options":{"label":{"aggregations":"Összegzések","macros":"Makrók"}}},"label":{"time":"idő","value":"érték"},"label-alias":"Alias","label-data-operations":"Adatműveletek","title-add-column":"Oszlop hozzáadása","title-remove-column":"Oszlop eltávolítása"},"settings":{"aria-label-conjunction":"Összekötés","aria-label-field":"Mező","aria-label-operator":"Műveleti jel","title-button-filter":"{{ buttonLabel }} szűrő"},"sql-query-editor-lazy":{"text-loading-editor":"Szerkesztő betöltése"},"table-selector":{"aria-label-table-selector":"Táblázatválasztó","placeholder-loading":"Táblázatok betöltése","placeholder-select-table":"Táblázat kiválasztása"},"tlssecrets-config":{"content-tlsssl-client-certificate":"A TLS-/SSL-klienstanúsítvánnyal történő hitelesítéshez adja meg itt a klienstanúsítványt.","content-tlsssl-client-key":"A TLS-/SSL-klienstanúsítvánnyal történő hitelesítéshez adja meg itt a kulcsot.","content-tlsssl-root-certificate":"Ha a kiválasztott TLS-/SSL-módhoz kiszolgáló-gyökértanúsítvány szükséges, adja meg itt","tlsssl-client-certificate":"TLS-/SSL-klienstanúsítvány","tlsssl-client-key":"TLS-/SSL-klienskulcs","tlsssl-root-certificate":"TLS-/SSL-gyökértanúsítvány"},"visual-editor":{"label-filter-by-column-value":"Szűrés oszlopérték szerint","label-group-by-column":"Csoportosítás oszlop szerint"},"widgets":{"aria-label-macros-value-selector":"Makróérték-választó"}},"utils":{"get-columns-width-indices":{"label-selected-columns":"Kijelölt oszlopok"}}}}')}}]);