// Code generated - EDITING IS FUTILE. DO NOT EDIT.
//
// Generated by:
//     public/app/plugins/gen.go
// Using jennies:
//     TSTypesJenny
//     PluginTsTypesJenny
//
// Run 'make gen-cue' from repository root to regenerate.

// Generated from public/app/plugins/datasource/parca/dataquery.cue file.

import * as common from '@grafana/schema';

export type ParcaQueryType = ('metrics' | 'profile' | 'both');

export const defaultParcaQueryType: ParcaQueryType = 'both';

export interface ParcaDataQuery extends common.DataQuery {
  /**
   * Specifies the query label selectors.
   */
  labelSelector: string;
  /**
   * Specifies the type of profile to query.
   */
  profileTypeId: string;
}

export const defaultParcaDataQuery: Partial<ParcaDataQuery> = {
  labelSelector: '{}',
};
