"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[8463],{21369(h,t,a){a.r(t),a.d(t,{plugin:()=>f});var i=a(16207),s=a(74848),e=a(22803),o=a(92745),r=a(63142),c=a(89640);const p=[{value:0,label:"Documentation",href:"https://grafana.com/docs/grafana/latest"},{value:1,label:"Tutorials",href:"https://grafana.com/tutorials"},{value:2,label:"Community",href:"https://community.grafana.com"},{value:3,label:"Public Slack",href:"http://slack.grafana.com"}],g=()=>{const n=(0,r.of)(d);return(0,s.jsxs)("div",{className:n.container,children:[(0,s.jsx)("h1",{className:n.title,children:(0,s.jsx)(o.x6,{i18nKey:"welcome.welcome-banner.welcome-to-grafana",children:"Welcome to Grafana"})}),(0,s.jsxs)("div",{className:n.help,children:[(0,s.jsx)("h2",{className:n.helpText,children:(0,s.jsx)(o.x6,{i18nKey:"welcome.welcome-banner.need-help",children:"Need help?"})}),(0,s.jsx)("div",{className:n.helpLinks,children:p.map((l,m)=>(0,s.jsx)(c.Y,{href:`${l.href}?utm_source=grafana_gettingstarted`,external:!0,inline:!1,children:l.label},`${l.label}-${m}`))})]})]})},d=n=>({container:(0,e.css)({display:"flex",backgroundSize:"cover",height:"100%",alignItems:"center",justifyContent:"space-between",padding:n.spacing(0,3),[n.breakpoints.down("lg")]:{backgroundPosition:"0px",flexDirection:"column",alignItems:"flex-start",justifyContent:"center"},[n.breakpoints.down("sm")]:{padding:n.spacing(0,1)}}),title:(0,e.css)({marginBottom:0,[n.breakpoints.down("lg")]:{marginBottom:n.spacing(1)},[n.breakpoints.down("md")]:{fontSize:n.typography.h2.fontSize},[n.breakpoints.down("sm")]:{fontSize:n.typography.h3.fontSize}}),help:(0,e.css)({display:"flex",alignItems:"baseline"}),helpText:(0,e.css)({...n.typography.h3,marginRight:n.spacing(2),marginBottom:0,[n.breakpoints.down("md")]:{fontSize:n.typography.h4.fontSize},[n.breakpoints.down("sm")]:{display:"none"}}),helpLinks:(0,e.css)({display:"flex",flexWrap:"wrap",gap:n.spacing(2),textWrap:"nowrap",[n.breakpoints.down("sm")]:{gap:n.spacing(1)}})}),f=new i.m(g).setNoPadding()}}]);

//# sourceMappingURL=welcomeBanner.5c1db04280547c11e886.js.map