"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[3511],{86559(e,a,s){e.exports=s.p+"static/img/loki_icon.abc01135.svg"},27323(e,a,s){e.exports=s.p+"static/img/mimir_logo.ac01ecbc.svg"},33611(e,a,s){e.exports=s.p+"static/img/prometheus_logo.3e9d4662.svg"}}]);

//# sourceMappingURL=AlertRuleListIndex-react19.2356635de8a332781144.js.map