// Code generated - EDITING IS FUTILE. DO NOT EDIT.
//
// Generated by:
//     public/app/plugins/gen.go
// Using jennies:
//     TSTypesJenny
//     PluginTsTypesJenny
//
// Run 'make gen-cue' from repository root to regenerate.

// Generated from public/app/plugins/panel/timeseries/panelcfg.cue file.

import * as common from '@grafana/schema';

export interface TimeSeriesLegendOptions extends common.VizLegendOptions {
  enableFacetedFilter?: boolean;
}

export const defaultTimeSeriesLegendOptions: Partial<TimeSeriesLegendOptions> = {
  enableFacetedFilter: true,
};

export interface Options extends common.OptionsWithTimezones, common.OptionsWithAnnotations {
  disableKeyboardEvents?: boolean;
  legend: TimeSeriesLegendOptions;
  orientation?: common.VizOrientation;
  timeCompare?: common.TimeCompareOptions;
  tooltip: common.VizTooltipOptions;
}

export interface FieldConfig extends common.GraphFieldConfig {}
