"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[5686],{88888(F,i,a){a.r(i),a.d(i,{plugin:()=>p});var s=a(83340),f=a(16207),o=a(97369),r=a(74848),m=a(1906),g=a(84033),u=a(27489),d=a(43173),v=a(14477);function l(e,n={}){(0,u.rR)(`grafana_flamegraph_${e}`,{app:m.Jk.Unknown,grafana_version:d.$.buildInfo.version,...n})}const h=e=>{const n=(0,o.oA)(e.data.series[0]);return n?(0,r.jsx)(v.a,{panelId:e.id,data:e.data,message:(0,o.Ik)(n)}):(0,r.jsx)(g.Ay,{data:e.data.series[0],stickyHeader:!1,getTheme:()=>d.$.theme2,showFlameGraphOnly:e.options?.showFlameGraphOnly??!1,onTableSymbolClick:()=>l("table_item_selected"),onViewSelected:t=>l("view_selected",{view:t}),onTextAlignSelected:t=>l("text_align_selected",{align:t}),onTableSort:t=>l("table_sort_selected",{sort:t})})},c=[s.uo.Unit,s.uo.Decimals],p=new f.m(h).setSuggestionsSupplier(e=>{if(e.rawFrames?.some(n=>(0,o.oA)(n)===void 0))return[{cardOptions:{previewModifier:n=>{n.options=n.options||{},n.options.showFlameGraphOnly=!0}}}]}).useFieldConfig({disableStandardOptions:Object.values(s.uo).filter(e=>!c.includes(e))})}}]);

//# sourceMappingURL=flamegraphPanel.067335d514200367dc12.js.map