"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[7659],{29154(Y,a,n){n.r(a),n.d(a,{default:()=>T});var t=n(74848),x=n(16817),g=n(92745),v=n(68143),f=n(34999),o=n(13791),c=n(96540),d=n(70255),h=n(92807),m=n(66404),y=n(21348);const A=({settings:e})=>(0,t.jsx)(h.P,{overflowY:"visible",overflowX:"auto",width:"100%",children:(0,t.jsx)("table",{className:"filter-table",children:(0,t.jsx)("tbody",{children:Object.entries(e).map(([s,i],l)=>(0,t.jsxs)(c.Fragment,{children:[(0,t.jsxs)("tr",{children:[(0,t.jsx)("td",{children:(0,t.jsx)(m.E,{color:"info",weight:"bold",children:s})}),(0,t.jsx)("td",{})]}),Object.entries(i).map(([p,C],N)=>(0,t.jsxs)("tr",{children:[(0,t.jsx)("td",{style:{paddingLeft:"25px"},children:p}),(0,t.jsx)("td",{style:{whiteSpace:"break-spaces"},children:C})]},`property-${N}`))]},`section-${l}`))})})}),S=new Array(50).fill(null).map(()=>Math.random()),u=({rootProps:e})=>(0,t.jsx)(h.P,{overflowY:"visible",overflowX:"auto",width:"100%",children:(0,t.jsx)("table",{className:"filter-table",...e,children:(0,t.jsx)("tbody",{children:S.map((s,i)=>{const l=i===0||s>.9;return(0,t.jsxs)(c.Fragment,{children:[l&&(0,t.jsxs)("tr",{children:[(0,t.jsx)("td",{className:"admin-settings-section",children:(0,t.jsx)(d.A,{width:r(40,80,s)})}),(0,t.jsx)("td",{})]}),(0,t.jsxs)("tr",{children:[(0,t.jsx)("td",{style:{paddingLeft:"25px"},children:(0,t.jsx)(d.A,{width:r(60,100,s)})}),(0,t.jsx)("td",{children:(0,t.jsx)(d.A,{width:r(80,320,s)})})]})]},i)})})})});function r(e,s,i){return i*(s-e)+e}const j=(0,y.j)(A,u);function b(){const{loading:e,value:s}=(0,x.A)(()=>(0,v.AI)().get("/api/admin/settings"),[]);return(0,t.jsx)(o.Y,{navId:"server-settings",children:(0,t.jsxs)(o.Y.Contents,{children:[(0,t.jsx)(f.F,{severity:"info",title:"",children:(0,t.jsx)(g.x6,{i18nKey:"admin.settings.info-description",children:"These system settings are defined in grafana.ini or custom.ini (or overridden in ENV variables). To change these you currently need to restart Grafana."})}),e&&(0,t.jsx)(j.Skeleton,{}),s&&(0,t.jsx)(j,{settings:s})]})})}const T=b}}]);

//# sourceMappingURL=AdminSettings.a22806de83d848536ec0.js.map