"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[5173],{77267(t,a,n){n.r(a),n.d(a,{default:()=>i});var e=n(14553),l=n(92745),o=n(13791),r=n(45776);function i(){const s=(0,r.C)("profile-settings");return(0,e.jsx)(o.Y,{navModel:s,children:(0,e.jsxs)(o.Y.Contents,{children:[(0,e.jsx)("h1",{children:(0,e.jsx)(l.x6,{i18nKey:"profile.feature-toggle-page.profile-not-enabled",children:"Profile is not enabled."})}),(0,e.jsx)(l.x6,{i18nKey:"profile.feature-toggles-age.enable-in-config",children:"Enable profile in the Grafana config file."}),(0,e.jsx)("div",{children:(0,e.jsx)("pre",{children:`[profile]
enable = true
`})})]})})}}}]);

//# sourceMappingURL=ProfileFeatureTogglePage-react19.2e40a3f434579a2ba6df.js.map