"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[2910,5790,8926],{75521(x,a,t){t.r(a),t.d(a,{completionItemProvider:()=>b,language:()=>h,languageConfiguration:()=>u});for(var o=t(10858),u={wordPattern:/(-?\d*\.\d\w*)|([^`~!#%^&*()\-=+\[{\]}\\|;:'",.<>\/?\s]+)/g,comments:{lineComment:"#"},brackets:[["{","}"],["[","]"],["(",")"]],autoClosingPairs:[{open:"{",close:"}"},{open:"[",close:"]"},{open:"(",close:")"},{open:'"',close:'"'},{open:"'",close:"'"}],surroundingPairs:[{open:"{",close:"}"},{open:"[",close:"]"},{open:"(",close:")"},{open:'"',close:'"'},{open:"'",close:"'"},{open:"<",close:">"}],folding:{}},i=["sum","min","max","avg","group","stddev","stdvar","count","count_values","bottomk","topk","quantile","limitk","limit_ratio"],d=["abs","absent","ceil","changes","clamp","clamp_max","clamp_min","day_of_month","day_of_week","day_of_year","days_in_month","delta","deriv","exp","floor","histogram_avg","histogram_count","histogram_sum","histogram_fraction","histogram_quantile","histogram_stddev","histogram_stdvar","double_exponential_smoothing","holt_winters","hour","idelta","increase","info","irate","label_join","label_replace","ln","log2","log10","minute","month","predict_linear","rate","resets","round","scalar","sgn","sort","sort_desc","sort_by_label","sort_by_label_desc","sqrt","time","timestamp","vector","year"],f=["acos","acosh","asin","asinh","atan","atanh","cos","cosh","sin","sinh","tan","tanh","deg","pi","rad"],r=[],n=0,l=i;n<l.length;n++){var m=l[n];r.push(m+"_over_time")}var c=["on","ignoring","group_right","group_left","by","without"],p="(".concat(c.reduce(function(s,e){return"".concat(s,"|").concat(e)}),")"),_=["+","-","*","/","%","^","==","!=",">","<",">=","<=","and","or","unless"],v=["offset"],g=i.concat(d).concat(f).concat(r).concat(c).concat(v),h={ignoreCase:!1,defaultToken:"",tokenPostfix:".promql",keywords:g,operators:_,vectorMatching:p,symbols:/[=><!~?:&|+\-*\/^%]+/,escapes:/\\(?:[abfnrtv\\"']|x[0-9A-Fa-f]{1,4}|u[0-9A-Fa-f]{4}|U[0-9A-Fa-f]{8})/,digits:/\d+(_+\d+)*/,octaldigits:/[0-7]+(_+[0-7]+)*/,binarydigits:/[0-1]+(_+[0-1]+)*/,hexdigits:/[0-9a-fA-F]+(_+[0-9a-fA-F]+)*/,integersuffix:/(ll|LL|u|U|l|L)?(ll|LL|u|U|l|L)?/,floatsuffix:/[fFlL]?/,tokenizer:{root:[[/@vectorMatching\s*(?=\()/,"type","@clauses"],[/[a-z_]\w*(?=\s*(=|!=|=~|!~))/,"tag"],[/(^#.*$)/,"comment"],[/[a-zA-Z_]\w*/,{cases:{"@keywords":"type","@default":"identifier"}}],[/"([^"\\]|\\.)*$/,"string.invalid"],[/'([^'\\]|\\.)*$/,"string.invalid"],[/"/,"string","@string_double"],[/'/,"string","@string_single"],[/`/,"string","@string_backtick"],{include:"@whitespace"},[/[{}()\[\]]/,"@brackets"],[/[<>](?!@symbols)/,"@brackets"],[/@symbols/,{cases:{"@operators":"delimiter","@default":""}}],[/\d+[smhdwy]/,"number"],[/\d*\d+[eE]([\-+]?\d+)?(@floatsuffix)/,"number.float"],[/\d*\.\d+([eE][\-+]?\d+)?(@floatsuffix)/,"number.float"],[/0[xX][0-9a-fA-F']*[0-9a-fA-F](@integersuffix)/,"number.hex"],[/0[0-7']*[0-7](@integersuffix)/,"number.octal"],[/0[bB][0-1']*[0-1](@integersuffix)/,"number.binary"],[/\d[\d']*\d(@integersuffix)/,"number"],[/\d(@integersuffix)/,"number"]],string_double:[[/[^\\"]+/,"string"],[/@escapes/,"string.escape"],[/\\./,"string.escape.invalid"],[/"/,"string","@pop"]],string_single:[[/[^\\']+/,"string"],[/@escapes/,"string.escape"],[/\\./,"string.escape.invalid"],[/'/,"string","@pop"]],string_backtick:[[/[^\\`$]+/,"string"],[/@escapes/,"string.escape"],[/\\./,"string.escape.invalid"],[/`/,"string","@pop"]],clauses:[[/[^(,)]/,"tag"],[/\)/,"identifier","@pop"]],whitespace:[[/[ \t\r\n]+/,"white"]]}},b={provideCompletionItems:function(){var s=g.map(function(e){return{label:e,kind:o.languages.CompletionItemKind.Keyword,insertText:e,insertTextRules:o.languages.CompletionItemInsertTextRule.InsertAsSnippet}});return{suggestions:s}}}}}]);

//# sourceMappingURL=5790-react19.075ef29172cdecdbabb7.js.map