"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[2513],{95637(E,n,a){a.r(n),a.d(n,{plugin:()=>D});var u=a(94644),s=a(18615);const D=new u.tD(s.N8)}}]);

//# sourceMappingURL=mixedPlugin.05053e8b7167d1155395.js.map