"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[1649],{29668(v,a,e){e.r(a),e.d(a,{default:()=>m});var n=e(74848),i=e(54148),s=e(92745),r=e(39947),o=e(95443),c=e(63895),E=e(36826),u=e(35526),P=e(24158),g=e(5358),M=e(88547);const D=()=>{const t=(0,i.zy)(),l=new URLSearchParams(t.search),{selectedAlertmanager:_=""}=(0,g.VI)(),f=(0,c.ou)(l.getAll("matcher")).find(d=>d.name===o.kW)?.value,A=(0,r.Di)((0,r.LP)(l));return(0,n.jsxs)(n.Fragment,{children:[(0,n.jsx)(u.e,{currentAlertmanager:_}),(0,n.jsx)(P.SilencesEditor,{formValues:A,alertManagerSourceName:_,ruleUid:f})]})};function O(){const t={id:"silence-new",text:(0,s.t)("alerting.new-silence-page.page-nav.text.silence-alert-rule","Silence alert rule"),subTitle:(0,s.t)("alerting.new-silence-page.page-nav.subTitle.configure-silences-notifications-particular-alert","Configure silences to stop notifications from a particular alert rule")};return(0,n.jsx)(E.y,{navId:"silences",pageNav:t,accessType:"instance",children:(0,n.jsx)(D,{})})}const m=(0,M.S)(O)}}]);

//# sourceMappingURL=NewSilencePage.29a9546a2bb5dabcc1fb.js.map