"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[2699],{2699(r,o,i){i.d(o,{ApplicationInsightsBackend:()=>e});var a=i(33749),s=i(78646);class e{constructor(t){this.options=t,this.supportedEvents=[s.on.Pageview,s.on.Interaction],this.addEvent=n=>{window.applicationInsights&&((0,a.nB)(n)&&window.applicationInsights.trackPageView?.(),(0,a.fX)(n)&&window.applicationInsights.trackEvent?.({name:n.payload.interactionName,properties:n.payload.properties}))},this.flush=()=>{};const c={config:{connectionString:t.connectionString,endpointUrl:t.endpointUrl,enableAutoRouteTracking:t.autoRouteTracking}};System.import("https://js.monitor.azure.com/scripts/b/ai.2.min.js").then(n=>n.default?n.default:n).then(({ApplicationInsights:n})=>{const p=new n(c);window.applicationInsights=p.loadAppInsights()})}}}}]);

//# sourceMappingURL=2699.aeb05f01a43dfb722144.js.map