"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[5747],{75747(c,i,o){o.d(i,{BrowserConsoleBackend:()=>p});var a=o(33749),t=o(78646);class p{constructor(){this.options={},this.supportedEvents=[t.on.Pageview,t.on.Interaction,t.on.ExperimentView],this.addEvent=e=>{if((0,a.nB)(e)&&console.log("[EchoSrv:pageview]",e.payload.page),(0,a.fX)(e)){const r=e.payload.interactionName;console.log("[EchoSrv:event]",r,e.payload.properties);const s=Object.entries(e.payload.properties??{}).filter(([v,l])=>{const n=typeof l;return!(n==="string"||n==="number"||n==="boolean"||n==="undefined")});s.length>0&&console.warn("Event",r,"has invalid property types. Event properties should only be string, number or boolean. Invalid properties:",Object.fromEntries(s))}(0,a.p6)(e)&&console.log("[EchoSrv:experiment]",e.payload)},this.flush=()=>{}}}}}]);

//# sourceMappingURL=5747.d5d40c1f1a3d9b02e184.js.map