"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[7616],{95594(d,s,a){a.r(s),a.d(s,{PlaylistNewPage:()=>n,default:()=>y});var t=a(14553),_=a(55501),o=a(92745),E=a(36490),l=a(13791),P=a(45397),i=a(81425),e=a(18984);const n=()=>{const[r]=(0,_.useState)((0,e.aV)()),[D]=(0,P.Dp)(),O=async p=>{await D({playlist:p}),E.Ny.push("/playlists")},M={text:(0,o.t)("playlist.playlist-new-page.page-nav.text.new-playlist","New playlist"),subTitle:"A playlist rotates through a pre-selected list of dashboards. A playlist can be a great way to build situational awareness, or just show off your metrics to your team or visitors."};return(0,t.jsx)(l.Y,{navId:"dashboards/playlists",pageNav:M,children:(0,t.jsx)(l.Y.Contents,{children:(0,t.jsx)(i.$,{onSubmit:O,playlist:r})})})},y=n}}]);

//# sourceMappingURL=PlaylistNewPage-react19.cd4cff3085f43a963286.js.map