"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[3085,9604],{58045(t,n,a){a.r(n),a.d(n,{TableNG:()=>s.i,attachSkeleton:()=>l.j,skeletonAnimation:()=>l.Q});var l=a(21348),s=a(12880)},53097(){}}]);
