"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[6857],{73194(M,i,a){a.r(i),a.d(i,{default:()=>O});var t=a(14553),s=a(92745),_=a(45861),E=a(66404),o=a(41654),r=a(16692),l=a(57276),P=a(88547),c=a(36826),g=a(35526),d=a(16704);function f(){const[n,e]=(0,r.L7)(r.QI.CreateNotificationTemplate);return(0,t.jsxs)(o.B,{direction:"column",gap:1,children:[(0,t.jsxs)(o.B,{direction:"row",alignItems:"center",justifyContent:"space-between",children:[(0,t.jsx)(E.E,{variant:"body",color:"secondary",children:(0,t.jsx)(s.x6,{i18nKey:"alerting.templates-page.description",children:"Create notification templates to customize your notifications."})}),n&&(0,t.jsx)(_.z9,{icon:"plus",variant:"primary",href:"/alerting/notifications/templates/new",disabled:!e,children:(0,t.jsx)(s.x6,{i18nKey:"alerting.templates-page.add-template",children:"New notification template"})})]}),(0,t.jsx)(d.v,{})]})}function D(){const{navId:n,pageNav:e}=(0,l.yy)();return(0,t.jsxs)(c.y,{navId:n,pageNav:e,accessType:"notification",children:[(0,t.jsx)(g.e,{currentAlertmanager:"grafana"}),(0,t.jsx)(f,{})]})}const O=(0,P.S)(D)}}]);

//# sourceMappingURL=TemplatesPage-react19.c097b6f32719bc57584b.js.map