"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[7430],{31830(T,o,n){n.r(o),n.d(o,{SelectOrgPage:()=>i,default:()=>U});var a=n(14553),_=n(71468),g=n(94701),E=n(92745),P=n(43173),O=n(45861),l=n(41654),s=n(13791),r=n(93922);const d={main:{icon:"grafana",subTitle:"Preferences",text:"Select active organization"},node:{text:"Select active organization"}},h=e=>({userOrgs:e.organization.userOrgs}),D={setUserOrganization:r.bA,getUserOrganizations:r.wV},M=(0,_.connect)(h,D),i=({setUserOrganization:e,getUserOrganizations:v,userOrgs:c})=>{const C=async t=>{await e(t.orgId),window.location.href=P.$.appSubUrl+"/"};return(0,g.A)(()=>{v()}),(0,a.jsx)(s.Y,{navModel:d,children:(0,a.jsx)(s.Y.Contents,{children:(0,a.jsxs)("div",{children:[(0,a.jsx)("p",{children:(0,a.jsx)(E.x6,{i18nKey:"org.select-org-page.description",children:"You have been invited to another organization! Please select which organization that you want to use right now. You can change this later at any time."})}),(0,a.jsx)(l.B,{wrap:"wrap",children:c&&c.map(t=>(0,a.jsx)(O.$n,{icon:"signin",onClick:()=>C(t),children:t.name},t.orgId))})]})})})},U=M(i)}}]);

//# sourceMappingURL=SelectOrgPage-react19.3566c9895c6f2b32bb13.js.map