"use strict";(globalThis.webpackChunkmssql=globalThis.webpackChunkmssql||[]).push([[540],{1540(e){e.exports=JSON.parse('{"grafana-sql":{"components":{"confirm-modal":{"builder-mode":"生成器模式不显示代码中所做的更改。查询生成器将显示您在生成器模式下所做的最后一次更改。","cancel":"取消","clipboard":"您想将代码复制到剪贴板吗？","copy-code-and-switch":"复制代码并切换","discard-code-and-switch":"丢弃代码并切换","warning":"警告"},"connection-limits":{"auto-max-idle":"自动最大空闲","content-auto-max-idle":"","content-max-idle":"空闲连接池中的最大连接数。如果<i>最大打开连接数</i>大于 0 但小于<i>最大空闲连接数</i>，则<i>最大空闲连接数</i>将减少，以匹配<i>最大打开连接数</i>限制。如果设置为 0，则不保留任何空闲连接。","content-max-lifetime":"连接可以重复使用的最长时间（秒）。如果设置为 0，则连接将永久重复使用。","content-max-open":"","max-idle":"最大空闲","max-lifetime":"最大生命周期","max-open":"最大打开","title-connection-limits":"连接限制"},"dataset-selector":{"aria-label-dataset-selector":"数据集选择器"},"error-boundary":{"fall-back":{"error":"错误"}},"get-custom-operators":{"custom-operators":{"label":{"macros":"宏指令"}}},"make-render-column":{"render-column":{"aria-label-group-by":"分组依据","title-remove-group-by-column":"按列移除组"}},"order-by-row":{"aria-label-order-by":"排序条件","label-limit":"限制","label-offset":"偏移","label-order-by":"排序条件"},"preview":{"label-element":{"preview":"预览","tooltip-copy-to-clipboard":"复制到剪贴板"}},"query-header":{"content-invalid-query":"","editor-modes":{"label-builder":"构建器","label-code":"代码"},"label-dataset":"数据集","label-filter":"筛选条件","label-format":"格式","label-group":"组","label-order":"顺序","label-preview":"预览","label-table":"表格","placeholder-select-format":"选择格式","run-query":"运行查询"},"query-meta":{"variables":{"textField":"文本字段","valueField":"值字段"}},"query-toolbox":{"content-hit-ctrlcmdreturn-to-run-query":"按 CTRL/CMD+Return 运行查询","tooltip-collapse":"折叠编辑器","tooltip-expand":"展开编辑器","tooltip-format-query":"格式化查询"},"query-validator":{"query-will-process":"<0></0> 此查询将在运行时处理 <strong>{{bytes}}</strong>。","validating-query":"正在验证查询..."},"raw-editor":{"render-placeholder":{"editing-in-expanded-code-editor":"在展开的代码编辑器中编辑"},"title-query-num":"查询 {{queryNum}}"},"select-column":{"label-column":"列"},"select-custom-function-parameters":{"aria-label-parameter":"{{columnIndex}} 列的参数 {{index}}","render-parameters":{"params":{"title-remove-parameter":"移除参数"}},"title-add-parameter":"添加参数"},"select-row":{"aggregate-options":{"options":{"label":{"aggregations":"聚合","macros":"宏指令"}}},"label":{"time":"时间","value":"值"},"label-alias":"别名","label-data-operations":"数据操作","title-add-column":"添加列","title-remove-column":"移除列"},"settings":{"aria-label-conjunction":"连词","aria-label-field":"字段","aria-label-operator":"运算符","title-button-filter":"{{ buttonLabel }}筛选器"},"sql-query-editor-lazy":{"text-loading-editor":"正在加载编辑器"},"table-selector":{"aria-label-table-selector":"表选择器","placeholder-loading":"正在加载表格","placeholder-select-table":"选择表格"},"tlssecrets-config":{"content-tlsssl-client-certificate":"要使用 TLS/SSL 客户端证书进行身份验证，请在此处提供客户端证书。","content-tlsssl-client-key":"要使用客户端 TLS/SSL 证书进行身份验证，请在此处提供密钥。","content-tlsssl-root-certificate":"如果所选的 TLS/SSL 模式需要服务器根证书，请在此处提供","tlsssl-client-certificate":"TLS/SSL 客户端证书","tlsssl-client-key":"TLS 客户端密钥","tlsssl-root-certificate":"TLS/SSL 根证书"},"visual-editor":{"label-filter-by-column-value":"按列值筛选","label-group-by-column":"按列分组"},"widgets":{"aria-label-macros-value-selector":"宏值选择器"}},"utils":{"get-columns-width-indices":{"label-selected-columns":"选定的列"}}}}')}}]);