"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[3659],{33659(m,t,a){a.r(t),a.d(t,{default:()=>e});var _=a(74848),n=a(54078),r=a(35748);const e=({query:E})=>(0,_.jsx)("pre",{children:(0,_.jsx)(n.Z,{query:E,language:{grammar:r.n6,name:"promql"}})})},54078(m,t,a){a.d(t,{Z:()=>E});var _=a(96540),n=a(22803),r=a(28848),o=a(63704),e=a(63142);function E({query:s,language:l,className:i}){const g=(0,e.$j)(),h=P(g),M=r.highlight(s,l.grammar,l.name);return _.createElement("div",{className:(0,n.cx)(h.editorField,"prism-syntax-highlight",i),"aria-label":"selector",dangerouslySetInnerHTML:{__html:o.sQ.sanitize(M)}})}const P=s=>({editorField:(0,n.css)({fontFamily:s.typography.fontFamilyMonospace,fontSize:s.typography.bodySmall.fontSize})})}}]);

//# sourceMappingURL=3659.df72f606d572b8948b4b.js.map