<!doctype html>
<html lang="en">
  <head>
    [[ if and .CSPEnabled .IsDevelopmentEnv ]]
    <!-- Cypress overwrites CSP headers in HTTP requests, so this is required for e2e tests-->
    <meta http-equiv="Content-Security-Policy" content="[[.CSPContent]]" />
    [[ end ]]
    <meta charset="utf-8" />
    <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1" />
    <meta name="viewport" content="width=device-width" />
    <meta name="theme-color" content="#000" />

    <title>Grafana API Reference</title>

    [[range $asset := .Assets.SwaggerCSSFiles]]
    <link rel="stylesheet" href="[[$asset.FilePath]]" />
    [[end]]

    <link rel="stylesheet" href="[[.Assets.Light]]" />

    <link rel="icon" type="image/png" href="[[.Assets.ContentDeliveryURL]]public/build/img/fav32.png" />
    <link
      rel="apple-touch-icon"
      sizes="180x180"
      href="[[.Assets.ContentDeliveryURL]]public/build/img/apple-touch-icon.png"
    />
    <link rel="mask-icon" href="[[.Assets.ContentDeliveryURL]]public/build/img/grafana_mask_icon.svg" color="#F05A28" />
  </head>

  <body>
    <noscript> You need to enable JavaScript to run this app. </noscript>
    <script nonce="[[$.Nonce]]">
      [[if .Assets.ContentDeliveryURL]]
          window.public_cdn_path = '[[.Assets.ContentDeliveryURL]]public/build/';
      [[end]]
    </script>
    <div id="root"></div>
    [[range $asset := .Assets.Swagger]]
    <script nonce="[[$.Nonce]]" src="[[$asset.FilePath]]" type="text/javascript"></script>
    [[end]]
    <script></script>
  </body>
</html>
