"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[904],{80904(L,f,g){g.d(f,{default:()=>_});var c={a:"\u03B1",b:"\u1E05",c:"\u037C",d:"\u1E0D",e:"\u1E1B",f:"\u03DD",g:"\u1E21",h:"\u1E25",i:"\u1E2D",j:"\u0135",k:"\u1E33",l:"\u1E3D",m:"\u1E43",n:"\u1E47",o:"\u1E53",p:"\u1E57",q:"\u02A0",r:"\u1E5B",s:"\u1E61",t:"\u0163",u:"\u1E75",v:"\u1E7D",w:"\u1E81",x:"\u1E8B",y:"\u1E8F",z:"\u1E93",A:"\xC0",B:"\u03B2",C:"\u1E08",D:"\u1E0C",E:"\u1E1A",F:"\u1E1E",G:"\u1E20",H:"\u1E24",I:"\u1E2C",J:"\u0134",K:"\u1E30",L:"\u1E3A",M:"\u1E40",N:"\u1E44",O:"\u1E4E",P:"\u1E54",Q:"\u01EA",R:"\u1E5A",S:"\u1E62",T:"\u1E6A",U:"\u1E72",V:"\u1E7E",W:"\u0174",X:"\u1E8A",Y:"\u0176",Z:"\u017B"},h=["a","e","i","o","u","y","A","E","I","O","U","Y"],b=function(e){var t=e.shouldWrap,a=e.string;return t?"["+a+"]":a},v=Object.assign||function(r){for(var e=1;e<arguments.length;e++){var t=arguments[e];for(var a in t)Object.prototype.hasOwnProperty.call(t,a)&&(r[a]=t[a])}return r};function w(r,e){if(!(r instanceof e))throw new TypeError("Cannot call a class as a function")}var _=function(){function r(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},t=e.languageToPseudo,a=t===void 0?"en":t,p=e.letterMultiplier,o=p===void 0?2:p,s=e.repeatedLetters,i=s===void 0?h:s,u=e.uglifedLetterObject,n=u===void 0?c:u,d=e.wrapped,P=d===void 0?!1:d,l=e.enabled,m=l===void 0?!0:l;w(this,r),this.name="pseudo",this.type="postProcessor",this.options={languageToPseudo:a,letterMultiplier:o,wrapped:P,repeatedLetters:i,letters:n,enabled:m}}return r.prototype.configurePseudo=function(t){this.options=v({},this.options,t)},r.prototype.process=function(t,a,p,o){var s=this;if(o.language&&this.options.languageToPseudo!==o.language||!this.options.enabled)return t;var i=0,u=t.split("").map(function(n){return n==="}"?(i=0,n):n==="{"?(i++,n):i===2?n:s.options.repeatedLetters.indexOf(n)!==-1?s.options.letters[n].repeat(s.options.letterMultiplier):s.options.letters[n]||n}).join("");return b({shouldWrap:this.options.wrapped,string:u})},r}()}}]);

//# sourceMappingURL=904-react19.57e2eeb1540c29f8f2a3.js.map