"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[9820],{59820(r,d,e){e.d(d,{GAEchoBackend:()=>u});var i=e(78646),o=e(25240);class u{constructor(s){this.options=s,this.supportedEvents=[i.on.Pageview],this.trackedUserId=null,this.addEvent=n=>{if(!window.ga)return;window.ga("set",{page:n.payload.page}),window.ga("send","pageview");const{userSignedIn:c,userId:t}=n.meta;c&&t!==this.trackedUserId&&(this.trackedUserId=t,window.ga("set","userId",t))},this.flush=()=>{};const g=`https://www.google-analytics.com/analytics${s.debug?"_debug":""}.js`;(0,o.k0)(g);const a=window.ga=window.ga||function(){(a.q=a.q||[]).push(arguments)};a.l=+new Date,a("create",s.googleAnalyticsId,"auto"),a("set","anonymizeIp",!0)}}}}]);

//# sourceMappingURL=9820.a8dce2eb8f5f51344898.js.map