"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[5106],{25106(l,e,a){a.r(e),a.d(e,{default:()=>r});var r={"grafana-scenes":{components:{"adhoc-filter-pill":{"edit-filter-with-key":"Editar filtro com chave {{keyLabel}}","managed-filter":"Filtro gerenciado de {{origin}}","non-applicable":"","remove-filter-with-key":"Remover filtro com chave {{keyLabel}}"},"adhoc-filters-combobox":{"remove-filter-value":"Remover valor do filtro: {{itemLabel}}","use-custom-value":"Usar valor personalizado: {{itemLabel}}"},"fallback-page":{content:"Se voc\xEA chegou aqui usando um link, pode haver um bug neste aplicativo.",subTitle:"O URL n\xE3o corresponde a nenhuma p\xE1gina",title:"N\xE3o encontrado"},"lazy-loader":{placeholder:""},"nested-scene-renderer":{"collapse-button-label":"Recolher cena","expand-button-label":"Expandir cena","remove-button-label":"Remover cena"},"scene-debugger":{"object-details":"Detalhes do objeto","scene-graph":"Gr\xE1fico de cena","title-scene-debugger":"Depurador de cena"},"scene-grid-row":{"collapse-row":"Recolher linha","expand-row":"Expandir linha"},"scene-refresh-picker":{"text-cancel":"Cancelar","text-refresh":"Atualizar","tooltip-cancel":""},"scene-time-range-compare-renderer":{"button-label":"Compara\xE7\xE3o","button-tooltip":"Ativar compara\xE7\xE3o de intervalo de tempo"},splitter:{"aria-label-pane-resize-widget":"Widget de redimensionamento do painel"},"time-picker":{"move-backward-tooltip":"","move-forward-tooltip":""},"viz-panel":{title:{title:"T\xEDtulo"}},"viz-panel-explore-button":{explore:"Explorar"},"viz-panel-renderer":{"loading-plugin-panel":"Carregando painel do plug-in\u2026","panel-plugin-has-no-panel-component":"O plug-in do painel n\xE3o possui componente de painel"},"viz-panel-series-limit":{"content-rendering-series-single-panel-impact-performance":"Renderizar muitas s\xE9ries em um \xFAnico painel pode afetar o desempenho e dificultar a leitura dos dados.","warning-message":"Mostrando apenas {{seriesLimit}} s\xE9rie(s)"}},utils:{"controls-label":{"tooltip-remove":"Remover"},"loading-indicator":{"content-cancel-query":"Cancelar consulta"}},variables:{"ad-hoc-combobox":{"aria-label-edit-filter-operator":"Editar operador de filtro"},"ad-hoc-filter-builder":{"aria-label-add-filter":"Adicionar filtro","title-add-filter":"Adicionar filtro"},"ad-hoc-filter-renderer":{"aria-label-remove-filter":"Remover filtro","key-select":{"placeholder-select-label":"Selecionar r\xF3tulo"},"label-select-label":"Selecionar r\xF3tulo","title-remove-filter":"Remover filtro","value-select":{"placeholder-select-value":"Selecionar valor"}},"data-source-variable":{label:{default:"padr\xE3o"}},"default-group-by-custom-indicator-container":{"aria-label-clear":"limpar",tooltip:"Aplicado por padr\xE3o neste painel. Se editado, ele \xE9 transferido para outros pain\xE9is.","tooltip-restore-groupby-set-by-this-dashboard":"Restaura a fun\xE7\xE3o groupby definida por este painel."},"format-registry":{formats:{description:{"commaseparated-values":"Valores separados por v\xEDrgula","double-quoted-values":"Valores entre aspas duplas","format-date-in-different-ways":"Formatar data de diferentes maneiras","format-multivalued-variables-using-syntax-example":"Formatar vari\xE1veis de m\xFAltiplos valores usando a sintaxe glob. Por exemplo: {value1,value2}","html-escaping-of-values":"Escape HTML de valores","join-values-with-a-comma":"","json-stringify-value":"Valor convertido em string JSON","keep-value-as-is":"Manter o valor como est\xE1","multiple-values-are-formatted-like-variablevalue":"V\xE1rios valores s\xE3o formatados como vari\xE1vel=valor","single-quoted-values":"Valores entre aspas simples","useful-escaping-values-taking-syntax-characters":"\xDAtil para valores de escape de URL, levando em considera\xE7\xE3o caracteres de sintaxe URI","useful-for-url-escaping-values":"\xDAtil para valores de escape de URL","values-are-separated-by-character":'Os valores s\xE3o separados pelo caractere "|"'}}},"group-by-variable-renderer":{"aria-label-group-by-selector":"Agrupar por seletor","placeholder-group-by-label":"Agrupar por r\xF3tulo"},"interval-variable":{"placeholder-select-value":"Selecionar valor"},"loading-options-placeholder":{"loading-options":"Carregando op\xE7\xF5es\u2026"},"multi-value-apply-button":{apply:"Aplicar"},"no-options-placeholder":{"no-options-found":"Nenhuma op\xE7\xE3o encontrada"},"options-error-placeholder":{"error-occurred-fetching-labels-click-retry":"Ocorreu um erro ao buscar r\xF3tulos. Clique para tentar novamente"},"test-object-with-variable-dependency":{title:{hello:"Ol\xE1"}},"test-variable":{text:{text:"Texto"}},"variable-value-input":{"placeholder-enter-value":"Digite um valor"},"variable-value-select":{"placeholder-select-value":"Selecionar valor"}}}}}}]);

//# sourceMappingURL=5106.3f8cb518215ee9310a97.js.map