"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[5446],{2286(r,e){/**
 * @license React
 * react-jsx-dev-runtime.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var a=Symbol.for("react.fragment");e.Fragment=a,e.jsxDEV=void 0},85446(r,e,a){r.exports=a(2286)}}]);

//# sourceMappingURL=5446.81e767db2588114ba97f.js.map