"use strict";(globalThis.webpackChunkmssql=globalThis.webpackChunkmssql||[]).push([[654],{2654(e){e.exports=JSON.parse('{"grafana-sql":{"components":{"confirm-modal":{"builder-mode":"El modo Creador no muestra los cambios realizados en el código. El creador de consultas mostrará los últimos cambios que haya realizado en el modo Creador.","cancel":"Cancelar","clipboard":"¿Quiere copiar su código en el portapapeles?","copy-code-and-switch":"Copiar código y cambiar","discard-code-and-switch":"Descartar código y cambiar","warning":"Advertencia"},"connection-limits":{"auto-max-idle":"Inactividad máxima automática","content-auto-max-idle":"","content-max-idle":"El número máximo de conexiones en el grupo de conexiones inactivas. Si <i>Conexiones abiertas máximas</i> es mayor que 0 pero menor que <i>Conexiones inactivas máximas</i>, entonces <i>Conexiones inactivas máximas</i> se reducirá para que coincida con el límite de <i>Conexiones abiertas máximas</i>. Si se establece en 0, no se conservan las conexiones inactivas.","content-max-lifetime":"La cantidad máxima de tiempo en segundos que se puede reutilizar una conexión. Si se establece en 0, las conexiones se reutilizan para siempre.","content-max-open":"","max-idle":"Máx. inactivas","max-lifetime":"Vida útil máxima","max-open":"Máx. abiertas","title-connection-limits":"Límites de la conexión"},"dataset-selector":{"aria-label-dataset-selector":"Selector de conjuntos de datos"},"error-boundary":{"fall-back":{"error":"Error"}},"get-custom-operators":{"custom-operators":{"label":{"macros":"Macros"}}},"make-render-column":{"render-column":{"aria-label-group-by":"Agrupar por","title-remove-group-by-column":"Eliminar grupo por columna"}},"order-by-row":{"aria-label-order-by":"Ordenar por","label-limit":"Límite","label-offset":"Separación","label-order-by":"Ordenar por"},"preview":{"label-element":{"preview":"Vista previa","tooltip-copy-to-clipboard":"Copiar al portapapeles"}},"query-header":{"content-invalid-query":"","editor-modes":{"label-builder":"Constructor","label-code":"Código"},"label-dataset":"Conjunto de datos","label-filter":"Filtro","label-format":"Formato","label-group":"Grupo","label-order":"Orden","label-preview":"Vista previa","label-table":"Tabla","placeholder-select-format":"Seleccionar formato","run-query":"Ejecutar consulta"},"query-meta":{"variables":{"textField":"Campo de texto","valueField":"Campo de valor"}},"query-toolbox":{"content-hit-ctrlcmdreturn-to-run-query":"Pulse CTRL/CMD+Return para ejecutar la consulta","tooltip-collapse":"Contraer editor","tooltip-expand":"Expandir editor","tooltip-format-query":"Formatear consulta"},"query-validator":{"query-will-process":"<0></0> Esta consulta procesará <strong>{{bytes}}</strong> cuando se ejecute.","validating-query":"Validando consulta..."},"raw-editor":{"render-placeholder":{"editing-in-expanded-code-editor":"Editando en el editor de código expandido"},"title-query-num":"Consulta {{queryNum}}"},"select-column":{"label-column":"Columna"},"select-custom-function-parameters":{"aria-label-parameter":"Parámetro {{index}} para la columna {{columnIndex}}","render-parameters":{"params":{"title-remove-parameter":"Eliminar parámetro"}},"title-add-parameter":"Añadir parámetro"},"select-row":{"aggregate-options":{"options":{"label":{"aggregations":"Agregaciones","macros":"Macros"}}},"label":{"time":"tiempo","value":"valor"},"label-alias":"Alias","label-data-operations":"Operaciones de datos","title-add-column":"Añadir columna","title-remove-column":"Eliminar columna"},"settings":{"aria-label-conjunction":"Conjunción","aria-label-field":"Campo","aria-label-operator":"Operador","title-button-filter":"Filtro {{ buttonLabel }}"},"sql-query-editor-lazy":{"text-loading-editor":"Cargando editor"},"table-selector":{"aria-label-table-selector":"Selector de tablas","placeholder-loading":"Cargando tablas","placeholder-select-table":"Seleccionar tabla"},"tlssecrets-config":{"content-tlsssl-client-certificate":"Para autenticarse con un certificado de cliente TLS/SSL, proporciónelo aquí.","content-tlsssl-client-key":"Para autenticarse con un certificado TLS/SSL de cliente, proporcione la clave aquí.","content-tlsssl-root-certificate":"Si el modo TLS/SSL seleccionado requiere un certificado raíz del servidor, proporciónelo aquí","tlsssl-client-certificate":"Certificado de cliente TLS/SSL","tlsssl-client-key":"Clave de cliente TLS/SSL","tlsssl-root-certificate":"Certificado raíz TLS/SSL"},"visual-editor":{"label-filter-by-column-value":"Filtrar por valor de columna","label-group-by-column":"Agrupar por columna"},"widgets":{"aria-label-macros-value-selector":"Selector de valores de macros"}},"utils":{"get-columns-width-indices":{"label-selected-columns":"Columnas seleccionadas"}}}}')}}]);