"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[558],{9334(U,i,t){t.r(i),t.d(i,{default:()=>f});var a=t(74848),E=t(54148),r=t(92745),o=t(43173),g=t(65819),P=t(20962),l=t(22628),m=t(57276),M=t(5358),u=t(29442),v=t(88547),D=t(36826),O=t(83192);const d=()=>{const[e]=(0,P.l)(),{selectedAlertmanager:_}=(0,M.VI)(),n=e.get("muteName"),{isLoading:A,data:s,isError:C}=(0,g.zm)({alertmanager:_,name:n});if(!n)return(0,a.jsx)(E.C5,{replace:!0,to:"/alerting/routes"});const I=s?.metadata?.annotations?.[l.T.Provenance];return(0,a.jsx)(O.A,{editMode:!0,loading:A,showError:C,muteTiming:s,provenance:I})};function T(){const{navId:e}=(0,m.s4)(),_=o.$.featureToggles.alertingNavigationV2,n=(0,u.aU)(_);return(0,a.jsx)(D.y,{navId:e,pageNav:{id:"alert-policy-edit",text:(0,r.t)("alerting.edit-mute-timing-page.text.edit-time-interval","Edit time interval"),parentItem:{text:(0,r.t)("alerting.time-intervals.title","Time intervals"),url:n}},accessType:"notification",children:(0,a.jsx)(d,{})})}const f=(0,v.S)(T)}}]);

//# sourceMappingURL=EditMuteTiming.04c322075302e3c14760.js.map