// This is specified in runtime Emotion GlobalStyles but we need them for the preloader styles
html {
  font-size: $font-size-base;
  height: 100%;
}

// This is specified in runtime Emotion GlobalStyles but we need them for the preloader styles
body {
  font-family: $font-family-sans-serif;
  font-size: $font-size-base;
  line-height: $line-height-base;
  color: $text-color;
  background-color: $body-bg;
  height: 100%;
  width: 100%;
  margin: 0;
  position: absolute;
}

// BASE
@import 'base/font_awesome';

// ANGULAR
@import 'angular';
