import { type DataFrameJSON } from '@grafana/data';

const data: DataFrameJSON = {
  data: {
    values: [
      [
        1727189670000, 1727189670000, 1727189670000, 1727189670000, 1727189670000, 1727189670000, 1727189670000,
        1727189670000, 1727189670000, 1727189670000, 1727189680000, 1727189700000, 1727189690000, 1727189690000,
        1727189690000, 1727189690000, 1727189690000, 1727189690000, 1727189690000, 1727189690000, 1727189680000,
        1727189670000, 1727189700000, 1727189710000, 1727189710000, 1727189710000, 1727189710000, 1727189710000,
        1727189710000, 1727189710000, 1727189710000, 1727189710000,
      ],
      [
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'NoData',
          values: {},
          condition: 'C',
          dashboardUID: '-W9yw_X7k',
          panelID: 3,
          fingerprint: '009794b4d3074732',
          ruleTitle: 'CPU Usage 6',
          ruleID: 10605,
          ruleUID: '-rRg_HcVz',
          labels: {
            alertname: 'CPU Usage 6',
            datasource_uid: 'gdev-prometheus',
            grafana_folder: 'Demonstrations',
            ref_id: 'A',
            team: 'operations',
            type: 'test',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'NoData',
          values: {},
          condition: 'C',
          dashboardUID: 'stzZvVw7k',
          panelID: 8,
          fingerprint: '186e7d0c9173ddf4',
          ruleTitle: 'CPU Usage 4',
          ruleID: 150,
          ruleUID: 's984p7n4k',
          labels: {
            alertname: 'CPU Usage 4',
            datasource_uid: 'gdev-prometheus',
            grafana_folder: 'Demonstrations',
            ref_id: 'A',
            region: 'EMEA',
            team: 'operations',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Alerting',
          current: 'Normal',
          values: {
            B: -6.886811320670644,
            C: 0,
          },
          condition: 'C',
          dashboardUID: '',
          panelID: 0,
          fingerprint: 'bcf38efbb616d8d2',
          ruleTitle: 'direct to contact point',
          ruleID: 10656,
          ruleUID: 'ddc5zx4l3zls0a',
          labels: {
            alertname: 'direct to contact point',
            grafana_folder: 'Testing and reproducing',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'NoData',
          values: {},
          condition: 'D',
          dashboardUID: 'iYSJkozVk',
          panelID: 13,
          fingerprint: 'd3f3ec7ad90a37fd',
          ruleTitle: 'With Disabled Query',
          ruleID: 10631,
          ruleUID: 'ab2ac850-1669-4dac-b566-c91728425f39',
          labels: {
            alertname: 'With Disabled Query',
            datasource_uid: 'gdev-prometheus',
            grafana_folder: 'Testing and reproducing',
            ref_id: 'A,B',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'NoData',
          values: {},
          condition: 'C',
          dashboardUID: '',
          panelID: 0,
          fingerprint: 'f4c073dd4c60e897',
          ruleTitle: 'Alert State History Random',
          ruleID: 10614,
          ruleUID: 'e4d7252b-c66f-4f5c-b7db-28b9936150bd',
          labels: {
            alertname: 'Alert State History Random',
            datasource_uid: 'gdev-prometheus',
            grafana_folder: 'Testing and reproducing',
            ref_id: 'A',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            B: 54.137611900622574,
            C: 1,
          },
          condition: 'C',
          dashboardUID: '',
          panelID: 0,
          fingerprint: 'e11b0ff29886e8f2',
          ruleTitle: 'node_namespace_pod_container:container_cpu_usage_seconds_total:sum_irate4',
          ruleID: 10662,
          ruleUID: 'ddgfq7c57fqpsc',
          labels: {
            alertname: 'node_namespace_pod_container:container_cpu_usage_seconds_total:sum_irate4',
            grafana_folder: 'Testing and reproducing',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'NoData',
          values: {},
          condition: 'B',
          dashboardUID: '',
          panelID: 0,
          fingerprint: 'd6d7e256298c8ea8',
          ruleTitle: 'delete-ds',
          ruleID: 144,
          ruleUID: 'wXS0SnWVz',
          labels: {
            alertname: 'delete-ds',
            datasource_uid: 'gdev-prometheus',
            grafana_folder: 'Testing and reproducing',
            ref_id: 'A',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'NoData',
          values: {},
          condition: 'C',
          dashboardUID: '',
          panelID: 0,
          fingerprint: 'bf1b309577b4fc0c',
          ruleTitle: 'Threshold Test',
          ruleID: 148,
          ruleUID: 'u0QVOqMVz',
          labels: {
            alertname: 'Threshold Test',
            datasource_uid: 'gdev-prometheus',
            grafana_folder: 'Testing and reproducing',
            ref_id: 'A',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'NoData',
          values: {},
          condition: 'C',
          dashboardUID: '',
          panelID: 0,
          fingerprint: 'd6b1cde6ab63bfca',
          ruleTitle: 'Alert History fingerprint',
          ruleID: 10619,
          ruleUID: 'c7a7989f-f12c-4aa5-8fb3-27f4509a5809',
          labels: {
            alertname: 'Alert History fingerprint',
            datasource_uid: 'PDDA8E780A17E7EF1',
            grafana_folder: 'Testing and reproducing',
            ref_id: 'A',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            B: 0.0020875,
            C: 1,
          },
          condition: 'C',
          dashboardUID: 'd4c61dfe-24d8-433e-984e-701a2f14faab',
          panelID: 1,
          fingerprint: '40ddb0f6e97426e9',
          ruleTitle: 'UniFi CPU Usage',
          ruleID: 10638,
          ruleUID: 'd10fc550-3552-4083-a3aa-f6b68058bc7f',
          labels: {
            alertname: 'UniFi CPU Usage',
            grafana_folder: 'UniFi',
            instance: 'unpoller:9130',
            job: 'unifipoller',
            name: 'Dream Router',
            site_name: 'Default (default)',
            source: 'https://192.168.1.1',
            type: 'udm',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Pending',
          current: 'Alerting',
          values: {
            B0: 36.956598435022215,
          },
          condition: 'B',
          dashboardUID: '',
          panelID: 0,
          fingerprint: '10716534bbf8086b',
          ruleTitle: 'XSS attack vector',
          ruleID: 37,
          ruleUID: 'p8TBxnq7k',
          labels: {
            alertname: 'XSS attack vector',
            grafana_folder: 'gdev dashboards',
          },
        },
        {
          schemaVersion: 1,
          previous: 'NoData',
          current: 'Normal (MissingSeries)',
          values: {},
          condition: 'C',
          dashboardUID: 'iYSJkozVk',
          panelID: 5,
          fingerprint: '4a5519a42c382438',
          ruleTitle: 'CPU Usage',
          ruleID: 7635,
          ruleUID: '8nEmeE44k',
          labels: {
            alertname: 'CPU Usage',
            datasource_uid: 'gdev-prometheus',
            grafana_folder: 'Demonstrations',
            ref_id: 'A',
            team: 'operations',
            type: 'cpu',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Alerting',
          values: {
            B: 0.067575,
            C: 1,
          },
          condition: 'C',
          dashboardUID: 'iYSJkozVk',
          panelID: 5,
          fingerprint: 'cc4aceb3018da2e8',
          ruleTitle: 'CPU Usage',
          ruleID: 7635,
          ruleUID: '8nEmeE44k',
          labels: {
            alertname: 'CPU Usage',
            cpu: '7',
            grafana_folder: 'Demonstrations',
            team: 'operations',
            type: 'cpu',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Alerting',
          values: {
            B: 0.06216900000000001,
            C: 1,
          },
          condition: 'C',
          dashboardUID: 'iYSJkozVk',
          panelID: 5,
          fingerprint: '20dfd648881e698f',
          ruleTitle: 'CPU Usage',
          ruleID: 7635,
          ruleUID: '8nEmeE44k',
          labels: {
            alertname: 'CPU Usage',
            cpu: '2',
            grafana_folder: 'Demonstrations',
            team: 'operations',
            type: 'cpu',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Alerting',
          values: {
            B: 0.07027800000000001,
            C: 1,
          },
          condition: 'C',
          dashboardUID: 'iYSJkozVk',
          panelID: 5,
          fingerprint: '234db8cd1e160f6a',
          ruleTitle: 'CPU Usage',
          ruleID: 7635,
          ruleUID: '8nEmeE44k',
          labels: {
            alertname: 'CPU Usage',
            cpu: '1',
            grafana_folder: 'Demonstrations',
            team: 'operations',
            type: 'cpu',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Alerting',
          values: {
            B: 0.06727466666666668,
            C: 1,
          },
          condition: 'C',
          dashboardUID: 'iYSJkozVk',
          panelID: 5,
          fingerprint: '4e1f85b7efc3ed75',
          ruleTitle: 'CPU Usage',
          ruleID: 7635,
          ruleUID: '8nEmeE44k',
          labels: {
            alertname: 'CPU Usage',
            cpu: '0',
            grafana_folder: 'Demonstrations',
            team: 'operations',
            type: 'cpu',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Alerting',
          values: {
            B: 0.05315900000000001,
            C: 1,
          },
          condition: 'C',
          dashboardUID: 'iYSJkozVk',
          panelID: 5,
          fingerprint: '50bf9f17b9534593',
          ruleTitle: 'CPU Usage',
          ruleID: 7635,
          ruleUID: '8nEmeE44k',
          labels: {
            alertname: 'CPU Usage',
            cpu: '6',
            grafana_folder: 'Demonstrations',
            team: 'operations',
            type: 'cpu',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Alerting',
          values: {
            B: 0.046551666666666686,
            C: 1,
          },
          condition: 'C',
          dashboardUID: 'iYSJkozVk',
          panelID: 5,
          fingerprint: '2fb0af95df9b2bfe',
          ruleTitle: 'CPU Usage',
          ruleID: 7635,
          ruleUID: '8nEmeE44k',
          labels: {
            alertname: 'CPU Usage',
            cpu: '5',
            grafana_folder: 'Demonstrations',
            team: 'operations',
            type: 'cpu',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Alerting',
          values: {
            B: 0.05466066666666668,
            C: 1,
          },
          condition: 'C',
          dashboardUID: 'iYSJkozVk',
          panelID: 5,
          fingerprint: 'd0303e26f6c376e9',
          ruleTitle: 'CPU Usage',
          ruleID: 7635,
          ruleUID: '8nEmeE44k',
          labels: {
            alertname: 'CPU Usage',
            cpu: '4',
            grafana_folder: 'Demonstrations',
            team: 'operations',
            type: 'cpu',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Alerting',
          values: {
            B: 0.059165666666666665,
            C: 1,
          },
          condition: 'C',
          dashboardUID: 'iYSJkozVk',
          panelID: 5,
          fingerprint: 'af82d1220e70c3b4',
          ruleTitle: 'CPU Usage',
          ruleID: 7635,
          ruleUID: '8nEmeE44k',
          labels: {
            alertname: 'CPU Usage',
            cpu: '3',
            grafana_folder: 'Demonstrations',
            team: 'operations',
            type: 'cpu',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Error',
          current: 'Normal (MissingSeries)',
          values: {},
          condition: 'C',
          dashboardUID: 'iYSJkozVk',
          panelID: 5,
          fingerprint: '4a5519a42c382438',
          ruleTitle: 'CPU Usage',
          ruleID: 7635,
          ruleUID: '8nEmeE44k',
          labels: {
            alertname: 'CPU Usage',
            datasource_uid: 'gdev-prometheus',
            grafana_folder: 'Demonstrations',
            ref_id: 'A',
            team: 'operations',
            type: 'cpu',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'NoData',
          values: {},
          condition: 'C',
          dashboardUID: 'iYSJkozVk',
          panelID: 5,
          fingerprint: '4a5519a42c382438',
          ruleTitle: 'CPU Usage',
          ruleID: 7635,
          ruleUID: '8nEmeE44k',
          labels: {
            alertname: 'CPU Usage',
            datasource_uid: 'gdev-prometheus',
            grafana_folder: 'Demonstrations',
            ref_id: 'A',
            team: 'operations',
            type: 'cpu',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            C0: 9.319781010143206,
          },
          condition: 'C',
          dashboardUID: '',
          panelID: 0,
          fingerprint: 'abeb2444057d7f01',
          ruleTitle: 'test-read-only-threshold-indicators',
          ruleID: 10606,
          ruleUID: 'Wq5jJlc4k',
          labels: {
            alertname: 'test-read-only-threshold-indicators',
            grafana_folder: 'Testing and reproducing',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            reducer: 0.07305751032920166,
            threshold: 1,
          },
          condition: 'threshold',
          dashboardUID: '',
          panelID: 0,
          fingerprint: '2e7f5e6f5a5f2902',
          ruleTitle: 'CPU Usage 5',
          ruleID: 10583,
          ruleUID: 'LvMu_IHVz',
          labels: {
            alertname: 'CPU Usage 5',
            cpu: '6',
            grafana_folder: 'Demonstrations',
            team: 'operations',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            reducer: 0.06820108623217379,
            threshold: 1,
          },
          condition: 'threshold',
          dashboardUID: '',
          panelID: 0,
          fingerprint: '1c1b21f738468ea5',
          ruleTitle: 'CPU Usage 5',
          ruleID: 10583,
          ruleUID: 'LvMu_IHVz',
          labels: {
            alertname: 'CPU Usage 5',
            cpu: '5',
            grafana_folder: 'Demonstrations',
            team: 'operations',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            reducer: 0.07685819005731041,
            threshold: 1,
          },
          condition: 'threshold',
          dashboardUID: '',
          panelID: 0,
          fingerprint: '23ee5f02850e38f8',
          ruleTitle: 'CPU Usage 5',
          ruleID: 10583,
          ruleUID: 'LvMu_IHVz',
          labels: {
            alertname: 'CPU Usage 5',
            cpu: '4',
            grafana_folder: 'Demonstrations',
            team: 'operations',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            reducer: 0.08614874050379848,
            threshold: 1,
          },
          condition: 'threshold',
          dashboardUID: '',
          panelID: 0,
          fingerprint: '2a810fa152d6407b',
          ruleTitle: 'CPU Usage 5',
          ruleID: 10583,
          ruleUID: 'LvMu_IHVz',
          labels: {
            alertname: 'CPU Usage 5',
            cpu: '3',
            grafana_folder: 'Demonstrations',
            team: 'operations',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            reducer: 0.0926943555910969,
            threshold: 1,
          },
          condition: 'threshold',
          dashboardUID: '',
          panelID: 0,
          fingerprint: '48d127e8826c6fb6',
          ruleTitle: 'CPU Usage 5',
          ruleID: 10583,
          ruleUID: 'LvMu_IHVz',
          labels: {
            alertname: 'CPU Usage 5',
            cpu: '2',
            grafana_folder: 'Demonstrations',
            team: 'operations',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            reducer: 0.1097974143675863,
            threshold: 1,
          },
          condition: 'threshold',
          dashboardUID: '',
          panelID: 0,
          fingerprint: 'c9582e3d28402fe9',
          ruleTitle: 'CPU Usage 5',
          ruleID: 10583,
          ruleUID: 'LvMu_IHVz',
          labels: {
            alertname: 'CPU Usage 5',
            cpu: '1',
            grafana_folder: 'Demonstrations',
            team: 'operations',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            reducer: 0.12985655737704918,
            threshold: 1,
          },
          condition: 'threshold',
          dashboardUID: '',
          panelID: 0,
          fingerprint: 'fb3e32509acec7ec',
          ruleTitle: 'CPU Usage 5',
          ruleID: 10583,
          ruleUID: 'LvMu_IHVz',
          labels: {
            alertname: 'CPU Usage 5',
            cpu: '0',
            grafana_folder: 'Demonstrations',
            team: 'operations',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            B0: 8.406488484275167,
          },
          condition: 'B',
          dashboardUID: '',
          panelID: 0,
          fingerprint: '91a8126f41689d98',
          ruleTitle: 'Random Values',
          ruleID: 146,
          ruleUID: 'I1M_iCWVk',
          labels: {
            alertname: 'Random Values',
            grafana_folder: 'Demonstrations',
          },
        },
        {
          schemaVersion: 1,
          previous: 'Normal',
          current: 'Pending',
          values: {
            reducer: 0.08973827135812343,
            threshold: 1,
          },
          condition: 'threshold',
          dashboardUID: '',
          panelID: 0,
          fingerprint: '6cd2ab200de1cf77',
          ruleTitle: 'CPU Usage 5',
          ruleID: 10583,
          ruleUID: 'LvMu_IHVz',
          labels: {
            alertname: 'CPU Usage 5',
            cpu: '7',
            grafana_folder: 'Demonstrations',
            team: 'operations',
          },
        },
      ],
      [
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'system-metrics',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'default 2',
          orgID: '1',
        },
        {
          folderUID: 'a3eyrfqnk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'a3eyrfqnk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'a3eyrfqnk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'a3eyrfqnk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'a3eyrfqnk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'a3eyrfqnk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'a3eyrfqnk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'Qh4ctRZVz',
          from: 'state-history',
          group: 'system',
          orgID: '1',
        },
        {
          folderUID: 'pPyg37Qnk',
          from: 'state-history',
          group: 'security',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'usage',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'usage',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'usage',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'usage',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'usage',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'usage',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'usage',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'usage',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'usage',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'usage',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'usage',
          orgID: '1',
        },
        {
          folderUID: 'a3eyrfqnk',
          from: 'state-history',
          group: 'thresholds',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
        {
          folderUID: 'lG5pfeRVk',
          from: 'state-history',
          group: 'default',
          orgID: '1',
        },
      ],
    ],
  },
};

export default data;
