"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[4964],{24964(i,_,a){a.r(_),a.d(_,{default:()=>l});var t=a(14553),n=a(54078),r=a(77236);const l=({query:o})=>(0,t.jsx)("pre",{children:(0,t.jsx)(n.Z,{query:o,language:{grammar:r.Ay,name:"promql"}})})},54078(i,_,a){a.d(_,{Z:()=>o});var t=a(55501),n=a(22803),r=a(28848),E=a(63704),l=a(63142);function o({query:s,language:e,className:g}){const m=(0,l.$j)(),d=P(m),M=r.highlight(s,e.grammar,e.name);return t.createElement("div",{className:(0,n.cx)(d.editorField,"prism-syntax-highlight",g),"aria-label":"selector",dangerouslySetInnerHTML:{__html:E.sQ.sanitize(M)}})}const P=s=>({editorField:(0,n.css)({fontFamily:s.typography.fontFamilyMonospace,fontSize:s.typography.bodySmall.fontSize})})}}]);

//# sourceMappingURL=4964-react19.c57dc378cde001d1dfd3.js.map