"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[44],{31542(K,i,a){a.r(i),a.d(i,{default:()=>T});var t=a(14553),_=a(22803),e=a(92745),E=a(43173),f=a(54314),o=a(60208),P=a(13941),c=a(66404),g=a(31286),v=a(63142),D=a(70958),O=a(75780),M=a(88547),d=a(36826),y=a(15784),C=(n=>(n.AlertEvents="alert_events",n.Notifications="notifications",n))(C||{});function h(){const n=(0,v.of)(A),[B,l]=(0,D.s)(),r=E.$.featureToggles.alertingNotificationHistoryGlobal,s=r&&B.tab==="notifications"?"notifications":"alert_events";return(0,t.jsxs)(d.d,{navId:"alerts-history",isLoading:!1,children:[r&&(0,t.jsxs)(f.U,{children:[(0,t.jsx)(o.o,{label:(0,e.t)("alerting.central-alert-history.tab.alert-events","Alert events"),active:s==="alert_events",onChangeTab:()=>l({tab:"alert_events"})}),(0,t.jsx)(o.o,{label:(0,e.t)("alerting.central-alert-history.tab.notifications","Notifications"),active:s==="notifications",onChangeTab:()=>l({tab:"notifications"})})]}),(0,t.jsxs)(P.J,{className:n.tabContent,children:[(0,t.jsx)(g.a,{paddingBottom:2,children:(0,t.jsx)(c.E,{color:"secondary",variant:"body",children:s==="alert_events"?(0,t.jsx)(e.x6,{i18nKey:"alerting.central-alert-history.description.alert-events",children:"View a history of all alert events generated by your Grafana-managed alert rules. All alert events are displayed regardless of whether silences or mute timings are set."}):(0,t.jsx)(e.x6,{i18nKey:"alerting.central-alert-history.description.notifications",children:"View a history of all notifications sent from your Grafana-managed alert rules."})})}),s==="alert_events"&&(0,t.jsx)(y.CentralAlertHistoryScene,{}),s==="notifications"&&r&&(0,t.jsx)(O.tE,{})]})]})}const A=n=>({tabContent:(0,_.css)({marginTop:n.spacing(2)})}),T=(0,M.S)(h)}}]);

//# sourceMappingURL=HistoryPage-react19.a11bd17d105baee4826f.js.map