"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[7018],{47018(w,g,e){e.d(g,{GA4EchoBackend:()=>o});var i=e(78646),s=e(25240);class o{constructor(a){this.options=a,this.supportedEvents=[i.on.Pageview],this.googleAnalytics4SendManualPageViews=!1,this.addEvent=t=>{window.gtag&&this.googleAnalytics4SendManualPageViews&&window.gtag("event","page_view",{page_path:t.payload.page})},this.flush=()=>{};const d=`https://www.googletagmanager.com/gtag/js?id=${a.googleAnalyticsId}`;(0,s.k0)(d,!0),window.dataLayer=window.dataLayer||[],window.gtag=function(){window.dataLayer.push(arguments)},window.gtag("js",new Date);const n={page_path:window.location.pathname};a.user&&(n.user_id=a.user.analytics.identifier),this.googleAnalytics4SendManualPageViews=a.googleAnalytics4SendManualPageViews,window.gtag("config",a.googleAnalyticsId,n)}}}}]);

//# sourceMappingURL=7018-react19.3b8f418f8a63eecf5357.js.map