"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[3963],{67023(r,e){/**
 * @license React
 * react-jsx-dev-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var a=Symbol.for("react.fragment");e.Fragment=a,e.jsxDEV=void 0},63963(r,e,a){r.exports=a(67023)}}]);

//# sourceMappingURL=3963-react19.de9c838c9aba7f8b9d3e.js.map