"use strict";(globalThis.webpackChunktempo=globalThis.webpackChunktempo||[]).push([[6948],{6948(e,a,t){t.r(a),t.d(a,{default:()=>i});var r=t(6308),s=t(8531),n=t(2007);function i(){return(0,s.reportInteraction)("grafana_traces_cheatsheet_clicked",{datasourceType:"tempo",grafana_version:s.config.buildInfo.version}),(0,r.jsxs)(r.Fragment,{children:[(0,r.jsx)("h2",{children:"Tempo Cheat Sheet"}),(0,r.jsxs)("p",{children:[(0,r.jsx)(n.TextLink,{href:"https://grafana.com/docs/tempo/latest/",external:!0,children:"Grafana Tempo"})," ","is an open source, easy-to-use, and high-volume distributed tracing backend."]}),(0,r.jsxs)("p",{children:["Tempo implements"," ",(0,r.jsx)(n.TextLink,{href:"https://grafana.com/docs/tempo/latest/traceql",external:!0,children:"TraceQL"}),", a traces-first query language inspired by LogQL and PromQL. This query language allows users to precisely and easily select spans and jump directly to the spans fulfilling the specified conditions."]}),(0,r.jsxs)("p",{children:["You can compose TraceQL queries using either the Search tab (the TraceQL query builder) or the TraceQL tab (the TraceQL query editor). Both of these methods let you build queries and drill-down into result sets. (",(0,r.jsx)(n.TextLink,{href:"https://grafana.com/docs/grafana/latest/datasources/tempo/query-editor/",external:!0,children:"Learn more"}),")"]})]})}}}]);
//# sourceMappingURL=6948.js.map