"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[7759],{27906(C,t,a){a.r(t),a.d(t,{BookmarksPage:()=>r,default:()=>v});var s=a(14553),g=a(22803),_=a(92745),i=a(50201),l=a(63142),p=a(5071),P=a(30362),c=a(21234),e=a(13791),m=a(52763);function r(){const o=(0,l.of)(M),D=(0,p.s)(),O=(0,m.d4)(n=>n.navBarTree),E=D.reduce((n,u)=>{const d=(0,P.Wk)(O,u);return d&&n.push(d),n},[]);return(0,s.jsx)(e.Y,{navId:"bookmarks",children:(0,s.jsx)(e.Y.Contents,{children:E.length===0?(0,s.jsx)(i.p,{variant:"call-to-action",message:(0,_.t)("bookmarks-page.empty.message","It looks like you haven\u2019t created any bookmarks yet"),children:(0,s.jsx)(_.x6,{i18nKey:"bookmarks-page.empty.tip",children:"Hover over any item in the nav menu and click on the bookmark icon to add it here."})}):(0,s.jsx)("section",{className:o.grid,children:E.map(n=>(0,s.jsx)(c.I,{description:n.subTitle,text:n.text,url:n.url??""},n.id||n.url))})})})}const M=o=>({grid:(0,g.css)({display:"grid",gap:o.spacing(3),gridTemplateColumns:"repeat(auto-fill, minmax(300px, 1fr))",gridAutoRows:"138px",padding:o.spacing(2,0)})}),v=r}}]);

//# sourceMappingURL=BookmarksPage-react19.33d40ff6cb93b7b8f74c.js.map