"use strict";(globalThis.webpackChunkmysql=globalThis.webpackChunkmysql||[]).push([[360],{8360(e,t,o){o.r(t),o.d(t,{default:()=>s});var r={a:"α",b:"ḅ",c:"ͼ",d:"ḍ",e:"ḛ",f:"ϝ",g:"ḡ",h:"ḥ",i:"ḭ",j:"ĵ",k:"ḳ",l:"ḽ",m:"ṃ",n:"ṇ",o:"ṓ",p:"ṗ",q:"ʠ",r:"ṛ",s:"ṡ",t:"ţ",u:"ṵ",v:"ṽ",w:"ẁ",x:"ẋ",y:"ẏ",z:"ẓ",A:"À",B:"β",C:"Ḉ",D:"Ḍ",E:"Ḛ",F:"Ḟ",G:"Ḡ",H:"Ḥ",I:"Ḭ",J:"Ĵ",K:"Ḱ",L:"Ḻ",M:"Ṁ",N:"Ṅ",O:"Ṏ",P:"Ṕ",Q:"Ǫ",R:"Ṛ",S:"Ṣ",T:"Ṫ",U:"Ṳ",V:"Ṿ",W:"Ŵ",X:"Ẋ",Y:"Ŷ",Z:"Ż"},n=["a","e","i","o","u","y","A","E","I","O","U","Y"],i=Object.assign||function(e){for(var t=1;t<arguments.length;t++){var o=arguments[t];for(var r in o)Object.prototype.hasOwnProperty.call(o,r)&&(e[r]=o[r])}return e};var s=function(){function e(){var t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},o=t.languageToPseudo,i=void 0===o?"en":o,s=t.letterMultiplier,a=void 0===s?2:s,p=t.repeatedLetters,l=void 0===p?n:p,u=t.uglifedLetterObject,d=void 0===u?r:u,c=t.wrapped,h=void 0!==c&&c,g=t.enabled,f=void 0===g||g;!function(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}(this,e),this.name="pseudo",this.type="postProcessor",this.options={languageToPseudo:i,letterMultiplier:a,wrapped:h,repeatedLetters:l,letters:d,enabled:f}}return e.prototype.configurePseudo=function(e){this.options=i({},this.options,e)},e.prototype.process=function(e,t,o,r){var n=this;if(r.language&&this.options.languageToPseudo!==r.language||!this.options.enabled)return e;var i,s,a,p=0,l=e.split("").map((function(e){return"}"===e?(p=0,e):"{"===e?(p++,e):2===p?e:-1!==n.options.repeatedLetters.indexOf(e)?n.options.letters[e].repeat(n.options.letterMultiplier):n.options.letters[e]||e})).join("");return i={shouldWrap:this.options.wrapped,string:l},s=i.shouldWrap,a=i.string,s?"["+a+"]":a},e}()}}]);
//# sourceMappingURL=360.js.map