"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[7620],{33876(p,l,t){t.r(l),t.d(l,{PlaylistEditPage:()=>r,default:()=>P});var a=t(74848),e=t(54148),s=t(92745),E=t(36490),i=t(13791),n=t(45397),d=t(81425);const r=()=>{const{uid:y=""}=(0,e.g)(),{data:o,isLoading:c,isError:u,error:O}=(0,n.Uu)({name:y}),[g]=(0,n.Lv)(),D=async _=>{g({name:_.metadata?.name??"",playlist:_}),E.Ny.push("/playlists")},m={text:(0,s.t)("playlist-edit.title","Edit playlist"),subTitle:(0,s.t)("playlist-edit.sub-title","A playlist rotates through a pre-selected list of dashboards. A playlist can be a great way to build situational awareness, or just show off your metrics to your team or visitors.")};return(0,a.jsx)(i.Y,{navId:"dashboards/playlists",pageNav:m,children:(0,a.jsxs)(i.Y.Contents,{isLoading:c,children:[u&&(0,a.jsxs)("div",{children:[(0,a.jsx)(s.x6,{i18nKey:"playlist-edit.error-prefix",children:"Error loading playlist:"}),JSON.stringify(O)]}),o&&(0,a.jsx)(d.$,{onSubmit:D,playlist:o})]})})},P=r}}]);

//# sourceMappingURL=PlaylistEditPage.43c1adf7992bd1bb6358.js.map