"use strict";(globalThis.webpackChunkmssql=globalThis.webpackChunkmssql||[]).push([[454],{2454(e){e.exports=JSON.parse('{"grafana-sql":{"components":{"confirm-modal":{"builder-mode":"O modo de construção não exibe as alterações feitas no código. O construtor de consultas exibirá as últimas alterações feitas no modo de construção.","cancel":"Cancelar","clipboard":"Deseja copiar seu código para a área de transferência?","copy-code-and-switch":"Copiar código e trocar","discard-code-and-switch":"Descartar código e trocar","warning":"Aviso"},"connection-limits":{"auto-max-idle":"Inatividade máxima automática","content-auto-max-idle":"","content-max-idle":"O número máximo de conexões no grupo de conexões ociosas. Se o <i>Máximo de conexões abertas</i> for maior que 0, mas menor que o <i>Máximo de conexões ociosas</i>, o <i>Máximo de conexões ociosas</i> será reduzido para corresponder ao limite do <i>Máximo de conexões abertas</i>. Se estiver definido como 0, nenhuma conexão ociosa será mantida.","content-max-lifetime":"A quantidade máxima de tempo em segundos que uma conexão pode ser reutilizada. Se estiver definido como 0, as conexões serão reutilizadas indefinidamente.","content-max-open":"","max-idle":"Máximo de ociosidade","max-lifetime":"Tempo de vida máximo","max-open":"Máximo de aberturas","title-connection-limits":"Limites de conexão"},"dataset-selector":{"aria-label-dataset-selector":"Seletor de conjunto de dados"},"error-boundary":{"fall-back":{"error":"Erro"}},"get-custom-operators":{"custom-operators":{"label":{"macros":"Macros"}}},"make-render-column":{"render-column":{"aria-label-group-by":"Agrupar por","title-remove-group-by-column":"Remover grupo por coluna"}},"order-by-row":{"aria-label-order-by":"Ordenar por","label-limit":"Limite","label-offset":"Desvio","label-order-by":"Ordenar por"},"preview":{"label-element":{"preview":"Visualizar","tooltip-copy-to-clipboard":"Copiar para a área de transferência"}},"query-header":{"content-invalid-query":"","editor-modes":{"label-builder":"Construtor","label-code":"Código"},"label-dataset":"Conjunto de dados","label-filter":"Filtro","label-format":"Formato","label-group":"Grupo","label-order":"Ordem","label-preview":"Visualizar","label-table":"Tabela","placeholder-select-format":"Selecionar formato","run-query":"Executar consulta"},"query-meta":{"variables":{"textField":"Campo de texto","valueField":"Campo de valor"}},"query-toolbox":{"content-hit-ctrlcmdreturn-to-run-query":"Pressione CTRL/CMD+Return para executar a consulta","tooltip-collapse":"Recolher editor","tooltip-expand":"Expandir editor","tooltip-format-query":"Formatar consulta"},"query-validator":{"query-will-process":"<0></0> Esta consulta processará <strong>{{bytes}}</strong> ao ser executada.","validating-query":"Validando consulta…"},"raw-editor":{"render-placeholder":{"editing-in-expanded-code-editor":"Editando no editor de código expandido"},"title-query-num":"Consulta {{queryNum}}"},"select-column":{"label-column":"Coluna"},"select-custom-function-parameters":{"aria-label-parameter":"Parâmetro {{index}} para coluna {{columnIndex}}","render-parameters":{"params":{"title-remove-parameter":"Remover parâmetro"}},"title-add-parameter":"Adicionar parâmetro"},"select-row":{"aggregate-options":{"options":{"label":{"aggregations":"Agregações","macros":"Macros"}}},"label":{"time":"tempo","value":"valor"},"label-alias":"Apelido","label-data-operations":"Operações de dados","title-add-column":"Adicionar coluna","title-remove-column":"Remover coluna"},"settings":{"aria-label-conjunction":"Conjunção","aria-label-field":"Campo","aria-label-operator":"Operador","title-button-filter":"Filtro {{ buttonLabel }}"},"sql-query-editor-lazy":{"text-loading-editor":"Carregando editor..."},"table-selector":{"aria-label-table-selector":"Seletor de tabela","placeholder-loading":"Carregando tabelas","placeholder-select-table":"Selecionar tabela"},"tlssecrets-config":{"content-tlsssl-client-certificate":"Para realizar a autenticação com um certificado de cliente TLS/SSL, insira o certificado de cliente aqui.","content-tlsssl-client-key":"Para realizar a autenticação com um certificado TLS/SSL de cliente, insira a chave aqui.","content-tlsssl-root-certificate":"Se o modo TLS/SSL selecionado exigir um certificado raiz do servidor, insira-o aqui","tlsssl-client-certificate":"Certificado de cliente TLS/SSL","tlsssl-client-key":"Chave de cliente TLS/SSL","tlsssl-root-certificate":"Certificado raiz TLS/SSL"},"visual-editor":{"label-filter-by-column-value":"Filtrar por valor da coluna","label-group-by-column":"Agrupar por coluna"},"widgets":{"aria-label-macros-value-selector":"Seletor de valor de macros"}},"utils":{"get-columns-width-indices":{"label-selected-columns":"Colunas selecionadas"}}}}')}}]);