"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[4857],{4857(h,e,s){s.d(e,{PerformanceBackend:()=>a});var r=s(78646),t=s(83873);class a{constructor(n){this.options=n,this.buffer=[],this.supportedEvents=[r.on.Performance],this.addEvent=f=>{this.buffer.push(f.payload)},this.flush=()=>{this.buffer.length!==0&&(t.IB.post("/api/frontend-metrics",{events:this.buffer},{showErrorAlert:!1}).catch(()=>{}),this.buffer=[])}}}}}]);

//# sourceMappingURL=4857-react19.cfdae62e6ae760747558.js.map