"use strict";(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[9136],{75103(_,e,a){_.exports=a.p+"static/img/dashboard_library_1.93b7e752.jpg"},7728(_,e,a){_.exports=a.p+"static/img/dashboard_library_2.e540bb1d.jpg"},52621(_,e,a){_.exports=a.p+"static/img/dashboard_library_3.8f41d293.jpg"},36422(_,e,a){_.exports=a.p+"static/img/dashboard_library_4.0a1e811b.jpg"},26531(_,e,a){_.exports=a.p+"static/img/dashboard_library_5.7f4bc831.jpg"},96820(_,e,a){_.exports=a.p+"static/img/dashboard_library_6.cf5e2209.jpg"},20230(_,e,a){_.exports=a.p+"static/img/grab_dark.4ee8d28b.svg"},5878(_,e,a){_.exports=a.p+"static/img/grab_light.322c3334.svg"},12326(_,e,a){_.exports=a.p+"static/img/grafana_text_logo_dark.70a1e5ee.svg"},71542(_,e,a){_.exports=a.p+"static/img/grafana_text_logo_light.467e047c.svg"}}]);

//# sourceMappingURL=DashboardPageProxy.d03872e39f54608bfd62.js.map