(self.webpackChunkgrafana=self.webpackChunkgrafana||[]).push([[6029],{25177(L,f,c){//! moment.js locale configuration
//! locale : Afrikaans [af]
//! author : Werner Mollentze : https://github.com/wernerm
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("af",{months:"Januarie_Februarie_Maart_April_Mei_Junie_Julie_Augustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mrt_Apr_Mei_Jun_Jul_Aug_Sep_Okt_Nov_Des".split("_"),weekdays:"Sondag_Maandag_Dinsdag_Woensdag_Donderdag_Vrydag_Saterdag".split("_"),weekdaysShort:"Son_Maa_Din_Woe_Don_Vry_Sat".split("_"),weekdaysMin:"So_Ma_Di_Wo_Do_Vr_Sa".split("_"),meridiemParse:/vm|nm/i,isPM:function(r){return/^nm$/i.test(r)},meridiem:function(r,s,n){return r<12?n?"vm":"VM":n?"nm":"NM"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Vandag om] LT",nextDay:"[M\xF4re om] LT",nextWeek:"dddd [om] LT",lastDay:"[Gister om] LT",lastWeek:"[Laas] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oor %s",past:"%s gelede",s:"'n paar sekondes",ss:"%d sekondes",m:"'n minuut",mm:"%d minute",h:"'n uur",hh:"%d ure",d:"'n dag",dd:"%d dae",M:"'n maand",MM:"%d maande",y:"'n jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(r){return r+(r===1||r===8||r>=20?"ste":"de")},week:{dow:1,doy:4}});return a})},41488(L,f,c){//! moment.js locale configuration
//! locale : Arabic (Algeria) [ar-dz]
//! author : Amine Roukh: https://github.com/Amine27
//! author : Abdel Said: https://github.com/abdelsaid
//! author : Ahmed Elkhatib
//! author : forabi https://github.com/forabi
//! author : Noureddine LOUAHEDJ : https://github.com/noureddinem
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=function(u){return u===0?0:u===1?1:u===2?2:u%100>=3&&u%100<=10?3:u%100>=11?4:5},r={s:["\u0623\u0642\u0644 \u0645\u0646 \u062B\u0627\u0646\u064A\u0629","\u062B\u0627\u0646\u064A\u0629 \u0648\u0627\u062D\u062F\u0629",["\u062B\u0627\u0646\u064A\u062A\u0627\u0646","\u062B\u0627\u0646\u064A\u062A\u064A\u0646"],"%d \u062B\u0648\u0627\u0646","%d \u062B\u0627\u0646\u064A\u0629","%d \u062B\u0627\u0646\u064A\u0629"],m:["\u0623\u0642\u0644 \u0645\u0646 \u062F\u0642\u064A\u0642\u0629","\u062F\u0642\u064A\u0642\u0629 \u0648\u0627\u062D\u062F\u0629",["\u062F\u0642\u064A\u0642\u062A\u0627\u0646","\u062F\u0642\u064A\u0642\u062A\u064A\u0646"],"%d \u062F\u0642\u0627\u0626\u0642","%d \u062F\u0642\u064A\u0642\u0629","%d \u062F\u0642\u064A\u0642\u0629"],h:["\u0623\u0642\u0644 \u0645\u0646 \u0633\u0627\u0639\u0629","\u0633\u0627\u0639\u0629 \u0648\u0627\u062D\u062F\u0629",["\u0633\u0627\u0639\u062A\u0627\u0646","\u0633\u0627\u0639\u062A\u064A\u0646"],"%d \u0633\u0627\u0639\u0627\u062A","%d \u0633\u0627\u0639\u0629","%d \u0633\u0627\u0639\u0629"],d:["\u0623\u0642\u0644 \u0645\u0646 \u064A\u0648\u0645","\u064A\u0648\u0645 \u0648\u0627\u062D\u062F",["\u064A\u0648\u0645\u0627\u0646","\u064A\u0648\u0645\u064A\u0646"],"%d \u0623\u064A\u0627\u0645","%d \u064A\u0648\u0645\u064B\u0627","%d \u064A\u0648\u0645"],M:["\u0623\u0642\u0644 \u0645\u0646 \u0634\u0647\u0631","\u0634\u0647\u0631 \u0648\u0627\u062D\u062F",["\u0634\u0647\u0631\u0627\u0646","\u0634\u0647\u0631\u064A\u0646"],"%d \u0623\u0634\u0647\u0631","%d \u0634\u0647\u0631\u0627","%d \u0634\u0647\u0631"],y:["\u0623\u0642\u0644 \u0645\u0646 \u0639\u0627\u0645","\u0639\u0627\u0645 \u0648\u0627\u062D\u062F",["\u0639\u0627\u0645\u0627\u0646","\u0639\u0627\u0645\u064A\u0646"],"%d \u0623\u0639\u0648\u0627\u0645","%d \u0639\u0627\u0645\u064B\u0627","%d \u0639\u0627\u0645"]},s=function(u){return function(m,h,y,g){var v=a(m),H=r[u][a(m)];return v===2&&(H=H[h?0:1]),H.replace(/%d/i,m)}},n=["\u062C\u0627\u0646\u0641\u064A","\u0641\u064A\u0641\u0631\u064A","\u0645\u0627\u0631\u0633","\u0623\u0641\u0631\u064A\u0644","\u0645\u0627\u064A","\u062C\u0648\u0627\u0646","\u062C\u0648\u064A\u0644\u064A\u0629","\u0623\u0648\u062A","\u0633\u0628\u062A\u0645\u0628\u0631","\u0623\u0643\u062A\u0648\u0628\u0631","\u0646\u0648\u0641\u0645\u0628\u0631","\u062F\u064A\u0633\u0645\u0628\u0631"],d=o.defineLocale("ar-dz",{months:n,monthsShort:n,weekdays:"\u0627\u0644\u0623\u062D\u062F_\u0627\u0644\u0625\u062B\u0646\u064A\u0646_\u0627\u0644\u062B\u0644\u0627\u062B\u0627\u0621_\u0627\u0644\u0623\u0631\u0628\u0639\u0627\u0621_\u0627\u0644\u062E\u0645\u064A\u0633_\u0627\u0644\u062C\u0645\u0639\u0629_\u0627\u0644\u0633\u0628\u062A".split("_"),weekdaysShort:"\u0623\u062D\u062F_\u0625\u062B\u0646\u064A\u0646_\u062B\u0644\u0627\u062B\u0627\u0621_\u0623\u0631\u0628\u0639\u0627\u0621_\u062E\u0645\u064A\u0633_\u062C\u0645\u0639\u0629_\u0633\u0628\u062A".split("_"),weekdaysMin:"\u062D_\u0646_\u062B_\u0631_\u062E_\u062C_\u0633".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/\u200FM/\u200FYYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(u){return u==="\u0645"},meridiem:function(u,m,h){return u<12?"\u0635":"\u0645"},calendar:{sameDay:"[\u0627\u0644\u064A\u0648\u0645 \u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextDay:"[\u063A\u062F\u064B\u0627 \u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextWeek:"dddd [\u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastDay:"[\u0623\u0645\u0633 \u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastWeek:"dddd [\u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",sameElse:"L"},relativeTime:{future:"\u0628\u0639\u062F %s",past:"\u0645\u0646\u0630 %s",s:s("s"),ss:s("s"),m:s("m"),mm:s("m"),h:s("h"),hh:s("h"),d:s("d"),dd:s("d"),M:s("M"),MM:s("M"),y:s("y"),yy:s("y")},postformat:function(u){return u.replace(/,/g,"\u060C")},week:{dow:0,doy:4}});return d})},58676(L,f,c){//! moment.js locale configuration
//! locale : Arabic (Kuwait) [ar-kw]
//! author : Nusret Parlak: https://github.com/nusretparlak
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ar-kw",{months:"\u064A\u0646\u0627\u064A\u0631_\u0641\u0628\u0631\u0627\u064A\u0631_\u0645\u0627\u0631\u0633_\u0623\u0628\u0631\u064A\u0644_\u0645\u0627\u064A_\u064A\u0648\u0646\u064A\u0648_\u064A\u0648\u0644\u064A\u0648\u0632_\u063A\u0634\u062A_\u0634\u062A\u0646\u0628\u0631_\u0623\u0643\u062A\u0648\u0628\u0631_\u0646\u0648\u0646\u0628\u0631_\u062F\u062C\u0646\u0628\u0631".split("_"),monthsShort:"\u064A\u0646\u0627\u064A\u0631_\u0641\u0628\u0631\u0627\u064A\u0631_\u0645\u0627\u0631\u0633_\u0623\u0628\u0631\u064A\u0644_\u0645\u0627\u064A_\u064A\u0648\u0646\u064A\u0648_\u064A\u0648\u0644\u064A\u0648\u0632_\u063A\u0634\u062A_\u0634\u062A\u0646\u0628\u0631_\u0623\u0643\u062A\u0648\u0628\u0631_\u0646\u0648\u0646\u0628\u0631_\u062F\u062C\u0646\u0628\u0631".split("_"),weekdays:"\u0627\u0644\u0623\u062D\u062F_\u0627\u0644\u0625\u062A\u0646\u064A\u0646_\u0627\u0644\u062B\u0644\u0627\u062B\u0627\u0621_\u0627\u0644\u0623\u0631\u0628\u0639\u0627\u0621_\u0627\u0644\u062E\u0645\u064A\u0633_\u0627\u0644\u062C\u0645\u0639\u0629_\u0627\u0644\u0633\u0628\u062A".split("_"),weekdaysShort:"\u0627\u062D\u062F_\u0627\u062A\u0646\u064A\u0646_\u062B\u0644\u0627\u062B\u0627\u0621_\u0627\u0631\u0628\u0639\u0627\u0621_\u062E\u0645\u064A\u0633_\u062C\u0645\u0639\u0629_\u0633\u0628\u062A".split("_"),weekdaysMin:"\u062D_\u0646_\u062B_\u0631_\u062E_\u062C_\u0633".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[\u0627\u0644\u064A\u0648\u0645 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextDay:"[\u063A\u062F\u0627 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextWeek:"dddd [\u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastDay:"[\u0623\u0645\u0633 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastWeek:"dddd [\u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",sameElse:"L"},relativeTime:{future:"\u0641\u064A %s",past:"\u0645\u0646\u0630 %s",s:"\u062B\u0648\u0627\u0646",ss:"%d \u062B\u0627\u0646\u064A\u0629",m:"\u062F\u0642\u064A\u0642\u0629",mm:"%d \u062F\u0642\u0627\u0626\u0642",h:"\u0633\u0627\u0639\u0629",hh:"%d \u0633\u0627\u0639\u0627\u062A",d:"\u064A\u0648\u0645",dd:"%d \u0623\u064A\u0627\u0645",M:"\u0634\u0647\u0631",MM:"%d \u0623\u0634\u0647\u0631",y:"\u0633\u0646\u0629",yy:"%d \u0633\u0646\u0648\u0627\u062A"},week:{dow:0,doy:12}});return a})},42353(L,f,c){//! moment.js locale configuration
//! locale : Arabic (Libya) [ar-ly]
//! author : Ali Hmer: https://github.com/kikoanis
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"1",2:"2",3:"3",4:"4",5:"5",6:"6",7:"7",8:"8",9:"9",0:"0"},r=function(m){return m===0?0:m===1?1:m===2?2:m%100>=3&&m%100<=10?3:m%100>=11?4:5},s={s:["\u0623\u0642\u0644 \u0645\u0646 \u062B\u0627\u0646\u064A\u0629","\u062B\u0627\u0646\u064A\u0629 \u0648\u0627\u062D\u062F\u0629",["\u062B\u0627\u0646\u064A\u062A\u0627\u0646","\u062B\u0627\u0646\u064A\u062A\u064A\u0646"],"%d \u062B\u0648\u0627\u0646","%d \u062B\u0627\u0646\u064A\u0629","%d \u062B\u0627\u0646\u064A\u0629"],m:["\u0623\u0642\u0644 \u0645\u0646 \u062F\u0642\u064A\u0642\u0629","\u062F\u0642\u064A\u0642\u0629 \u0648\u0627\u062D\u062F\u0629",["\u062F\u0642\u064A\u0642\u062A\u0627\u0646","\u062F\u0642\u064A\u0642\u062A\u064A\u0646"],"%d \u062F\u0642\u0627\u0626\u0642","%d \u062F\u0642\u064A\u0642\u0629","%d \u062F\u0642\u064A\u0642\u0629"],h:["\u0623\u0642\u0644 \u0645\u0646 \u0633\u0627\u0639\u0629","\u0633\u0627\u0639\u0629 \u0648\u0627\u062D\u062F\u0629",["\u0633\u0627\u0639\u062A\u0627\u0646","\u0633\u0627\u0639\u062A\u064A\u0646"],"%d \u0633\u0627\u0639\u0627\u062A","%d \u0633\u0627\u0639\u0629","%d \u0633\u0627\u0639\u0629"],d:["\u0623\u0642\u0644 \u0645\u0646 \u064A\u0648\u0645","\u064A\u0648\u0645 \u0648\u0627\u062D\u062F",["\u064A\u0648\u0645\u0627\u0646","\u064A\u0648\u0645\u064A\u0646"],"%d \u0623\u064A\u0627\u0645","%d \u064A\u0648\u0645\u064B\u0627","%d \u064A\u0648\u0645"],M:["\u0623\u0642\u0644 \u0645\u0646 \u0634\u0647\u0631","\u0634\u0647\u0631 \u0648\u0627\u062D\u062F",["\u0634\u0647\u0631\u0627\u0646","\u0634\u0647\u0631\u064A\u0646"],"%d \u0623\u0634\u0647\u0631","%d \u0634\u0647\u0631\u0627","%d \u0634\u0647\u0631"],y:["\u0623\u0642\u0644 \u0645\u0646 \u0639\u0627\u0645","\u0639\u0627\u0645 \u0648\u0627\u062D\u062F",["\u0639\u0627\u0645\u0627\u0646","\u0639\u0627\u0645\u064A\u0646"],"%d \u0623\u0639\u0648\u0627\u0645","%d \u0639\u0627\u0645\u064B\u0627","%d \u0639\u0627\u0645"]},n=function(m){return function(h,y,g,v){var H=r(h),Q=s[m][r(h)];return H===2&&(Q=Q[y?0:1]),Q.replace(/%d/i,h)}},d=["\u064A\u0646\u0627\u064A\u0631","\u0641\u0628\u0631\u0627\u064A\u0631","\u0645\u0627\u0631\u0633","\u0623\u0628\u0631\u064A\u0644","\u0645\u0627\u064A\u0648","\u064A\u0648\u0646\u064A\u0648","\u064A\u0648\u0644\u064A\u0648","\u0623\u063A\u0633\u0637\u0633","\u0633\u0628\u062A\u0645\u0628\u0631","\u0623\u0643\u062A\u0648\u0628\u0631","\u0646\u0648\u0641\u0645\u0628\u0631","\u062F\u064A\u0633\u0645\u0628\u0631"],u=o.defineLocale("ar-ly",{months:d,monthsShort:d,weekdays:"\u0627\u0644\u0623\u062D\u062F_\u0627\u0644\u0625\u062B\u0646\u064A\u0646_\u0627\u0644\u062B\u0644\u0627\u062B\u0627\u0621_\u0627\u0644\u0623\u0631\u0628\u0639\u0627\u0621_\u0627\u0644\u062E\u0645\u064A\u0633_\u0627\u0644\u062C\u0645\u0639\u0629_\u0627\u0644\u0633\u0628\u062A".split("_"),weekdaysShort:"\u0623\u062D\u062F_\u0625\u062B\u0646\u064A\u0646_\u062B\u0644\u0627\u062B\u0627\u0621_\u0623\u0631\u0628\u0639\u0627\u0621_\u062E\u0645\u064A\u0633_\u062C\u0645\u0639\u0629_\u0633\u0628\u062A".split("_"),weekdaysMin:"\u062D_\u0646_\u062B_\u0631_\u062E_\u062C_\u0633".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/\u200FM/\u200FYYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(m){return m==="\u0645"},meridiem:function(m,h,y){return m<12?"\u0635":"\u0645"},calendar:{sameDay:"[\u0627\u0644\u064A\u0648\u0645 \u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextDay:"[\u063A\u062F\u064B\u0627 \u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextWeek:"dddd [\u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastDay:"[\u0623\u0645\u0633 \u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastWeek:"dddd [\u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",sameElse:"L"},relativeTime:{future:"\u0628\u0639\u062F %s",past:"\u0645\u0646\u0630 %s",s:n("s"),ss:n("s"),m:n("m"),mm:n("m"),h:n("h"),hh:n("h"),d:n("d"),dd:n("d"),M:n("M"),MM:n("M"),y:n("y"),yy:n("y")},preparse:function(m){return m.replace(/،/g,",")},postformat:function(m){return m.replace(/\d/g,function(h){return a[h]}).replace(/,/g,"\u060C")},week:{dow:6,doy:12}});return u})},24496(L,f,c){//! moment.js locale configuration
//! locale : Arabic (Morocco) [ar-ma]
//! author : ElFadili Yassine : https://github.com/ElFadiliY
//! author : Abdel Said : https://github.com/abdelsaid
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ar-ma",{months:"\u064A\u0646\u0627\u064A\u0631_\u0641\u0628\u0631\u0627\u064A\u0631_\u0645\u0627\u0631\u0633_\u0623\u0628\u0631\u064A\u0644_\u0645\u0627\u064A_\u064A\u0648\u0646\u064A\u0648_\u064A\u0648\u0644\u064A\u0648\u0632_\u063A\u0634\u062A_\u0634\u062A\u0646\u0628\u0631_\u0623\u0643\u062A\u0648\u0628\u0631_\u0646\u0648\u0646\u0628\u0631_\u062F\u062C\u0646\u0628\u0631".split("_"),monthsShort:"\u064A\u0646\u0627\u064A\u0631_\u0641\u0628\u0631\u0627\u064A\u0631_\u0645\u0627\u0631\u0633_\u0623\u0628\u0631\u064A\u0644_\u0645\u0627\u064A_\u064A\u0648\u0646\u064A\u0648_\u064A\u0648\u0644\u064A\u0648\u0632_\u063A\u0634\u062A_\u0634\u062A\u0646\u0628\u0631_\u0623\u0643\u062A\u0648\u0628\u0631_\u0646\u0648\u0646\u0628\u0631_\u062F\u062C\u0646\u0628\u0631".split("_"),weekdays:"\u0627\u0644\u0623\u062D\u062F_\u0627\u0644\u0625\u062B\u0646\u064A\u0646_\u0627\u0644\u062B\u0644\u0627\u062B\u0627\u0621_\u0627\u0644\u0623\u0631\u0628\u0639\u0627\u0621_\u0627\u0644\u062E\u0645\u064A\u0633_\u0627\u0644\u062C\u0645\u0639\u0629_\u0627\u0644\u0633\u0628\u062A".split("_"),weekdaysShort:"\u0627\u062D\u062F_\u0627\u062B\u0646\u064A\u0646_\u062B\u0644\u0627\u062B\u0627\u0621_\u0627\u0631\u0628\u0639\u0627\u0621_\u062E\u0645\u064A\u0633_\u062C\u0645\u0639\u0629_\u0633\u0628\u062A".split("_"),weekdaysMin:"\u062D_\u0646_\u062B_\u0631_\u062E_\u062C_\u0633".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[\u0627\u0644\u064A\u0648\u0645 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextDay:"[\u063A\u062F\u0627 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextWeek:"dddd [\u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastDay:"[\u0623\u0645\u0633 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastWeek:"dddd [\u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",sameElse:"L"},relativeTime:{future:"\u0641\u064A %s",past:"\u0645\u0646\u0630 %s",s:"\u062B\u0648\u0627\u0646",ss:"%d \u062B\u0627\u0646\u064A\u0629",m:"\u062F\u0642\u064A\u0642\u0629",mm:"%d \u062F\u0642\u0627\u0626\u0642",h:"\u0633\u0627\u0639\u0629",hh:"%d \u0633\u0627\u0639\u0627\u062A",d:"\u064A\u0648\u0645",dd:"%d \u0623\u064A\u0627\u0645",M:"\u0634\u0647\u0631",MM:"%d \u0623\u0634\u0647\u0631",y:"\u0633\u0646\u0629",yy:"%d \u0633\u0646\u0648\u0627\u062A"},week:{dow:1,doy:4}});return a})},6947(L,f,c){//! moment.js locale configuration
//! locale : Arabic (Palestine) [ar-ps]
//! author : Majd Al-Shihabi : https://github.com/majdal
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0661",2:"\u0662",3:"\u0663",4:"\u0664",5:"\u0665",6:"\u0666",7:"\u0667",8:"\u0668",9:"\u0669",0:"\u0660"},r={"\u0661":"1","\u0662":"2","\u0663":"3","\u0664":"4","\u0665":"5","\u0666":"6","\u0667":"7","\u0668":"8","\u0669":"9","\u0660":"0"},s=o.defineLocale("ar-ps",{months:"\u0643\u0627\u0646\u0648\u0646 \u0627\u0644\u062B\u0627\u0646\u064A_\u0634\u0628\u0627\u0637_\u0622\u0630\u0627\u0631_\u0646\u064A\u0633\u0627\u0646_\u0623\u064A\u0651\u0627\u0631_\u062D\u0632\u064A\u0631\u0627\u0646_\u062A\u0645\u0651\u0648\u0632_\u0622\u0628_\u0623\u064A\u0644\u0648\u0644_\u062A\u0634\u0631\u064A \u0627\u0644\u0623\u0648\u0651\u0644_\u062A\u0634\u0631\u064A\u0646 \u0627\u0644\u062B\u0627\u0646\u064A_\u0643\u0627\u0646\u0648\u0646 \u0627\u0644\u0623\u0648\u0651\u0644".split("_"),monthsShort:"\u0643\u0662_\u0634\u0628\u0627\u0637_\u0622\u0630\u0627\u0631_\u0646\u064A\u0633\u0627\u0646_\u0623\u064A\u0651\u0627\u0631_\u062D\u0632\u064A\u0631\u0627\u0646_\u062A\u0645\u0651\u0648\u0632_\u0622\u0628_\u0623\u064A\u0644\u0648\u0644_\u062A\u0661_\u062A\u0662_\u0643\u0661".split("_"),weekdays:"\u0627\u0644\u0623\u062D\u062F_\u0627\u0644\u0625\u062B\u0646\u064A\u0646_\u0627\u0644\u062B\u0644\u0627\u062B\u0627\u0621_\u0627\u0644\u0623\u0631\u0628\u0639\u0627\u0621_\u0627\u0644\u062E\u0645\u064A\u0633_\u0627\u0644\u062C\u0645\u0639\u0629_\u0627\u0644\u0633\u0628\u062A".split("_"),weekdaysShort:"\u0623\u062D\u062F_\u0625\u062B\u0646\u064A\u0646_\u062B\u0644\u0627\u062B\u0627\u0621_\u0623\u0631\u0628\u0639\u0627\u0621_\u062E\u0645\u064A\u0633_\u062C\u0645\u0639\u0629_\u0633\u0628\u062A".split("_"),weekdaysMin:"\u062D_\u0646_\u062B_\u0631_\u062E_\u062C_\u0633".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(n){return n==="\u0645"},meridiem:function(n,d,u){return n<12?"\u0635":"\u0645"},calendar:{sameDay:"[\u0627\u0644\u064A\u0648\u0645 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextDay:"[\u063A\u062F\u0627 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextWeek:"dddd [\u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastDay:"[\u0623\u0645\u0633 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastWeek:"dddd [\u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",sameElse:"L"},relativeTime:{future:"\u0641\u064A %s",past:"\u0645\u0646\u0630 %s",s:"\u062B\u0648\u0627\u0646",ss:"%d \u062B\u0627\u0646\u064A\u0629",m:"\u062F\u0642\u064A\u0642\u0629",mm:"%d \u062F\u0642\u0627\u0626\u0642",h:"\u0633\u0627\u0639\u0629",hh:"%d \u0633\u0627\u0639\u0627\u062A",d:"\u064A\u0648\u0645",dd:"%d \u0623\u064A\u0627\u0645",M:"\u0634\u0647\u0631",MM:"%d \u0623\u0634\u0647\u0631",y:"\u0633\u0646\u0629",yy:"%d \u0633\u0646\u0648\u0627\u062A"},preparse:function(n){return n.replace(/[٣٤٥٦٧٨٩٠]/g,function(d){return r[d]}).split("").reverse().join("").replace(/[١٢](?![\u062a\u0643])/g,function(d){return r[d]}).split("").reverse().join("").replace(/،/g,",")},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]}).replace(/,/g,"\u060C")},week:{dow:0,doy:6}});return s})},82682(L,f,c){//! moment.js locale configuration
//! locale : Arabic (Saudi Arabia) [ar-sa]
//! author : Suhail Alkowaileet : https://github.com/xsoh
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0661",2:"\u0662",3:"\u0663",4:"\u0664",5:"\u0665",6:"\u0666",7:"\u0667",8:"\u0668",9:"\u0669",0:"\u0660"},r={"\u0661":"1","\u0662":"2","\u0663":"3","\u0664":"4","\u0665":"5","\u0666":"6","\u0667":"7","\u0668":"8","\u0669":"9","\u0660":"0"},s=o.defineLocale("ar-sa",{months:"\u064A\u0646\u0627\u064A\u0631_\u0641\u0628\u0631\u0627\u064A\u0631_\u0645\u0627\u0631\u0633_\u0623\u0628\u0631\u064A\u0644_\u0645\u0627\u064A\u0648_\u064A\u0648\u0646\u064A\u0648_\u064A\u0648\u0644\u064A\u0648_\u0623\u063A\u0633\u0637\u0633_\u0633\u0628\u062A\u0645\u0628\u0631_\u0623\u0643\u062A\u0648\u0628\u0631_\u0646\u0648\u0641\u0645\u0628\u0631_\u062F\u064A\u0633\u0645\u0628\u0631".split("_"),monthsShort:"\u064A\u0646\u0627\u064A\u0631_\u0641\u0628\u0631\u0627\u064A\u0631_\u0645\u0627\u0631\u0633_\u0623\u0628\u0631\u064A\u0644_\u0645\u0627\u064A\u0648_\u064A\u0648\u0646\u064A\u0648_\u064A\u0648\u0644\u064A\u0648_\u0623\u063A\u0633\u0637\u0633_\u0633\u0628\u062A\u0645\u0628\u0631_\u0623\u0643\u062A\u0648\u0628\u0631_\u0646\u0648\u0641\u0645\u0628\u0631_\u062F\u064A\u0633\u0645\u0628\u0631".split("_"),weekdays:"\u0627\u0644\u0623\u062D\u062F_\u0627\u0644\u0625\u062B\u0646\u064A\u0646_\u0627\u0644\u062B\u0644\u0627\u062B\u0627\u0621_\u0627\u0644\u0623\u0631\u0628\u0639\u0627\u0621_\u0627\u0644\u062E\u0645\u064A\u0633_\u0627\u0644\u062C\u0645\u0639\u0629_\u0627\u0644\u0633\u0628\u062A".split("_"),weekdaysShort:"\u0623\u062D\u062F_\u0625\u062B\u0646\u064A\u0646_\u062B\u0644\u0627\u062B\u0627\u0621_\u0623\u0631\u0628\u0639\u0627\u0621_\u062E\u0645\u064A\u0633_\u062C\u0645\u0639\u0629_\u0633\u0628\u062A".split("_"),weekdaysMin:"\u062D_\u0646_\u062B_\u0631_\u062E_\u062C_\u0633".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(n){return n==="\u0645"},meridiem:function(n,d,u){return n<12?"\u0635":"\u0645"},calendar:{sameDay:"[\u0627\u0644\u064A\u0648\u0645 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextDay:"[\u063A\u062F\u0627 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextWeek:"dddd [\u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastDay:"[\u0623\u0645\u0633 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastWeek:"dddd [\u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",sameElse:"L"},relativeTime:{future:"\u0641\u064A %s",past:"\u0645\u0646\u0630 %s",s:"\u062B\u0648\u0627\u0646",ss:"%d \u062B\u0627\u0646\u064A\u0629",m:"\u062F\u0642\u064A\u0642\u0629",mm:"%d \u062F\u0642\u0627\u0626\u0642",h:"\u0633\u0627\u0639\u0629",hh:"%d \u0633\u0627\u0639\u0627\u062A",d:"\u064A\u0648\u0645",dd:"%d \u0623\u064A\u0627\u0645",M:"\u0634\u0647\u0631",MM:"%d \u0623\u0634\u0647\u0631",y:"\u0633\u0646\u0629",yy:"%d \u0633\u0646\u0648\u0627\u062A"},preparse:function(n){return n.replace(/[١٢٣٤٥٦٧٨٩٠]/g,function(d){return r[d]}).replace(/،/g,",")},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]}).replace(/,/g,"\u060C")},week:{dow:0,doy:6}});return s})},89756(L,f,c){//! moment.js locale configuration
//! locale  :  Arabic (Tunisia) [ar-tn]
//! author : Nader Toukabri : https://github.com/naderio
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ar-tn",{months:"\u062C\u0627\u0646\u0641\u064A_\u0641\u064A\u0641\u0631\u064A_\u0645\u0627\u0631\u0633_\u0623\u0641\u0631\u064A\u0644_\u0645\u0627\u064A_\u062C\u0648\u0627\u0646_\u062C\u0648\u064A\u0644\u064A\u0629_\u0623\u0648\u062A_\u0633\u0628\u062A\u0645\u0628\u0631_\u0623\u0643\u062A\u0648\u0628\u0631_\u0646\u0648\u0641\u0645\u0628\u0631_\u062F\u064A\u0633\u0645\u0628\u0631".split("_"),monthsShort:"\u062C\u0627\u0646\u0641\u064A_\u0641\u064A\u0641\u0631\u064A_\u0645\u0627\u0631\u0633_\u0623\u0641\u0631\u064A\u0644_\u0645\u0627\u064A_\u062C\u0648\u0627\u0646_\u062C\u0648\u064A\u0644\u064A\u0629_\u0623\u0648\u062A_\u0633\u0628\u062A\u0645\u0628\u0631_\u0623\u0643\u062A\u0648\u0628\u0631_\u0646\u0648\u0641\u0645\u0628\u0631_\u062F\u064A\u0633\u0645\u0628\u0631".split("_"),weekdays:"\u0627\u0644\u0623\u062D\u062F_\u0627\u0644\u0625\u062B\u0646\u064A\u0646_\u0627\u0644\u062B\u0644\u0627\u062B\u0627\u0621_\u0627\u0644\u0623\u0631\u0628\u0639\u0627\u0621_\u0627\u0644\u062E\u0645\u064A\u0633_\u0627\u0644\u062C\u0645\u0639\u0629_\u0627\u0644\u0633\u0628\u062A".split("_"),weekdaysShort:"\u0623\u062D\u062F_\u0625\u062B\u0646\u064A\u0646_\u062B\u0644\u0627\u062B\u0627\u0621_\u0623\u0631\u0628\u0639\u0627\u0621_\u062E\u0645\u064A\u0633_\u062C\u0645\u0639\u0629_\u0633\u0628\u062A".split("_"),weekdaysMin:"\u062D_\u0646_\u062B_\u0631_\u062E_\u062C_\u0633".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[\u0627\u0644\u064A\u0648\u0645 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextDay:"[\u063A\u062F\u0627 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextWeek:"dddd [\u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastDay:"[\u0623\u0645\u0633 \u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastWeek:"dddd [\u0639\u0644\u0649 \u0627\u0644\u0633\u0627\u0639\u0629] LT",sameElse:"L"},relativeTime:{future:"\u0641\u064A %s",past:"\u0645\u0646\u0630 %s",s:"\u062B\u0648\u0627\u0646",ss:"%d \u062B\u0627\u0646\u064A\u0629",m:"\u062F\u0642\u064A\u0642\u0629",mm:"%d \u062F\u0642\u0627\u0626\u0642",h:"\u0633\u0627\u0639\u0629",hh:"%d \u0633\u0627\u0639\u0627\u062A",d:"\u064A\u0648\u0645",dd:"%d \u0623\u064A\u0627\u0645",M:"\u0634\u0647\u0631",MM:"%d \u0623\u0634\u0647\u0631",y:"\u0633\u0646\u0629",yy:"%d \u0633\u0646\u0648\u0627\u062A"},week:{dow:1,doy:4}});return a})},61509(L,f,c){//! moment.js locale configuration
//! locale : Arabic [ar]
//! author : Abdel Said: https://github.com/abdelsaid
//! author : Ahmed Elkhatib
//! author : forabi https://github.com/forabi
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0661",2:"\u0662",3:"\u0663",4:"\u0664",5:"\u0665",6:"\u0666",7:"\u0667",8:"\u0668",9:"\u0669",0:"\u0660"},r={"\u0661":"1","\u0662":"2","\u0663":"3","\u0664":"4","\u0665":"5","\u0666":"6","\u0667":"7","\u0668":"8","\u0669":"9","\u0660":"0"},s=function(h){return h===0?0:h===1?1:h===2?2:h%100>=3&&h%100<=10?3:h%100>=11?4:5},n={s:["\u0623\u0642\u0644 \u0645\u0646 \u062B\u0627\u0646\u064A\u0629","\u062B\u0627\u0646\u064A\u0629 \u0648\u0627\u062D\u062F\u0629",["\u062B\u0627\u0646\u064A\u062A\u0627\u0646","\u062B\u0627\u0646\u064A\u062A\u064A\u0646"],"%d \u062B\u0648\u0627\u0646","%d \u062B\u0627\u0646\u064A\u0629","%d \u062B\u0627\u0646\u064A\u0629"],m:["\u0623\u0642\u0644 \u0645\u0646 \u062F\u0642\u064A\u0642\u0629","\u062F\u0642\u064A\u0642\u0629 \u0648\u0627\u062D\u062F\u0629",["\u062F\u0642\u064A\u0642\u062A\u0627\u0646","\u062F\u0642\u064A\u0642\u062A\u064A\u0646"],"%d \u062F\u0642\u0627\u0626\u0642","%d \u062F\u0642\u064A\u0642\u0629","%d \u062F\u0642\u064A\u0642\u0629"],h:["\u0623\u0642\u0644 \u0645\u0646 \u0633\u0627\u0639\u0629","\u0633\u0627\u0639\u0629 \u0648\u0627\u062D\u062F\u0629",["\u0633\u0627\u0639\u062A\u0627\u0646","\u0633\u0627\u0639\u062A\u064A\u0646"],"%d \u0633\u0627\u0639\u0627\u062A","%d \u0633\u0627\u0639\u0629","%d \u0633\u0627\u0639\u0629"],d:["\u0623\u0642\u0644 \u0645\u0646 \u064A\u0648\u0645","\u064A\u0648\u0645 \u0648\u0627\u062D\u062F",["\u064A\u0648\u0645\u0627\u0646","\u064A\u0648\u0645\u064A\u0646"],"%d \u0623\u064A\u0627\u0645","%d \u064A\u0648\u0645\u064B\u0627","%d \u064A\u0648\u0645"],M:["\u0623\u0642\u0644 \u0645\u0646 \u0634\u0647\u0631","\u0634\u0647\u0631 \u0648\u0627\u062D\u062F",["\u0634\u0647\u0631\u0627\u0646","\u0634\u0647\u0631\u064A\u0646"],"%d \u0623\u0634\u0647\u0631","%d \u0634\u0647\u0631\u0627","%d \u0634\u0647\u0631"],y:["\u0623\u0642\u0644 \u0645\u0646 \u0639\u0627\u0645","\u0639\u0627\u0645 \u0648\u0627\u062D\u062F",["\u0639\u0627\u0645\u0627\u0646","\u0639\u0627\u0645\u064A\u0646"],"%d \u0623\u0639\u0648\u0627\u0645","%d \u0639\u0627\u0645\u064B\u0627","%d \u0639\u0627\u0645"]},d=function(h){return function(y,g,v,H){var Q=s(y),k=n[h][s(y)];return Q===2&&(k=k[g?0:1]),k.replace(/%d/i,y)}},u=["\u064A\u0646\u0627\u064A\u0631","\u0641\u0628\u0631\u0627\u064A\u0631","\u0645\u0627\u0631\u0633","\u0623\u0628\u0631\u064A\u0644","\u0645\u0627\u064A\u0648","\u064A\u0648\u0646\u064A\u0648","\u064A\u0648\u0644\u064A\u0648","\u0623\u063A\u0633\u0637\u0633","\u0633\u0628\u062A\u0645\u0628\u0631","\u0623\u0643\u062A\u0648\u0628\u0631","\u0646\u0648\u0641\u0645\u0628\u0631","\u062F\u064A\u0633\u0645\u0628\u0631"],m=o.defineLocale("ar",{months:u,monthsShort:u,weekdays:"\u0627\u0644\u0623\u062D\u062F_\u0627\u0644\u0625\u062B\u0646\u064A\u0646_\u0627\u0644\u062B\u0644\u0627\u062B\u0627\u0621_\u0627\u0644\u0623\u0631\u0628\u0639\u0627\u0621_\u0627\u0644\u062E\u0645\u064A\u0633_\u0627\u0644\u062C\u0645\u0639\u0629_\u0627\u0644\u0633\u0628\u062A".split("_"),weekdaysShort:"\u0623\u062D\u062F_\u0625\u062B\u0646\u064A\u0646_\u062B\u0644\u0627\u062B\u0627\u0621_\u0623\u0631\u0628\u0639\u0627\u0621_\u062E\u0645\u064A\u0633_\u062C\u0645\u0639\u0629_\u0633\u0628\u062A".split("_"),weekdaysMin:"\u062D_\u0646_\u062B_\u0631_\u062E_\u062C_\u0633".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/\u200FM/\u200FYYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/ص|م/,isPM:function(h){return h==="\u0645"},meridiem:function(h,y,g){return h<12?"\u0635":"\u0645"},calendar:{sameDay:"[\u0627\u0644\u064A\u0648\u0645 \u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextDay:"[\u063A\u062F\u064B\u0627 \u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",nextWeek:"dddd [\u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastDay:"[\u0623\u0645\u0633 \u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",lastWeek:"dddd [\u0639\u0646\u062F \u0627\u0644\u0633\u0627\u0639\u0629] LT",sameElse:"L"},relativeTime:{future:"\u0628\u0639\u062F %s",past:"\u0645\u0646\u0630 %s",s:d("s"),ss:d("s"),m:d("m"),mm:d("m"),h:d("h"),hh:d("h"),d:d("d"),dd:d("d"),M:d("M"),MM:d("M"),y:d("y"),yy:d("y")},preparse:function(h){return h.replace(/[١٢٣٤٥٦٧٨٩٠]/g,function(y){return r[y]}).replace(/،/g,",")},postformat:function(h){return h.replace(/\d/g,function(y){return a[y]}).replace(/,/g,"\u060C")},week:{dow:6,doy:12}});return m})},95533(L,f,c){//! moment.js locale configuration
//! locale : Azerbaijani [az]
//! author : topchiyev : https://github.com/topchiyev
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"-inci",5:"-inci",8:"-inci",70:"-inci",80:"-inci",2:"-nci",7:"-nci",20:"-nci",50:"-nci",3:"-\xFCnc\xFC",4:"-\xFCnc\xFC",100:"-\xFCnc\xFC",6:"-nc\u0131",9:"-uncu",10:"-uncu",30:"-uncu",60:"-\u0131nc\u0131",90:"-\u0131nc\u0131"},r=o.defineLocale("az",{months:"yanvar_fevral_mart_aprel_may_iyun_iyul_avqust_sentyabr_oktyabr_noyabr_dekabr".split("_"),monthsShort:"yan_fev_mar_apr_may_iyn_iyl_avq_sen_okt_noy_dek".split("_"),weekdays:"Bazar_Bazar ert\u0259si_\xC7\u0259r\u015F\u0259nb\u0259 ax\u015Fam\u0131_\xC7\u0259r\u015F\u0259nb\u0259_C\xFCm\u0259 ax\u015Fam\u0131_C\xFCm\u0259_\u015E\u0259nb\u0259".split("_"),weekdaysShort:"Baz_BzE_\xC7Ax_\xC7\u0259r_CAx_C\xFCm_\u015E\u0259n".split("_"),weekdaysMin:"Bz_BE_\xC7A_\xC7\u0259_CA_C\xFC_\u015E\u0259".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bug\xFCn saat] LT",nextDay:"[sabah saat] LT",nextWeek:"[g\u0259l\u0259n h\u0259ft\u0259] dddd [saat] LT",lastDay:"[d\xFCn\u0259n] LT",lastWeek:"[ke\xE7\u0259n h\u0259ft\u0259] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s \u0259vv\u0259l",s:"bir ne\xE7\u0259 saniy\u0259",ss:"%d saniy\u0259",m:"bir d\u0259qiq\u0259",mm:"%d d\u0259qiq\u0259",h:"bir saat",hh:"%d saat",d:"bir g\xFCn",dd:"%d g\xFCn",M:"bir ay",MM:"%d ay",y:"bir il",yy:"%d il"},meridiemParse:/gecə|səhər|gündüz|axşam/,isPM:function(s){return/^(gündüz|axşam)$/.test(s)},meridiem:function(s,n,d){return s<4?"gec\u0259":s<12?"s\u0259h\u0259r":s<17?"g\xFCnd\xFCz":"ax\u015Fam"},dayOfMonthOrdinalParse:/\d{1,2}-(ıncı|inci|nci|üncü|ncı|uncu)/,ordinal:function(s){if(s===0)return s+"-\u0131nc\u0131";var n=s%10,d=s%100-n,u=s>=100?100:null;return s+(a[n]||a[d]||a[u])},week:{dow:1,doy:7}});return r})},28959(L,f,c){//! moment.js locale configuration
//! locale : Belarusian [be]
//! author : Dmitry Demidov : https://github.com/demidov91
//! author: Praleska: http://praleska.pro/
//! Author : Menelion Elensúle : https://github.com/Oire
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(n,d){var u=n.split("_");return d%10===1&&d%100!==11?u[0]:d%10>=2&&d%10<=4&&(d%100<10||d%100>=20)?u[1]:u[2]}function r(n,d,u){var m={ss:d?"\u0441\u0435\u043A\u0443\u043D\u0434\u0430_\u0441\u0435\u043A\u0443\u043D\u0434\u044B_\u0441\u0435\u043A\u0443\u043D\u0434":"\u0441\u0435\u043A\u0443\u043D\u0434\u0443_\u0441\u0435\u043A\u0443\u043D\u0434\u044B_\u0441\u0435\u043A\u0443\u043D\u0434",mm:d?"\u0445\u0432\u0456\u043B\u0456\u043D\u0430_\u0445\u0432\u0456\u043B\u0456\u043D\u044B_\u0445\u0432\u0456\u043B\u0456\u043D":"\u0445\u0432\u0456\u043B\u0456\u043D\u0443_\u0445\u0432\u0456\u043B\u0456\u043D\u044B_\u0445\u0432\u0456\u043B\u0456\u043D",hh:d?"\u0433\u0430\u0434\u0437\u0456\u043D\u0430_\u0433\u0430\u0434\u0437\u0456\u043D\u044B_\u0433\u0430\u0434\u0437\u0456\u043D":"\u0433\u0430\u0434\u0437\u0456\u043D\u0443_\u0433\u0430\u0434\u0437\u0456\u043D\u044B_\u0433\u0430\u0434\u0437\u0456\u043D",dd:"\u0434\u0437\u0435\u043D\u044C_\u0434\u043D\u0456_\u0434\u0437\u0451\u043D",MM:"\u043C\u0435\u0441\u044F\u0446_\u043C\u0435\u0441\u044F\u0446\u044B_\u043C\u0435\u0441\u044F\u0446\u0430\u045E",yy:"\u0433\u043E\u0434_\u0433\u0430\u0434\u044B_\u0433\u0430\u0434\u043E\u045E"};return u==="m"?d?"\u0445\u0432\u0456\u043B\u0456\u043D\u0430":"\u0445\u0432\u0456\u043B\u0456\u043D\u0443":u==="h"?d?"\u0433\u0430\u0434\u0437\u0456\u043D\u0430":"\u0433\u0430\u0434\u0437\u0456\u043D\u0443":n+" "+a(m[u],+n)}var s=o.defineLocale("be",{months:{format:"\u0441\u0442\u0443\u0434\u0437\u0435\u043D\u044F_\u043B\u044E\u0442\u0430\u0433\u0430_\u0441\u0430\u043A\u0430\u0432\u0456\u043A\u0430_\u043A\u0440\u0430\u0441\u0430\u0432\u0456\u043A\u0430_\u0442\u0440\u0430\u045E\u043D\u044F_\u0447\u044D\u0440\u0432\u0435\u043D\u044F_\u043B\u0456\u043F\u0435\u043D\u044F_\u0436\u043D\u0456\u045E\u043D\u044F_\u0432\u0435\u0440\u0430\u0441\u043D\u044F_\u043A\u0430\u0441\u0442\u0440\u044B\u0447\u043D\u0456\u043A\u0430_\u043B\u0456\u0441\u0442\u0430\u043F\u0430\u0434\u0430_\u0441\u043D\u0435\u0436\u043D\u044F".split("_"),standalone:"\u0441\u0442\u0443\u0434\u0437\u0435\u043D\u044C_\u043B\u044E\u0442\u044B_\u0441\u0430\u043A\u0430\u0432\u0456\u043A_\u043A\u0440\u0430\u0441\u0430\u0432\u0456\u043A_\u0442\u0440\u0430\u0432\u0435\u043D\u044C_\u0447\u044D\u0440\u0432\u0435\u043D\u044C_\u043B\u0456\u043F\u0435\u043D\u044C_\u0436\u043D\u0456\u0432\u0435\u043D\u044C_\u0432\u0435\u0440\u0430\u0441\u0435\u043D\u044C_\u043A\u0430\u0441\u0442\u0440\u044B\u0447\u043D\u0456\u043A_\u043B\u0456\u0441\u0442\u0430\u043F\u0430\u0434_\u0441\u043D\u0435\u0436\u0430\u043D\u044C".split("_")},monthsShort:"\u0441\u0442\u0443\u0434_\u043B\u044E\u0442_\u0441\u0430\u043A_\u043A\u0440\u0430\u0441_\u0442\u0440\u0430\u0432_\u0447\u044D\u0440\u0432_\u043B\u0456\u043F_\u0436\u043D\u0456\u0432_\u0432\u0435\u0440_\u043A\u0430\u0441\u0442_\u043B\u0456\u0441\u0442_\u0441\u043D\u0435\u0436".split("_"),weekdays:{format:"\u043D\u044F\u0434\u0437\u0435\u043B\u044E_\u043F\u0430\u043D\u044F\u0434\u0437\u0435\u043B\u0430\u043A_\u0430\u045E\u0442\u043E\u0440\u0430\u043A_\u0441\u0435\u0440\u0430\u0434\u0443_\u0447\u0430\u0446\u0432\u0435\u0440_\u043F\u044F\u0442\u043D\u0456\u0446\u0443_\u0441\u0443\u0431\u043E\u0442\u0443".split("_"),standalone:"\u043D\u044F\u0434\u0437\u0435\u043B\u044F_\u043F\u0430\u043D\u044F\u0434\u0437\u0435\u043B\u0430\u043A_\u0430\u045E\u0442\u043E\u0440\u0430\u043A_\u0441\u0435\u0440\u0430\u0434\u0430_\u0447\u0430\u0446\u0432\u0435\u0440_\u043F\u044F\u0442\u043D\u0456\u0446\u0430_\u0441\u0443\u0431\u043E\u0442\u0430".split("_"),isFormat:/\[ ?[Ууў] ?(?:мінулую|наступную)? ?\] ?dddd/},weekdaysShort:"\u043D\u0434_\u043F\u043D_\u0430\u0442_\u0441\u0440_\u0447\u0446_\u043F\u0442_\u0441\u0431".split("_"),weekdaysMin:"\u043D\u0434_\u043F\u043D_\u0430\u0442_\u0441\u0440_\u0447\u0446_\u043F\u0442_\u0441\u0431".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY \u0433.",LLL:"D MMMM YYYY \u0433., HH:mm",LLLL:"dddd, D MMMM YYYY \u0433., HH:mm"},calendar:{sameDay:"[\u0421\u0451\u043D\u043D\u044F \u045E] LT",nextDay:"[\u0417\u0430\u045E\u0442\u0440\u0430 \u045E] LT",lastDay:"[\u0423\u0447\u043E\u0440\u0430 \u045E] LT",nextWeek:function(){return"[\u0423] dddd [\u045E] LT"},lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return"[\u0423 \u043C\u0456\u043D\u0443\u043B\u0443\u044E] dddd [\u045E] LT";case 1:case 2:case 4:return"[\u0423 \u043C\u0456\u043D\u0443\u043B\u044B] dddd [\u045E] LT"}},sameElse:"L"},relativeTime:{future:"\u043F\u0440\u0430\u0437 %s",past:"%s \u0442\u0430\u043C\u0443",s:"\u043D\u0435\u043A\u0430\u043B\u044C\u043A\u0456 \u0441\u0435\u043A\u0443\u043D\u0434",m:r,mm:r,h:r,hh:r,d:"\u0434\u0437\u0435\u043D\u044C",dd:r,M:"\u043C\u0435\u0441\u044F\u0446",MM:r,y:"\u0433\u043E\u0434",yy:r},meridiemParse:/ночы|раніцы|дня|вечара/,isPM:function(n){return/^(дня|вечара)$/.test(n)},meridiem:function(n,d,u){return n<4?"\u043D\u043E\u0447\u044B":n<12?"\u0440\u0430\u043D\u0456\u0446\u044B":n<17?"\u0434\u043D\u044F":"\u0432\u0435\u0447\u0430\u0440\u0430"},dayOfMonthOrdinalParse:/\d{1,2}-(і|ы|га)/,ordinal:function(n,d){switch(d){case"M":case"d":case"DDD":case"w":case"W":return(n%10===2||n%10===3)&&n%100!==12&&n%100!==13?n+"-\u0456":n+"-\u044B";case"D":return n+"-\u0433\u0430";default:return n}},week:{dow:1,doy:7}});return s})},47777(L,f,c){//! moment.js locale configuration
//! locale : Bulgarian [bg]
//! author : Krasen Borisov : https://github.com/kraz
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("bg",{months:"\u044F\u043D\u0443\u0430\u0440\u0438_\u0444\u0435\u0432\u0440\u0443\u0430\u0440\u0438_\u043C\u0430\u0440\u0442_\u0430\u043F\u0440\u0438\u043B_\u043C\u0430\u0439_\u044E\u043D\u0438_\u044E\u043B\u0438_\u0430\u0432\u0433\u0443\u0441\u0442_\u0441\u0435\u043F\u0442\u0435\u043C\u0432\u0440\u0438_\u043E\u043A\u0442\u043E\u043C\u0432\u0440\u0438_\u043D\u043E\u0435\u043C\u0432\u0440\u0438_\u0434\u0435\u043A\u0435\u043C\u0432\u0440\u0438".split("_"),monthsShort:"\u044F\u043D\u0443_\u0444\u0435\u0432_\u043C\u0430\u0440_\u0430\u043F\u0440_\u043C\u0430\u0439_\u044E\u043D\u0438_\u044E\u043B\u0438_\u0430\u0432\u0433_\u0441\u0435\u043F_\u043E\u043A\u0442_\u043D\u043E\u0435_\u0434\u0435\u043A".split("_"),weekdays:"\u043D\u0435\u0434\u0435\u043B\u044F_\u043F\u043E\u043D\u0435\u0434\u0435\u043B\u043D\u0438\u043A_\u0432\u0442\u043E\u0440\u043D\u0438\u043A_\u0441\u0440\u044F\u0434\u0430_\u0447\u0435\u0442\u0432\u044A\u0440\u0442\u044A\u043A_\u043F\u0435\u0442\u044A\u043A_\u0441\u044A\u0431\u043E\u0442\u0430".split("_"),weekdaysShort:"\u043D\u0435\u0434_\u043F\u043E\u043D_\u0432\u0442\u043E_\u0441\u0440\u044F_\u0447\u0435\u0442_\u043F\u0435\u0442_\u0441\u044A\u0431".split("_"),weekdaysMin:"\u043D\u0434_\u043F\u043D_\u0432\u0442_\u0441\u0440_\u0447\u0442_\u043F\u0442_\u0441\u0431".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[\u0414\u043D\u0435\u0441 \u0432] LT",nextDay:"[\u0423\u0442\u0440\u0435 \u0432] LT",nextWeek:"dddd [\u0432] LT",lastDay:"[\u0412\u0447\u0435\u0440\u0430 \u0432] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[\u041C\u0438\u043D\u0430\u043B\u0430\u0442\u0430] dddd [\u0432] LT";case 1:case 2:case 4:case 5:return"[\u041C\u0438\u043D\u0430\u043B\u0438\u044F] dddd [\u0432] LT"}},sameElse:"L"},relativeTime:{future:"\u0441\u043B\u0435\u0434 %s",past:"\u043F\u0440\u0435\u0434\u0438 %s",s:"\u043D\u044F\u043A\u043E\u043B\u043A\u043E \u0441\u0435\u043A\u0443\u043D\u0434\u0438",ss:"%d \u0441\u0435\u043A\u0443\u043D\u0434\u0438",m:"\u043C\u0438\u043D\u0443\u0442\u0430",mm:"%d \u043C\u0438\u043D\u0443\u0442\u0438",h:"\u0447\u0430\u0441",hh:"%d \u0447\u0430\u0441\u0430",d:"\u0434\u0435\u043D",dd:"%d \u0434\u0435\u043D\u0430",w:"\u0441\u0435\u0434\u043C\u0438\u0446\u0430",ww:"%d \u0441\u0435\u0434\u043C\u0438\u0446\u0438",M:"\u043C\u0435\u0441\u0435\u0446",MM:"%d \u043C\u0435\u0441\u0435\u0446\u0430",y:"\u0433\u043E\u0434\u0438\u043D\u0430",yy:"%d \u0433\u043E\u0434\u0438\u043D\u0438"},dayOfMonthOrdinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(r){var s=r%10,n=r%100;return r===0?r+"-\u0435\u0432":n===0?r+"-\u0435\u043D":n>10&&n<20?r+"-\u0442\u0438":s===1?r+"-\u0432\u0438":s===2?r+"-\u0440\u0438":s===7||s===8?r+"-\u043C\u0438":r+"-\u0442\u0438"},week:{dow:1,doy:7}});return a})},54903(L,f,c){//! moment.js locale configuration
//! locale : Bambara [bm]
//! author : Estelle Comment : https://github.com/estellecomment
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("bm",{months:"Zanwuyekalo_Fewuruyekalo_Marisikalo_Awirilikalo_M\u025Bkalo_Zuw\u025Bnkalo_Zuluyekalo_Utikalo_S\u025Btanburukalo_\u0254kut\u0254burukalo_Nowanburukalo_Desanburukalo".split("_"),monthsShort:"Zan_Few_Mar_Awi_M\u025B_Zuw_Zul_Uti_S\u025Bt_\u0254ku_Now_Des".split("_"),weekdays:"Kari_Nt\u025Bn\u025Bn_Tarata_Araba_Alamisa_Juma_Sibiri".split("_"),weekdaysShort:"Kar_Nt\u025B_Tar_Ara_Ala_Jum_Sib".split("_"),weekdaysMin:"Ka_Nt_Ta_Ar_Al_Ju_Si".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"MMMM [tile] D [san] YYYY",LLL:"MMMM [tile] D [san] YYYY [l\u025Br\u025B] HH:mm",LLLL:"dddd MMMM [tile] D [san] YYYY [l\u025Br\u025B] HH:mm"},calendar:{sameDay:"[Bi l\u025Br\u025B] LT",nextDay:"[Sini l\u025Br\u025B] LT",nextWeek:"dddd [don l\u025Br\u025B] LT",lastDay:"[Kunu l\u025Br\u025B] LT",lastWeek:"dddd [t\u025Bm\u025Bnen l\u025Br\u025B] LT",sameElse:"L"},relativeTime:{future:"%s k\u0254n\u0254",past:"a b\u025B %s b\u0254",s:"sanga dama dama",ss:"sekondi %d",m:"miniti kelen",mm:"miniti %d",h:"l\u025Br\u025B kelen",hh:"l\u025Br\u025B %d",d:"tile kelen",dd:"tile %d",M:"kalo kelen",MM:"kalo %d",y:"san kelen",yy:"san %d"},week:{dow:1,doy:4}});return a})},17357(L,f,c){//! moment.js locale configuration
//! locale : Bengali (Bangladesh) [bn-bd]
//! author : Asraf Hossain Patoary : https://github.com/ashwoolford
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u09E7",2:"\u09E8",3:"\u09E9",4:"\u09EA",5:"\u09EB",6:"\u09EC",7:"\u09ED",8:"\u09EE",9:"\u09EF",0:"\u09E6"},r={"\u09E7":"1","\u09E8":"2","\u09E9":"3","\u09EA":"4","\u09EB":"5","\u09EC":"6","\u09ED":"7","\u09EE":"8","\u09EF":"9","\u09E6":"0"},s=o.defineLocale("bn-bd",{months:"\u099C\u09BE\u09A8\u09C1\u09DF\u09BE\u09B0\u09BF_\u09AB\u09C7\u09AC\u09CD\u09B0\u09C1\u09DF\u09BE\u09B0\u09BF_\u09AE\u09BE\u09B0\u09CD\u099A_\u098F\u09AA\u09CD\u09B0\u09BF\u09B2_\u09AE\u09C7_\u099C\u09C1\u09A8_\u099C\u09C1\u09B2\u09BE\u0987_\u0986\u0997\u09B8\u09CD\u099F_\u09B8\u09C7\u09AA\u09CD\u099F\u09C7\u09AE\u09CD\u09AC\u09B0_\u0985\u0995\u09CD\u099F\u09CB\u09AC\u09B0_\u09A8\u09AD\u09C7\u09AE\u09CD\u09AC\u09B0_\u09A1\u09BF\u09B8\u09C7\u09AE\u09CD\u09AC\u09B0".split("_"),monthsShort:"\u099C\u09BE\u09A8\u09C1_\u09AB\u09C7\u09AC\u09CD\u09B0\u09C1_\u09AE\u09BE\u09B0\u09CD\u099A_\u098F\u09AA\u09CD\u09B0\u09BF\u09B2_\u09AE\u09C7_\u099C\u09C1\u09A8_\u099C\u09C1\u09B2\u09BE\u0987_\u0986\u0997\u09B8\u09CD\u099F_\u09B8\u09C7\u09AA\u09CD\u099F_\u0985\u0995\u09CD\u099F\u09CB_\u09A8\u09AD\u09C7_\u09A1\u09BF\u09B8\u09C7".split("_"),weekdays:"\u09B0\u09AC\u09BF\u09AC\u09BE\u09B0_\u09B8\u09CB\u09AE\u09AC\u09BE\u09B0_\u09AE\u0999\u09CD\u0997\u09B2\u09AC\u09BE\u09B0_\u09AC\u09C1\u09A7\u09AC\u09BE\u09B0_\u09AC\u09C3\u09B9\u09B8\u09CD\u09AA\u09A4\u09BF\u09AC\u09BE\u09B0_\u09B6\u09C1\u0995\u09CD\u09B0\u09AC\u09BE\u09B0_\u09B6\u09A8\u09BF\u09AC\u09BE\u09B0".split("_"),weekdaysShort:"\u09B0\u09AC\u09BF_\u09B8\u09CB\u09AE_\u09AE\u0999\u09CD\u0997\u09B2_\u09AC\u09C1\u09A7_\u09AC\u09C3\u09B9\u09B8\u09CD\u09AA\u09A4\u09BF_\u09B6\u09C1\u0995\u09CD\u09B0_\u09B6\u09A8\u09BF".split("_"),weekdaysMin:"\u09B0\u09AC\u09BF_\u09B8\u09CB\u09AE_\u09AE\u0999\u09CD\u0997\u09B2_\u09AC\u09C1\u09A7_\u09AC\u09C3\u09B9_\u09B6\u09C1\u0995\u09CD\u09B0_\u09B6\u09A8\u09BF".split("_"),longDateFormat:{LT:"A h:mm \u09B8\u09AE\u09DF",LTS:"A h:mm:ss \u09B8\u09AE\u09DF",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm \u09B8\u09AE\u09DF",LLLL:"dddd, D MMMM YYYY, A h:mm \u09B8\u09AE\u09DF"},calendar:{sameDay:"[\u0986\u099C] LT",nextDay:"[\u0986\u0997\u09BE\u09AE\u09C0\u0995\u09BE\u09B2] LT",nextWeek:"dddd, LT",lastDay:"[\u0997\u09A4\u0995\u09BE\u09B2] LT",lastWeek:"[\u0997\u09A4] dddd, LT",sameElse:"L"},relativeTime:{future:"%s \u09AA\u09B0\u09C7",past:"%s \u0986\u0997\u09C7",s:"\u0995\u09DF\u09C7\u0995 \u09B8\u09C7\u0995\u09C7\u09A8\u09CD\u09A1",ss:"%d \u09B8\u09C7\u0995\u09C7\u09A8\u09CD\u09A1",m:"\u098F\u0995 \u09AE\u09BF\u09A8\u09BF\u099F",mm:"%d \u09AE\u09BF\u09A8\u09BF\u099F",h:"\u098F\u0995 \u0998\u09A8\u09CD\u099F\u09BE",hh:"%d \u0998\u09A8\u09CD\u099F\u09BE",d:"\u098F\u0995 \u09A6\u09BF\u09A8",dd:"%d \u09A6\u09BF\u09A8",M:"\u098F\u0995 \u09AE\u09BE\u09B8",MM:"%d \u09AE\u09BE\u09B8",y:"\u098F\u0995 \u09AC\u099B\u09B0",yy:"%d \u09AC\u099B\u09B0"},preparse:function(n){return n.replace(/[১২৩৪৫৬৭৮৯০]/g,function(d){return r[d]})},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]})},meridiemParse:/রাত|ভোর|সকাল|দুপুর|বিকাল|সন্ধ্যা|রাত/,meridiemHour:function(n,d){if(n===12&&(n=0),d==="\u09B0\u09BE\u09A4")return n<4?n:n+12;if(d==="\u09AD\u09CB\u09B0")return n;if(d==="\u09B8\u0995\u09BE\u09B2")return n;if(d==="\u09A6\u09C1\u09AA\u09C1\u09B0")return n>=3?n:n+12;if(d==="\u09AC\u09BF\u0995\u09BE\u09B2")return n+12;if(d==="\u09B8\u09A8\u09CD\u09A7\u09CD\u09AF\u09BE")return n+12},meridiem:function(n,d,u){return n<4?"\u09B0\u09BE\u09A4":n<6?"\u09AD\u09CB\u09B0":n<12?"\u09B8\u0995\u09BE\u09B2":n<15?"\u09A6\u09C1\u09AA\u09C1\u09B0":n<18?"\u09AC\u09BF\u0995\u09BE\u09B2":n<20?"\u09B8\u09A8\u09CD\u09A7\u09CD\u09AF\u09BE":"\u09B0\u09BE\u09A4"},week:{dow:0,doy:6}});return s})},61290(L,f,c){//! moment.js locale configuration
//! locale : Bengali [bn]
//! author : Kaushik Gandhi : https://github.com/kaushikgandhi
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u09E7",2:"\u09E8",3:"\u09E9",4:"\u09EA",5:"\u09EB",6:"\u09EC",7:"\u09ED",8:"\u09EE",9:"\u09EF",0:"\u09E6"},r={"\u09E7":"1","\u09E8":"2","\u09E9":"3","\u09EA":"4","\u09EB":"5","\u09EC":"6","\u09ED":"7","\u09EE":"8","\u09EF":"9","\u09E6":"0"},s=o.defineLocale("bn",{months:"\u099C\u09BE\u09A8\u09C1\u09DF\u09BE\u09B0\u09BF_\u09AB\u09C7\u09AC\u09CD\u09B0\u09C1\u09DF\u09BE\u09B0\u09BF_\u09AE\u09BE\u09B0\u09CD\u099A_\u098F\u09AA\u09CD\u09B0\u09BF\u09B2_\u09AE\u09C7_\u099C\u09C1\u09A8_\u099C\u09C1\u09B2\u09BE\u0987_\u0986\u0997\u09B8\u09CD\u099F_\u09B8\u09C7\u09AA\u09CD\u099F\u09C7\u09AE\u09CD\u09AC\u09B0_\u0985\u0995\u09CD\u099F\u09CB\u09AC\u09B0_\u09A8\u09AD\u09C7\u09AE\u09CD\u09AC\u09B0_\u09A1\u09BF\u09B8\u09C7\u09AE\u09CD\u09AC\u09B0".split("_"),monthsShort:"\u099C\u09BE\u09A8\u09C1_\u09AB\u09C7\u09AC\u09CD\u09B0\u09C1_\u09AE\u09BE\u09B0\u09CD\u099A_\u098F\u09AA\u09CD\u09B0\u09BF\u09B2_\u09AE\u09C7_\u099C\u09C1\u09A8_\u099C\u09C1\u09B2\u09BE\u0987_\u0986\u0997\u09B8\u09CD\u099F_\u09B8\u09C7\u09AA\u09CD\u099F_\u0985\u0995\u09CD\u099F\u09CB_\u09A8\u09AD\u09C7_\u09A1\u09BF\u09B8\u09C7".split("_"),weekdays:"\u09B0\u09AC\u09BF\u09AC\u09BE\u09B0_\u09B8\u09CB\u09AE\u09AC\u09BE\u09B0_\u09AE\u0999\u09CD\u0997\u09B2\u09AC\u09BE\u09B0_\u09AC\u09C1\u09A7\u09AC\u09BE\u09B0_\u09AC\u09C3\u09B9\u09B8\u09CD\u09AA\u09A4\u09BF\u09AC\u09BE\u09B0_\u09B6\u09C1\u0995\u09CD\u09B0\u09AC\u09BE\u09B0_\u09B6\u09A8\u09BF\u09AC\u09BE\u09B0".split("_"),weekdaysShort:"\u09B0\u09AC\u09BF_\u09B8\u09CB\u09AE_\u09AE\u0999\u09CD\u0997\u09B2_\u09AC\u09C1\u09A7_\u09AC\u09C3\u09B9\u09B8\u09CD\u09AA\u09A4\u09BF_\u09B6\u09C1\u0995\u09CD\u09B0_\u09B6\u09A8\u09BF".split("_"),weekdaysMin:"\u09B0\u09AC\u09BF_\u09B8\u09CB\u09AE_\u09AE\u0999\u09CD\u0997\u09B2_\u09AC\u09C1\u09A7_\u09AC\u09C3\u09B9_\u09B6\u09C1\u0995\u09CD\u09B0_\u09B6\u09A8\u09BF".split("_"),longDateFormat:{LT:"A h:mm \u09B8\u09AE\u09DF",LTS:"A h:mm:ss \u09B8\u09AE\u09DF",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm \u09B8\u09AE\u09DF",LLLL:"dddd, D MMMM YYYY, A h:mm \u09B8\u09AE\u09DF"},calendar:{sameDay:"[\u0986\u099C] LT",nextDay:"[\u0986\u0997\u09BE\u09AE\u09C0\u0995\u09BE\u09B2] LT",nextWeek:"dddd, LT",lastDay:"[\u0997\u09A4\u0995\u09BE\u09B2] LT",lastWeek:"[\u0997\u09A4] dddd, LT",sameElse:"L"},relativeTime:{future:"%s \u09AA\u09B0\u09C7",past:"%s \u0986\u0997\u09C7",s:"\u0995\u09DF\u09C7\u0995 \u09B8\u09C7\u0995\u09C7\u09A8\u09CD\u09A1",ss:"%d \u09B8\u09C7\u0995\u09C7\u09A8\u09CD\u09A1",m:"\u098F\u0995 \u09AE\u09BF\u09A8\u09BF\u099F",mm:"%d \u09AE\u09BF\u09A8\u09BF\u099F",h:"\u098F\u0995 \u0998\u09A8\u09CD\u099F\u09BE",hh:"%d \u0998\u09A8\u09CD\u099F\u09BE",d:"\u098F\u0995 \u09A6\u09BF\u09A8",dd:"%d \u09A6\u09BF\u09A8",M:"\u098F\u0995 \u09AE\u09BE\u09B8",MM:"%d \u09AE\u09BE\u09B8",y:"\u098F\u0995 \u09AC\u099B\u09B0",yy:"%d \u09AC\u099B\u09B0"},preparse:function(n){return n.replace(/[১২৩৪৫৬৭৮৯০]/g,function(d){return r[d]})},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]})},meridiemParse:/রাত|সকাল|দুপুর|বিকাল|রাত/,meridiemHour:function(n,d){return n===12&&(n=0),d==="\u09B0\u09BE\u09A4"&&n>=4||d==="\u09A6\u09C1\u09AA\u09C1\u09B0"&&n<5||d==="\u09AC\u09BF\u0995\u09BE\u09B2"?n+12:n},meridiem:function(n,d,u){return n<4?"\u09B0\u09BE\u09A4":n<10?"\u09B8\u0995\u09BE\u09B2":n<17?"\u09A6\u09C1\u09AA\u09C1\u09B0":n<20?"\u09AC\u09BF\u0995\u09BE\u09B2":"\u09B0\u09BE\u09A4"},week:{dow:0,doy:6}});return s})},31545(L,f,c){//! moment.js locale configuration
//! locale : Tibetan [bo]
//! author : Thupten N. Chakrishar : https://github.com/vajradog
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0F21",2:"\u0F22",3:"\u0F23",4:"\u0F24",5:"\u0F25",6:"\u0F26",7:"\u0F27",8:"\u0F28",9:"\u0F29",0:"\u0F20"},r={"\u0F21":"1","\u0F22":"2","\u0F23":"3","\u0F24":"4","\u0F25":"5","\u0F26":"6","\u0F27":"7","\u0F28":"8","\u0F29":"9","\u0F20":"0"},s=o.defineLocale("bo",{months:"\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F51\u0F44\u0F0B\u0F54\u0F7C_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F42\u0F49\u0F72\u0F66\u0F0B\u0F54_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F42\u0F66\u0F74\u0F58\u0F0B\u0F54_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F56\u0F5E\u0F72\u0F0B\u0F54_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F63\u0F94\u0F0B\u0F54_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F51\u0FB2\u0F74\u0F42\u0F0B\u0F54_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F56\u0F51\u0F74\u0F53\u0F0B\u0F54_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F56\u0F62\u0F92\u0FB1\u0F51\u0F0B\u0F54_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F51\u0F42\u0F74\u0F0B\u0F54_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F56\u0F45\u0F74\u0F0B\u0F54_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F56\u0F45\u0F74\u0F0B\u0F42\u0F45\u0F72\u0F42\u0F0B\u0F54_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F56\u0F45\u0F74\u0F0B\u0F42\u0F49\u0F72\u0F66\u0F0B\u0F54".split("_"),monthsShort:"\u0F5F\u0FB3\u0F0B1_\u0F5F\u0FB3\u0F0B2_\u0F5F\u0FB3\u0F0B3_\u0F5F\u0FB3\u0F0B4_\u0F5F\u0FB3\u0F0B5_\u0F5F\u0FB3\u0F0B6_\u0F5F\u0FB3\u0F0B7_\u0F5F\u0FB3\u0F0B8_\u0F5F\u0FB3\u0F0B9_\u0F5F\u0FB3\u0F0B10_\u0F5F\u0FB3\u0F0B11_\u0F5F\u0FB3\u0F0B12".split("_"),monthsShortRegex:/^(ཟླ་\d{1,2})/,monthsParseExact:!0,weekdays:"\u0F42\u0F5F\u0F60\u0F0B\u0F49\u0F72\u0F0B\u0F58\u0F0B_\u0F42\u0F5F\u0F60\u0F0B\u0F5F\u0FB3\u0F0B\u0F56\u0F0B_\u0F42\u0F5F\u0F60\u0F0B\u0F58\u0F72\u0F42\u0F0B\u0F51\u0F58\u0F62\u0F0B_\u0F42\u0F5F\u0F60\u0F0B\u0F63\u0FB7\u0F42\u0F0B\u0F54\u0F0B_\u0F42\u0F5F\u0F60\u0F0B\u0F55\u0F74\u0F62\u0F0B\u0F56\u0F74_\u0F42\u0F5F\u0F60\u0F0B\u0F54\u0F0B\u0F66\u0F44\u0F66\u0F0B_\u0F42\u0F5F\u0F60\u0F0B\u0F66\u0FA4\u0F7A\u0F53\u0F0B\u0F54\u0F0B".split("_"),weekdaysShort:"\u0F49\u0F72\u0F0B\u0F58\u0F0B_\u0F5F\u0FB3\u0F0B\u0F56\u0F0B_\u0F58\u0F72\u0F42\u0F0B\u0F51\u0F58\u0F62\u0F0B_\u0F63\u0FB7\u0F42\u0F0B\u0F54\u0F0B_\u0F55\u0F74\u0F62\u0F0B\u0F56\u0F74_\u0F54\u0F0B\u0F66\u0F44\u0F66\u0F0B_\u0F66\u0FA4\u0F7A\u0F53\u0F0B\u0F54\u0F0B".split("_"),weekdaysMin:"\u0F49\u0F72_\u0F5F\u0FB3_\u0F58\u0F72\u0F42_\u0F63\u0FB7\u0F42_\u0F55\u0F74\u0F62_\u0F66\u0F44\u0F66_\u0F66\u0FA4\u0F7A\u0F53".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[\u0F51\u0F72\u0F0B\u0F62\u0F72\u0F44] LT",nextDay:"[\u0F66\u0F44\u0F0B\u0F49\u0F72\u0F53] LT",nextWeek:"[\u0F56\u0F51\u0F74\u0F53\u0F0B\u0F55\u0FB2\u0F42\u0F0B\u0F62\u0F97\u0F7A\u0F66\u0F0B\u0F58], LT",lastDay:"[\u0F41\u0F0B\u0F66\u0F44] LT",lastWeek:"[\u0F56\u0F51\u0F74\u0F53\u0F0B\u0F55\u0FB2\u0F42\u0F0B\u0F58\u0F50\u0F60\u0F0B\u0F58] dddd, LT",sameElse:"L"},relativeTime:{future:"%s \u0F63\u0F0B",past:"%s \u0F66\u0F94\u0F53\u0F0B\u0F63",s:"\u0F63\u0F58\u0F0B\u0F66\u0F44",ss:"%d \u0F66\u0F90\u0F62\u0F0B\u0F46\u0F0D",m:"\u0F66\u0F90\u0F62\u0F0B\u0F58\u0F0B\u0F42\u0F45\u0F72\u0F42",mm:"%d \u0F66\u0F90\u0F62\u0F0B\u0F58",h:"\u0F46\u0F74\u0F0B\u0F5A\u0F7C\u0F51\u0F0B\u0F42\u0F45\u0F72\u0F42",hh:"%d \u0F46\u0F74\u0F0B\u0F5A\u0F7C\u0F51",d:"\u0F49\u0F72\u0F53\u0F0B\u0F42\u0F45\u0F72\u0F42",dd:"%d \u0F49\u0F72\u0F53\u0F0B",M:"\u0F5F\u0FB3\u0F0B\u0F56\u0F0B\u0F42\u0F45\u0F72\u0F42",MM:"%d \u0F5F\u0FB3\u0F0B\u0F56",y:"\u0F63\u0F7C\u0F0B\u0F42\u0F45\u0F72\u0F42",yy:"%d \u0F63\u0F7C"},preparse:function(n){return n.replace(/[༡༢༣༤༥༦༧༨༩༠]/g,function(d){return r[d]})},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]})},meridiemParse:/མཚན་མོ|ཞོགས་ཀས|ཉིན་གུང|དགོང་དག|མཚན་མོ/,meridiemHour:function(n,d){return n===12&&(n=0),d==="\u0F58\u0F5A\u0F53\u0F0B\u0F58\u0F7C"&&n>=4||d==="\u0F49\u0F72\u0F53\u0F0B\u0F42\u0F74\u0F44"&&n<5||d==="\u0F51\u0F42\u0F7C\u0F44\u0F0B\u0F51\u0F42"?n+12:n},meridiem:function(n,d,u){return n<4?"\u0F58\u0F5A\u0F53\u0F0B\u0F58\u0F7C":n<10?"\u0F5E\u0F7C\u0F42\u0F66\u0F0B\u0F40\u0F66":n<17?"\u0F49\u0F72\u0F53\u0F0B\u0F42\u0F74\u0F44":n<20?"\u0F51\u0F42\u0F7C\u0F44\u0F0B\u0F51\u0F42":"\u0F58\u0F5A\u0F53\u0F0B\u0F58\u0F7C"},week:{dow:0,doy:6}});return s})},11470(L,f,c){//! moment.js locale configuration
//! locale : Breton [br]
//! author : Jean-Baptiste Le Duigou : https://github.com/jbleduigou
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(k,I,Me){var he={mm:"munutenn",MM:"miz",dd:"devezh"};return k+" "+n(he[Me],k)}function r(k){switch(s(k)){case 1:case 3:case 4:case 5:case 9:return k+" bloaz";default:return k+" vloaz"}}function s(k){return k>9?s(k%10):k}function n(k,I){return I===2?d(k):k}function d(k){var I={m:"v",b:"v",d:"z"};return I[k.charAt(0)]===void 0?k:I[k.charAt(0)]+k.substring(1)}var u=[/^gen/i,/^c[ʼ\']hwe/i,/^meu/i,/^ebr/i,/^mae/i,/^(mez|eve)/i,/^gou/i,/^eos/i,/^gwe/i,/^her/i,/^du/i,/^ker/i],m=/^(genver|c[ʼ\']hwevrer|meurzh|ebrel|mae|mezheven|gouere|eost|gwengolo|here|du|kerzu|gen|c[ʼ\']hwe|meu|ebr|mae|eve|gou|eos|gwe|her|du|ker)/i,h=/^(genver|c[ʼ\']hwevrer|meurzh|ebrel|mae|mezheven|gouere|eost|gwengolo|here|du|kerzu)/i,y=/^(gen|c[ʼ\']hwe|meu|ebr|mae|eve|gou|eos|gwe|her|du|ker)/i,g=[/^sul/i,/^lun/i,/^meurzh/i,/^merc[ʼ\']her/i,/^yaou/i,/^gwener/i,/^sadorn/i],v=[/^Sul/i,/^Lun/i,/^Meu/i,/^Mer/i,/^Yao/i,/^Gwe/i,/^Sad/i],H=[/^Su/i,/^Lu/i,/^Me([^r]|$)/i,/^Mer/i,/^Ya/i,/^Gw/i,/^Sa/i],Q=o.defineLocale("br",{months:"Genver_C\u02BChwevrer_Meurzh_Ebrel_Mae_Mezheven_Gouere_Eost_Gwengolo_Here_Du_Kerzu".split("_"),monthsShort:"Gen_C\u02BChwe_Meu_Ebr_Mae_Eve_Gou_Eos_Gwe_Her_Du_Ker".split("_"),weekdays:"Sul_Lun_Meurzh_Merc\u02BCher_Yaou_Gwener_Sadorn".split("_"),weekdaysShort:"Sul_Lun_Meu_Mer_Yao_Gwe_Sad".split("_"),weekdaysMin:"Su_Lu_Me_Mer_Ya_Gw_Sa".split("_"),weekdaysParse:H,fullWeekdaysParse:g,shortWeekdaysParse:v,minWeekdaysParse:H,monthsRegex:m,monthsShortRegex:m,monthsStrictRegex:h,monthsShortStrictRegex:y,monthsParse:u,longMonthsParse:u,shortMonthsParse:u,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [a viz] MMMM YYYY",LLL:"D [a viz] MMMM YYYY HH:mm",LLLL:"dddd, D [a viz] MMMM YYYY HH:mm"},calendar:{sameDay:"[Hiziv da] LT",nextDay:"[Warc\u02BChoazh da] LT",nextWeek:"dddd [da] LT",lastDay:"[Dec\u02BCh da] LT",lastWeek:"dddd [paset da] LT",sameElse:"L"},relativeTime:{future:"a-benn %s",past:"%s \u02BCzo",s:"un nebeud segondenno\xF9",ss:"%d eilenn",m:"ur vunutenn",mm:a,h:"un eur",hh:"%d eur",d:"un devezh",dd:a,M:"ur miz",MM:a,y:"ur bloaz",yy:r},dayOfMonthOrdinalParse:/\d{1,2}(añ|vet)/,ordinal:function(k){var I=k===1?"a\xF1":"vet";return k+I},week:{dow:1,doy:4},meridiemParse:/a.m.|g.m./,isPM:function(k){return k==="g.m."},meridiem:function(k,I,Me){return k<12?"a.m.":"g.m."}});return Q})},44429(L,f,c){//! moment.js locale configuration
//! locale : Bosnian [bs]
//! author : Nedim Cholich : https://github.com/frontyard
//! author : Rasid Redzic : https://github.com/rasidre
//! based on (hr) translation by Bojan Marković
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(n,d,u,m){switch(u){case"m":return d?"jedna minuta":m?"jednu minutu":"jedne minute"}}function r(n,d,u){var m=n+" ";switch(u){case"ss":return n===1?m+="sekunda":n===2||n===3||n===4?m+="sekunde":m+="sekundi",m;case"mm":return n===1?m+="minuta":n===2||n===3||n===4?m+="minute":m+="minuta",m;case"h":return"jedan sat";case"hh":return n===1?m+="sat":n===2||n===3||n===4?m+="sata":m+="sati",m;case"dd":return n===1?m+="dan":m+="dana",m;case"MM":return n===1?m+="mjesec":n===2||n===3||n===4?m+="mjeseca":m+="mjeseci",m;case"yy":return n===1?m+="godina":n===2||n===3||n===4?m+="godine":m+="godina",m}}var s=o.defineLocale("bs",{months:"januar_februar_mart_april_maj_juni_juli_august_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._aug._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_\u010Detvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._\u010Det._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_\u010De_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[ju\u010Der u] LT",lastWeek:function(){switch(this.day()){case 0:case 3:return"[pro\u0161lu] dddd [u] LT";case 6:return"[pro\u0161le] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[pro\u0161li] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",ss:r,m:a,mm:r,h:r,hh:r,d:"dan",dd:r,M:"mjesec",MM:r,y:"godinu",yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return s})},7306(L,f,c){//! moment.js locale configuration
//! locale : Catalan [ca]
//! author : Juan G. Hurtado : https://github.com/juanghurtado
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ca",{months:{standalone:"gener_febrer_mar\xE7_abril_maig_juny_juliol_agost_setembre_octubre_novembre_desembre".split("_"),format:"de gener_de febrer_de mar\xE7_d'abril_de maig_de juny_de juliol_d'agost_de setembre_d'octubre_de novembre_de desembre".split("_"),isFormat:/D[oD]?(\s)+MMMM/},monthsShort:"gen._febr._mar\xE7_abr._maig_juny_jul._ag._set._oct._nov._des.".split("_"),monthsParseExact:!0,weekdays:"diumenge_dilluns_dimarts_dimecres_dijous_divendres_dissabte".split("_"),weekdaysShort:"dg._dl._dt._dc._dj._dv._ds.".split("_"),weekdaysMin:"dg_dl_dt_dc_dj_dv_ds".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [de] YYYY",ll:"D MMM YYYY",LLL:"D MMMM [de] YYYY [a les] H:mm",lll:"D MMM YYYY, H:mm",LLLL:"dddd D MMMM [de] YYYY [a les] H:mm",llll:"ddd D MMM YYYY, H:mm"},calendar:{sameDay:function(){return"[avui a "+(this.hours()!==1?"les":"la")+"] LT"},nextDay:function(){return"[dem\xE0 a "+(this.hours()!==1?"les":"la")+"] LT"},nextWeek:function(){return"dddd [a "+(this.hours()!==1?"les":"la")+"] LT"},lastDay:function(){return"[ahir a "+(this.hours()!==1?"les":"la")+"] LT"},lastWeek:function(){return"[el] dddd [passat a "+(this.hours()!==1?"les":"la")+"] LT"},sameElse:"L"},relativeTime:{future:"d'aqu\xED %s",past:"fa %s",s:"uns segons",ss:"%d segons",m:"un minut",mm:"%d minuts",h:"una hora",hh:"%d hores",d:"un dia",dd:"%d dies",M:"un mes",MM:"%d mesos",y:"un any",yy:"%d anys"},dayOfMonthOrdinalParse:/\d{1,2}(r|n|t|è|a)/,ordinal:function(r,s){var n=r===1?"r":r===2?"n":r===3?"r":r===4?"t":"\xE8";return(s==="w"||s==="W")&&(n="a"),r+n},week:{dow:1,doy:4}});return a})},56464(L,f,c){//! moment.js locale configuration
//! locale : Czech [cs]
//! author : petrbela : https://github.com/petrbela
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={standalone:"leden_\xFAnor_b\u0159ezen_duben_kv\u011Bten_\u010Derven_\u010Dervenec_srpen_z\xE1\u0159\xED_\u0159\xEDjen_listopad_prosinec".split("_"),format:"ledna_\xFAnora_b\u0159ezna_dubna_kv\u011Btna_\u010Dervna_\u010Dervence_srpna_z\xE1\u0159\xED_\u0159\xEDjna_listopadu_prosince".split("_"),isFormat:/DD?[o.]?(\[[^\[\]]*\]|\s)+MMMM/},r="led_\xFAno_b\u0159e_dub_kv\u011B_\u010Dvn_\u010Dvc_srp_z\xE1\u0159_\u0159\xEDj_lis_pro".split("_"),s=[/^led/i,/^úno/i,/^bře/i,/^dub/i,/^kvě/i,/^(čvn|červen$|června)/i,/^(čvc|červenec|července)/i,/^srp/i,/^zář/i,/^říj/i,/^lis/i,/^pro/i],n=/^(leden|únor|březen|duben|květen|červenec|července|červen|června|srpen|září|říjen|listopad|prosinec|led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i;function d(h){return h>1&&h<5&&~~(h/10)!==1}function u(h,y,g,v){var H=h+" ";switch(g){case"s":return y||v?"p\xE1r sekund":"p\xE1r sekundami";case"ss":return y||v?H+(d(h)?"sekundy":"sekund"):H+"sekundami";case"m":return y?"minuta":v?"minutu":"minutou";case"mm":return y||v?H+(d(h)?"minuty":"minut"):H+"minutami";case"h":return y?"hodina":v?"hodinu":"hodinou";case"hh":return y||v?H+(d(h)?"hodiny":"hodin"):H+"hodinami";case"d":return y||v?"den":"dnem";case"dd":return y||v?H+(d(h)?"dny":"dn\xED"):H+"dny";case"M":return y||v?"m\u011Bs\xEDc":"m\u011Bs\xEDcem";case"MM":return y||v?H+(d(h)?"m\u011Bs\xEDce":"m\u011Bs\xEDc\u016F"):H+"m\u011Bs\xEDci";case"y":return y||v?"rok":"rokem";case"yy":return y||v?H+(d(h)?"roky":"let"):H+"lety"}}var m=o.defineLocale("cs",{months:a,monthsShort:r,monthsRegex:n,monthsShortRegex:n,monthsStrictRegex:/^(leden|ledna|února|únor|březen|března|duben|dubna|květen|května|červenec|července|červen|června|srpen|srpna|září|říjen|října|listopadu|listopad|prosinec|prosince)/i,monthsShortStrictRegex:/^(led|úno|bře|dub|kvě|čvn|čvc|srp|zář|říj|lis|pro)/i,monthsParse:s,longMonthsParse:s,shortMonthsParse:s,weekdays:"ned\u011Ble_pond\u011Bl\xED_\xFAter\xFD_st\u0159eda_\u010Dtvrtek_p\xE1tek_sobota".split("_"),weekdaysShort:"ne_po_\xFAt_st_\u010Dt_p\xE1_so".split("_"),weekdaysMin:"ne_po_\xFAt_st_\u010Dt_p\xE1_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm",l:"D. M. YYYY"},calendar:{sameDay:"[dnes v] LT",nextDay:"[z\xEDtra v] LT",nextWeek:function(){switch(this.day()){case 0:return"[v ned\u011Bli v] LT";case 1:case 2:return"[v] dddd [v] LT";case 3:return"[ve st\u0159edu v] LT";case 4:return"[ve \u010Dtvrtek v] LT";case 5:return"[v p\xE1tek v] LT";case 6:return"[v sobotu v] LT"}},lastDay:"[v\u010Dera v] LT",lastWeek:function(){switch(this.day()){case 0:return"[minulou ned\u011Bli v] LT";case 1:case 2:return"[minul\xE9] dddd [v] LT";case 3:return"[minulou st\u0159edu v] LT";case 4:case 5:return"[minul\xFD] dddd [v] LT";case 6:return"[minulou sobotu v] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"p\u0159ed %s",s:u,ss:u,m:u,mm:u,h:u,hh:u,d:u,dd:u,M:u,MM:u,y:u,yy:u},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return m})},73635(L,f,c){//! moment.js locale configuration
//! locale : Chuvash [cv]
//! author : Anatoly Mironov : https://github.com/mirontoli
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("cv",{months:"\u043A\u04D1\u0440\u043B\u0430\u0447_\u043D\u0430\u0440\u04D1\u0441_\u043F\u0443\u0448_\u0430\u043A\u0430_\u043C\u0430\u0439_\u04AB\u04D7\u0440\u0442\u043C\u0435_\u0443\u0442\u04D1_\u04AB\u0443\u0440\u043B\u0430_\u0430\u0432\u04D1\u043D_\u044E\u043F\u0430_\u0447\u04F3\u043A_\u0440\u0430\u0448\u0442\u0430\u0432".split("_"),monthsShort:"\u043A\u04D1\u0440_\u043D\u0430\u0440_\u043F\u0443\u0448_\u0430\u043A\u0430_\u043C\u0430\u0439_\u04AB\u04D7\u0440_\u0443\u0442\u04D1_\u04AB\u0443\u0440_\u0430\u0432\u043D_\u044E\u043F\u0430_\u0447\u04F3\u043A_\u0440\u0430\u0448".split("_"),weekdays:"\u0432\u044B\u0440\u0441\u0430\u0440\u043D\u0438\u043A\u0443\u043D_\u0442\u0443\u043D\u0442\u0438\u043A\u0443\u043D_\u044B\u0442\u043B\u0430\u0440\u0438\u043A\u0443\u043D_\u044E\u043D\u043A\u0443\u043D_\u043A\u04D7\u04AB\u043D\u0435\u0440\u043D\u0438\u043A\u0443\u043D_\u044D\u0440\u043D\u0435\u043A\u0443\u043D_\u0448\u04D1\u043C\u0430\u0442\u043A\u0443\u043D".split("_"),weekdaysShort:"\u0432\u044B\u0440_\u0442\u0443\u043D_\u044B\u0442\u043B_\u044E\u043D_\u043A\u04D7\u04AB_\u044D\u0440\u043D_\u0448\u04D1\u043C".split("_"),weekdaysMin:"\u0432\u0440_\u0442\u043D_\u044B\u0442_\u044E\u043D_\u043A\u04AB_\u044D\u0440_\u0448\u043C".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"YYYY [\u04AB\u0443\u043B\u0445\u0438] MMMM [\u0443\u0439\u04D1\u0445\u04D7\u043D] D[-\u043C\u04D7\u0448\u04D7]",LLL:"YYYY [\u04AB\u0443\u043B\u0445\u0438] MMMM [\u0443\u0439\u04D1\u0445\u04D7\u043D] D[-\u043C\u04D7\u0448\u04D7], HH:mm",LLLL:"dddd, YYYY [\u04AB\u0443\u043B\u0445\u0438] MMMM [\u0443\u0439\u04D1\u0445\u04D7\u043D] D[-\u043C\u04D7\u0448\u04D7], HH:mm"},calendar:{sameDay:"[\u041F\u0430\u044F\u043D] LT [\u0441\u0435\u0445\u0435\u0442\u0440\u0435]",nextDay:"[\u042B\u0440\u0430\u043D] LT [\u0441\u0435\u0445\u0435\u0442\u0440\u0435]",lastDay:"[\u04D6\u043D\u0435\u0440] LT [\u0441\u0435\u0445\u0435\u0442\u0440\u0435]",nextWeek:"[\u04AA\u0438\u0442\u0435\u0441] dddd LT [\u0441\u0435\u0445\u0435\u0442\u0440\u0435]",lastWeek:"[\u0418\u0440\u0442\u043D\u04D7] dddd LT [\u0441\u0435\u0445\u0435\u0442\u0440\u0435]",sameElse:"L"},relativeTime:{future:function(r){var s=/сехет$/i.exec(r)?"\u0440\u0435\u043D":/ҫул$/i.exec(r)?"\u0442\u0430\u043D":"\u0440\u0430\u043D";return r+s},past:"%s \u043A\u0430\u044F\u043B\u043B\u0430",s:"\u043F\u04D7\u0440-\u0438\u043A \u04AB\u0435\u043A\u043A\u0443\u043D\u0442",ss:"%d \u04AB\u0435\u043A\u043A\u0443\u043D\u0442",m:"\u043F\u04D7\u0440 \u043C\u0438\u043D\u0443\u0442",mm:"%d \u043C\u0438\u043D\u0443\u0442",h:"\u043F\u04D7\u0440 \u0441\u0435\u0445\u0435\u0442",hh:"%d \u0441\u0435\u0445\u0435\u0442",d:"\u043F\u04D7\u0440 \u043A\u0443\u043D",dd:"%d \u043A\u0443\u043D",M:"\u043F\u04D7\u0440 \u0443\u0439\u04D1\u0445",MM:"%d \u0443\u0439\u04D1\u0445",y:"\u043F\u04D7\u0440 \u04AB\u0443\u043B",yy:"%d \u04AB\u0443\u043B"},dayOfMonthOrdinalParse:/\d{1,2}-мӗш/,ordinal:"%d-\u043C\u04D7\u0448",week:{dow:1,doy:7}});return a})},64226(L,f,c){//! moment.js locale configuration
//! locale : Welsh [cy]
//! author : Robert Allen : https://github.com/robgallen
//! author : https://github.com/ryangreaves
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("cy",{months:"Ionawr_Chwefror_Mawrth_Ebrill_Mai_Mehefin_Gorffennaf_Awst_Medi_Hydref_Tachwedd_Rhagfyr".split("_"),monthsShort:"Ion_Chwe_Maw_Ebr_Mai_Meh_Gor_Aws_Med_Hyd_Tach_Rhag".split("_"),weekdays:"Dydd Sul_Dydd Llun_Dydd Mawrth_Dydd Mercher_Dydd Iau_Dydd Gwener_Dydd Sadwrn".split("_"),weekdaysShort:"Sul_Llun_Maw_Mer_Iau_Gwe_Sad".split("_"),weekdaysMin:"Su_Ll_Ma_Me_Ia_Gw_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Heddiw am] LT",nextDay:"[Yfory am] LT",nextWeek:"dddd [am] LT",lastDay:"[Ddoe am] LT",lastWeek:"dddd [diwethaf am] LT",sameElse:"L"},relativeTime:{future:"mewn %s",past:"%s yn \xF4l",s:"ychydig eiliadau",ss:"%d eiliad",m:"munud",mm:"%d munud",h:"awr",hh:"%d awr",d:"diwrnod",dd:"%d diwrnod",M:"mis",MM:"%d mis",y:"blwyddyn",yy:"%d flynedd"},dayOfMonthOrdinalParse:/\d{1,2}(fed|ain|af|il|ydd|ed|eg)/,ordinal:function(r){var s=r,n="",d=["","af","il","ydd","ydd","ed","ed","ed","fed","fed","fed","eg","fed","eg","eg","fed","eg","eg","fed","eg","fed"];return s>20?s===40||s===50||s===60||s===80||s===100?n="fed":n="ain":s>0&&(n=d[s]),r+n},week:{dow:1,doy:4}});return a})},93601(L,f,c){//! moment.js locale configuration
//! locale : Danish [da]
//! author : Ulrik Nielsen : https://github.com/mrbase
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("da",{months:"januar_februar_marts_april_maj_juni_juli_august_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"s\xF8ndag_mandag_tirsdag_onsdag_torsdag_fredag_l\xF8rdag".split("_"),weekdaysShort:"s\xF8n_man_tir_ons_tor_fre_l\xF8r".split("_"),weekdaysMin:"s\xF8_ma_ti_on_to_fr_l\xF8".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd [d.] D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[i dag kl.] LT",nextDay:"[i morgen kl.] LT",nextWeek:"p\xE5 dddd [kl.] LT",lastDay:"[i g\xE5r kl.] LT",lastWeek:"[i] dddd[s kl.] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"f\xE5 sekunder",ss:"%d sekunder",m:"et minut",mm:"%d minutter",h:"en time",hh:"%d timer",d:"en dag",dd:"%d dage",M:"en m\xE5ned",MM:"%d m\xE5neder",y:"et \xE5r",yy:"%d \xE5r"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a})},26111(L,f,c){//! moment.js locale configuration
//! locale : German (Austria) [de-at]
//! author : lluchs : https://github.com/lluchs
//! author: Menelion Elensúle: https://github.com/Oire
//! author : Martin Groller : https://github.com/MadMG
//! author : Mikolaj Dadela : https://github.com/mik01aj
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(s,n,d,u){var m={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[s+" Tage",s+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[s+" Monate",s+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[s+" Jahre",s+" Jahren"]};return n?m[d][0]:m[d][1]}var r=o.defineLocale("de-at",{months:"J\xE4nner_Februar_M\xE4rz_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"J\xE4n._Feb._M\xE4rz_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:a,mm:"%d Minuten",h:a,hh:"%d Stunden",d:a,dd:a,w:a,ww:"%d Wochen",M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return r})},54697(L,f,c){//! moment.js locale configuration
//! locale : German (Switzerland) [de-ch]
//! author : sschueller : https://github.com/sschueller
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(s,n,d,u){var m={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[s+" Tage",s+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[s+" Monate",s+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[s+" Jahre",s+" Jahren"]};return n?m[d][0]:m[d][1]}var r=o.defineLocale("de-ch",{months:"Januar_Februar_M\xE4rz_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Feb._M\xE4rz_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:a,mm:"%d Minuten",h:a,hh:"%d Stunden",d:a,dd:a,w:a,ww:"%d Wochen",M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return r})},77853(L,f,c){//! moment.js locale configuration
//! locale : German [de]
//! author : lluchs : https://github.com/lluchs
//! author: Menelion Elensúle: https://github.com/Oire
//! author : Mikolaj Dadela : https://github.com/mik01aj
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(s,n,d,u){var m={m:["eine Minute","einer Minute"],h:["eine Stunde","einer Stunde"],d:["ein Tag","einem Tag"],dd:[s+" Tage",s+" Tagen"],w:["eine Woche","einer Woche"],M:["ein Monat","einem Monat"],MM:[s+" Monate",s+" Monaten"],y:["ein Jahr","einem Jahr"],yy:[s+" Jahre",s+" Jahren"]};return n?m[d][0]:m[d][1]}var r=o.defineLocale("de",{months:"Januar_Februar_M\xE4rz_April_Mai_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Feb._M\xE4rz_Apr._Mai_Juni_Juli_Aug._Sep._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonntag_Montag_Dienstag_Mittwoch_Donnerstag_Freitag_Samstag".split("_"),weekdaysShort:"So._Mo._Di._Mi._Do._Fr._Sa.".split("_"),weekdaysMin:"So_Mo_Di_Mi_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY HH:mm",LLLL:"dddd, D. MMMM YYYY HH:mm"},calendar:{sameDay:"[heute um] LT [Uhr]",sameElse:"L",nextDay:"[morgen um] LT [Uhr]",nextWeek:"dddd [um] LT [Uhr]",lastDay:"[gestern um] LT [Uhr]",lastWeek:"[letzten] dddd [um] LT [Uhr]"},relativeTime:{future:"in %s",past:"vor %s",s:"ein paar Sekunden",ss:"%d Sekunden",m:a,mm:"%d Minuten",h:a,hh:"%d Stunden",d:a,dd:a,w:a,ww:"%d Wochen",M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return r})},60708(L,f,c){//! moment.js locale configuration
//! locale : Maldivian [dv]
//! author : Jawish Hameed : https://github.com/jawish
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=["\u0796\u07AC\u0782\u07AA\u0787\u07A6\u0783\u07A9","\u078A\u07AC\u0784\u07B0\u0783\u07AA\u0787\u07A6\u0783\u07A9","\u0789\u07A7\u0783\u07A8\u0797\u07AA","\u0787\u07AD\u0795\u07B0\u0783\u07A9\u078D\u07AA","\u0789\u07AD","\u0796\u07AB\u0782\u07B0","\u0796\u07AA\u078D\u07A6\u0787\u07A8","\u0787\u07AF\u078E\u07A6\u0790\u07B0\u0793\u07AA","\u0790\u07AC\u0795\u07B0\u0793\u07AC\u0789\u07B0\u0784\u07A6\u0783\u07AA","\u0787\u07AE\u0786\u07B0\u0793\u07AF\u0784\u07A6\u0783\u07AA","\u0782\u07AE\u0788\u07AC\u0789\u07B0\u0784\u07A6\u0783\u07AA","\u0791\u07A8\u0790\u07AC\u0789\u07B0\u0784\u07A6\u0783\u07AA"],r=["\u0787\u07A7\u078B\u07A8\u0787\u07B0\u078C\u07A6","\u0780\u07AF\u0789\u07A6","\u0787\u07A6\u0782\u07B0\u078E\u07A7\u0783\u07A6","\u0784\u07AA\u078B\u07A6","\u0784\u07AA\u0783\u07A7\u0790\u07B0\u078A\u07A6\u078C\u07A8","\u0780\u07AA\u0786\u07AA\u0783\u07AA","\u0780\u07AE\u0782\u07A8\u0780\u07A8\u0783\u07AA"],s=o.defineLocale("dv",{months:a,monthsShort:a,weekdays:r,weekdaysShort:r,weekdaysMin:"\u0787\u07A7\u078B\u07A8_\u0780\u07AF\u0789\u07A6_\u0787\u07A6\u0782\u07B0_\u0784\u07AA\u078B\u07A6_\u0784\u07AA\u0783\u07A7_\u0780\u07AA\u0786\u07AA_\u0780\u07AE\u0782\u07A8".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"D/M/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},meridiemParse:/މކ|މފ/,isPM:function(n){return n==="\u0789\u078A"},meridiem:function(n,d,u){return n<12?"\u0789\u0786":"\u0789\u078A"},calendar:{sameDay:"[\u0789\u07A8\u0787\u07A6\u078B\u07AA] LT",nextDay:"[\u0789\u07A7\u078B\u07A6\u0789\u07A7] LT",nextWeek:"dddd LT",lastDay:"[\u0787\u07A8\u0787\u07B0\u0794\u07AC] LT",lastWeek:"[\u078A\u07A7\u0787\u07A8\u078C\u07AA\u0788\u07A8] dddd LT",sameElse:"L"},relativeTime:{future:"\u078C\u07AC\u0783\u07AD\u078E\u07A6\u0787\u07A8 %s",past:"\u0786\u07AA\u0783\u07A8\u0782\u07B0 %s",s:"\u0790\u07A8\u0786\u07AA\u0782\u07B0\u078C\u07AA\u0786\u07AE\u0785\u07AC\u0787\u07B0",ss:"d% \u0790\u07A8\u0786\u07AA\u0782\u07B0\u078C\u07AA",m:"\u0789\u07A8\u0782\u07A8\u0793\u07AC\u0787\u07B0",mm:"\u0789\u07A8\u0782\u07A8\u0793\u07AA %d",h:"\u078E\u07A6\u0791\u07A8\u0787\u07A8\u0783\u07AC\u0787\u07B0",hh:"\u078E\u07A6\u0791\u07A8\u0787\u07A8\u0783\u07AA %d",d:"\u078B\u07AA\u0788\u07A6\u0780\u07AC\u0787\u07B0",dd:"\u078B\u07AA\u0788\u07A6\u0790\u07B0 %d",M:"\u0789\u07A6\u0780\u07AC\u0787\u07B0",MM:"\u0789\u07A6\u0790\u07B0 %d",y:"\u0787\u07A6\u0780\u07A6\u0783\u07AC\u0787\u07B0",yy:"\u0787\u07A6\u0780\u07A6\u0783\u07AA %d"},preparse:function(n){return n.replace(/،/g,",")},postformat:function(n){return n.replace(/,/g,"\u060C")},week:{dow:7,doy:12}});return s})},54691(L,f,c){//! moment.js locale configuration
//! locale : Greek [el]
//! author : Aggelos Karalias : https://github.com/mehiel
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(s){return typeof Function<"u"&&s instanceof Function||Object.prototype.toString.call(s)==="[object Function]"}var r=o.defineLocale("el",{monthsNominativeEl:"\u0399\u03B1\u03BD\u03BF\u03C5\u03AC\u03C1\u03B9\u03BF\u03C2_\u03A6\u03B5\u03B2\u03C1\u03BF\u03C5\u03AC\u03C1\u03B9\u03BF\u03C2_\u039C\u03AC\u03C1\u03C4\u03B9\u03BF\u03C2_\u0391\u03C0\u03C1\u03AF\u03BB\u03B9\u03BF\u03C2_\u039C\u03AC\u03B9\u03BF\u03C2_\u0399\u03BF\u03CD\u03BD\u03B9\u03BF\u03C2_\u0399\u03BF\u03CD\u03BB\u03B9\u03BF\u03C2_\u0391\u03CD\u03B3\u03BF\u03C5\u03C3\u03C4\u03BF\u03C2_\u03A3\u03B5\u03C0\u03C4\u03AD\u03BC\u03B2\u03C1\u03B9\u03BF\u03C2_\u039F\u03BA\u03C4\u03CE\u03B2\u03C1\u03B9\u03BF\u03C2_\u039D\u03BF\u03AD\u03BC\u03B2\u03C1\u03B9\u03BF\u03C2_\u0394\u03B5\u03BA\u03AD\u03BC\u03B2\u03C1\u03B9\u03BF\u03C2".split("_"),monthsGenitiveEl:"\u0399\u03B1\u03BD\u03BF\u03C5\u03B1\u03C1\u03AF\u03BF\u03C5_\u03A6\u03B5\u03B2\u03C1\u03BF\u03C5\u03B1\u03C1\u03AF\u03BF\u03C5_\u039C\u03B1\u03C1\u03C4\u03AF\u03BF\u03C5_\u0391\u03C0\u03C1\u03B9\u03BB\u03AF\u03BF\u03C5_\u039C\u03B1\u0390\u03BF\u03C5_\u0399\u03BF\u03C5\u03BD\u03AF\u03BF\u03C5_\u0399\u03BF\u03C5\u03BB\u03AF\u03BF\u03C5_\u0391\u03C5\u03B3\u03BF\u03CD\u03C3\u03C4\u03BF\u03C5_\u03A3\u03B5\u03C0\u03C4\u03B5\u03BC\u03B2\u03C1\u03AF\u03BF\u03C5_\u039F\u03BA\u03C4\u03C9\u03B2\u03C1\u03AF\u03BF\u03C5_\u039D\u03BF\u03B5\u03BC\u03B2\u03C1\u03AF\u03BF\u03C5_\u0394\u03B5\u03BA\u03B5\u03BC\u03B2\u03C1\u03AF\u03BF\u03C5".split("_"),months:function(s,n){return s?typeof n=="string"&&/D/.test(n.substring(0,n.indexOf("MMMM")))?this._monthsGenitiveEl[s.month()]:this._monthsNominativeEl[s.month()]:this._monthsNominativeEl},monthsShort:"\u0399\u03B1\u03BD_\u03A6\u03B5\u03B2_\u039C\u03B1\u03C1_\u0391\u03C0\u03C1_\u039C\u03B1\u03CA_\u0399\u03BF\u03C5\u03BD_\u0399\u03BF\u03C5\u03BB_\u0391\u03C5\u03B3_\u03A3\u03B5\u03C0_\u039F\u03BA\u03C4_\u039D\u03BF\u03B5_\u0394\u03B5\u03BA".split("_"),weekdays:"\u039A\u03C5\u03C1\u03B9\u03B1\u03BA\u03AE_\u0394\u03B5\u03C5\u03C4\u03AD\u03C1\u03B1_\u03A4\u03C1\u03AF\u03C4\u03B7_\u03A4\u03B5\u03C4\u03AC\u03C1\u03C4\u03B7_\u03A0\u03AD\u03BC\u03C0\u03C4\u03B7_\u03A0\u03B1\u03C1\u03B1\u03C3\u03BA\u03B5\u03C5\u03AE_\u03A3\u03AC\u03B2\u03B2\u03B1\u03C4\u03BF".split("_"),weekdaysShort:"\u039A\u03C5\u03C1_\u0394\u03B5\u03C5_\u03A4\u03C1\u03B9_\u03A4\u03B5\u03C4_\u03A0\u03B5\u03BC_\u03A0\u03B1\u03C1_\u03A3\u03B1\u03B2".split("_"),weekdaysMin:"\u039A\u03C5_\u0394\u03B5_\u03A4\u03C1_\u03A4\u03B5_\u03A0\u03B5_\u03A0\u03B1_\u03A3\u03B1".split("_"),meridiem:function(s,n,d){return s>11?d?"\u03BC\u03BC":"\u039C\u039C":d?"\u03C0\u03BC":"\u03A0\u039C"},isPM:function(s){return(s+"").toLowerCase()[0]==="\u03BC"},meridiemParse:/[ΠΜ]\.?Μ?\.?/i,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendarEl:{sameDay:"[\u03A3\u03AE\u03BC\u03B5\u03C1\u03B1 {}] LT",nextDay:"[\u0391\u03CD\u03C1\u03B9\u03BF {}] LT",nextWeek:"dddd [{}] LT",lastDay:"[\u03A7\u03B8\u03B5\u03C2 {}] LT",lastWeek:function(){switch(this.day()){case 6:return"[\u03C4\u03BF \u03C0\u03C1\u03BF\u03B7\u03B3\u03BF\u03CD\u03BC\u03B5\u03BD\u03BF] dddd [{}] LT";default:return"[\u03C4\u03B7\u03BD \u03C0\u03C1\u03BF\u03B7\u03B3\u03BF\u03CD\u03BC\u03B5\u03BD\u03B7] dddd [{}] LT"}},sameElse:"L"},calendar:function(s,n){var d=this._calendarEl[s],u=n&&n.hours();return a(d)&&(d=d.apply(n)),d.replace("{}",u%12===1?"\u03C3\u03C4\u03B7":"\u03C3\u03C4\u03B9\u03C2")},relativeTime:{future:"\u03C3\u03B5 %s",past:"%s \u03C0\u03C1\u03B9\u03BD",s:"\u03BB\u03AF\u03B3\u03B1 \u03B4\u03B5\u03C5\u03C4\u03B5\u03C1\u03CC\u03BB\u03B5\u03C0\u03C4\u03B1",ss:"%d \u03B4\u03B5\u03C5\u03C4\u03B5\u03C1\u03CC\u03BB\u03B5\u03C0\u03C4\u03B1",m:"\u03AD\u03BD\u03B1 \u03BB\u03B5\u03C0\u03C4\u03CC",mm:"%d \u03BB\u03B5\u03C0\u03C4\u03AC",h:"\u03BC\u03AF\u03B1 \u03CE\u03C1\u03B1",hh:"%d \u03CE\u03C1\u03B5\u03C2",d:"\u03BC\u03AF\u03B1 \u03BC\u03AD\u03C1\u03B1",dd:"%d \u03BC\u03AD\u03C1\u03B5\u03C2",M:"\u03AD\u03BD\u03B1\u03C2 \u03BC\u03AE\u03BD\u03B1\u03C2",MM:"%d \u03BC\u03AE\u03BD\u03B5\u03C2",y:"\u03AD\u03BD\u03B1\u03C2 \u03C7\u03C1\u03CC\u03BD\u03BF\u03C2",yy:"%d \u03C7\u03C1\u03CC\u03BD\u03B9\u03B1"},dayOfMonthOrdinalParse:/\d{1,2}η/,ordinal:"%d\u03B7",week:{dow:1,doy:4}});return r})},53872(L,f,c){//! moment.js locale configuration
//! locale : English (Australia) [en-au]
//! author : Jared Morse : https://github.com/jarcoal
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("en-au",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(r){var s=r%10,n=~~(r%100/10)===1?"th":s===1?"st":s===2?"nd":s===3?"rd":"th";return r+n},week:{dow:0,doy:4}});return a})},28298(L,f,c){//! moment.js locale configuration
//! locale : English (Canada) [en-ca]
//! author : Jonathan Abourbih : https://github.com/jonbca
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("en-ca",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"YYYY-MM-DD",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(r){var s=r%10,n=~~(r%100/10)===1?"th":s===1?"st":s===2?"nd":s===3?"rd":"th";return r+n}});return a})},56195(L,f,c){//! moment.js locale configuration
//! locale : English (United Kingdom) [en-gb]
//! author : Chris Gedrim : https://github.com/chrisgedrim
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("en-gb",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(r){var s=r%10,n=~~(r%100/10)===1?"th":s===1?"st":s===2?"nd":s===3?"rd":"th";return r+n},week:{dow:1,doy:4}});return a})},66584(L,f,c){//! moment.js locale configuration
//! locale : English (Ireland) [en-ie]
//! author : Chris Cartlidge : https://github.com/chriscartlidge
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("en-ie",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(r){var s=r%10,n=~~(r%100/10)===1?"th":s===1?"st":s===2?"nd":s===3?"rd":"th";return r+n},week:{dow:1,doy:4}});return a})},65543(L,f,c){//! moment.js locale configuration
//! locale : English (Israel) [en-il]
//! author : Chris Gedrim : https://github.com/chrisgedrim
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("en-il",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(r){var s=r%10,n=~~(r%100/10)===1?"th":s===1?"st":s===2?"nd":s===3?"rd":"th";return r+n}});return a})},9033(L,f,c){//! moment.js locale configuration
//! locale : English (India) [en-in]
//! author : Jatin Agrawal : https://github.com/jatinag22
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("en-in",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(r){var s=r%10,n=~~(r%100/10)===1?"th":s===1?"st":s===2?"nd":s===3?"rd":"th";return r+n},week:{dow:0,doy:6}});return a})},79402(L,f,c){//! moment.js locale configuration
//! locale : English (New Zealand) [en-nz]
//! author : Luke McGregor : https://github.com/lukemcgregor
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("en-nz",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(r){var s=r%10,n=~~(r%100/10)===1?"th":s===1?"st":s===2?"nd":s===3?"rd":"th";return r+n},week:{dow:1,doy:4}});return a})},43004(L,f,c){//! moment.js locale configuration
//! locale : English (Singapore) [en-sg]
//! author : Matthew Castrillon-Madrigal : https://github.com/techdimension
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("en-sg",{months:"January_February_March_April_May_June_July_August_September_October_November_December".split("_"),monthsShort:"Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),weekdays:"Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),weekdaysShort:"Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),weekdaysMin:"Su_Mo_Tu_We_Th_Fr_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"},relativeTime:{future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",M:"a month",MM:"%d months",y:"a year",yy:"%d years"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(r){var s=r%10,n=~~(r%100/10)===1?"th":s===1?"st":s===2?"nd":s===3?"rd":"th";return r+n},week:{dow:1,doy:4}});return a})},32934(L,f,c){//! moment.js locale configuration
//! locale : Esperanto [eo]
//! author : Colin Dean : https://github.com/colindean
//! author : Mia Nordentoft Imperatori : https://github.com/miestasmia
//! comment : miestasmia corrected the translation by colindean
//! comment : Vivakvo corrected the translation by colindean and miestasmia
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("eo",{months:"januaro_februaro_marto_aprilo_majo_junio_julio_a\u016Dgusto_septembro_oktobro_novembro_decembro".split("_"),monthsShort:"jan_feb_mart_apr_maj_jun_jul_a\u016Dg_sept_okt_nov_dec".split("_"),weekdays:"diman\u0109o_lundo_mardo_merkredo_\u0135a\u016Ddo_vendredo_sabato".split("_"),weekdaysShort:"dim_lun_mard_merk_\u0135a\u016D_ven_sab".split("_"),weekdaysMin:"di_lu_ma_me_\u0135a_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"[la] D[-an de] MMMM, YYYY",LLL:"[la] D[-an de] MMMM, YYYY HH:mm",LLLL:"dddd[n], [la] D[-an de] MMMM, YYYY HH:mm",llll:"ddd, [la] D[-an de] MMM, YYYY HH:mm"},meridiemParse:/[ap]\.t\.m/i,isPM:function(r){return r.charAt(0).toLowerCase()==="p"},meridiem:function(r,s,n){return r>11?n?"p.t.m.":"P.T.M.":n?"a.t.m.":"A.T.M."},calendar:{sameDay:"[Hodia\u016D je] LT",nextDay:"[Morga\u016D je] LT",nextWeek:"dddd[n je] LT",lastDay:"[Hiera\u016D je] LT",lastWeek:"[pasintan] dddd[n je] LT",sameElse:"L"},relativeTime:{future:"post %s",past:"anta\u016D %s",s:"kelkaj sekundoj",ss:"%d sekundoj",m:"unu minuto",mm:"%d minutoj",h:"unu horo",hh:"%d horoj",d:"unu tago",dd:"%d tagoj",M:"unu monato",MM:"%d monatoj",y:"unu jaro",yy:"%d jaroj"},dayOfMonthOrdinalParse:/\d{1,2}a/,ordinal:"%da",week:{dow:1,doy:7}});return a})},20838(L,f,c){//! moment.js locale configuration
//! locale : Spanish (Dominican Republic) [es-do]
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),r="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),s=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],n=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,d=o.defineLocale("es-do",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(u,m){return u?/-MMM-/.test(m)?r[u.month()]:a[u.month()]:a},monthsRegex:n,monthsShortRegex:n,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:s,longMonthsParse:s,shortMonthsParse:s,weekdays:"domingo_lunes_martes_mi\xE9rcoles_jueves_viernes_s\xE1bado".split("_"),weekdaysShort:"dom._lun._mar._mi\xE9._jue._vie._s\xE1b.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_s\xE1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY h:mm A",LLLL:"dddd, D [de] MMMM [de] YYYY h:mm A"},calendar:{sameDay:function(){return"[hoy a la"+(this.hours()!==1?"s":"")+"] LT"},nextDay:function(){return"[ma\xF1ana a la"+(this.hours()!==1?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(this.hours()!==1?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(this.hours()!==1?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(this.hours()!==1?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un d\xEDa",dd:"%d d\xEDas",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un a\xF1o",yy:"%d a\xF1os"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%d\xBA",week:{dow:1,doy:4}});return d})},17730(L,f,c){//! moment.js locale configuration
//! locale : Spanish (Mexico) [es-mx]
//! author : JC Franco : https://github.com/jcfranco
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),r="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),s=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],n=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,d=o.defineLocale("es-mx",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(u,m){return u?/-MMM-/.test(m)?r[u.month()]:a[u.month()]:a},monthsRegex:n,monthsShortRegex:n,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:s,longMonthsParse:s,shortMonthsParse:s,weekdays:"domingo_lunes_martes_mi\xE9rcoles_jueves_viernes_s\xE1bado".split("_"),weekdaysShort:"dom._lun._mar._mi\xE9._jue._vie._s\xE1b.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_s\xE1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoy a la"+(this.hours()!==1?"s":"")+"] LT"},nextDay:function(){return"[ma\xF1ana a la"+(this.hours()!==1?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(this.hours()!==1?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(this.hours()!==1?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(this.hours()!==1?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un d\xEDa",dd:"%d d\xEDas",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un a\xF1o",yy:"%d a\xF1os"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%d\xBA",week:{dow:0,doy:4},invalidDate:"Fecha inv\xE1lida"});return d})},56575(L,f,c){//! moment.js locale configuration
//! locale : Spanish (United States) [es-us]
//! author : bustta : https://github.com/bustta
//! author : chrisrodz : https://github.com/chrisrodz
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),r="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),s=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],n=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,d=o.defineLocale("es-us",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(u,m){return u?/-MMM-/.test(m)?r[u.month()]:a[u.month()]:a},monthsRegex:n,monthsShortRegex:n,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:s,longMonthsParse:s,shortMonthsParse:s,weekdays:"domingo_lunes_martes_mi\xE9rcoles_jueves_viernes_s\xE1bado".split("_"),weekdaysShort:"dom._lun._mar._mi\xE9._jue._vie._s\xE1b.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_s\xE1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"MM/DD/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY h:mm A",LLLL:"dddd, D [de] MMMM [de] YYYY h:mm A"},calendar:{sameDay:function(){return"[hoy a la"+(this.hours()!==1?"s":"")+"] LT"},nextDay:function(){return"[ma\xF1ana a la"+(this.hours()!==1?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(this.hours()!==1?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(this.hours()!==1?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(this.hours()!==1?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un d\xEDa",dd:"%d d\xEDas",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un a\xF1o",yy:"%d a\xF1os"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%d\xBA",week:{dow:0,doy:6}});return d})},97650(L,f,c){//! moment.js locale configuration
//! locale : Spanish [es]
//! author : Julio Napurí : https://github.com/julionc
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="ene._feb._mar._abr._may._jun._jul._ago._sep._oct._nov._dic.".split("_"),r="ene_feb_mar_abr_may_jun_jul_ago_sep_oct_nov_dic".split("_"),s=[/^ene/i,/^feb/i,/^mar/i,/^abr/i,/^may/i,/^jun/i,/^jul/i,/^ago/i,/^sep/i,/^oct/i,/^nov/i,/^dic/i],n=/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre|ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,d=o.defineLocale("es",{months:"enero_febrero_marzo_abril_mayo_junio_julio_agosto_septiembre_octubre_noviembre_diciembre".split("_"),monthsShort:function(u,m){return u?/-MMM-/.test(m)?r[u.month()]:a[u.month()]:a},monthsRegex:n,monthsShortRegex:n,monthsStrictRegex:/^(enero|febrero|marzo|abril|mayo|junio|julio|agosto|septiembre|octubre|noviembre|diciembre)/i,monthsShortStrictRegex:/^(ene\.?|feb\.?|mar\.?|abr\.?|may\.?|jun\.?|jul\.?|ago\.?|sep\.?|oct\.?|nov\.?|dic\.?)/i,monthsParse:s,longMonthsParse:s,shortMonthsParse:s,weekdays:"domingo_lunes_martes_mi\xE9rcoles_jueves_viernes_s\xE1bado".split("_"),weekdaysShort:"dom._lun._mar._mi\xE9._jue._vie._s\xE1b.".split("_"),weekdaysMin:"do_lu_ma_mi_ju_vi_s\xE1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoy a la"+(this.hours()!==1?"s":"")+"] LT"},nextDay:function(){return"[ma\xF1ana a la"+(this.hours()!==1?"s":"")+"] LT"},nextWeek:function(){return"dddd [a la"+(this.hours()!==1?"s":"")+"] LT"},lastDay:function(){return"[ayer a la"+(this.hours()!==1?"s":"")+"] LT"},lastWeek:function(){return"[el] dddd [pasado a la"+(this.hours()!==1?"s":"")+"] LT"},sameElse:"L"},relativeTime:{future:"en %s",past:"hace %s",s:"unos segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"una hora",hh:"%d horas",d:"un d\xEDa",dd:"%d d\xEDas",w:"una semana",ww:"%d semanas",M:"un mes",MM:"%d meses",y:"un a\xF1o",yy:"%d a\xF1os"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%d\xBA",week:{dow:1,doy:4},invalidDate:"Fecha inv\xE1lida"});return d})},3035(L,f,c){//! moment.js locale configuration
//! locale : Estonian [et]
//! author : Henry Kehlmann : https://github.com/madhenry
//! improvements : Illimar Tambek : https://github.com/ragulka
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(s,n,d,u){var m={s:["m\xF5ne sekundi","m\xF5ni sekund","paar sekundit"],ss:[s+"sekundi",s+"sekundit"],m:["\xFChe minuti","\xFCks minut"],mm:[s+" minuti",s+" minutit"],h:["\xFChe tunni","tund aega","\xFCks tund"],hh:[s+" tunni",s+" tundi"],d:["\xFChe p\xE4eva","\xFCks p\xE4ev"],M:["kuu aja","kuu aega","\xFCks kuu"],MM:[s+" kuu",s+" kuud"],y:["\xFChe aasta","aasta","\xFCks aasta"],yy:[s+" aasta",s+" aastat"]};return n?m[d][2]?m[d][2]:m[d][1]:u?m[d][0]:m[d][1]}var r=o.defineLocale("et",{months:"jaanuar_veebruar_m\xE4rts_aprill_mai_juuni_juuli_august_september_oktoober_november_detsember".split("_"),monthsShort:"jaan_veebr_m\xE4rts_apr_mai_juuni_juuli_aug_sept_okt_nov_dets".split("_"),weekdays:"p\xFChap\xE4ev_esmasp\xE4ev_teisip\xE4ev_kolmap\xE4ev_neljap\xE4ev_reede_laup\xE4ev".split("_"),weekdaysShort:"P_E_T_K_N_R_L".split("_"),weekdaysMin:"P_E_T_K_N_R_L".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[T\xE4na,] LT",nextDay:"[Homme,] LT",nextWeek:"[J\xE4rgmine] dddd LT",lastDay:"[Eile,] LT",lastWeek:"[Eelmine] dddd LT",sameElse:"L"},relativeTime:{future:"%s p\xE4rast",past:"%s tagasi",s:a,ss:a,m:a,mm:a,h:a,hh:a,d:a,dd:"%d p\xE4eva",M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return r})},3508(L,f,c){//! moment.js locale configuration
//! locale : Basque [eu]
//! author : Eneko Illarramendi : https://github.com/eillarra
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("eu",{months:"urtarrila_otsaila_martxoa_apirila_maiatza_ekaina_uztaila_abuztua_iraila_urria_azaroa_abendua".split("_"),monthsShort:"urt._ots._mar._api._mai._eka._uzt._abu._ira._urr._aza._abe.".split("_"),monthsParseExact:!0,weekdays:"igandea_astelehena_asteartea_asteazkena_osteguna_ostirala_larunbata".split("_"),weekdaysShort:"ig._al._ar._az._og._ol._lr.".split("_"),weekdaysMin:"ig_al_ar_az_og_ol_lr".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY[ko] MMMM[ren] D[a]",LLL:"YYYY[ko] MMMM[ren] D[a] HH:mm",LLLL:"dddd, YYYY[ko] MMMM[ren] D[a] HH:mm",l:"YYYY-M-D",ll:"YYYY[ko] MMM D[a]",lll:"YYYY[ko] MMM D[a] HH:mm",llll:"ddd, YYYY[ko] MMM D[a] HH:mm"},calendar:{sameDay:"[gaur] LT[etan]",nextDay:"[bihar] LT[etan]",nextWeek:"dddd LT[etan]",lastDay:"[atzo] LT[etan]",lastWeek:"[aurreko] dddd LT[etan]",sameElse:"L"},relativeTime:{future:"%s barru",past:"duela %s",s:"segundo batzuk",ss:"%d segundo",m:"minutu bat",mm:"%d minutu",h:"ordu bat",hh:"%d ordu",d:"egun bat",dd:"%d egun",M:"hilabete bat",MM:"%d hilabete",y:"urte bat",yy:"%d urte"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return a})},119(L,f,c){//! moment.js locale configuration
//! locale : Persian [fa]
//! author : Ebrahim Byagowi : https://github.com/ebraminio
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u06F1",2:"\u06F2",3:"\u06F3",4:"\u06F4",5:"\u06F5",6:"\u06F6",7:"\u06F7",8:"\u06F8",9:"\u06F9",0:"\u06F0"},r={"\u06F1":"1","\u06F2":"2","\u06F3":"3","\u06F4":"4","\u06F5":"5","\u06F6":"6","\u06F7":"7","\u06F8":"8","\u06F9":"9","\u06F0":"0"},s=o.defineLocale("fa",{months:"\u0698\u0627\u0646\u0648\u06CC\u0647_\u0641\u0648\u0631\u06CC\u0647_\u0645\u0627\u0631\u0633_\u0622\u0648\u0631\u06CC\u0644_\u0645\u0647_\u0698\u0648\u0626\u0646_\u0698\u0648\u0626\u06CC\u0647_\u0627\u0648\u062A_\u0633\u067E\u062A\u0627\u0645\u0628\u0631_\u0627\u06A9\u062A\u0628\u0631_\u0646\u0648\u0627\u0645\u0628\u0631_\u062F\u0633\u0627\u0645\u0628\u0631".split("_"),monthsShort:"\u0698\u0627\u0646\u0648\u06CC\u0647_\u0641\u0648\u0631\u06CC\u0647_\u0645\u0627\u0631\u0633_\u0622\u0648\u0631\u06CC\u0644_\u0645\u0647_\u0698\u0648\u0626\u0646_\u0698\u0648\u0626\u06CC\u0647_\u0627\u0648\u062A_\u0633\u067E\u062A\u0627\u0645\u0628\u0631_\u0627\u06A9\u062A\u0628\u0631_\u0646\u0648\u0627\u0645\u0628\u0631_\u062F\u0633\u0627\u0645\u0628\u0631".split("_"),weekdays:"\u06CC\u06A9\u200C\u0634\u0646\u0628\u0647_\u062F\u0648\u0634\u0646\u0628\u0647_\u0633\u0647\u200C\u0634\u0646\u0628\u0647_\u0686\u0647\u0627\u0631\u0634\u0646\u0628\u0647_\u067E\u0646\u062C\u200C\u0634\u0646\u0628\u0647_\u062C\u0645\u0639\u0647_\u0634\u0646\u0628\u0647".split("_"),weekdaysShort:"\u06CC\u06A9\u200C\u0634\u0646\u0628\u0647_\u062F\u0648\u0634\u0646\u0628\u0647_\u0633\u0647\u200C\u0634\u0646\u0628\u0647_\u0686\u0647\u0627\u0631\u0634\u0646\u0628\u0647_\u067E\u0646\u062C\u200C\u0634\u0646\u0628\u0647_\u062C\u0645\u0639\u0647_\u0634\u0646\u0628\u0647".split("_"),weekdaysMin:"\u06CC_\u062F_\u0633_\u0686_\u067E_\u062C_\u0634".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/قبل از ظهر|بعد از ظهر/,isPM:function(n){return/بعد از ظهر/.test(n)},meridiem:function(n,d,u){return n<12?"\u0642\u0628\u0644 \u0627\u0632 \u0638\u0647\u0631":"\u0628\u0639\u062F \u0627\u0632 \u0638\u0647\u0631"},calendar:{sameDay:"[\u0627\u0645\u0631\u0648\u0632 \u0633\u0627\u0639\u062A] LT",nextDay:"[\u0641\u0631\u062F\u0627 \u0633\u0627\u0639\u062A] LT",nextWeek:"dddd [\u0633\u0627\u0639\u062A] LT",lastDay:"[\u062F\u06CC\u0631\u0648\u0632 \u0633\u0627\u0639\u062A] LT",lastWeek:"dddd [\u067E\u06CC\u0634] [\u0633\u0627\u0639\u062A] LT",sameElse:"L"},relativeTime:{future:"\u062F\u0631 %s",past:"%s \u067E\u06CC\u0634",s:"\u0686\u0646\u062F \u062B\u0627\u0646\u06CC\u0647",ss:"%d \u062B\u0627\u0646\u06CC\u0647",m:"\u06CC\u06A9 \u062F\u0642\u06CC\u0642\u0647",mm:"%d \u062F\u0642\u06CC\u0642\u0647",h:"\u06CC\u06A9 \u0633\u0627\u0639\u062A",hh:"%d \u0633\u0627\u0639\u062A",d:"\u06CC\u06A9 \u0631\u0648\u0632",dd:"%d \u0631\u0648\u0632",M:"\u06CC\u06A9 \u0645\u0627\u0647",MM:"%d \u0645\u0627\u0647",y:"\u06CC\u06A9 \u0633\u0627\u0644",yy:"%d \u0633\u0627\u0644"},preparse:function(n){return n.replace(/[۰-۹]/g,function(d){return r[d]}).replace(/،/g,",")},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]}).replace(/,/g,"\u060C")},dayOfMonthOrdinalParse:/\d{1,2}م/,ordinal:"%d\u0645",week:{dow:6,doy:12}});return s})},90527(L,f,c){//! moment.js locale configuration
//! locale : Finnish [fi]
//! author : Tarmo Aidantausta : https://github.com/bleadof
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="nolla yksi kaksi kolme nelj\xE4 viisi kuusi seitsem\xE4n kahdeksan yhdeks\xE4n".split(" "),r=["nolla","yhden","kahden","kolmen","nelj\xE4n","viiden","kuuden",a[7],a[8],a[9]];function s(u,m,h,y){var g="";switch(h){case"s":return y?"muutaman sekunnin":"muutama sekunti";case"ss":g=y?"sekunnin":"sekuntia";break;case"m":return y?"minuutin":"minuutti";case"mm":g=y?"minuutin":"minuuttia";break;case"h":return y?"tunnin":"tunti";case"hh":g=y?"tunnin":"tuntia";break;case"d":return y?"p\xE4iv\xE4n":"p\xE4iv\xE4";case"dd":g=y?"p\xE4iv\xE4n":"p\xE4iv\xE4\xE4";break;case"M":return y?"kuukauden":"kuukausi";case"MM":g=y?"kuukauden":"kuukautta";break;case"y":return y?"vuoden":"vuosi";case"yy":g=y?"vuoden":"vuotta";break}return g=n(u,y)+" "+g,g}function n(u,m){return u<10?m?r[u]:a[u]:u}var d=o.defineLocale("fi",{months:"tammikuu_helmikuu_maaliskuu_huhtikuu_toukokuu_kes\xE4kuu_hein\xE4kuu_elokuu_syyskuu_lokakuu_marraskuu_joulukuu".split("_"),monthsShort:"tammi_helmi_maalis_huhti_touko_kes\xE4_hein\xE4_elo_syys_loka_marras_joulu".split("_"),weekdays:"sunnuntai_maanantai_tiistai_keskiviikko_torstai_perjantai_lauantai".split("_"),weekdaysShort:"su_ma_ti_ke_to_pe_la".split("_"),weekdaysMin:"su_ma_ti_ke_to_pe_la".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"Do MMMM[ta] YYYY",LLL:"Do MMMM[ta] YYYY, [klo] HH.mm",LLLL:"dddd, Do MMMM[ta] YYYY, [klo] HH.mm",l:"D.M.YYYY",ll:"Do MMM YYYY",lll:"Do MMM YYYY, [klo] HH.mm",llll:"ddd, Do MMM YYYY, [klo] HH.mm"},calendar:{sameDay:"[t\xE4n\xE4\xE4n] [klo] LT",nextDay:"[huomenna] [klo] LT",nextWeek:"dddd [klo] LT",lastDay:"[eilen] [klo] LT",lastWeek:"[viime] dddd[na] [klo] LT",sameElse:"L"},relativeTime:{future:"%s p\xE4\xE4st\xE4",past:"%s sitten",s,ss:s,m:s,mm:s,h:s,hh:s,d:s,dd:s,M:s,MM:s,y:s,yy:s},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return d})},95995(L,f,c){//! moment.js locale configuration
//! locale : Filipino [fil]
//! author : Dan Hagman : https://github.com/hagmandan
//! author : Matthew Co : https://github.com/matthewdeeco
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("fil",{months:"Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre".split("_"),monthsShort:"Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis".split("_"),weekdays:"Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado".split("_"),weekdaysShort:"Lin_Lun_Mar_Miy_Huw_Biy_Sab".split("_"),weekdaysMin:"Li_Lu_Ma_Mi_Hu_Bi_Sab".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"MM/D/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY HH:mm",LLLL:"dddd, MMMM DD, YYYY HH:mm"},calendar:{sameDay:"LT [ngayong araw]",nextDay:"[Bukas ng] LT",nextWeek:"LT [sa susunod na] dddd",lastDay:"LT [kahapon]",lastWeek:"LT [noong nakaraang] dddd",sameElse:"L"},relativeTime:{future:"sa loob ng %s",past:"%s ang nakalipas",s:"ilang segundo",ss:"%d segundo",m:"isang minuto",mm:"%d minuto",h:"isang oras",hh:"%d oras",d:"isang araw",dd:"%d araw",M:"isang buwan",MM:"%d buwan",y:"isang taon",yy:"%d taon"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(r){return r},week:{dow:1,doy:4}});return a})},52477(L,f,c){//! moment.js locale configuration
//! locale : Faroese [fo]
//! author : Ragnar Johannesen : https://github.com/ragnar123
//! author : Kristian Sakarisson : https://github.com/sakarisson
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("fo",{months:"januar_februar_mars_apr\xEDl_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan_feb_mar_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),weekdays:"sunnudagur_m\xE1nadagur_t\xFDsdagur_mikudagur_h\xF3sdagur_fr\xEDggjadagur_leygardagur".split("_"),weekdaysShort:"sun_m\xE1n_t\xFDs_mik_h\xF3s_fr\xED_ley".split("_"),weekdaysMin:"su_m\xE1_t\xFD_mi_h\xF3_fr_le".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D. MMMM, YYYY HH:mm"},calendar:{sameDay:"[\xCD dag kl.] LT",nextDay:"[\xCD morgin kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[\xCD gj\xE1r kl.] LT",lastWeek:"[s\xED\xF0stu] dddd [kl] LT",sameElse:"L"},relativeTime:{future:"um %s",past:"%s s\xED\xF0ani",s:"f\xE1 sekund",ss:"%d sekundir",m:"ein minuttur",mm:"%d minuttir",h:"ein t\xEDmi",hh:"%d t\xEDmar",d:"ein dagur",dd:"%d dagar",M:"ein m\xE1na\xF0ur",MM:"%d m\xE1na\xF0ir",y:"eitt \xE1r",yy:"%d \xE1r"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a})},26435(L,f,c){//! moment.js locale configuration
//! locale : French (Canada) [fr-ca]
//! author : Jonathan Abourbih : https://github.com/jonbca
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("fr-ca",{months:"janvier_f\xE9vrier_mars_avril_mai_juin_juillet_ao\xFBt_septembre_octobre_novembre_d\xE9cembre".split("_"),monthsShort:"janv._f\xE9vr._mars_avr._mai_juin_juil._ao\xFBt_sept._oct._nov._d\xE9c.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd\u2019hui \xE0] LT",nextDay:"[Demain \xE0] LT",nextWeek:"dddd [\xE0] LT",lastDay:"[Hier \xE0] LT",lastWeek:"dddd [dernier \xE0] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|e)/,ordinal:function(r,s){switch(s){default:case"M":case"Q":case"D":case"DDD":case"d":return r+(r===1?"er":"e");case"w":case"W":return r+(r===1?"re":"e")}}});return a})},37892(L,f,c){//! moment.js locale configuration
//! locale : French (Switzerland) [fr-ch]
//! author : Gaspard Bucher : https://github.com/gaspard
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("fr-ch",{months:"janvier_f\xE9vrier_mars_avril_mai_juin_juillet_ao\xFBt_septembre_octobre_novembre_d\xE9cembre".split("_"),monthsShort:"janv._f\xE9vr._mars_avr._mai_juin_juil._ao\xFBt_sept._oct._nov._d\xE9c.".split("_"),monthsParseExact:!0,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd\u2019hui \xE0] LT",nextDay:"[Demain \xE0] LT",nextWeek:"dddd [\xE0] LT",lastDay:"[Hier \xE0] LT",lastWeek:"dddd [dernier \xE0] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|e)/,ordinal:function(r,s){switch(s){default:case"M":case"Q":case"D":case"DDD":case"d":return r+(r===1?"er":"e");case"w":case"W":return r+(r===1?"re":"e")}},week:{dow:1,doy:4}});return a})},85498(L,f,c){//! moment.js locale configuration
//! locale : French [fr]
//! author : John Fischer : https://github.com/jfroffice
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=/^(janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)/i,r=/(janv\.?|févr\.?|mars|avr\.?|mai|juin|juil\.?|août|sept\.?|oct\.?|nov\.?|déc\.?)/i,s=/(janv\.?|févr\.?|mars|avr\.?|mai|juin|juil\.?|août|sept\.?|oct\.?|nov\.?|déc\.?|janvier|février|mars|avril|mai|juin|juillet|août|septembre|octobre|novembre|décembre)/i,n=[/^janv/i,/^févr/i,/^mars/i,/^avr/i,/^mai/i,/^juin/i,/^juil/i,/^août/i,/^sept/i,/^oct/i,/^nov/i,/^déc/i],d=o.defineLocale("fr",{months:"janvier_f\xE9vrier_mars_avril_mai_juin_juillet_ao\xFBt_septembre_octobre_novembre_d\xE9cembre".split("_"),monthsShort:"janv._f\xE9vr._mars_avr._mai_juin_juil._ao\xFBt_sept._oct._nov._d\xE9c.".split("_"),monthsRegex:s,monthsShortRegex:s,monthsStrictRegex:a,monthsShortStrictRegex:r,monthsParse:n,longMonthsParse:n,shortMonthsParse:n,weekdays:"dimanche_lundi_mardi_mercredi_jeudi_vendredi_samedi".split("_"),weekdaysShort:"dim._lun._mar._mer._jeu._ven._sam.".split("_"),weekdaysMin:"di_lu_ma_me_je_ve_sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Aujourd\u2019hui \xE0] LT",nextDay:"[Demain \xE0] LT",nextWeek:"dddd [\xE0] LT",lastDay:"[Hier \xE0] LT",lastWeek:"dddd [dernier \xE0] LT",sameElse:"L"},relativeTime:{future:"dans %s",past:"il y a %s",s:"quelques secondes",ss:"%d secondes",m:"une minute",mm:"%d minutes",h:"une heure",hh:"%d heures",d:"un jour",dd:"%d jours",w:"une semaine",ww:"%d semaines",M:"un mois",MM:"%d mois",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(er|)/,ordinal:function(u,m){switch(m){case"D":return u+(u===1?"er":"");default:case"M":case"Q":case"DDD":case"d":return u+(u===1?"er":"e");case"w":case"W":return u+(u===1?"re":"e")}},week:{dow:1,doy:4}});return d})},37071(L,f,c){//! moment.js locale configuration
//! locale : Frisian [fy]
//! author : Robin van der Vliet : https://github.com/robin0van0der0v
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="jan._feb._mrt._apr._mai_jun._jul._aug._sep._okt._nov._des.".split("_"),r="jan_feb_mrt_apr_mai_jun_jul_aug_sep_okt_nov_des".split("_"),s=o.defineLocale("fy",{months:"jannewaris_febrewaris_maart_april_maaie_juny_july_augustus_septimber_oktober_novimber_desimber".split("_"),monthsShort:function(n,d){return n?/-MMM-/.test(d)?r[n.month()]:a[n.month()]:a},monthsParseExact:!0,weekdays:"snein_moandei_tiisdei_woansdei_tongersdei_freed_sneon".split("_"),weekdaysShort:"si._mo._ti._wo._to._fr._so.".split("_"),weekdaysMin:"Si_Mo_Ti_Wo_To_Fr_So".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[hjoed om] LT",nextDay:"[moarn om] LT",nextWeek:"dddd [om] LT",lastDay:"[juster om] LT",lastWeek:"[\xF4fr\xFBne] dddd [om] LT",sameElse:"L"},relativeTime:{future:"oer %s",past:"%s lyn",s:"in pear sekonden",ss:"%d sekonden",m:"ien min\xFAt",mm:"%d minuten",h:"ien oere",hh:"%d oeren",d:"ien dei",dd:"%d dagen",M:"ien moanne",MM:"%d moannen",y:"ien jier",yy:"%d jierren"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(n){return n+(n===1||n===8||n>=20?"ste":"de")},week:{dow:1,doy:4}});return s})},41734(L,f,c){//! moment.js locale configuration
//! locale : Irish or Irish Gaelic [ga]
//! author : André Silva : https://github.com/askpt
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=["Ean\xE1ir","Feabhra","M\xE1rta","Aibre\xE1n","Bealtaine","Meitheamh","I\xFAil","L\xFAnasa","Me\xE1n F\xF3mhair","Deireadh F\xF3mhair","Samhain","Nollaig"],r=["Ean","Feabh","M\xE1rt","Aib","Beal","Meith","I\xFAil","L\xFAn","M.F.","D.F.","Samh","Noll"],s=["D\xE9 Domhnaigh","D\xE9 Luain","D\xE9 M\xE1irt","D\xE9 C\xE9adaoin","D\xE9ardaoin","D\xE9 hAoine","D\xE9 Sathairn"],n=["Domh","Luan","M\xE1irt","C\xE9ad","D\xE9ar","Aoine","Sath"],d=["Do","Lu","M\xE1","C\xE9","D\xE9","A","Sa"],u=o.defineLocale("ga",{months:a,monthsShort:r,monthsParseExact:!0,weekdays:s,weekdaysShort:n,weekdaysMin:d,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Inniu ag] LT",nextDay:"[Am\xE1rach ag] LT",nextWeek:"dddd [ag] LT",lastDay:"[Inn\xE9 ag] LT",lastWeek:"dddd [seo caite] [ag] LT",sameElse:"L"},relativeTime:{future:"i %s",past:"%s \xF3 shin",s:"c\xFApla soicind",ss:"%d soicind",m:"n\xF3im\xE9ad",mm:"%d n\xF3im\xE9ad",h:"uair an chloig",hh:"%d uair an chloig",d:"l\xE1",dd:"%d l\xE1",M:"m\xED",MM:"%d m\xEDonna",y:"bliain",yy:"%d bliain"},dayOfMonthOrdinalParse:/\d{1,2}(d|na|mh)/,ordinal:function(m){var h=m===1?"d":m%10===2?"na":"mh";return m+h},week:{dow:1,doy:4}});return u})},70217(L,f,c){//! moment.js locale configuration
//! locale : Scottish Gaelic [gd]
//! author : Jon Ashdown : https://github.com/jonashdown
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=["Am Faoilleach","An Gearran","Am M\xE0rt","An Giblean","An C\xE8itean","An t-\xD2gmhios","An t-Iuchar","An L\xF9nastal","An t-Sultain","An D\xE0mhair","An t-Samhain","An D\xF9bhlachd"],r=["Faoi","Gear","M\xE0rt","Gibl","C\xE8it","\xD2gmh","Iuch","L\xF9n","Sult","D\xE0mh","Samh","D\xF9bh"],s=["Did\xF2mhnaich","Diluain","Dim\xE0irt","Diciadain","Diardaoin","Dihaoine","Disathairne"],n=["Did","Dil","Dim","Dic","Dia","Dih","Dis"],d=["D\xF2","Lu","M\xE0","Ci","Ar","Ha","Sa"],u=o.defineLocale("gd",{months:a,monthsShort:r,monthsParseExact:!0,weekdays:s,weekdaysShort:n,weekdaysMin:d,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[An-diugh aig] LT",nextDay:"[A-m\xE0ireach aig] LT",nextWeek:"dddd [aig] LT",lastDay:"[An-d\xE8 aig] LT",lastWeek:"dddd [seo chaidh] [aig] LT",sameElse:"L"},relativeTime:{future:"ann an %s",past:"bho chionn %s",s:"beagan diogan",ss:"%d diogan",m:"mionaid",mm:"%d mionaidean",h:"uair",hh:"%d uairean",d:"latha",dd:"%d latha",M:"m\xECos",MM:"%d m\xECosan",y:"bliadhna",yy:"%d bliadhna"},dayOfMonthOrdinalParse:/\d{1,2}(d|na|mh)/,ordinal:function(m){var h=m===1?"d":m%10===2?"na":"mh";return m+h},week:{dow:1,doy:4}});return u})},77329(L,f,c){//! moment.js locale configuration
//! locale : Galician [gl]
//! author : Juan G. Hurtado : https://github.com/juanghurtado
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("gl",{months:"xaneiro_febreiro_marzo_abril_maio_xu\xF1o_xullo_agosto_setembro_outubro_novembro_decembro".split("_"),monthsShort:"xan._feb._mar._abr._mai._xu\xF1._xul._ago._set._out._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"domingo_luns_martes_m\xE9rcores_xoves_venres_s\xE1bado".split("_"),weekdaysShort:"dom._lun._mar._m\xE9r._xov._ven._s\xE1b.".split("_"),weekdaysMin:"do_lu_ma_m\xE9_xo_ve_s\xE1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY H:mm",LLLL:"dddd, D [de] MMMM [de] YYYY H:mm"},calendar:{sameDay:function(){return"[hoxe "+(this.hours()!==1?"\xE1s":"\xE1")+"] LT"},nextDay:function(){return"[ma\xF1\xE1 "+(this.hours()!==1?"\xE1s":"\xE1")+"] LT"},nextWeek:function(){return"dddd ["+(this.hours()!==1?"\xE1s":"a")+"] LT"},lastDay:function(){return"[onte "+(this.hours()!==1?"\xE1":"a")+"] LT"},lastWeek:function(){return"[o] dddd [pasado "+(this.hours()!==1?"\xE1s":"a")+"] LT"},sameElse:"L"},relativeTime:{future:function(r){return r.indexOf("un")===0?"n"+r:"en "+r},past:"hai %s",s:"uns segundos",ss:"%d segundos",m:"un minuto",mm:"%d minutos",h:"unha hora",hh:"%d horas",d:"un d\xEDa",dd:"%d d\xEDas",M:"un mes",MM:"%d meses",y:"un ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%d\xBA",week:{dow:1,doy:4}});return a})},32124(L,f,c){//! moment.js locale configuration
//! locale : Konkani Devanagari script [gom-deva]
//! author : The Discoverer : https://github.com/WikiDiscoverer
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(s,n,d,u){var m={s:["\u0925\u094B\u0921\u092F\u093E \u0938\u0945\u0915\u0902\u0921\u093E\u0902\u0928\u0940","\u0925\u094B\u0921\u0947 \u0938\u0945\u0915\u0902\u0921"],ss:[s+" \u0938\u0945\u0915\u0902\u0921\u093E\u0902\u0928\u0940",s+" \u0938\u0945\u0915\u0902\u0921"],m:["\u090F\u0915\u093E \u092E\u093F\u0923\u091F\u093E\u0928","\u090F\u0915 \u092E\u093F\u0928\u0942\u091F"],mm:[s+" \u092E\u093F\u0923\u091F\u093E\u0902\u0928\u0940",s+" \u092E\u093F\u0923\u091F\u093E\u0902"],h:["\u090F\u0915\u093E \u0935\u0930\u093E\u0928","\u090F\u0915 \u0935\u0930"],hh:[s+" \u0935\u0930\u093E\u0902\u0928\u0940",s+" \u0935\u0930\u093E\u0902"],d:["\u090F\u0915\u093E \u0926\u093F\u0938\u093E\u0928","\u090F\u0915 \u0926\u0940\u0938"],dd:[s+" \u0926\u093F\u0938\u093E\u0902\u0928\u0940",s+" \u0926\u0940\u0938"],M:["\u090F\u0915\u093E \u092E\u094D\u0939\u092F\u0928\u094D\u092F\u093E\u0928","\u090F\u0915 \u092E\u094D\u0939\u092F\u0928\u094B"],MM:[s+" \u092E\u094D\u0939\u092F\u0928\u094D\u092F\u093E\u0928\u0940",s+" \u092E\u094D\u0939\u092F\u0928\u0947"],y:["\u090F\u0915\u093E \u0935\u0930\u094D\u0938\u093E\u0928","\u090F\u0915 \u0935\u0930\u094D\u0938"],yy:[s+" \u0935\u0930\u094D\u0938\u093E\u0902\u0928\u0940",s+" \u0935\u0930\u094D\u0938\u093E\u0902"]};return u?m[d][0]:m[d][1]}var r=o.defineLocale("gom-deva",{months:{standalone:"\u091C\u093E\u0928\u0947\u0935\u093E\u0930\u0940_\u092B\u0947\u092C\u094D\u0930\u0941\u0935\u093E\u0930\u0940_\u092E\u093E\u0930\u094D\u091A_\u090F\u092A\u094D\u0930\u0940\u0932_\u092E\u0947_\u091C\u0942\u0928_\u091C\u0941\u0932\u092F_\u0911\u0917\u0938\u094D\u091F_\u0938\u092A\u094D\u091F\u0947\u0902\u092C\u0930_\u0911\u0915\u094D\u091F\u094B\u092C\u0930_\u0928\u094B\u0935\u094D\u0939\u0947\u0902\u092C\u0930_\u0921\u093F\u0938\u0947\u0902\u092C\u0930".split("_"),format:"\u091C\u093E\u0928\u0947\u0935\u093E\u0930\u0940\u091A\u094D\u092F\u093E_\u092B\u0947\u092C\u094D\u0930\u0941\u0935\u093E\u0930\u0940\u091A\u094D\u092F\u093E_\u092E\u093E\u0930\u094D\u091A\u093E\u091A\u094D\u092F\u093E_\u090F\u092A\u094D\u0930\u0940\u0932\u093E\u091A\u094D\u092F\u093E_\u092E\u0947\u092F\u093E\u091A\u094D\u092F\u093E_\u091C\u0942\u0928\u093E\u091A\u094D\u092F\u093E_\u091C\u0941\u0932\u092F\u093E\u091A\u094D\u092F\u093E_\u0911\u0917\u0938\u094D\u091F\u093E\u091A\u094D\u092F\u093E_\u0938\u092A\u094D\u091F\u0947\u0902\u092C\u0930\u093E\u091A\u094D\u092F\u093E_\u0911\u0915\u094D\u091F\u094B\u092C\u0930\u093E\u091A\u094D\u092F\u093E_\u0928\u094B\u0935\u094D\u0939\u0947\u0902\u092C\u0930\u093E\u091A\u094D\u092F\u093E_\u0921\u093F\u0938\u0947\u0902\u092C\u0930\u093E\u091A\u094D\u092F\u093E".split("_"),isFormat:/MMMM(\s)+D[oD]?/},monthsShort:"\u091C\u093E\u0928\u0947._\u092B\u0947\u092C\u094D\u0930\u0941._\u092E\u093E\u0930\u094D\u091A_\u090F\u092A\u094D\u0930\u0940._\u092E\u0947_\u091C\u0942\u0928_\u091C\u0941\u0932._\u0911\u0917._\u0938\u092A\u094D\u091F\u0947\u0902._\u0911\u0915\u094D\u091F\u094B._\u0928\u094B\u0935\u094D\u0939\u0947\u0902._\u0921\u093F\u0938\u0947\u0902.".split("_"),monthsParseExact:!0,weekdays:"\u0906\u092F\u0924\u093E\u0930_\u0938\u094B\u092E\u093E\u0930_\u092E\u0902\u0917\u0933\u093E\u0930_\u092C\u0941\u0927\u0935\u093E\u0930_\u092C\u093F\u0930\u0947\u0938\u094D\u0924\u093E\u0930_\u0938\u0941\u0915\u094D\u0930\u093E\u0930_\u0936\u0947\u0928\u0935\u093E\u0930".split("_"),weekdaysShort:"\u0906\u092F\u0924._\u0938\u094B\u092E._\u092E\u0902\u0917\u0933._\u092C\u0941\u0927._\u092C\u094D\u0930\u0947\u0938\u094D\u0924._\u0938\u0941\u0915\u094D\u0930._\u0936\u0947\u0928.".split("_"),weekdaysMin:"\u0906_\u0938\u094B_\u092E\u0902_\u092C\u0941_\u092C\u094D\u0930\u0947_\u0938\u0941_\u0936\u0947".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"A h:mm [\u0935\u093E\u091C\u0924\u093E\u0902]",LTS:"A h:mm:ss [\u0935\u093E\u091C\u0924\u093E\u0902]",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY A h:mm [\u0935\u093E\u091C\u0924\u093E\u0902]",LLLL:"dddd, MMMM Do, YYYY, A h:mm [\u0935\u093E\u091C\u0924\u093E\u0902]",llll:"ddd, D MMM YYYY, A h:mm [\u0935\u093E\u091C\u0924\u093E\u0902]"},calendar:{sameDay:"[\u0906\u092F\u091C] LT",nextDay:"[\u092B\u093E\u0932\u094D\u092F\u093E\u0902] LT",nextWeek:"[\u092B\u0941\u0921\u0932\u094B] dddd[,] LT",lastDay:"[\u0915\u093E\u0932] LT",lastWeek:"[\u092B\u093E\u091F\u0932\u094B] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%s",past:"%s \u0906\u0926\u0940\u0902",s:a,ss:a,m:a,mm:a,h:a,hh:a,d:a,dd:a,M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}(वेर)/,ordinal:function(s,n){switch(n){case"D":return s+"\u0935\u0947\u0930";default:case"M":case"Q":case"DDD":case"d":case"w":case"W":return s}},week:{dow:0,doy:3},meridiemParse:/राती|सकाळीं|दनपारां|सांजे/,meridiemHour:function(s,n){if(s===12&&(s=0),n==="\u0930\u093E\u0924\u0940")return s<4?s:s+12;if(n==="\u0938\u0915\u093E\u0933\u0940\u0902")return s;if(n==="\u0926\u0928\u092A\u093E\u0930\u093E\u0902")return s>12?s:s+12;if(n==="\u0938\u093E\u0902\u091C\u0947")return s+12},meridiem:function(s,n,d){return s<4?"\u0930\u093E\u0924\u0940":s<12?"\u0938\u0915\u093E\u0933\u0940\u0902":s<16?"\u0926\u0928\u092A\u093E\u0930\u093E\u0902":s<20?"\u0938\u093E\u0902\u091C\u0947":"\u0930\u093E\u0924\u0940"}});return r})},93383(L,f,c){//! moment.js locale configuration
//! locale : Konkani Latin script [gom-latn]
//! author : The Discoverer : https://github.com/WikiDiscoverer
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(s,n,d,u){var m={s:["thoddea sekondamni","thodde sekond"],ss:[s+" sekondamni",s+" sekond"],m:["eka mintan","ek minut"],mm:[s+" mintamni",s+" mintam"],h:["eka voran","ek vor"],hh:[s+" voramni",s+" voram"],d:["eka disan","ek dis"],dd:[s+" disamni",s+" dis"],M:["eka mhoinean","ek mhoino"],MM:[s+" mhoineamni",s+" mhoine"],y:["eka vorsan","ek voros"],yy:[s+" vorsamni",s+" vorsam"]};return u?m[d][0]:m[d][1]}var r=o.defineLocale("gom-latn",{months:{standalone:"Janer_Febrer_Mars_Abril_Mai_Jun_Julai_Agost_Setembr_Otubr_Novembr_Dezembr".split("_"),format:"Janerachea_Febrerachea_Marsachea_Abrilachea_Maiachea_Junachea_Julaiachea_Agostachea_Setembrachea_Otubrachea_Novembrachea_Dezembrachea".split("_"),isFormat:/MMMM(\s)+D[oD]?/},monthsShort:"Jan._Feb._Mars_Abr._Mai_Jun_Jul._Ago._Set._Otu._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Aitar_Somar_Mongllar_Budhvar_Birestar_Sukrar_Son'var".split("_"),weekdaysShort:"Ait._Som._Mon._Bud._Bre._Suk._Son.".split("_"),weekdaysMin:"Ai_Sm_Mo_Bu_Br_Su_Sn".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"A h:mm [vazta]",LTS:"A h:mm:ss [vazta]",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY A h:mm [vazta]",LLLL:"dddd, MMMM Do, YYYY, A h:mm [vazta]",llll:"ddd, D MMM YYYY, A h:mm [vazta]"},calendar:{sameDay:"[Aiz] LT",nextDay:"[Faleam] LT",nextWeek:"[Fuddlo] dddd[,] LT",lastDay:"[Kal] LT",lastWeek:"[Fattlo] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%s",past:"%s adim",s:a,ss:a,m:a,mm:a,h:a,hh:a,d:a,dd:a,M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}(er)/,ordinal:function(s,n){switch(n){case"D":return s+"er";default:case"M":case"Q":case"DDD":case"d":case"w":case"W":return s}},week:{dow:0,doy:3},meridiemParse:/rati|sokallim|donparam|sanje/,meridiemHour:function(s,n){if(s===12&&(s=0),n==="rati")return s<4?s:s+12;if(n==="sokallim")return s;if(n==="donparam")return s>12?s:s+12;if(n==="sanje")return s+12},meridiem:function(s,n,d){return s<4?"rati":s<12?"sokallim":s<16?"donparam":s<20?"sanje":"rati"}});return r})},95050(L,f,c){//! moment.js locale configuration
//! locale : Gujarati [gu]
//! author : Kaushik Thanki : https://github.com/Kaushik1987
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0AE7",2:"\u0AE8",3:"\u0AE9",4:"\u0AEA",5:"\u0AEB",6:"\u0AEC",7:"\u0AED",8:"\u0AEE",9:"\u0AEF",0:"\u0AE6"},r={"\u0AE7":"1","\u0AE8":"2","\u0AE9":"3","\u0AEA":"4","\u0AEB":"5","\u0AEC":"6","\u0AED":"7","\u0AEE":"8","\u0AEF":"9","\u0AE6":"0"},s=o.defineLocale("gu",{months:"\u0A9C\u0ABE\u0AA8\u0ACD\u0AAF\u0AC1\u0A86\u0AB0\u0AC0_\u0AAB\u0AC7\u0AAC\u0ACD\u0AB0\u0AC1\u0A86\u0AB0\u0AC0_\u0AAE\u0ABE\u0AB0\u0ACD\u0A9A_\u0A8F\u0AAA\u0ACD\u0AB0\u0ABF\u0AB2_\u0AAE\u0AC7_\u0A9C\u0AC2\u0AA8_\u0A9C\u0AC1\u0AB2\u0ABE\u0A88_\u0A91\u0A97\u0AB8\u0ACD\u0A9F_\u0AB8\u0AAA\u0ACD\u0A9F\u0AC7\u0AAE\u0ACD\u0AAC\u0AB0_\u0A91\u0A95\u0ACD\u0A9F\u0ACD\u0AAC\u0AB0_\u0AA8\u0AB5\u0AC7\u0AAE\u0ACD\u0AAC\u0AB0_\u0AA1\u0ABF\u0AB8\u0AC7\u0AAE\u0ACD\u0AAC\u0AB0".split("_"),monthsShort:"\u0A9C\u0ABE\u0AA8\u0ACD\u0AAF\u0AC1._\u0AAB\u0AC7\u0AAC\u0ACD\u0AB0\u0AC1._\u0AAE\u0ABE\u0AB0\u0ACD\u0A9A_\u0A8F\u0AAA\u0ACD\u0AB0\u0ABF._\u0AAE\u0AC7_\u0A9C\u0AC2\u0AA8_\u0A9C\u0AC1\u0AB2\u0ABE._\u0A91\u0A97._\u0AB8\u0AAA\u0ACD\u0A9F\u0AC7._\u0A91\u0A95\u0ACD\u0A9F\u0ACD._\u0AA8\u0AB5\u0AC7._\u0AA1\u0ABF\u0AB8\u0AC7.".split("_"),monthsParseExact:!0,weekdays:"\u0AB0\u0AB5\u0ABF\u0AB5\u0ABE\u0AB0_\u0AB8\u0ACB\u0AAE\u0AB5\u0ABE\u0AB0_\u0AAE\u0A82\u0A97\u0AB3\u0AB5\u0ABE\u0AB0_\u0AAC\u0AC1\u0AA7\u0ACD\u0AB5\u0ABE\u0AB0_\u0A97\u0AC1\u0AB0\u0AC1\u0AB5\u0ABE\u0AB0_\u0AB6\u0AC1\u0A95\u0ACD\u0AB0\u0AB5\u0ABE\u0AB0_\u0AB6\u0AA8\u0ABF\u0AB5\u0ABE\u0AB0".split("_"),weekdaysShort:"\u0AB0\u0AB5\u0ABF_\u0AB8\u0ACB\u0AAE_\u0AAE\u0A82\u0A97\u0AB3_\u0AAC\u0AC1\u0AA7\u0ACD_\u0A97\u0AC1\u0AB0\u0AC1_\u0AB6\u0AC1\u0A95\u0ACD\u0AB0_\u0AB6\u0AA8\u0ABF".split("_"),weekdaysMin:"\u0AB0_\u0AB8\u0ACB_\u0AAE\u0A82_\u0AAC\u0AC1_\u0A97\u0AC1_\u0AB6\u0AC1_\u0AB6".split("_"),longDateFormat:{LT:"A h:mm \u0AB5\u0ABE\u0A97\u0ACD\u0AAF\u0AC7",LTS:"A h:mm:ss \u0AB5\u0ABE\u0A97\u0ACD\u0AAF\u0AC7",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm \u0AB5\u0ABE\u0A97\u0ACD\u0AAF\u0AC7",LLLL:"dddd, D MMMM YYYY, A h:mm \u0AB5\u0ABE\u0A97\u0ACD\u0AAF\u0AC7"},calendar:{sameDay:"[\u0A86\u0A9C] LT",nextDay:"[\u0A95\u0ABE\u0AB2\u0AC7] LT",nextWeek:"dddd, LT",lastDay:"[\u0A97\u0A87\u0A95\u0ABE\u0AB2\u0AC7] LT",lastWeek:"[\u0AAA\u0ABE\u0A9B\u0AB2\u0ABE] dddd, LT",sameElse:"L"},relativeTime:{future:"%s \u0AAE\u0ABE",past:"%s \u0AAA\u0AB9\u0AC7\u0AB2\u0ABE",s:"\u0A85\u0AAE\u0AC1\u0A95 \u0AAA\u0AB3\u0ACB",ss:"%d \u0AB8\u0AC7\u0A95\u0A82\u0AA1",m:"\u0A8F\u0A95 \u0AAE\u0ABF\u0AA8\u0ABF\u0A9F",mm:"%d \u0AAE\u0ABF\u0AA8\u0ABF\u0A9F",h:"\u0A8F\u0A95 \u0A95\u0AB2\u0ABE\u0A95",hh:"%d \u0A95\u0AB2\u0ABE\u0A95",d:"\u0A8F\u0A95 \u0AA6\u0ABF\u0AB5\u0AB8",dd:"%d \u0AA6\u0ABF\u0AB5\u0AB8",M:"\u0A8F\u0A95 \u0AAE\u0AB9\u0ABF\u0AA8\u0ACB",MM:"%d \u0AAE\u0AB9\u0ABF\u0AA8\u0ACB",y:"\u0A8F\u0A95 \u0AB5\u0AB0\u0ACD\u0AB7",yy:"%d \u0AB5\u0AB0\u0ACD\u0AB7"},preparse:function(n){return n.replace(/[૧૨૩૪૫૬૭૮૯૦]/g,function(d){return r[d]})},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]})},meridiemParse:/રાત|બપોર|સવાર|સાંજ/,meridiemHour:function(n,d){if(n===12&&(n=0),d==="\u0AB0\u0ABE\u0AA4")return n<4?n:n+12;if(d==="\u0AB8\u0AB5\u0ABE\u0AB0")return n;if(d==="\u0AAC\u0AAA\u0ACB\u0AB0")return n>=10?n:n+12;if(d==="\u0AB8\u0ABE\u0A82\u0A9C")return n+12},meridiem:function(n,d,u){return n<4?"\u0AB0\u0ABE\u0AA4":n<10?"\u0AB8\u0AB5\u0ABE\u0AB0":n<17?"\u0AAC\u0AAA\u0ACB\u0AB0":n<20?"\u0AB8\u0ABE\u0A82\u0A9C":"\u0AB0\u0ABE\u0AA4"},week:{dow:0,doy:6}});return s})},11713(L,f,c){//! moment.js locale configuration
//! locale : Hebrew [he]
//! author : Tomer Cohen : https://github.com/tomer
//! author : Moshe Simantov : https://github.com/DevelopmentIL
//! author : Tal Ater : https://github.com/TalAter
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("he",{months:"\u05D9\u05E0\u05D5\u05D0\u05E8_\u05E4\u05D1\u05E8\u05D5\u05D0\u05E8_\u05DE\u05E8\u05E5_\u05D0\u05E4\u05E8\u05D9\u05DC_\u05DE\u05D0\u05D9_\u05D9\u05D5\u05E0\u05D9_\u05D9\u05D5\u05DC\u05D9_\u05D0\u05D5\u05D2\u05D5\u05E1\u05D8_\u05E1\u05E4\u05D8\u05DE\u05D1\u05E8_\u05D0\u05D5\u05E7\u05D8\u05D5\u05D1\u05E8_\u05E0\u05D5\u05D1\u05DE\u05D1\u05E8_\u05D3\u05E6\u05DE\u05D1\u05E8".split("_"),monthsShort:"\u05D9\u05E0\u05D5\u05F3_\u05E4\u05D1\u05E8\u05F3_\u05DE\u05E8\u05E5_\u05D0\u05E4\u05E8\u05F3_\u05DE\u05D0\u05D9_\u05D9\u05D5\u05E0\u05D9_\u05D9\u05D5\u05DC\u05D9_\u05D0\u05D5\u05D2\u05F3_\u05E1\u05E4\u05D8\u05F3_\u05D0\u05D5\u05E7\u05F3_\u05E0\u05D5\u05D1\u05F3_\u05D3\u05E6\u05DE\u05F3".split("_"),weekdays:"\u05E8\u05D0\u05E9\u05D5\u05DF_\u05E9\u05E0\u05D9_\u05E9\u05DC\u05D9\u05E9\u05D9_\u05E8\u05D1\u05D9\u05E2\u05D9_\u05D7\u05DE\u05D9\u05E9\u05D9_\u05E9\u05D9\u05E9\u05D9_\u05E9\u05D1\u05EA".split("_"),weekdaysShort:"\u05D0\u05F3_\u05D1\u05F3_\u05D2\u05F3_\u05D3\u05F3_\u05D4\u05F3_\u05D5\u05F3_\u05E9\u05F3".split("_"),weekdaysMin:"\u05D0_\u05D1_\u05D2_\u05D3_\u05D4_\u05D5_\u05E9".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [\u05D1]MMMM YYYY",LLL:"D [\u05D1]MMMM YYYY HH:mm",LLLL:"dddd, D [\u05D1]MMMM YYYY HH:mm",l:"D/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[\u05D4\u05D9\u05D5\u05DD \u05D1\u05BE]LT",nextDay:"[\u05DE\u05D7\u05E8 \u05D1\u05BE]LT",nextWeek:"dddd [\u05D1\u05E9\u05E2\u05D4] LT",lastDay:"[\u05D0\u05EA\u05DE\u05D5\u05DC \u05D1\u05BE]LT",lastWeek:"[\u05D1\u05D9\u05D5\u05DD] dddd [\u05D4\u05D0\u05D7\u05E8\u05D5\u05DF \u05D1\u05E9\u05E2\u05D4] LT",sameElse:"L"},relativeTime:{future:"\u05D1\u05E2\u05D5\u05D3 %s",past:"\u05DC\u05E4\u05E0\u05D9 %s",s:"\u05DE\u05E1\u05E4\u05E8 \u05E9\u05E0\u05D9\u05D5\u05EA",ss:"%d \u05E9\u05E0\u05D9\u05D5\u05EA",m:"\u05D3\u05E7\u05D4",mm:"%d \u05D3\u05E7\u05D5\u05EA",h:"\u05E9\u05E2\u05D4",hh:function(r){return r===2?"\u05E9\u05E2\u05EA\u05D9\u05D9\u05DD":r+" \u05E9\u05E2\u05D5\u05EA"},d:"\u05D9\u05D5\u05DD",dd:function(r){return r===2?"\u05D9\u05D5\u05DE\u05D9\u05D9\u05DD":r+" \u05D9\u05DE\u05D9\u05DD"},M:"\u05D7\u05D5\u05D3\u05E9",MM:function(r){return r===2?"\u05D7\u05D5\u05D3\u05E9\u05D9\u05D9\u05DD":r+" \u05D7\u05D5\u05D3\u05E9\u05D9\u05DD"},y:"\u05E9\u05E0\u05D4",yy:function(r){return r===2?"\u05E9\u05E0\u05EA\u05D9\u05D9\u05DD":r%10===0&&r!==10?r+" \u05E9\u05E0\u05D4":r+" \u05E9\u05E0\u05D9\u05DD"}},meridiemParse:/אחה"צ|לפנה"צ|אחרי הצהריים|לפני הצהריים|לפנות בוקר|בבוקר|בערב/i,isPM:function(r){return/^(אחה"צ|אחרי הצהריים|בערב)$/.test(r)},meridiem:function(r,s,n){return r<5?"\u05DC\u05E4\u05E0\u05D5\u05EA \u05D1\u05D5\u05E7\u05E8":r<10?"\u05D1\u05D1\u05D5\u05E7\u05E8":r<12?n?'\u05DC\u05E4\u05E0\u05D4"\u05E6':"\u05DC\u05E4\u05E0\u05D9 \u05D4\u05E6\u05D4\u05E8\u05D9\u05D9\u05DD":r<18?n?'\u05D0\u05D7\u05D4"\u05E6':"\u05D0\u05D7\u05E8\u05D9 \u05D4\u05E6\u05D4\u05E8\u05D9\u05D9\u05DD":"\u05D1\u05E2\u05E8\u05D1"}});return a})},43861(L,f,c){//! moment.js locale configuration
//! locale : Hindi [hi]
//! author : Mayank Singhal : https://github.com/mayanksinghal
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0967",2:"\u0968",3:"\u0969",4:"\u096A",5:"\u096B",6:"\u096C",7:"\u096D",8:"\u096E",9:"\u096F",0:"\u0966"},r={"\u0967":"1","\u0968":"2","\u0969":"3","\u096A":"4","\u096B":"5","\u096C":"6","\u096D":"7","\u096E":"8","\u096F":"9","\u0966":"0"},s=[/^जन/i,/^फ़र|फर/i,/^मार्च/i,/^अप्रै/i,/^मई/i,/^जून/i,/^जुल/i,/^अग/i,/^सितं|सित/i,/^अक्टू/i,/^नव|नवं/i,/^दिसं|दिस/i],n=[/^जन/i,/^फ़र/i,/^मार्च/i,/^अप्रै/i,/^मई/i,/^जून/i,/^जुल/i,/^अग/i,/^सित/i,/^अक्टू/i,/^नव/i,/^दिस/i],d=o.defineLocale("hi",{months:{format:"\u091C\u0928\u0935\u0930\u0940_\u092B\u093C\u0930\u0935\u0930\u0940_\u092E\u093E\u0930\u094D\u091A_\u0905\u092A\u094D\u0930\u0948\u0932_\u092E\u0908_\u091C\u0942\u0928_\u091C\u0941\u0932\u093E\u0908_\u0905\u0917\u0938\u094D\u0924_\u0938\u093F\u0924\u092E\u094D\u092C\u0930_\u0905\u0915\u094D\u091F\u0942\u092C\u0930_\u0928\u0935\u092E\u094D\u092C\u0930_\u0926\u093F\u0938\u092E\u094D\u092C\u0930".split("_"),standalone:"\u091C\u0928\u0935\u0930\u0940_\u092B\u0930\u0935\u0930\u0940_\u092E\u093E\u0930\u094D\u091A_\u0905\u092A\u094D\u0930\u0948\u0932_\u092E\u0908_\u091C\u0942\u0928_\u091C\u0941\u0932\u093E\u0908_\u0905\u0917\u0938\u094D\u0924_\u0938\u093F\u0924\u0902\u092C\u0930_\u0905\u0915\u094D\u091F\u0942\u092C\u0930_\u0928\u0935\u0902\u092C\u0930_\u0926\u093F\u0938\u0902\u092C\u0930".split("_")},monthsShort:"\u091C\u0928._\u092B\u093C\u0930._\u092E\u093E\u0930\u094D\u091A_\u0905\u092A\u094D\u0930\u0948._\u092E\u0908_\u091C\u0942\u0928_\u091C\u0941\u0932._\u0905\u0917._\u0938\u093F\u0924._\u0905\u0915\u094D\u091F\u0942._\u0928\u0935._\u0926\u093F\u0938.".split("_"),weekdays:"\u0930\u0935\u093F\u0935\u093E\u0930_\u0938\u094B\u092E\u0935\u093E\u0930_\u092E\u0902\u0917\u0932\u0935\u093E\u0930_\u092C\u0941\u0927\u0935\u093E\u0930_\u0917\u0941\u0930\u0942\u0935\u093E\u0930_\u0936\u0941\u0915\u094D\u0930\u0935\u093E\u0930_\u0936\u0928\u093F\u0935\u093E\u0930".split("_"),weekdaysShort:"\u0930\u0935\u093F_\u0938\u094B\u092E_\u092E\u0902\u0917\u0932_\u092C\u0941\u0927_\u0917\u0941\u0930\u0942_\u0936\u0941\u0915\u094D\u0930_\u0936\u0928\u093F".split("_"),weekdaysMin:"\u0930_\u0938\u094B_\u092E\u0902_\u092C\u0941_\u0917\u0941_\u0936\u0941_\u0936".split("_"),longDateFormat:{LT:"A h:mm \u092C\u091C\u0947",LTS:"A h:mm:ss \u092C\u091C\u0947",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm \u092C\u091C\u0947",LLLL:"dddd, D MMMM YYYY, A h:mm \u092C\u091C\u0947"},monthsParse:s,longMonthsParse:s,shortMonthsParse:n,monthsRegex:/^(जनवरी|जन\.?|फ़रवरी|फरवरी|फ़र\.?|मार्च?|अप्रैल|अप्रै\.?|मई?|जून?|जुलाई|जुल\.?|अगस्त|अग\.?|सितम्बर|सितंबर|सित\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर|नव\.?|दिसम्बर|दिसंबर|दिस\.?)/i,monthsShortRegex:/^(जनवरी|जन\.?|फ़रवरी|फरवरी|फ़र\.?|मार्च?|अप्रैल|अप्रै\.?|मई?|जून?|जुलाई|जुल\.?|अगस्त|अग\.?|सितम्बर|सितंबर|सित\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर|नव\.?|दिसम्बर|दिसंबर|दिस\.?)/i,monthsStrictRegex:/^(जनवरी?|फ़रवरी|फरवरी?|मार्च?|अप्रैल?|मई?|जून?|जुलाई?|अगस्त?|सितम्बर|सितंबर|सित?\.?|अक्टूबर|अक्टू\.?|नवम्बर|नवंबर?|दिसम्बर|दिसंबर?)/i,monthsShortStrictRegex:/^(जन\.?|फ़र\.?|मार्च?|अप्रै\.?|मई?|जून?|जुल\.?|अग\.?|सित\.?|अक्टू\.?|नव\.?|दिस\.?)/i,calendar:{sameDay:"[\u0906\u091C] LT",nextDay:"[\u0915\u0932] LT",nextWeek:"dddd, LT",lastDay:"[\u0915\u0932] LT",lastWeek:"[\u092A\u093F\u091B\u0932\u0947] dddd, LT",sameElse:"L"},relativeTime:{future:"%s \u092E\u0947\u0902",past:"%s \u092A\u0939\u0932\u0947",s:"\u0915\u0941\u091B \u0939\u0940 \u0915\u094D\u0937\u0923",ss:"%d \u0938\u0947\u0915\u0902\u0921",m:"\u090F\u0915 \u092E\u093F\u0928\u091F",mm:"%d \u092E\u093F\u0928\u091F",h:"\u090F\u0915 \u0918\u0902\u091F\u093E",hh:"%d \u0918\u0902\u091F\u0947",d:"\u090F\u0915 \u0926\u093F\u0928",dd:"%d \u0926\u093F\u0928",M:"\u090F\u0915 \u092E\u0939\u0940\u0928\u0947",MM:"%d \u092E\u0939\u0940\u0928\u0947",y:"\u090F\u0915 \u0935\u0930\u094D\u0937",yy:"%d \u0935\u0930\u094D\u0937"},preparse:function(u){return u.replace(/[१२३४५६७८९०]/g,function(m){return r[m]})},postformat:function(u){return u.replace(/\d/g,function(m){return a[m]})},meridiemParse:/रात|सुबह|दोपहर|शाम/,meridiemHour:function(u,m){if(u===12&&(u=0),m==="\u0930\u093E\u0924")return u<4?u:u+12;if(m==="\u0938\u0941\u092C\u0939")return u;if(m==="\u0926\u094B\u092A\u0939\u0930")return u>=10?u:u+12;if(m==="\u0936\u093E\u092E")return u+12},meridiem:function(u,m,h){return u<4?"\u0930\u093E\u0924":u<10?"\u0938\u0941\u092C\u0939":u<17?"\u0926\u094B\u092A\u0939\u0930":u<20?"\u0936\u093E\u092E":"\u0930\u093E\u0924"},week:{dow:0,doy:6}});return d})},26308(L,f,c){//! moment.js locale configuration
//! locale : Croatian [hr]
//! author : Bojan Marković : https://github.com/bmarkovic
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(s,n,d){var u=s+" ";switch(d){case"ss":return s===1?u+="sekunda":s===2||s===3||s===4?u+="sekunde":u+="sekundi",u;case"m":return n?"jedna minuta":"jedne minute";case"mm":return s===1?u+="minuta":s===2||s===3||s===4?u+="minute":u+="minuta",u;case"h":return n?"jedan sat":"jednog sata";case"hh":return s===1?u+="sat":s===2||s===3||s===4?u+="sata":u+="sati",u;case"dd":return s===1?u+="dan":u+="dana",u;case"MM":return s===1?u+="mjesec":s===2||s===3||s===4?u+="mjeseca":u+="mjeseci",u;case"yy":return s===1?u+="godina":s===2||s===3||s===4?u+="godine":u+="godina",u}}var r=o.defineLocale("hr",{months:{format:"sije\u010Dnja_velja\u010De_o\u017Eujka_travnja_svibnja_lipnja_srpnja_kolovoza_rujna_listopada_studenoga_prosinca".split("_"),standalone:"sije\u010Danj_velja\u010Da_o\u017Eujak_travanj_svibanj_lipanj_srpanj_kolovoz_rujan_listopad_studeni_prosinac".split("_")},monthsShort:"sij._velj._o\u017Eu._tra._svi._lip._srp._kol._ruj._lis._stu._pro.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_\u010Detvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._\u010Det._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_\u010De_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"Do MMMM YYYY",LLL:"Do MMMM YYYY H:mm",LLLL:"dddd, Do MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[ju\u010Der u] LT",lastWeek:function(){switch(this.day()){case 0:return"[pro\u0161lu] [nedjelju] [u] LT";case 3:return"[pro\u0161lu] [srijedu] [u] LT";case 6:return"[pro\u0161le] [subote] [u] LT";case 1:case 2:case 4:case 5:return"[pro\u0161li] dddd [u] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"par sekundi",ss:a,m:a,mm:a,h:a,hh:a,d:"dan",dd:a,M:"mjesec",MM:a,y:"godinu",yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return r})},90609(L,f,c){//! moment.js locale configuration
//! locale : Hungarian [hu]
//! author : Adam Brunner : https://github.com/adambrunner
//! author : Peter Viszt  : https://github.com/passatgt
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="vas\xE1rnap h\xE9tf\u0151n kedden szerd\xE1n cs\xFCt\xF6rt\xF6k\xF6n p\xE9nteken szombaton".split(" ");function r(d,u,m,h){var y=d;switch(m){case"s":return h||u?"n\xE9h\xE1ny m\xE1sodperc":"n\xE9h\xE1ny m\xE1sodperce";case"ss":return y+(h||u)?" m\xE1sodperc":" m\xE1sodperce";case"m":return"egy"+(h||u?" perc":" perce");case"mm":return y+(h||u?" perc":" perce");case"h":return"egy"+(h||u?" \xF3ra":" \xF3r\xE1ja");case"hh":return y+(h||u?" \xF3ra":" \xF3r\xE1ja");case"d":return"egy"+(h||u?" nap":" napja");case"dd":return y+(h||u?" nap":" napja");case"M":return"egy"+(h||u?" h\xF3nap":" h\xF3napja");case"MM":return y+(h||u?" h\xF3nap":" h\xF3napja");case"y":return"egy"+(h||u?" \xE9v":" \xE9ve");case"yy":return y+(h||u?" \xE9v":" \xE9ve")}return""}function s(d){return(d?"":"[m\xFAlt] ")+"["+a[this.day()]+"] LT[-kor]"}var n=o.defineLocale("hu",{months:"janu\xE1r_febru\xE1r_m\xE1rcius_\xE1prilis_m\xE1jus_j\xFAnius_j\xFAlius_augusztus_szeptember_okt\xF3ber_november_december".split("_"),monthsShort:"jan._feb._m\xE1rc._\xE1pr._m\xE1j._j\xFAn._j\xFAl._aug._szept._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"vas\xE1rnap_h\xE9tf\u0151_kedd_szerda_cs\xFCt\xF6rt\xF6k_p\xE9ntek_szombat".split("_"),weekdaysShort:"vas_h\xE9t_kedd_sze_cs\xFCt_p\xE9n_szo".split("_"),weekdaysMin:"v_h_k_sze_cs_p_szo".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"YYYY.MM.DD.",LL:"YYYY. MMMM D.",LLL:"YYYY. MMMM D. H:mm",LLLL:"YYYY. MMMM D., dddd H:mm"},meridiemParse:/de|du/i,isPM:function(d){return d.charAt(1).toLowerCase()==="u"},meridiem:function(d,u,m){return d<12?m===!0?"de":"DE":m===!0?"du":"DU"},calendar:{sameDay:"[ma] LT[-kor]",nextDay:"[holnap] LT[-kor]",nextWeek:function(){return s.call(this,!0)},lastDay:"[tegnap] LT[-kor]",lastWeek:function(){return s.call(this,!1)},sameElse:"L"},relativeTime:{future:"%s m\xFAlva",past:"%s",s:r,ss:r,m:r,mm:r,h:r,hh:r,d:r,dd:r,M:r,MM:r,y:r,yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return n})},17160(L,f,c){//! moment.js locale configuration
//! locale : Armenian [hy-am]
//! author : Armendarabyan : https://github.com/armendarabyan
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("hy-am",{months:{format:"\u0570\u0578\u0582\u0576\u057E\u0561\u0580\u056B_\u0583\u0565\u057F\u0580\u057E\u0561\u0580\u056B_\u0574\u0561\u0580\u057F\u056B_\u0561\u057A\u0580\u056B\u056C\u056B_\u0574\u0561\u0575\u056B\u057D\u056B_\u0570\u0578\u0582\u0576\u056B\u057D\u056B_\u0570\u0578\u0582\u056C\u056B\u057D\u056B_\u0585\u0563\u0578\u057D\u057F\u0578\u057D\u056B_\u057D\u0565\u057A\u057F\u0565\u0574\u0562\u0565\u0580\u056B_\u0570\u0578\u056F\u057F\u0565\u0574\u0562\u0565\u0580\u056B_\u0576\u0578\u0575\u0565\u0574\u0562\u0565\u0580\u056B_\u0564\u0565\u056F\u057F\u0565\u0574\u0562\u0565\u0580\u056B".split("_"),standalone:"\u0570\u0578\u0582\u0576\u057E\u0561\u0580_\u0583\u0565\u057F\u0580\u057E\u0561\u0580_\u0574\u0561\u0580\u057F_\u0561\u057A\u0580\u056B\u056C_\u0574\u0561\u0575\u056B\u057D_\u0570\u0578\u0582\u0576\u056B\u057D_\u0570\u0578\u0582\u056C\u056B\u057D_\u0585\u0563\u0578\u057D\u057F\u0578\u057D_\u057D\u0565\u057A\u057F\u0565\u0574\u0562\u0565\u0580_\u0570\u0578\u056F\u057F\u0565\u0574\u0562\u0565\u0580_\u0576\u0578\u0575\u0565\u0574\u0562\u0565\u0580_\u0564\u0565\u056F\u057F\u0565\u0574\u0562\u0565\u0580".split("_")},monthsShort:"\u0570\u0576\u057E_\u0583\u057F\u0580_\u0574\u0580\u057F_\u0561\u057A\u0580_\u0574\u0575\u057D_\u0570\u0576\u057D_\u0570\u056C\u057D_\u0585\u0563\u057D_\u057D\u057A\u057F_\u0570\u056F\u057F_\u0576\u0574\u0562_\u0564\u056F\u057F".split("_"),weekdays:"\u056F\u056B\u0580\u0561\u056F\u056B_\u0565\u0580\u056F\u0578\u0582\u0577\u0561\u0562\u0569\u056B_\u0565\u0580\u0565\u0584\u0577\u0561\u0562\u0569\u056B_\u0579\u0578\u0580\u0565\u0584\u0577\u0561\u0562\u0569\u056B_\u0570\u056B\u0576\u0563\u0577\u0561\u0562\u0569\u056B_\u0578\u0582\u0580\u0562\u0561\u0569_\u0577\u0561\u0562\u0561\u0569".split("_"),weekdaysShort:"\u056F\u0580\u056F_\u0565\u0580\u056F_\u0565\u0580\u0584_\u0579\u0580\u0584_\u0570\u0576\u0563_\u0578\u0582\u0580\u0562_\u0577\u0562\u0569".split("_"),weekdaysMin:"\u056F\u0580\u056F_\u0565\u0580\u056F_\u0565\u0580\u0584_\u0579\u0580\u0584_\u0570\u0576\u0563_\u0578\u0582\u0580\u0562_\u0577\u0562\u0569".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY \u0569.",LLL:"D MMMM YYYY \u0569., HH:mm",LLLL:"dddd, D MMMM YYYY \u0569., HH:mm"},calendar:{sameDay:"[\u0561\u0575\u057D\u0585\u0580] LT",nextDay:"[\u057E\u0561\u0572\u0568] LT",lastDay:"[\u0565\u0580\u0565\u056F] LT",nextWeek:function(){return"dddd [\u0585\u0580\u0568 \u056A\u0561\u0574\u0568] LT"},lastWeek:function(){return"[\u0561\u0576\u0581\u0561\u056E] dddd [\u0585\u0580\u0568 \u056A\u0561\u0574\u0568] LT"},sameElse:"L"},relativeTime:{future:"%s \u0570\u0565\u057F\u0578",past:"%s \u0561\u057C\u0561\u057B",s:"\u0574\u056B \u0584\u0561\u0576\u056B \u057E\u0561\u0575\u0580\u056F\u0575\u0561\u0576",ss:"%d \u057E\u0561\u0575\u0580\u056F\u0575\u0561\u0576",m:"\u0580\u0578\u057A\u0565",mm:"%d \u0580\u0578\u057A\u0565",h:"\u056A\u0561\u0574",hh:"%d \u056A\u0561\u0574",d:"\u0585\u0580",dd:"%d \u0585\u0580",M:"\u0561\u0574\u056B\u057D",MM:"%d \u0561\u0574\u056B\u057D",y:"\u057F\u0561\u0580\u056B",yy:"%d \u057F\u0561\u0580\u056B"},meridiemParse:/գիշերվա|առավոտվա|ցերեկվա|երեկոյան/,isPM:function(r){return/^(ցերեկվա|երեկոյան)$/.test(r)},meridiem:function(r){return r<4?"\u0563\u056B\u0577\u0565\u0580\u057E\u0561":r<12?"\u0561\u057C\u0561\u057E\u0578\u057F\u057E\u0561":r<17?"\u0581\u0565\u0580\u0565\u056F\u057E\u0561":"\u0565\u0580\u0565\u056F\u0578\u0575\u0561\u0576"},dayOfMonthOrdinalParse:/\d{1,2}|\d{1,2}-(ին|րդ)/,ordinal:function(r,s){switch(s){case"DDD":case"w":case"W":case"DDDo":return r===1?r+"-\u056B\u0576":r+"-\u0580\u0564";default:return r}},week:{dow:1,doy:7}});return a})},74063(L,f,c){//! moment.js locale configuration
//! locale : Indonesian [id]
//! author : Mohammad Satrio Utomo : https://github.com/tyok
//! reference: http://id.wikisource.org/wiki/Pedoman_Umum_Ejaan_Bahasa_Indonesia_yang_Disempurnakan
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("id",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_November_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Agt_Sep_Okt_Nov_Des".split("_"),weekdays:"Minggu_Senin_Selasa_Rabu_Kamis_Jumat_Sabtu".split("_"),weekdaysShort:"Min_Sen_Sel_Rab_Kam_Jum_Sab".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|siang|sore|malam/,meridiemHour:function(r,s){if(r===12&&(r=0),s==="pagi")return r;if(s==="siang")return r>=11?r:r+12;if(s==="sore"||s==="malam")return r+12},meridiem:function(r,s,n){return r<11?"pagi":r<15?"siang":r<19?"sore":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Besok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kemarin pukul] LT",lastWeek:"dddd [lalu pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lalu",s:"beberapa detik",ss:"%d detik",m:"semenit",mm:"%d menit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:0,doy:6}});return a})},89374(L,f,c){//! moment.js locale configuration
//! locale : Icelandic [is]
//! author : Hinrik Örn Sigurðsson : https://github.com/hinrik
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(n){return n%100===11?!0:n%10!==1}function r(n,d,u,m){var h=n+" ";switch(u){case"s":return d||m?"nokkrar sek\xFAndur":"nokkrum sek\xFAndum";case"ss":return a(n)?h+(d||m?"sek\xFAndur":"sek\xFAndum"):h+"sek\xFAnda";case"m":return d?"m\xEDn\xFAta":"m\xEDn\xFAtu";case"mm":return a(n)?h+(d||m?"m\xEDn\xFAtur":"m\xEDn\xFAtum"):d?h+"m\xEDn\xFAta":h+"m\xEDn\xFAtu";case"hh":return a(n)?h+(d||m?"klukkustundir":"klukkustundum"):h+"klukkustund";case"d":return d?"dagur":m?"dag":"degi";case"dd":return a(n)?d?h+"dagar":h+(m?"daga":"d\xF6gum"):d?h+"dagur":h+(m?"dag":"degi");case"M":return d?"m\xE1nu\xF0ur":m?"m\xE1nu\xF0":"m\xE1nu\xF0i";case"MM":return a(n)?d?h+"m\xE1nu\xF0ir":h+(m?"m\xE1nu\xF0i":"m\xE1nu\xF0um"):d?h+"m\xE1nu\xF0ur":h+(m?"m\xE1nu\xF0":"m\xE1nu\xF0i");case"y":return d||m?"\xE1r":"\xE1ri";case"yy":return a(n)?h+(d||m?"\xE1r":"\xE1rum"):h+(d||m?"\xE1r":"\xE1ri")}}var s=o.defineLocale("is",{months:"jan\xFAar_febr\xFAar_mars_apr\xEDl_ma\xED_j\xFAn\xED_j\xFAl\xED_\xE1g\xFAst_september_okt\xF3ber_n\xF3vember_desember".split("_"),monthsShort:"jan_feb_mar_apr_ma\xED_j\xFAn_j\xFAl_\xE1g\xFA_sep_okt_n\xF3v_des".split("_"),weekdays:"sunnudagur_m\xE1nudagur_\xFEri\xF0judagur_mi\xF0vikudagur_fimmtudagur_f\xF6studagur_laugardagur".split("_"),weekdaysShort:"sun_m\xE1n_\xFEri_mi\xF0_fim_f\xF6s_lau".split("_"),weekdaysMin:"Su_M\xE1_\xDEr_Mi_Fi_F\xF6_La".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd, D. MMMM YYYY [kl.] H:mm"},calendar:{sameDay:"[\xED dag kl.] LT",nextDay:"[\xE1 morgun kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[\xED g\xE6r kl.] LT",lastWeek:"[s\xED\xF0asta] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"eftir %s",past:"fyrir %s s\xED\xF0an",s:r,ss:r,m:r,mm:r,h:"klukkustund",hh:r,d:r,dd:r,M:r,MM:r,y:r,yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return s})},21827(L,f,c){//! moment.js locale configuration
//! locale : Italian (Switzerland) [it-ch]
//! author : xfh : https://github.com/xfh
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("it-ch",{months:"gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre".split("_"),monthsShort:"gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic".split("_"),weekdays:"domenica_luned\xEC_marted\xEC_mercoled\xEC_gioved\xEC_venerd\xEC_sabato".split("_"),weekdaysShort:"dom_lun_mar_mer_gio_ven_sab".split("_"),weekdaysMin:"do_lu_ma_me_gi_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[Oggi alle] LT",nextDay:"[Domani alle] LT",nextWeek:"dddd [alle] LT",lastDay:"[Ieri alle] LT",lastWeek:function(){switch(this.day()){case 0:return"[la scorsa] dddd [alle] LT";default:return"[lo scorso] dddd [alle] LT"}},sameElse:"L"},relativeTime:{future:function(r){return(/^[0-9].+$/.test(r)?"tra":"in")+" "+r},past:"%s fa",s:"alcuni secondi",ss:"%d secondi",m:"un minuto",mm:"%d minuti",h:"un'ora",hh:"%d ore",d:"un giorno",dd:"%d giorni",M:"un mese",MM:"%d mesi",y:"un anno",yy:"%d anni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%d\xBA",week:{dow:1,doy:4}});return a})},88383(L,f,c){//! moment.js locale configuration
//! locale : Italian [it]
//! author : Lorenzo : https://github.com/aliem
//! author: Mattia Larentis: https://github.com/nostalgiaz
//! author: Marco : https://github.com/Manfre98
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("it",{months:"gennaio_febbraio_marzo_aprile_maggio_giugno_luglio_agosto_settembre_ottobre_novembre_dicembre".split("_"),monthsShort:"gen_feb_mar_apr_mag_giu_lug_ago_set_ott_nov_dic".split("_"),weekdays:"domenica_luned\xEC_marted\xEC_mercoled\xEC_gioved\xEC_venerd\xEC_sabato".split("_"),weekdaysShort:"dom_lun_mar_mer_gio_ven_sab".split("_"),weekdaysMin:"do_lu_ma_me_gi_ve_sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:function(){return"[Oggi a"+(this.hours()>1?"lle ":this.hours()===0?" ":"ll'")+"]LT"},nextDay:function(){return"[Domani a"+(this.hours()>1?"lle ":this.hours()===0?" ":"ll'")+"]LT"},nextWeek:function(){return"dddd [a"+(this.hours()>1?"lle ":this.hours()===0?" ":"ll'")+"]LT"},lastDay:function(){return"[Ieri a"+(this.hours()>1?"lle ":this.hours()===0?" ":"ll'")+"]LT"},lastWeek:function(){switch(this.day()){case 0:return"[La scorsa] dddd [a"+(this.hours()>1?"lle ":this.hours()===0?" ":"ll'")+"]LT";default:return"[Lo scorso] dddd [a"+(this.hours()>1?"lle ":this.hours()===0?" ":"ll'")+"]LT"}},sameElse:"L"},relativeTime:{future:"tra %s",past:"%s fa",s:"alcuni secondi",ss:"%d secondi",m:"un minuto",mm:"%d minuti",h:"un'ora",hh:"%d ore",d:"un giorno",dd:"%d giorni",w:"una settimana",ww:"%d settimane",M:"un mese",MM:"%d mesi",y:"un anno",yy:"%d anni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%d\xBA",week:{dow:1,doy:4}});return a})},23827(L,f,c){//! moment.js locale configuration
//! locale : Japanese [ja]
//! author : LI Long : https://github.com/baryon
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ja",{eras:[{since:"2019-05-01",offset:1,name:"\u4EE4\u548C",narrow:"\u32FF",abbr:"R"},{since:"1989-01-08",until:"2019-04-30",offset:1,name:"\u5E73\u6210",narrow:"\u337B",abbr:"H"},{since:"1926-12-25",until:"1989-01-07",offset:1,name:"\u662D\u548C",narrow:"\u337C",abbr:"S"},{since:"1912-07-30",until:"1926-12-24",offset:1,name:"\u5927\u6B63",narrow:"\u337D",abbr:"T"},{since:"1873-01-01",until:"1912-07-29",offset:6,name:"\u660E\u6CBB",narrow:"\u337E",abbr:"M"},{since:"0001-01-01",until:"1873-12-31",offset:1,name:"\u897F\u66A6",narrow:"AD",abbr:"AD"},{since:"0000-12-31",until:-1/0,offset:1,name:"\u7D00\u5143\u524D",narrow:"BC",abbr:"BC"}],eraYearOrdinalRegex:/(元|\d+)年/,eraYearOrdinalParse:function(r,s){return s[1]==="\u5143"?1:parseInt(s[1]||r,10)},months:"1\u6708_2\u6708_3\u6708_4\u6708_5\u6708_6\u6708_7\u6708_8\u6708_9\u6708_10\u6708_11\u6708_12\u6708".split("_"),monthsShort:"1\u6708_2\u6708_3\u6708_4\u6708_5\u6708_6\u6708_7\u6708_8\u6708_9\u6708_10\u6708_11\u6708_12\u6708".split("_"),weekdays:"\u65E5\u66DC\u65E5_\u6708\u66DC\u65E5_\u706B\u66DC\u65E5_\u6C34\u66DC\u65E5_\u6728\u66DC\u65E5_\u91D1\u66DC\u65E5_\u571F\u66DC\u65E5".split("_"),weekdaysShort:"\u65E5_\u6708_\u706B_\u6C34_\u6728_\u91D1_\u571F".split("_"),weekdaysMin:"\u65E5_\u6708_\u706B_\u6C34_\u6728_\u91D1_\u571F".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY\u5E74M\u6708D\u65E5",LLL:"YYYY\u5E74M\u6708D\u65E5 HH:mm",LLLL:"YYYY\u5E74M\u6708D\u65E5 dddd HH:mm",l:"YYYY/MM/DD",ll:"YYYY\u5E74M\u6708D\u65E5",lll:"YYYY\u5E74M\u6708D\u65E5 HH:mm",llll:"YYYY\u5E74M\u6708D\u65E5(ddd) HH:mm"},meridiemParse:/午前|午後/i,isPM:function(r){return r==="\u5348\u5F8C"},meridiem:function(r,s,n){return r<12?"\u5348\u524D":"\u5348\u5F8C"},calendar:{sameDay:"[\u4ECA\u65E5] LT",nextDay:"[\u660E\u65E5] LT",nextWeek:function(r){return r.week()!==this.week()?"[\u6765\u9031]dddd LT":"dddd LT"},lastDay:"[\u6628\u65E5] LT",lastWeek:function(r){return this.week()!==r.week()?"[\u5148\u9031]dddd LT":"dddd LT"},sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}日/,ordinal:function(r,s){switch(s){case"y":return r===1?"\u5143\u5E74":r+"\u5E74";case"d":case"D":case"DDD":return r+"\u65E5";default:return r}},relativeTime:{future:"%s\u5F8C",past:"%s\u524D",s:"\u6570\u79D2",ss:"%d\u79D2",m:"1\u5206",mm:"%d\u5206",h:"1\u6642\u9593",hh:"%d\u6642\u9593",d:"1\u65E5",dd:"%d\u65E5",M:"1\u30F6\u6708",MM:"%d\u30F6\u6708",y:"1\u5E74",yy:"%d\u5E74"}});return a})},89722(L,f,c){//! moment.js locale configuration
//! locale : Javanese [jv]
//! author : Rony Lantip : https://github.com/lantip
//! reference: http://jv.wikipedia.org/wiki/Basa_Jawa
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("jv",{months:"Januari_Februari_Maret_April_Mei_Juni_Juli_Agustus_September_Oktober_Nopember_Desember".split("_"),monthsShort:"Jan_Feb_Mar_Apr_Mei_Jun_Jul_Ags_Sep_Okt_Nop_Des".split("_"),weekdays:"Minggu_Senen_Seloso_Rebu_Kemis_Jemuwah_Septu".split("_"),weekdaysShort:"Min_Sen_Sel_Reb_Kem_Jem_Sep".split("_"),weekdaysMin:"Mg_Sn_Sl_Rb_Km_Jm_Sp".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/enjing|siyang|sonten|ndalu/,meridiemHour:function(r,s){if(r===12&&(r=0),s==="enjing")return r;if(s==="siyang")return r>=11?r:r+12;if(s==="sonten"||s==="ndalu")return r+12},meridiem:function(r,s,n){return r<11?"enjing":r<15?"siyang":r<19?"sonten":"ndalu"},calendar:{sameDay:"[Dinten puniko pukul] LT",nextDay:"[Mbenjang pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kala wingi pukul] LT",lastWeek:"dddd [kepengker pukul] LT",sameElse:"L"},relativeTime:{future:"wonten ing %s",past:"%s ingkang kepengker",s:"sawetawis detik",ss:"%d detik",m:"setunggal menit",mm:"%d menit",h:"setunggal jam",hh:"%d jam",d:"sedinten",dd:"%d dinten",M:"sewulan",MM:"%d wulan",y:"setaun",yy:"%d taun"},week:{dow:1,doy:7}});return a})},41794(L,f,c){//! moment.js locale configuration
//! locale : Georgian [ka]
//! author : Irakli Janiashvili : https://github.com/IrakliJani
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ka",{months:"\u10D8\u10D0\u10DC\u10D5\u10D0\u10E0\u10D8_\u10D7\u10D4\u10D1\u10D4\u10E0\u10D5\u10D0\u10DA\u10D8_\u10DB\u10D0\u10E0\u10E2\u10D8_\u10D0\u10DE\u10E0\u10D8\u10DA\u10D8_\u10DB\u10D0\u10D8\u10E1\u10D8_\u10D8\u10D5\u10DC\u10D8\u10E1\u10D8_\u10D8\u10D5\u10DA\u10D8\u10E1\u10D8_\u10D0\u10D2\u10D5\u10D8\u10E1\u10E2\u10DD_\u10E1\u10D4\u10E5\u10E2\u10D4\u10DB\u10D1\u10D4\u10E0\u10D8_\u10DD\u10E5\u10E2\u10DD\u10DB\u10D1\u10D4\u10E0\u10D8_\u10DC\u10DD\u10D4\u10DB\u10D1\u10D4\u10E0\u10D8_\u10D3\u10D4\u10D9\u10D4\u10DB\u10D1\u10D4\u10E0\u10D8".split("_"),monthsShort:"\u10D8\u10D0\u10DC_\u10D7\u10D4\u10D1_\u10DB\u10D0\u10E0_\u10D0\u10DE\u10E0_\u10DB\u10D0\u10D8_\u10D8\u10D5\u10DC_\u10D8\u10D5\u10DA_\u10D0\u10D2\u10D5_\u10E1\u10D4\u10E5_\u10DD\u10E5\u10E2_\u10DC\u10DD\u10D4_\u10D3\u10D4\u10D9".split("_"),weekdays:{standalone:"\u10D9\u10D5\u10D8\u10E0\u10D0_\u10DD\u10E0\u10E8\u10D0\u10D1\u10D0\u10D7\u10D8_\u10E1\u10D0\u10DB\u10E8\u10D0\u10D1\u10D0\u10D7\u10D8_\u10DD\u10D7\u10EE\u10E8\u10D0\u10D1\u10D0\u10D7\u10D8_\u10EE\u10E3\u10D7\u10E8\u10D0\u10D1\u10D0\u10D7\u10D8_\u10DE\u10D0\u10E0\u10D0\u10E1\u10D9\u10D4\u10D5\u10D8_\u10E8\u10D0\u10D1\u10D0\u10D7\u10D8".split("_"),format:"\u10D9\u10D5\u10D8\u10E0\u10D0\u10E1_\u10DD\u10E0\u10E8\u10D0\u10D1\u10D0\u10D7\u10E1_\u10E1\u10D0\u10DB\u10E8\u10D0\u10D1\u10D0\u10D7\u10E1_\u10DD\u10D7\u10EE\u10E8\u10D0\u10D1\u10D0\u10D7\u10E1_\u10EE\u10E3\u10D7\u10E8\u10D0\u10D1\u10D0\u10D7\u10E1_\u10DE\u10D0\u10E0\u10D0\u10E1\u10D9\u10D4\u10D5\u10E1_\u10E8\u10D0\u10D1\u10D0\u10D7\u10E1".split("_"),isFormat:/(წინა|შემდეგ)/},weekdaysShort:"\u10D9\u10D5\u10D8_\u10DD\u10E0\u10E8_\u10E1\u10D0\u10DB_\u10DD\u10D7\u10EE_\u10EE\u10E3\u10D7_\u10DE\u10D0\u10E0_\u10E8\u10D0\u10D1".split("_"),weekdaysMin:"\u10D9\u10D5_\u10DD\u10E0_\u10E1\u10D0_\u10DD\u10D7_\u10EE\u10E3_\u10DE\u10D0_\u10E8\u10D0".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[\u10D3\u10E6\u10D4\u10E1] LT[-\u10D6\u10D4]",nextDay:"[\u10EE\u10D5\u10D0\u10DA] LT[-\u10D6\u10D4]",lastDay:"[\u10D2\u10E3\u10E8\u10D8\u10DC] LT[-\u10D6\u10D4]",nextWeek:"[\u10E8\u10D4\u10DB\u10D3\u10D4\u10D2] dddd LT[-\u10D6\u10D4]",lastWeek:"[\u10EC\u10D8\u10DC\u10D0] dddd LT-\u10D6\u10D4",sameElse:"L"},relativeTime:{future:function(r){return r.replace(/(წამ|წუთ|საათ|წელ|დღ|თვ)(ი|ე)/,function(s,n,d){return d==="\u10D8"?n+"\u10E8\u10D8":n+d+"\u10E8\u10D8"})},past:function(r){return/(წამი|წუთი|საათი|დღე|თვე)/.test(r)?r.replace(/(ი|ე)$/,"\u10D8\u10E1 \u10EC\u10D8\u10DC"):/წელი/.test(r)?r.replace(/წელი$/,"\u10EC\u10DA\u10D8\u10E1 \u10EC\u10D8\u10DC"):r},s:"\u10E0\u10D0\u10DB\u10D3\u10D4\u10DC\u10D8\u10DB\u10D4 \u10EC\u10D0\u10DB\u10D8",ss:"%d \u10EC\u10D0\u10DB\u10D8",m:"\u10EC\u10E3\u10D7\u10D8",mm:"%d \u10EC\u10E3\u10D7\u10D8",h:"\u10E1\u10D0\u10D0\u10D7\u10D8",hh:"%d \u10E1\u10D0\u10D0\u10D7\u10D8",d:"\u10D3\u10E6\u10D4",dd:"%d \u10D3\u10E6\u10D4",M:"\u10D7\u10D5\u10D4",MM:"%d \u10D7\u10D5\u10D4",y:"\u10EC\u10D4\u10DA\u10D8",yy:"%d \u10EC\u10D4\u10DA\u10D8"},dayOfMonthOrdinalParse:/0|1-ლი|მე-\d{1,2}|\d{1,2}-ე/,ordinal:function(r){return r===0?r:r===1?r+"-\u10DA\u10D8":r<20||r<=100&&r%20===0||r%100===0?"\u10DB\u10D4-"+r:r+"-\u10D4"},week:{dow:1,doy:7}});return a})},27088(L,f,c){//! moment.js locale configuration
//! locale : Kazakh [kk]
//! authors : Nurlan Rakhimzhanov : https://github.com/nurlan
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={0:"-\u0448\u0456",1:"-\u0448\u0456",2:"-\u0448\u0456",3:"-\u0448\u0456",4:"-\u0448\u0456",5:"-\u0448\u0456",6:"-\u0448\u044B",7:"-\u0448\u0456",8:"-\u0448\u0456",9:"-\u0448\u044B",10:"-\u0448\u044B",20:"-\u0448\u044B",30:"-\u0448\u044B",40:"-\u0448\u044B",50:"-\u0448\u0456",60:"-\u0448\u044B",70:"-\u0448\u0456",80:"-\u0448\u0456",90:"-\u0448\u044B",100:"-\u0448\u0456"},r=o.defineLocale("kk",{months:"\u049B\u0430\u04A3\u0442\u0430\u0440_\u0430\u049B\u043F\u0430\u043D_\u043D\u0430\u0443\u0440\u044B\u0437_\u0441\u04D9\u0443\u0456\u0440_\u043C\u0430\u043C\u044B\u0440_\u043C\u0430\u0443\u0441\u044B\u043C_\u0448\u0456\u043B\u0434\u0435_\u0442\u0430\u043C\u044B\u0437_\u049B\u044B\u0440\u043A\u04AF\u0439\u0435\u043A_\u049B\u0430\u0437\u0430\u043D_\u049B\u0430\u0440\u0430\u0448\u0430_\u0436\u0435\u043B\u0442\u043E\u049B\u0441\u0430\u043D".split("_"),monthsShort:"\u049B\u0430\u04A3_\u0430\u049B\u043F_\u043D\u0430\u0443_\u0441\u04D9\u0443_\u043C\u0430\u043C_\u043C\u0430\u0443_\u0448\u0456\u043B_\u0442\u0430\u043C_\u049B\u044B\u0440_\u049B\u0430\u0437_\u049B\u0430\u0440_\u0436\u0435\u043B".split("_"),weekdays:"\u0436\u0435\u043A\u0441\u0435\u043D\u0431\u0456_\u0434\u04AF\u0439\u0441\u0435\u043D\u0431\u0456_\u0441\u0435\u0439\u0441\u0435\u043D\u0431\u0456_\u0441\u04D9\u0440\u0441\u0435\u043D\u0431\u0456_\u0431\u0435\u0439\u0441\u0435\u043D\u0431\u0456_\u0436\u04B1\u043C\u0430_\u0441\u0435\u043D\u0431\u0456".split("_"),weekdaysShort:"\u0436\u0435\u043A_\u0434\u04AF\u0439_\u0441\u0435\u0439_\u0441\u04D9\u0440_\u0431\u0435\u0439_\u0436\u04B1\u043C_\u0441\u0435\u043D".split("_"),weekdaysMin:"\u0436\u043A_\u0434\u0439_\u0441\u0439_\u0441\u0440_\u0431\u0439_\u0436\u043C_\u0441\u043D".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[\u0411\u04AF\u0433\u0456\u043D \u0441\u0430\u0493\u0430\u0442] LT",nextDay:"[\u0415\u0440\u0442\u0435\u04A3 \u0441\u0430\u0493\u0430\u0442] LT",nextWeek:"dddd [\u0441\u0430\u0493\u0430\u0442] LT",lastDay:"[\u041A\u0435\u0448\u0435 \u0441\u0430\u0493\u0430\u0442] LT",lastWeek:"[\u04E8\u0442\u043A\u0435\u043D \u0430\u043F\u0442\u0430\u043D\u044B\u04A3] dddd [\u0441\u0430\u0493\u0430\u0442] LT",sameElse:"L"},relativeTime:{future:"%s \u0456\u0448\u0456\u043D\u0434\u0435",past:"%s \u0431\u04B1\u0440\u044B\u043D",s:"\u0431\u0456\u0440\u043D\u0435\u0448\u0435 \u0441\u0435\u043A\u0443\u043D\u0434",ss:"%d \u0441\u0435\u043A\u0443\u043D\u0434",m:"\u0431\u0456\u0440 \u043C\u0438\u043D\u0443\u0442",mm:"%d \u043C\u0438\u043D\u0443\u0442",h:"\u0431\u0456\u0440 \u0441\u0430\u0493\u0430\u0442",hh:"%d \u0441\u0430\u0493\u0430\u0442",d:"\u0431\u0456\u0440 \u043A\u04AF\u043D",dd:"%d \u043A\u04AF\u043D",M:"\u0431\u0456\u0440 \u0430\u0439",MM:"%d \u0430\u0439",y:"\u0431\u0456\u0440 \u0436\u044B\u043B",yy:"%d \u0436\u044B\u043B"},dayOfMonthOrdinalParse:/\d{1,2}-(ші|шы)/,ordinal:function(s){var n=s%10,d=s>=100?100:null;return s+(a[s]||a[n]||a[d])},week:{dow:1,doy:7}});return r})},96870(L,f,c){//! moment.js locale configuration
//! locale : Cambodian [km]
//! author : Kruy Vanna : https://github.com/kruyvanna
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u17E1",2:"\u17E2",3:"\u17E3",4:"\u17E4",5:"\u17E5",6:"\u17E6",7:"\u17E7",8:"\u17E8",9:"\u17E9",0:"\u17E0"},r={"\u17E1":"1","\u17E2":"2","\u17E3":"3","\u17E4":"4","\u17E5":"5","\u17E6":"6","\u17E7":"7","\u17E8":"8","\u17E9":"9","\u17E0":"0"},s=o.defineLocale("km",{months:"\u1798\u1780\u179A\u17B6_\u1780\u17BB\u1798\u17D2\u1797\u17C8_\u1798\u17B8\u1793\u17B6_\u1798\u17C1\u179F\u17B6_\u17A7\u179F\u1797\u17B6_\u1798\u17B7\u1790\u17BB\u1793\u17B6_\u1780\u1780\u17D2\u1780\u178A\u17B6_\u179F\u17B8\u17A0\u17B6_\u1780\u1789\u17D2\u1789\u17B6_\u178F\u17BB\u179B\u17B6_\u179C\u17B7\u1785\u17D2\u1786\u17B7\u1780\u17B6_\u1792\u17D2\u1793\u17BC".split("_"),monthsShort:"\u1798\u1780\u179A\u17B6_\u1780\u17BB\u1798\u17D2\u1797\u17C8_\u1798\u17B8\u1793\u17B6_\u1798\u17C1\u179F\u17B6_\u17A7\u179F\u1797\u17B6_\u1798\u17B7\u1790\u17BB\u1793\u17B6_\u1780\u1780\u17D2\u1780\u178A\u17B6_\u179F\u17B8\u17A0\u17B6_\u1780\u1789\u17D2\u1789\u17B6_\u178F\u17BB\u179B\u17B6_\u179C\u17B7\u1785\u17D2\u1786\u17B7\u1780\u17B6_\u1792\u17D2\u1793\u17BC".split("_"),weekdays:"\u17A2\u17B6\u1791\u17B7\u178F\u17D2\u1799_\u1785\u17D0\u1793\u17D2\u1791_\u17A2\u1784\u17D2\u1782\u17B6\u179A_\u1796\u17BB\u1792_\u1796\u17D2\u179A\u17A0\u179F\u17D2\u1794\u178F\u17B7\u17CD_\u179F\u17BB\u1780\u17D2\u179A_\u179F\u17C5\u179A\u17CD".split("_"),weekdaysShort:"\u17A2\u17B6_\u1785_\u17A2_\u1796_\u1796\u17D2\u179A_\u179F\u17BB_\u179F".split("_"),weekdaysMin:"\u17A2\u17B6_\u1785_\u17A2_\u1796_\u1796\u17D2\u179A_\u179F\u17BB_\u179F".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/ព្រឹក|ល្ងាច/,isPM:function(n){return n==="\u179B\u17D2\u1784\u17B6\u1785"},meridiem:function(n,d,u){return n<12?"\u1796\u17D2\u179A\u17B9\u1780":"\u179B\u17D2\u1784\u17B6\u1785"},calendar:{sameDay:"[\u1790\u17D2\u1784\u17C3\u1793\u17C1\u17C7 \u1798\u17C9\u17C4\u1784] LT",nextDay:"[\u179F\u17D2\u17A2\u17C2\u1780 \u1798\u17C9\u17C4\u1784] LT",nextWeek:"dddd [\u1798\u17C9\u17C4\u1784] LT",lastDay:"[\u1798\u17D2\u179F\u17B7\u179B\u1798\u17B7\u1789 \u1798\u17C9\u17C4\u1784] LT",lastWeek:"dddd [\u179F\u1794\u17D2\u178F\u17B6\u17A0\u17CD\u1798\u17BB\u1793] [\u1798\u17C9\u17C4\u1784] LT",sameElse:"L"},relativeTime:{future:"%s\u1791\u17C0\u178F",past:"%s\u1798\u17BB\u1793",s:"\u1794\u17C9\u17BB\u1793\u17D2\u1798\u17B6\u1793\u179C\u17B7\u1793\u17B6\u1791\u17B8",ss:"%d \u179C\u17B7\u1793\u17B6\u1791\u17B8",m:"\u1798\u17BD\u1799\u1793\u17B6\u1791\u17B8",mm:"%d \u1793\u17B6\u1791\u17B8",h:"\u1798\u17BD\u1799\u1798\u17C9\u17C4\u1784",hh:"%d \u1798\u17C9\u17C4\u1784",d:"\u1798\u17BD\u1799\u1790\u17D2\u1784\u17C3",dd:"%d \u1790\u17D2\u1784\u17C3",M:"\u1798\u17BD\u1799\u1781\u17C2",MM:"%d \u1781\u17C2",y:"\u1798\u17BD\u1799\u1786\u17D2\u1793\u17B6\u17C6",yy:"%d \u1786\u17D2\u1793\u17B6\u17C6"},dayOfMonthOrdinalParse:/ទី\d{1,2}/,ordinal:"\u1791\u17B8%d",preparse:function(n){return n.replace(/[១២៣៤៥៦៧៨៩០]/g,function(d){return r[d]})},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]})},week:{dow:1,doy:4}});return s})},84451(L,f,c){//! moment.js locale configuration
//! locale : Kannada [kn]
//! author : Rajeev Naik : https://github.com/rajeevnaikte
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0CE7",2:"\u0CE8",3:"\u0CE9",4:"\u0CEA",5:"\u0CEB",6:"\u0CEC",7:"\u0CED",8:"\u0CEE",9:"\u0CEF",0:"\u0CE6"},r={"\u0CE7":"1","\u0CE8":"2","\u0CE9":"3","\u0CEA":"4","\u0CEB":"5","\u0CEC":"6","\u0CED":"7","\u0CEE":"8","\u0CEF":"9","\u0CE6":"0"},s=o.defineLocale("kn",{months:"\u0C9C\u0CA8\u0CB5\u0CB0\u0CBF_\u0CAB\u0CC6\u0CAC\u0CCD\u0CB0\u0CB5\u0CB0\u0CBF_\u0CAE\u0CBE\u0CB0\u0CCD\u0C9A\u0CCD_\u0C8F\u0CAA\u0CCD\u0CB0\u0CBF\u0CB2\u0CCD_\u0CAE\u0CC6\u0CD5_\u0C9C\u0CC2\u0CA8\u0CCD_\u0C9C\u0CC1\u0CB2\u0CC6\u0CD6_\u0C86\u0C97\u0CB8\u0CCD\u0C9F\u0CCD_\u0CB8\u0CC6\u0CAA\u0CCD\u0C9F\u0CC6\u0C82\u0CAC\u0CB0\u0CCD_\u0C85\u0C95\u0CCD\u0C9F\u0CC6\u0CC2\u0CD5\u0CAC\u0CB0\u0CCD_\u0CA8\u0CB5\u0CC6\u0C82\u0CAC\u0CB0\u0CCD_\u0CA1\u0CBF\u0CB8\u0CC6\u0C82\u0CAC\u0CB0\u0CCD".split("_"),monthsShort:"\u0C9C\u0CA8_\u0CAB\u0CC6\u0CAC\u0CCD\u0CB0_\u0CAE\u0CBE\u0CB0\u0CCD\u0C9A\u0CCD_\u0C8F\u0CAA\u0CCD\u0CB0\u0CBF\u0CB2\u0CCD_\u0CAE\u0CC6\u0CD5_\u0C9C\u0CC2\u0CA8\u0CCD_\u0C9C\u0CC1\u0CB2\u0CC6\u0CD6_\u0C86\u0C97\u0CB8\u0CCD\u0C9F\u0CCD_\u0CB8\u0CC6\u0CAA\u0CCD\u0C9F\u0CC6\u0C82_\u0C85\u0C95\u0CCD\u0C9F\u0CC6\u0CC2\u0CD5_\u0CA8\u0CB5\u0CC6\u0C82_\u0CA1\u0CBF\u0CB8\u0CC6\u0C82".split("_"),monthsParseExact:!0,weekdays:"\u0CAD\u0CBE\u0CA8\u0CC1\u0CB5\u0CBE\u0CB0_\u0CB8\u0CC6\u0CC2\u0CD5\u0CAE\u0CB5\u0CBE\u0CB0_\u0CAE\u0C82\u0C97\u0CB3\u0CB5\u0CBE\u0CB0_\u0CAC\u0CC1\u0CA7\u0CB5\u0CBE\u0CB0_\u0C97\u0CC1\u0CB0\u0CC1\u0CB5\u0CBE\u0CB0_\u0CB6\u0CC1\u0C95\u0CCD\u0CB0\u0CB5\u0CBE\u0CB0_\u0CB6\u0CA8\u0CBF\u0CB5\u0CBE\u0CB0".split("_"),weekdaysShort:"\u0CAD\u0CBE\u0CA8\u0CC1_\u0CB8\u0CC6\u0CC2\u0CD5\u0CAE_\u0CAE\u0C82\u0C97\u0CB3_\u0CAC\u0CC1\u0CA7_\u0C97\u0CC1\u0CB0\u0CC1_\u0CB6\u0CC1\u0C95\u0CCD\u0CB0_\u0CB6\u0CA8\u0CBF".split("_"),weekdaysMin:"\u0CAD\u0CBE_\u0CB8\u0CC6\u0CC2\u0CD5_\u0CAE\u0C82_\u0CAC\u0CC1_\u0C97\u0CC1_\u0CB6\u0CC1_\u0CB6".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[\u0C87\u0C82\u0CA6\u0CC1] LT",nextDay:"[\u0CA8\u0CBE\u0CB3\u0CC6] LT",nextWeek:"dddd, LT",lastDay:"[\u0CA8\u0CBF\u0CA8\u0CCD\u0CA8\u0CC6] LT",lastWeek:"[\u0C95\u0CC6\u0CC2\u0CA8\u0CC6\u0CAF] dddd, LT",sameElse:"L"},relativeTime:{future:"%s \u0CA8\u0C82\u0CA4\u0CB0",past:"%s \u0CB9\u0CBF\u0C82\u0CA6\u0CC6",s:"\u0C95\u0CC6\u0CB2\u0CB5\u0CC1 \u0C95\u0CCD\u0CB7\u0CA3\u0C97\u0CB3\u0CC1",ss:"%d \u0CB8\u0CC6\u0C95\u0CC6\u0C82\u0CA1\u0CC1\u0C97\u0CB3\u0CC1",m:"\u0C92\u0C82\u0CA6\u0CC1 \u0CA8\u0CBF\u0CAE\u0CBF\u0CB7",mm:"%d \u0CA8\u0CBF\u0CAE\u0CBF\u0CB7",h:"\u0C92\u0C82\u0CA6\u0CC1 \u0C97\u0C82\u0C9F\u0CC6",hh:"%d \u0C97\u0C82\u0C9F\u0CC6",d:"\u0C92\u0C82\u0CA6\u0CC1 \u0CA6\u0CBF\u0CA8",dd:"%d \u0CA6\u0CBF\u0CA8",M:"\u0C92\u0C82\u0CA6\u0CC1 \u0CA4\u0CBF\u0C82\u0C97\u0CB3\u0CC1",MM:"%d \u0CA4\u0CBF\u0C82\u0C97\u0CB3\u0CC1",y:"\u0C92\u0C82\u0CA6\u0CC1 \u0CB5\u0CB0\u0CCD\u0CB7",yy:"%d \u0CB5\u0CB0\u0CCD\u0CB7"},preparse:function(n){return n.replace(/[೧೨೩೪೫೬೭೮೯೦]/g,function(d){return r[d]})},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]})},meridiemParse:/ರಾತ್ರಿ|ಬೆಳಿಗ್ಗೆ|ಮಧ್ಯಾಹ್ನ|ಸಂಜೆ/,meridiemHour:function(n,d){if(n===12&&(n=0),d==="\u0CB0\u0CBE\u0CA4\u0CCD\u0CB0\u0CBF")return n<4?n:n+12;if(d==="\u0CAC\u0CC6\u0CB3\u0CBF\u0C97\u0CCD\u0C97\u0CC6")return n;if(d==="\u0CAE\u0CA7\u0CCD\u0CAF\u0CBE\u0CB9\u0CCD\u0CA8")return n>=10?n:n+12;if(d==="\u0CB8\u0C82\u0C9C\u0CC6")return n+12},meridiem:function(n,d,u){return n<4?"\u0CB0\u0CBE\u0CA4\u0CCD\u0CB0\u0CBF":n<10?"\u0CAC\u0CC6\u0CB3\u0CBF\u0C97\u0CCD\u0C97\u0CC6":n<17?"\u0CAE\u0CA7\u0CCD\u0CAF\u0CBE\u0CB9\u0CCD\u0CA8":n<20?"\u0CB8\u0C82\u0C9C\u0CC6":"\u0CB0\u0CBE\u0CA4\u0CCD\u0CB0\u0CBF"},dayOfMonthOrdinalParse:/\d{1,2}(ನೇ)/,ordinal:function(n){return n+"\u0CA8\u0CC6\u0CD5"},week:{dow:0,doy:6}});return s})},63164(L,f,c){//! moment.js locale configuration
//! locale : Korean [ko]
//! author : Kyungwook, Park : https://github.com/kyungw00k
//! author : Jeeeyul Lee <jeeeyul@gmail.com>
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ko",{months:"1\uC6D4_2\uC6D4_3\uC6D4_4\uC6D4_5\uC6D4_6\uC6D4_7\uC6D4_8\uC6D4_9\uC6D4_10\uC6D4_11\uC6D4_12\uC6D4".split("_"),monthsShort:"1\uC6D4_2\uC6D4_3\uC6D4_4\uC6D4_5\uC6D4_6\uC6D4_7\uC6D4_8\uC6D4_9\uC6D4_10\uC6D4_11\uC6D4_12\uC6D4".split("_"),weekdays:"\uC77C\uC694\uC77C_\uC6D4\uC694\uC77C_\uD654\uC694\uC77C_\uC218\uC694\uC77C_\uBAA9\uC694\uC77C_\uAE08\uC694\uC77C_\uD1A0\uC694\uC77C".split("_"),weekdaysShort:"\uC77C_\uC6D4_\uD654_\uC218_\uBAA9_\uAE08_\uD1A0".split("_"),weekdaysMin:"\uC77C_\uC6D4_\uD654_\uC218_\uBAA9_\uAE08_\uD1A0".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"YYYY.MM.DD.",LL:"YYYY\uB144 MMMM D\uC77C",LLL:"YYYY\uB144 MMMM D\uC77C A h:mm",LLLL:"YYYY\uB144 MMMM D\uC77C dddd A h:mm",l:"YYYY.MM.DD.",ll:"YYYY\uB144 MMMM D\uC77C",lll:"YYYY\uB144 MMMM D\uC77C A h:mm",llll:"YYYY\uB144 MMMM D\uC77C dddd A h:mm"},calendar:{sameDay:"\uC624\uB298 LT",nextDay:"\uB0B4\uC77C LT",nextWeek:"dddd LT",lastDay:"\uC5B4\uC81C LT",lastWeek:"\uC9C0\uB09C\uC8FC dddd LT",sameElse:"L"},relativeTime:{future:"%s \uD6C4",past:"%s \uC804",s:"\uBA87 \uCD08",ss:"%d\uCD08",m:"1\uBD84",mm:"%d\uBD84",h:"\uD55C \uC2DC\uAC04",hh:"%d\uC2DC\uAC04",d:"\uD558\uB8E8",dd:"%d\uC77C",M:"\uD55C \uB2EC",MM:"%d\uB2EC",y:"\uC77C \uB144",yy:"%d\uB144"},dayOfMonthOrdinalParse:/\d{1,2}(일|월|주)/,ordinal:function(r,s){switch(s){case"d":case"D":case"DDD":return r+"\uC77C";case"M":return r+"\uC6D4";case"w":case"W":return r+"\uC8FC";default:return r}},meridiemParse:/오전|오후/,isPM:function(r){return r==="\uC624\uD6C4"},meridiem:function(r,s,n){return r<12?"\uC624\uC804":"\uC624\uD6C4"}});return a})},6181(L,f,c){//! moment.js locale configuration
//! locale : Northern Kurdish [ku-kmr]
//! authors : Mazlum Özdogan : https://github.com/mergehez
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(n,d,u,m){var h={s:["\xE7end san\xEEye","\xE7end san\xEEyeyan"],ss:[n+" san\xEEye",n+" san\xEEyeyan"],m:["deq\xEEqeyek","deq\xEEqeyek\xEA"],mm:[n+" deq\xEEqe",n+" deq\xEEqeyan"],h:["saetek","saetek\xEA"],hh:[n+" saet",n+" saetan"],d:["rojek","rojek\xEA"],dd:[n+" roj",n+" rojan"],w:["hefteyek","hefteyek\xEA"],ww:[n+" hefte",n+" hefteyan"],M:["mehek","mehek\xEA"],MM:[n+" meh",n+" mehan"],y:["salek","salek\xEA"],yy:[n+" sal",n+" salan"]};return d?h[u][0]:h[u][1]}function r(n){n=""+n;var d=n.substring(n.length-1),u=n.length>1?n.substring(n.length-2):"";return!(u==12||u==13)&&(d=="2"||d=="3"||u=="50"||d=="70"||d=="80")?"y\xEA":"\xEA"}var s=o.defineLocale("ku-kmr",{months:"R\xEAbendan_Sibat_Adar_N\xEEsan_Gulan_Hez\xEEran_T\xEErmeh_Tebax_\xCElon_Cotmeh_Mijdar_Berfanbar".split("_"),monthsShort:"R\xEAb_Sib_Ada_N\xEEs_Gul_Hez_T\xEEr_Teb_\xCElo_Cot_Mij_Ber".split("_"),monthsParseExact:!0,weekdays:"Yek\u015Fem_Du\u015Fem_S\xEA\u015Fem_\xC7ar\u015Fem_P\xEAnc\u015Fem_\xCEn_\u015Eem\xEE".split("_"),weekdaysShort:"Yek_Du_S\xEA_\xC7ar_P\xEAn_\xCEn_\u015Eem".split("_"),weekdaysMin:"Ye_Du_S\xEA_\xC7a_P\xEA_\xCEn_\u015Ee".split("_"),meridiem:function(n,d,u){return n<12?u?"bn":"BN":u?"pn":"PN"},meridiemParse:/bn|BN|pn|PN/,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"Do MMMM[a] YYYY[an]",LLL:"Do MMMM[a] YYYY[an] HH:mm",LLLL:"dddd, Do MMMM[a] YYYY[an] HH:mm",ll:"Do MMM[.] YYYY[an]",lll:"Do MMM[.] YYYY[an] HH:mm",llll:"ddd[.], Do MMM[.] YYYY[an] HH:mm"},calendar:{sameDay:"[\xCEro di saet] LT [de]",nextDay:"[Sib\xEA di saet] LT [de]",nextWeek:"dddd [di saet] LT [de]",lastDay:"[Duh di saet] LT [de]",lastWeek:"dddd[a bor\xEE di saet] LT [de]",sameElse:"L"},relativeTime:{future:"di %s de",past:"ber\xEE %s",s:a,ss:a,m:a,mm:a,h:a,hh:a,d:a,dd:a,w:a,ww:a,M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}(?:yê|ê|\.)/,ordinal:function(n,d){var u=d.toLowerCase();return u.includes("w")||u.includes("m")?n+".":n+r(n)},week:{dow:1,doy:4}});return s})},98174(L,f,c){//! moment.js locale configuration
//! locale : Kurdish [ku]
//! author : Shahram Mebashar : https://github.com/ShahramMebashar
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0661",2:"\u0662",3:"\u0663",4:"\u0664",5:"\u0665",6:"\u0666",7:"\u0667",8:"\u0668",9:"\u0669",0:"\u0660"},r={"\u0661":"1","\u0662":"2","\u0663":"3","\u0664":"4","\u0665":"5","\u0666":"6","\u0667":"7","\u0668":"8","\u0669":"9","\u0660":"0"},s=["\u06A9\u0627\u0646\u0648\u0646\u06CC \u062F\u0648\u0648\u06D5\u0645","\u0634\u0648\u0628\u0627\u062A","\u0626\u0627\u0632\u0627\u0631","\u0646\u06CC\u0633\u0627\u0646","\u0626\u0627\u06CC\u0627\u0631","\u062D\u0648\u0632\u06D5\u06CC\u0631\u0627\u0646","\u062A\u06D5\u0645\u0645\u0648\u0632","\u0626\u0627\u0628","\u0626\u06D5\u06CC\u0644\u0648\u0648\u0644","\u062A\u0634\u0631\u06CC\u0646\u06CC \u06CC\u06D5\u0643\u06D5\u0645","\u062A\u0634\u0631\u06CC\u0646\u06CC \u062F\u0648\u0648\u06D5\u0645","\u0643\u0627\u0646\u0648\u0646\u06CC \u06CC\u06D5\u06A9\u06D5\u0645"],n=o.defineLocale("ku",{months:s,monthsShort:s,weekdays:"\u06CC\u0647\u200C\u0643\u0634\u0647\u200C\u0645\u0645\u0647\u200C_\u062F\u0648\u0648\u0634\u0647\u200C\u0645\u0645\u0647\u200C_\u0633\u06CE\u0634\u0647\u200C\u0645\u0645\u0647\u200C_\u0686\u0648\u0627\u0631\u0634\u0647\u200C\u0645\u0645\u0647\u200C_\u067E\u06CE\u0646\u062C\u0634\u0647\u200C\u0645\u0645\u0647\u200C_\u0647\u0647\u200C\u06CC\u0646\u06CC_\u0634\u0647\u200C\u0645\u0645\u0647\u200C".split("_"),weekdaysShort:"\u06CC\u0647\u200C\u0643\u0634\u0647\u200C\u0645_\u062F\u0648\u0648\u0634\u0647\u200C\u0645_\u0633\u06CE\u0634\u0647\u200C\u0645_\u0686\u0648\u0627\u0631\u0634\u0647\u200C\u0645_\u067E\u06CE\u0646\u062C\u0634\u0647\u200C\u0645_\u0647\u0647\u200C\u06CC\u0646\u06CC_\u0634\u0647\u200C\u0645\u0645\u0647\u200C".split("_"),weekdaysMin:"\u06CC_\u062F_\u0633_\u0686_\u067E_\u0647_\u0634".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},meridiemParse:/ئێواره‌|به‌یانی/,isPM:function(d){return/ئێواره‌/.test(d)},meridiem:function(d,u,m){return d<12?"\u0628\u0647\u200C\u06CC\u0627\u0646\u06CC":"\u0626\u06CE\u0648\u0627\u0631\u0647\u200C"},calendar:{sameDay:"[\u0626\u0647\u200C\u0645\u0631\u06C6 \u0643\u0627\u062A\u0698\u0645\u06CE\u0631] LT",nextDay:"[\u0628\u0647\u200C\u06CC\u0627\u0646\u06CC \u0643\u0627\u062A\u0698\u0645\u06CE\u0631] LT",nextWeek:"dddd [\u0643\u0627\u062A\u0698\u0645\u06CE\u0631] LT",lastDay:"[\u062F\u0648\u06CE\u0646\u06CE \u0643\u0627\u062A\u0698\u0645\u06CE\u0631] LT",lastWeek:"dddd [\u0643\u0627\u062A\u0698\u0645\u06CE\u0631] LT",sameElse:"L"},relativeTime:{future:"\u0644\u0647\u200C %s",past:"%s",s:"\u0686\u0647\u200C\u0646\u062F \u0686\u0631\u0643\u0647\u200C\u06CC\u0647\u200C\u0643",ss:"\u0686\u0631\u0643\u0647\u200C %d",m:"\u06CC\u0647\u200C\u0643 \u062E\u0648\u0644\u0647\u200C\u0643",mm:"%d \u062E\u0648\u0644\u0647\u200C\u0643",h:"\u06CC\u0647\u200C\u0643 \u0643\u0627\u062A\u0698\u0645\u06CE\u0631",hh:"%d \u0643\u0627\u062A\u0698\u0645\u06CE\u0631",d:"\u06CC\u0647\u200C\u0643 \u0695\u06C6\u0698",dd:"%d \u0695\u06C6\u0698",M:"\u06CC\u0647\u200C\u0643 \u0645\u0627\u0646\u06AF",MM:"%d \u0645\u0627\u0646\u06AF",y:"\u06CC\u0647\u200C\u0643 \u0633\u0627\u06B5",yy:"%d \u0633\u0627\u06B5"},preparse:function(d){return d.replace(/[١٢٣٤٥٦٧٨٩٠]/g,function(u){return r[u]}).replace(/،/g,",")},postformat:function(d){return d.replace(/\d/g,function(u){return a[u]}).replace(/,/g,"\u060C")},week:{dow:6,doy:12}});return n})},78474(L,f,c){//! moment.js locale configuration
//! locale : Kyrgyz [ky]
//! author : Chyngyz Arystan uulu : https://github.com/chyngyz
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={0:"-\u0447\u04AF",1:"-\u0447\u0438",2:"-\u0447\u0438",3:"-\u0447\u04AF",4:"-\u0447\u04AF",5:"-\u0447\u0438",6:"-\u0447\u044B",7:"-\u0447\u0438",8:"-\u0447\u0438",9:"-\u0447\u0443",10:"-\u0447\u0443",20:"-\u0447\u044B",30:"-\u0447\u0443",40:"-\u0447\u044B",50:"-\u0447\u04AF",60:"-\u0447\u044B",70:"-\u0447\u0438",80:"-\u0447\u0438",90:"-\u0447\u0443",100:"-\u0447\u04AF"},r=o.defineLocale("ky",{months:"\u044F\u043D\u0432\u0430\u0440\u044C_\u0444\u0435\u0432\u0440\u0430\u043B\u044C_\u043C\u0430\u0440\u0442_\u0430\u043F\u0440\u0435\u043B\u044C_\u043C\u0430\u0439_\u0438\u044E\u043D\u044C_\u0438\u044E\u043B\u044C_\u0430\u0432\u0433\u0443\u0441\u0442_\u0441\u0435\u043D\u0442\u044F\u0431\u0440\u044C_\u043E\u043A\u0442\u044F\u0431\u0440\u044C_\u043D\u043E\u044F\u0431\u0440\u044C_\u0434\u0435\u043A\u0430\u0431\u0440\u044C".split("_"),monthsShort:"\u044F\u043D\u0432_\u0444\u0435\u0432_\u043C\u0430\u0440\u0442_\u0430\u043F\u0440_\u043C\u0430\u0439_\u0438\u044E\u043D\u044C_\u0438\u044E\u043B\u044C_\u0430\u0432\u0433_\u0441\u0435\u043D_\u043E\u043A\u0442_\u043D\u043E\u044F_\u0434\u0435\u043A".split("_"),weekdays:"\u0416\u0435\u043A\u0448\u0435\u043C\u0431\u0438_\u0414\u04AF\u0439\u0448\u04E9\u043C\u0431\u04AF_\u0428\u0435\u0439\u0448\u0435\u043C\u0431\u0438_\u0428\u0430\u0440\u0448\u0435\u043C\u0431\u0438_\u0411\u0435\u0439\u0448\u0435\u043C\u0431\u0438_\u0416\u0443\u043C\u0430_\u0418\u0448\u0435\u043C\u0431\u0438".split("_"),weekdaysShort:"\u0416\u0435\u043A_\u0414\u04AF\u0439_\u0428\u0435\u0439_\u0428\u0430\u0440_\u0411\u0435\u0439_\u0416\u0443\u043C_\u0418\u0448\u0435".split("_"),weekdaysMin:"\u0416\u043A_\u0414\u0439_\u0428\u0439_\u0428\u0440_\u0411\u0439_\u0416\u043C_\u0418\u0448".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[\u0411\u04AF\u0433\u04AF\u043D \u0441\u0430\u0430\u0442] LT",nextDay:"[\u042D\u0440\u0442\u0435\u04A3 \u0441\u0430\u0430\u0442] LT",nextWeek:"dddd [\u0441\u0430\u0430\u0442] LT",lastDay:"[\u041A\u0435\u0447\u044D\u044D \u0441\u0430\u0430\u0442] LT",lastWeek:"[\u04E8\u0442\u043A\u04E9\u043D \u0430\u043F\u0442\u0430\u043D\u044B\u043D] dddd [\u043A\u04AF\u043D\u04AF] [\u0441\u0430\u0430\u0442] LT",sameElse:"L"},relativeTime:{future:"%s \u0438\u0447\u0438\u043D\u0434\u0435",past:"%s \u043C\u0443\u0440\u0443\u043D",s:"\u0431\u0438\u0440\u043D\u0435\u0447\u0435 \u0441\u0435\u043A\u0443\u043D\u0434",ss:"%d \u0441\u0435\u043A\u0443\u043D\u0434",m:"\u0431\u0438\u0440 \u043C\u04AF\u043D\u04E9\u0442",mm:"%d \u043C\u04AF\u043D\u04E9\u0442",h:"\u0431\u0438\u0440 \u0441\u0430\u0430\u0442",hh:"%d \u0441\u0430\u0430\u0442",d:"\u0431\u0438\u0440 \u043A\u04AF\u043D",dd:"%d \u043A\u04AF\u043D",M:"\u0431\u0438\u0440 \u0430\u0439",MM:"%d \u0430\u0439",y:"\u0431\u0438\u0440 \u0436\u044B\u043B",yy:"%d \u0436\u044B\u043B"},dayOfMonthOrdinalParse:/\d{1,2}-(чи|чы|чү|чу)/,ordinal:function(s){var n=s%10,d=s>=100?100:null;return s+(a[s]||a[n]||a[d])},week:{dow:1,doy:7}});return r})},79680(L,f,c){//! moment.js locale configuration
//! locale : Luxembourgish [lb]
//! author : mweimerskirch : https://github.com/mweimerskirch
//! author : David Raison : https://github.com/kwisatz
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(u,m,h,y){var g={m:["eng Minutt","enger Minutt"],h:["eng Stonn","enger Stonn"],d:["een Dag","engem Dag"],M:["ee Mount","engem Mount"],y:["ee Joer","engem Joer"]};return m?g[h][0]:g[h][1]}function r(u){var m=u.substr(0,u.indexOf(" "));return n(m)?"a "+u:"an "+u}function s(u){var m=u.substr(0,u.indexOf(" "));return n(m)?"viru "+u:"virun "+u}function n(u){if(u=parseInt(u,10),isNaN(u))return!1;if(u<0)return!0;if(u<10)return 4<=u&&u<=7;if(u<100){var m=u%10,h=u/10;return n(m===0?h:m)}else if(u<1e4){for(;u>=10;)u=u/10;return n(u)}else return u=u/1e3,n(u)}var d=o.defineLocale("lb",{months:"Januar_Februar_M\xE4erz_Abr\xEBll_Mee_Juni_Juli_August_September_Oktober_November_Dezember".split("_"),monthsShort:"Jan._Febr._Mrz._Abr._Mee_Jun._Jul._Aug._Sept._Okt._Nov._Dez.".split("_"),monthsParseExact:!0,weekdays:"Sonndeg_M\xE9indeg_D\xEBnschdeg_M\xEBttwoch_Donneschdeg_Freideg_Samschdeg".split("_"),weekdaysShort:"So._M\xE9._D\xEB._M\xEB._Do._Fr._Sa.".split("_"),weekdaysMin:"So_M\xE9_D\xEB_M\xEB_Do_Fr_Sa".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm [Auer]",LTS:"H:mm:ss [Auer]",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm [Auer]",LLLL:"dddd, D. MMMM YYYY H:mm [Auer]"},calendar:{sameDay:"[Haut um] LT",sameElse:"L",nextDay:"[Muer um] LT",nextWeek:"dddd [um] LT",lastDay:"[G\xEBschter um] LT",lastWeek:function(){switch(this.day()){case 2:case 4:return"[Leschten] dddd [um] LT";default:return"[Leschte] dddd [um] LT"}}},relativeTime:{future:r,past:s,s:"e puer Sekonnen",ss:"%d Sekonnen",m:a,mm:"%d Minutten",h:a,hh:"%d Stonnen",d:a,dd:"%d Deeg",M:a,MM:"%d M\xE9int",y:a,yy:"%d Joer"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return d})},15867(L,f,c){//! moment.js locale configuration
//! locale : Lao [lo]
//! author : Ryan Hart : https://github.com/ryanhart2
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("lo",{months:"\u0EA1\u0EB1\u0E87\u0E81\u0EAD\u0E99_\u0E81\u0EB8\u0EA1\u0E9E\u0EB2_\u0EA1\u0EB5\u0E99\u0EB2_\u0EC0\u0EA1\u0EAA\u0EB2_\u0E9E\u0EB6\u0E94\u0EAA\u0EB0\u0E9E\u0EB2_\u0EA1\u0EB4\u0E96\u0EB8\u0E99\u0EB2_\u0E81\u0ECD\u0EA5\u0EB0\u0E81\u0EBB\u0E94_\u0EAA\u0EB4\u0E87\u0EAB\u0EB2_\u0E81\u0EB1\u0E99\u0E8D\u0EB2_\u0E95\u0EB8\u0EA5\u0EB2_\u0E9E\u0EB0\u0E88\u0EB4\u0E81_\u0E97\u0EB1\u0E99\u0EA7\u0EB2".split("_"),monthsShort:"\u0EA1\u0EB1\u0E87\u0E81\u0EAD\u0E99_\u0E81\u0EB8\u0EA1\u0E9E\u0EB2_\u0EA1\u0EB5\u0E99\u0EB2_\u0EC0\u0EA1\u0EAA\u0EB2_\u0E9E\u0EB6\u0E94\u0EAA\u0EB0\u0E9E\u0EB2_\u0EA1\u0EB4\u0E96\u0EB8\u0E99\u0EB2_\u0E81\u0ECD\u0EA5\u0EB0\u0E81\u0EBB\u0E94_\u0EAA\u0EB4\u0E87\u0EAB\u0EB2_\u0E81\u0EB1\u0E99\u0E8D\u0EB2_\u0E95\u0EB8\u0EA5\u0EB2_\u0E9E\u0EB0\u0E88\u0EB4\u0E81_\u0E97\u0EB1\u0E99\u0EA7\u0EB2".split("_"),weekdays:"\u0EAD\u0EB2\u0E97\u0EB4\u0E94_\u0E88\u0EB1\u0E99_\u0EAD\u0EB1\u0E87\u0E84\u0EB2\u0E99_\u0E9E\u0EB8\u0E94_\u0E9E\u0EB0\u0EAB\u0EB1\u0E94_\u0EAA\u0EB8\u0E81_\u0EC0\u0EAA\u0EBB\u0EB2".split("_"),weekdaysShort:"\u0E97\u0EB4\u0E94_\u0E88\u0EB1\u0E99_\u0EAD\u0EB1\u0E87\u0E84\u0EB2\u0E99_\u0E9E\u0EB8\u0E94_\u0E9E\u0EB0\u0EAB\u0EB1\u0E94_\u0EAA\u0EB8\u0E81_\u0EC0\u0EAA\u0EBB\u0EB2".split("_"),weekdaysMin:"\u0E97_\u0E88_\u0EAD\u0E84_\u0E9E_\u0E9E\u0EAB_\u0EAA\u0E81_\u0EAA".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"\u0EA7\u0EB1\u0E99dddd D MMMM YYYY HH:mm"},meridiemParse:/ຕອນເຊົ້າ|ຕອນແລງ/,isPM:function(r){return r==="\u0E95\u0EAD\u0E99\u0EC1\u0EA5\u0E87"},meridiem:function(r,s,n){return r<12?"\u0E95\u0EAD\u0E99\u0EC0\u0E8A\u0EBB\u0EC9\u0EB2":"\u0E95\u0EAD\u0E99\u0EC1\u0EA5\u0E87"},calendar:{sameDay:"[\u0EA1\u0EB7\u0EC9\u0E99\u0EB5\u0EC9\u0EC0\u0EA7\u0EA5\u0EB2] LT",nextDay:"[\u0EA1\u0EB7\u0EC9\u0EAD\u0EB7\u0EC8\u0E99\u0EC0\u0EA7\u0EA5\u0EB2] LT",nextWeek:"[\u0EA7\u0EB1\u0E99]dddd[\u0EDC\u0EC9\u0EB2\u0EC0\u0EA7\u0EA5\u0EB2] LT",lastDay:"[\u0EA1\u0EB7\u0EC9\u0EA7\u0EB2\u0E99\u0E99\u0EB5\u0EC9\u0EC0\u0EA7\u0EA5\u0EB2] LT",lastWeek:"[\u0EA7\u0EB1\u0E99]dddd[\u0EC1\u0EA5\u0EC9\u0EA7\u0E99\u0EB5\u0EC9\u0EC0\u0EA7\u0EA5\u0EB2] LT",sameElse:"L"},relativeTime:{future:"\u0EAD\u0EB5\u0E81 %s",past:"%s\u0E9C\u0EC8\u0EB2\u0E99\u0EA1\u0EB2",s:"\u0E9A\u0ECD\u0EC8\u0EC0\u0E97\u0EBB\u0EC8\u0EB2\u0EC3\u0E94\u0EA7\u0EB4\u0E99\u0EB2\u0E97\u0EB5",ss:"%d \u0EA7\u0EB4\u0E99\u0EB2\u0E97\u0EB5",m:"1 \u0E99\u0EB2\u0E97\u0EB5",mm:"%d \u0E99\u0EB2\u0E97\u0EB5",h:"1 \u0E8A\u0EBB\u0EC8\u0EA7\u0EC2\u0EA1\u0E87",hh:"%d \u0E8A\u0EBB\u0EC8\u0EA7\u0EC2\u0EA1\u0E87",d:"1 \u0EA1\u0EB7\u0EC9",dd:"%d \u0EA1\u0EB7\u0EC9",M:"1 \u0EC0\u0E94\u0EB7\u0EAD\u0E99",MM:"%d \u0EC0\u0E94\u0EB7\u0EAD\u0E99",y:"1 \u0E9B\u0EB5",yy:"%d \u0E9B\u0EB5"},dayOfMonthOrdinalParse:/(ທີ່)\d{1,2}/,ordinal:function(r){return"\u0E97\u0EB5\u0EC8"+r}});return a})},45766(L,f,c){//! moment.js locale configuration
//! locale : Lithuanian [lt]
//! author : Mindaugas Mozūras : https://github.com/mmozuras
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={ss:"sekund\u0117_sekund\u017Ei\u0173_sekundes",m:"minut\u0117_minut\u0117s_minut\u0119",mm:"minut\u0117s_minu\u010Di\u0173_minutes",h:"valanda_valandos_valand\u0105",hh:"valandos_valand\u0173_valandas",d:"diena_dienos_dien\u0105",dd:"dienos_dien\u0173_dienas",M:"m\u0117nuo_m\u0117nesio_m\u0117nes\u012F",MM:"m\u0117nesiai_m\u0117nesi\u0173_m\u0117nesius",y:"metai_met\u0173_metus",yy:"metai_met\u0173_metus"};function r(h,y,g,v){return y?"kelios sekund\u0117s":v?"keli\u0173 sekund\u017Ei\u0173":"kelias sekundes"}function s(h,y,g,v){return y?d(g)[0]:v?d(g)[1]:d(g)[2]}function n(h){return h%10===0||h>10&&h<20}function d(h){return a[h].split("_")}function u(h,y,g,v){var H=h+" ";return h===1?H+s(h,y,g[0],v):y?H+(n(h)?d(g)[1]:d(g)[0]):v?H+d(g)[1]:H+(n(h)?d(g)[1]:d(g)[2])}var m=o.defineLocale("lt",{months:{format:"sausio_vasario_kovo_baland\u017Eio_gegu\u017E\u0117s_bir\u017Eelio_liepos_rugpj\u016B\u010Dio_rugs\u0117jo_spalio_lapkri\u010Dio_gruod\u017Eio".split("_"),standalone:"sausis_vasaris_kovas_balandis_gegu\u017E\u0117_bir\u017Eelis_liepa_rugpj\u016Btis_rugs\u0117jis_spalis_lapkritis_gruodis".split("_"),isFormat:/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?|MMMM?(\[[^\[\]]*\]|\s)+D[oD]?/},monthsShort:"sau_vas_kov_bal_geg_bir_lie_rgp_rgs_spa_lap_grd".split("_"),weekdays:{format:"sekmadien\u012F_pirmadien\u012F_antradien\u012F_tre\u010Diadien\u012F_ketvirtadien\u012F_penktadien\u012F_\u0161e\u0161tadien\u012F".split("_"),standalone:"sekmadienis_pirmadienis_antradienis_tre\u010Diadienis_ketvirtadienis_penktadienis_\u0161e\u0161tadienis".split("_"),isFormat:/dddd HH:mm/},weekdaysShort:"Sek_Pir_Ant_Tre_Ket_Pen_\u0160e\u0161".split("_"),weekdaysMin:"S_P_A_T_K_Pn_\u0160".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY [m.] MMMM D [d.]",LLL:"YYYY [m.] MMMM D [d.], HH:mm [val.]",LLLL:"YYYY [m.] MMMM D [d.], dddd, HH:mm [val.]",l:"YYYY-MM-DD",ll:"YYYY [m.] MMMM D [d.]",lll:"YYYY [m.] MMMM D [d.], HH:mm [val.]",llll:"YYYY [m.] MMMM D [d.], ddd, HH:mm [val.]"},calendar:{sameDay:"[\u0160iandien] LT",nextDay:"[Rytoj] LT",nextWeek:"dddd LT",lastDay:"[Vakar] LT",lastWeek:"[Pra\u0117jus\u012F] dddd LT",sameElse:"L"},relativeTime:{future:"po %s",past:"prie\u0161 %s",s:r,ss:u,m:s,mm:u,h:s,hh:u,d:s,dd:u,M:s,MM:u,y:s,yy:u},dayOfMonthOrdinalParse:/\d{1,2}-oji/,ordinal:function(h){return h+"-oji"},week:{dow:1,doy:4}});return m})},69532(L,f,c){//! moment.js locale configuration
//! locale : Latvian [lv]
//! author : Kristaps Karlsons : https://github.com/skakri
//! author : Jānis Elmeris : https://github.com/JanisE
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={ss:"sekundes_sekund\u0113m_sekunde_sekundes".split("_"),m:"min\u016Btes_min\u016Bt\u0113m_min\u016Bte_min\u016Btes".split("_"),mm:"min\u016Btes_min\u016Bt\u0113m_min\u016Bte_min\u016Btes".split("_"),h:"stundas_stund\u0101m_stunda_stundas".split("_"),hh:"stundas_stund\u0101m_stunda_stundas".split("_"),d:"dienas_dien\u0101m_diena_dienas".split("_"),dd:"dienas_dien\u0101m_diena_dienas".split("_"),M:"m\u0113ne\u0161a_m\u0113ne\u0161iem_m\u0113nesis_m\u0113ne\u0161i".split("_"),MM:"m\u0113ne\u0161a_m\u0113ne\u0161iem_m\u0113nesis_m\u0113ne\u0161i".split("_"),y:"gada_gadiem_gads_gadi".split("_"),yy:"gada_gadiem_gads_gadi".split("_")};function r(m,h,y){return y?h%10===1&&h%100!==11?m[2]:m[3]:h%10===1&&h%100!==11?m[0]:m[1]}function s(m,h,y){return m+" "+r(a[y],m,h)}function n(m,h,y){return r(a[y],m,h)}function d(m,h){return h?"da\u017Eas sekundes":"da\u017E\u0101m sekund\u0113m"}var u=o.defineLocale("lv",{months:"janv\u0101ris_febru\u0101ris_marts_apr\u012Blis_maijs_j\u016Bnijs_j\u016Blijs_augusts_septembris_oktobris_novembris_decembris".split("_"),monthsShort:"jan_feb_mar_apr_mai_j\u016Bn_j\u016Bl_aug_sep_okt_nov_dec".split("_"),weekdays:"sv\u0113tdiena_pirmdiena_otrdiena_tre\u0161diena_ceturtdiena_piektdiena_sestdiena".split("_"),weekdaysShort:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysMin:"Sv_P_O_T_C_Pk_S".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY.",LL:"YYYY. [gada] D. MMMM",LLL:"YYYY. [gada] D. MMMM, HH:mm",LLLL:"YYYY. [gada] D. MMMM, dddd, HH:mm"},calendar:{sameDay:"[\u0160odien pulksten] LT",nextDay:"[R\u012Bt pulksten] LT",nextWeek:"dddd [pulksten] LT",lastDay:"[Vakar pulksten] LT",lastWeek:"[Pag\u0101ju\u0161\u0101] dddd [pulksten] LT",sameElse:"L"},relativeTime:{future:"p\u0113c %s",past:"pirms %s",s:d,ss:s,m:n,mm:s,h:n,hh:s,d:n,dd:s,M:n,MM:s,y:n,yy:s},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return u})},58076(L,f,c){//! moment.js locale configuration
//! locale : Montenegrin [me]
//! author : Miodrag Nikač <miodrag@restartit.me> : https://github.com/miodragnikac
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={words:{ss:["sekund","sekunda","sekundi"],m:["jedan minut","jednog minuta"],mm:["minut","minuta","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],dd:["dan","dana","dana"],MM:["mjesec","mjeseca","mjeseci"],yy:["godina","godine","godina"]},correctGrammaticalCase:function(s,n){return s===1?n[0]:s>=2&&s<=4?n[1]:n[2]},translate:function(s,n,d){var u=a.words[d];return d.length===1?n?u[0]:u[1]:s+" "+a.correctGrammaticalCase(s,u)}},r=o.defineLocale("me",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedjelja_ponedjeljak_utorak_srijeda_\u010Detvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sri._\u010Det._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_\u010De_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sjutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedjelju] [u] LT";case 3:return"[u] [srijedu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[ju\u010De u] LT",lastWeek:function(){var s=["[pro\u0161le] [nedjelje] [u] LT","[pro\u0161log] [ponedjeljka] [u] LT","[pro\u0161log] [utorka] [u] LT","[pro\u0161le] [srijede] [u] LT","[pro\u0161log] [\u010Detvrtka] [u] LT","[pro\u0161log] [petka] [u] LT","[pro\u0161le] [subote] [u] LT"];return s[this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"prije %s",s:"nekoliko sekundi",ss:a.translate,m:a.translate,mm:a.translate,h:a.translate,hh:a.translate,d:"dan",dd:a.translate,M:"mjesec",MM:a.translate,y:"godinu",yy:a.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return r})},41848(L,f,c){//! moment.js locale configuration
//! locale : Maori [mi]
//! author : John Corrigan <robbiecloset@gmail.com> : https://github.com/johnideal
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("mi",{months:"Kohi-t\u0101te_Hui-tanguru_Pout\u016B-te-rangi_Paenga-wh\u0101wh\u0101_Haratua_Pipiri_H\u014Dngoingoi_Here-turi-k\u014Dk\u0101_Mahuru_Whiringa-\u0101-nuku_Whiringa-\u0101-rangi_Hakihea".split("_"),monthsShort:"Kohi_Hui_Pou_Pae_Hara_Pipi_H\u014Dngoi_Here_Mahu_Whi-nu_Whi-ra_Haki".split("_"),monthsRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,3}/i,monthsShortStrictRegex:/(?:['a-z\u0101\u014D\u016B]+\-?){1,2}/i,weekdays:"R\u0101tapu_Mane_T\u016Brei_Wenerei_T\u0101ite_Paraire_H\u0101tarei".split("_"),weekdaysShort:"Ta_Ma_T\u016B_We_T\u0101i_Pa_H\u0101".split("_"),weekdaysMin:"Ta_Ma_T\u016B_We_T\u0101i_Pa_H\u0101".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [i] HH:mm",LLLL:"dddd, D MMMM YYYY [i] HH:mm"},calendar:{sameDay:"[i teie mahana, i] LT",nextDay:"[apopo i] LT",nextWeek:"dddd [i] LT",lastDay:"[inanahi i] LT",lastWeek:"dddd [whakamutunga i] LT",sameElse:"L"},relativeTime:{future:"i roto i %s",past:"%s i mua",s:"te h\u0113kona ruarua",ss:"%d h\u0113kona",m:"he meneti",mm:"%d meneti",h:"te haora",hh:"%d haora",d:"he ra",dd:"%d ra",M:"he marama",MM:"%d marama",y:"he tau",yy:"%d tau"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%d\xBA",week:{dow:1,doy:4}});return a})},30306(L,f,c){//! moment.js locale configuration
//! locale : Macedonian [mk]
//! author : Borislav Mickov : https://github.com/B0k0
//! author : Sashko Todorov : https://github.com/bkyceh
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("mk",{months:"\u0458\u0430\u043D\u0443\u0430\u0440\u0438_\u0444\u0435\u0432\u0440\u0443\u0430\u0440\u0438_\u043C\u0430\u0440\u0442_\u0430\u043F\u0440\u0438\u043B_\u043C\u0430\u0458_\u0458\u0443\u043D\u0438_\u0458\u0443\u043B\u0438_\u0430\u0432\u0433\u0443\u0441\u0442_\u0441\u0435\u043F\u0442\u0435\u043C\u0432\u0440\u0438_\u043E\u043A\u0442\u043E\u043C\u0432\u0440\u0438_\u043D\u043E\u0435\u043C\u0432\u0440\u0438_\u0434\u0435\u043A\u0435\u043C\u0432\u0440\u0438".split("_"),monthsShort:"\u0458\u0430\u043D_\u0444\u0435\u0432_\u043C\u0430\u0440_\u0430\u043F\u0440_\u043C\u0430\u0458_\u0458\u0443\u043D_\u0458\u0443\u043B_\u0430\u0432\u0433_\u0441\u0435\u043F_\u043E\u043A\u0442_\u043D\u043E\u0435_\u0434\u0435\u043A".split("_"),weekdays:"\u043D\u0435\u0434\u0435\u043B\u0430_\u043F\u043E\u043D\u0435\u0434\u0435\u043B\u043D\u0438\u043A_\u0432\u0442\u043E\u0440\u043D\u0438\u043A_\u0441\u0440\u0435\u0434\u0430_\u0447\u0435\u0442\u0432\u0440\u0442\u043E\u043A_\u043F\u0435\u0442\u043E\u043A_\u0441\u0430\u0431\u043E\u0442\u0430".split("_"),weekdaysShort:"\u043D\u0435\u0434_\u043F\u043E\u043D_\u0432\u0442\u043E_\u0441\u0440\u0435_\u0447\u0435\u0442_\u043F\u0435\u0442_\u0441\u0430\u0431".split("_"),weekdaysMin:"\u043De_\u043Fo_\u0432\u0442_\u0441\u0440_\u0447\u0435_\u043F\u0435_\u0441a".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[\u0414\u0435\u043D\u0435\u0441 \u0432\u043E] LT",nextDay:"[\u0423\u0442\u0440\u0435 \u0432\u043E] LT",nextWeek:"[\u0412\u043E] dddd [\u0432\u043E] LT",lastDay:"[\u0412\u0447\u0435\u0440\u0430 \u0432\u043E] LT",lastWeek:function(){switch(this.day()){case 0:case 3:case 6:return"[\u0418\u0437\u043C\u0438\u043D\u0430\u0442\u0430\u0442\u0430] dddd [\u0432\u043E] LT";case 1:case 2:case 4:case 5:return"[\u0418\u0437\u043C\u0438\u043D\u0430\u0442\u0438\u043E\u0442] dddd [\u0432\u043E] LT"}},sameElse:"L"},relativeTime:{future:"\u0437\u0430 %s",past:"\u043F\u0440\u0435\u0434 %s",s:"\u043D\u0435\u043A\u043E\u043B\u043A\u0443 \u0441\u0435\u043A\u0443\u043D\u0434\u0438",ss:"%d \u0441\u0435\u043A\u0443\u043D\u0434\u0438",m:"\u0435\u0434\u043D\u0430 \u043C\u0438\u043D\u0443\u0442\u0430",mm:"%d \u043C\u0438\u043D\u0443\u0442\u0438",h:"\u0435\u0434\u0435\u043D \u0447\u0430\u0441",hh:"%d \u0447\u0430\u0441\u0430",d:"\u0435\u0434\u0435\u043D \u0434\u0435\u043D",dd:"%d \u0434\u0435\u043D\u0430",M:"\u0435\u0434\u0435\u043D \u043C\u0435\u0441\u0435\u0446",MM:"%d \u043C\u0435\u0441\u0435\u0446\u0438",y:"\u0435\u0434\u043D\u0430 \u0433\u043E\u0434\u0438\u043D\u0430",yy:"%d \u0433\u043E\u0434\u0438\u043D\u0438"},dayOfMonthOrdinalParse:/\d{1,2}-(ев|ен|ти|ви|ри|ми)/,ordinal:function(r){var s=r%10,n=r%100;return r===0?r+"-\u0435\u0432":n===0?r+"-\u0435\u043D":n>10&&n<20?r+"-\u0442\u0438":s===1?r+"-\u0432\u0438":s===2?r+"-\u0440\u0438":s===7||s===8?r+"-\u043C\u0438":r+"-\u0442\u0438"},week:{dow:1,doy:7}});return a})},73739(L,f,c){//! moment.js locale configuration
//! locale : Malayalam [ml]
//! author : Floyd Pink : https://github.com/floydpink
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ml",{months:"\u0D1C\u0D28\u0D41\u0D35\u0D30\u0D3F_\u0D2B\u0D46\u0D2C\u0D4D\u0D30\u0D41\u0D35\u0D30\u0D3F_\u0D2E\u0D3E\u0D7C\u0D1A\u0D4D\u0D1A\u0D4D_\u0D0F\u0D2A\u0D4D\u0D30\u0D3F\u0D7D_\u0D2E\u0D47\u0D2F\u0D4D_\u0D1C\u0D42\u0D7A_\u0D1C\u0D42\u0D32\u0D48_\u0D13\u0D17\u0D38\u0D4D\u0D31\u0D4D\u0D31\u0D4D_\u0D38\u0D46\u0D2A\u0D4D\u0D31\u0D4D\u0D31\u0D02\u0D2C\u0D7C_\u0D12\u0D15\u0D4D\u0D1F\u0D4B\u0D2C\u0D7C_\u0D28\u0D35\u0D02\u0D2C\u0D7C_\u0D21\u0D3F\u0D38\u0D02\u0D2C\u0D7C".split("_"),monthsShort:"\u0D1C\u0D28\u0D41._\u0D2B\u0D46\u0D2C\u0D4D\u0D30\u0D41._\u0D2E\u0D3E\u0D7C._\u0D0F\u0D2A\u0D4D\u0D30\u0D3F._\u0D2E\u0D47\u0D2F\u0D4D_\u0D1C\u0D42\u0D7A_\u0D1C\u0D42\u0D32\u0D48._\u0D13\u0D17._\u0D38\u0D46\u0D2A\u0D4D\u0D31\u0D4D\u0D31._\u0D12\u0D15\u0D4D\u0D1F\u0D4B._\u0D28\u0D35\u0D02._\u0D21\u0D3F\u0D38\u0D02.".split("_"),monthsParseExact:!0,weekdays:"\u0D1E\u0D3E\u0D2F\u0D31\u0D3E\u0D34\u0D4D\u0D1A_\u0D24\u0D3F\u0D19\u0D4D\u0D15\u0D33\u0D3E\u0D34\u0D4D\u0D1A_\u0D1A\u0D4A\u0D35\u0D4D\u0D35\u0D3E\u0D34\u0D4D\u0D1A_\u0D2C\u0D41\u0D27\u0D28\u0D3E\u0D34\u0D4D\u0D1A_\u0D35\u0D4D\u0D2F\u0D3E\u0D34\u0D3E\u0D34\u0D4D\u0D1A_\u0D35\u0D46\u0D33\u0D4D\u0D33\u0D3F\u0D2F\u0D3E\u0D34\u0D4D\u0D1A_\u0D36\u0D28\u0D3F\u0D2F\u0D3E\u0D34\u0D4D\u0D1A".split("_"),weekdaysShort:"\u0D1E\u0D3E\u0D2F\u0D7C_\u0D24\u0D3F\u0D19\u0D4D\u0D15\u0D7E_\u0D1A\u0D4A\u0D35\u0D4D\u0D35_\u0D2C\u0D41\u0D27\u0D7B_\u0D35\u0D4D\u0D2F\u0D3E\u0D34\u0D02_\u0D35\u0D46\u0D33\u0D4D\u0D33\u0D3F_\u0D36\u0D28\u0D3F".split("_"),weekdaysMin:"\u0D1E\u0D3E_\u0D24\u0D3F_\u0D1A\u0D4A_\u0D2C\u0D41_\u0D35\u0D4D\u0D2F\u0D3E_\u0D35\u0D46_\u0D36".split("_"),longDateFormat:{LT:"A h:mm -\u0D28\u0D41",LTS:"A h:mm:ss -\u0D28\u0D41",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm -\u0D28\u0D41",LLLL:"dddd, D MMMM YYYY, A h:mm -\u0D28\u0D41"},calendar:{sameDay:"[\u0D07\u0D28\u0D4D\u0D28\u0D4D] LT",nextDay:"[\u0D28\u0D3E\u0D33\u0D46] LT",nextWeek:"dddd, LT",lastDay:"[\u0D07\u0D28\u0D4D\u0D28\u0D32\u0D46] LT",lastWeek:"[\u0D15\u0D34\u0D3F\u0D1E\u0D4D\u0D1E] dddd, LT",sameElse:"L"},relativeTime:{future:"%s \u0D15\u0D34\u0D3F\u0D1E\u0D4D\u0D1E\u0D4D",past:"%s \u0D2E\u0D41\u0D7B\u0D2A\u0D4D",s:"\u0D05\u0D7D\u0D2A \u0D28\u0D3F\u0D2E\u0D3F\u0D37\u0D19\u0D4D\u0D19\u0D7E",ss:"%d \u0D38\u0D46\u0D15\u0D4D\u0D15\u0D7B\u0D21\u0D4D",m:"\u0D12\u0D30\u0D41 \u0D2E\u0D3F\u0D28\u0D3F\u0D31\u0D4D\u0D31\u0D4D",mm:"%d \u0D2E\u0D3F\u0D28\u0D3F\u0D31\u0D4D\u0D31\u0D4D",h:"\u0D12\u0D30\u0D41 \u0D2E\u0D23\u0D3F\u0D15\u0D4D\u0D15\u0D42\u0D7C",hh:"%d \u0D2E\u0D23\u0D3F\u0D15\u0D4D\u0D15\u0D42\u0D7C",d:"\u0D12\u0D30\u0D41 \u0D26\u0D3F\u0D35\u0D38\u0D02",dd:"%d \u0D26\u0D3F\u0D35\u0D38\u0D02",M:"\u0D12\u0D30\u0D41 \u0D2E\u0D3E\u0D38\u0D02",MM:"%d \u0D2E\u0D3E\u0D38\u0D02",y:"\u0D12\u0D30\u0D41 \u0D35\u0D7C\u0D37\u0D02",yy:"%d \u0D35\u0D7C\u0D37\u0D02"},meridiemParse:/രാത്രി|രാവിലെ|ഉച്ച കഴിഞ്ഞ്|വൈകുന്നേരം|രാത്രി/i,meridiemHour:function(r,s){return r===12&&(r=0),s==="\u0D30\u0D3E\u0D24\u0D4D\u0D30\u0D3F"&&r>=4||s==="\u0D09\u0D1A\u0D4D\u0D1A \u0D15\u0D34\u0D3F\u0D1E\u0D4D\u0D1E\u0D4D"||s==="\u0D35\u0D48\u0D15\u0D41\u0D28\u0D4D\u0D28\u0D47\u0D30\u0D02"?r+12:r},meridiem:function(r,s,n){return r<4?"\u0D30\u0D3E\u0D24\u0D4D\u0D30\u0D3F":r<12?"\u0D30\u0D3E\u0D35\u0D3F\u0D32\u0D46":r<17?"\u0D09\u0D1A\u0D4D\u0D1A \u0D15\u0D34\u0D3F\u0D1E\u0D4D\u0D1E\u0D4D":r<20?"\u0D35\u0D48\u0D15\u0D41\u0D28\u0D4D\u0D28\u0D47\u0D30\u0D02":"\u0D30\u0D3E\u0D24\u0D4D\u0D30\u0D3F"}});return a})},99053(L,f,c){//! moment.js locale configuration
//! locale : Mongolian [mn]
//! author : Javkhlantugs Nyamdorj : https://github.com/javkhaanj7
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(s,n,d,u){switch(d){case"s":return n?"\u0445\u044D\u0434\u0445\u044D\u043D \u0441\u0435\u043A\u0443\u043D\u0434":"\u0445\u044D\u0434\u0445\u044D\u043D \u0441\u0435\u043A\u0443\u043D\u0434\u044B\u043D";case"ss":return s+(n?" \u0441\u0435\u043A\u0443\u043D\u0434":" \u0441\u0435\u043A\u0443\u043D\u0434\u044B\u043D");case"m":case"mm":return s+(n?" \u043C\u0438\u043D\u0443\u0442":" \u043C\u0438\u043D\u0443\u0442\u044B\u043D");case"h":case"hh":return s+(n?" \u0446\u0430\u0433":" \u0446\u0430\u0433\u0438\u0439\u043D");case"d":case"dd":return s+(n?" \u04E9\u0434\u04E9\u0440":" \u04E9\u0434\u0440\u0438\u0439\u043D");case"M":case"MM":return s+(n?" \u0441\u0430\u0440":" \u0441\u0430\u0440\u044B\u043D");case"y":case"yy":return s+(n?" \u0436\u0438\u043B":" \u0436\u0438\u043B\u0438\u0439\u043D");default:return s}}var r=o.defineLocale("mn",{months:"\u041D\u044D\u0433\u0434\u04AF\u0433\u044D\u044D\u0440 \u0441\u0430\u0440_\u0425\u043E\u0451\u0440\u0434\u0443\u0433\u0430\u0430\u0440 \u0441\u0430\u0440_\u0413\u0443\u0440\u0430\u0432\u0434\u0443\u0433\u0430\u0430\u0440 \u0441\u0430\u0440_\u0414\u04E9\u0440\u04E9\u0432\u0434\u04AF\u0433\u044D\u044D\u0440 \u0441\u0430\u0440_\u0422\u0430\u0432\u0434\u0443\u0433\u0430\u0430\u0440 \u0441\u0430\u0440_\u0417\u0443\u0440\u0433\u0430\u0434\u0443\u0433\u0430\u0430\u0440 \u0441\u0430\u0440_\u0414\u043E\u043B\u0434\u0443\u0433\u0430\u0430\u0440 \u0441\u0430\u0440_\u041D\u0430\u0439\u043C\u0434\u0443\u0433\u0430\u0430\u0440 \u0441\u0430\u0440_\u0415\u0441\u0434\u04AF\u0433\u044D\u044D\u0440 \u0441\u0430\u0440_\u0410\u0440\u0430\u0432\u0434\u0443\u0433\u0430\u0430\u0440 \u0441\u0430\u0440_\u0410\u0440\u0432\u0430\u043D \u043D\u044D\u0433\u0434\u04AF\u0433\u044D\u044D\u0440 \u0441\u0430\u0440_\u0410\u0440\u0432\u0430\u043D \u0445\u043E\u0451\u0440\u0434\u0443\u0433\u0430\u0430\u0440 \u0441\u0430\u0440".split("_"),monthsShort:"1 \u0441\u0430\u0440_2 \u0441\u0430\u0440_3 \u0441\u0430\u0440_4 \u0441\u0430\u0440_5 \u0441\u0430\u0440_6 \u0441\u0430\u0440_7 \u0441\u0430\u0440_8 \u0441\u0430\u0440_9 \u0441\u0430\u0440_10 \u0441\u0430\u0440_11 \u0441\u0430\u0440_12 \u0441\u0430\u0440".split("_"),monthsParseExact:!0,weekdays:"\u041D\u044F\u043C_\u0414\u0430\u0432\u0430\u0430_\u041C\u044F\u0433\u043C\u0430\u0440_\u041B\u0445\u0430\u0433\u0432\u0430_\u041F\u04AF\u0440\u044D\u0432_\u0411\u0430\u0430\u0441\u0430\u043D_\u0411\u044F\u043C\u0431\u0430".split("_"),weekdaysShort:"\u041D\u044F\u043C_\u0414\u0430\u0432_\u041C\u044F\u0433_\u041B\u0445\u0430_\u041F\u04AF\u0440_\u0411\u0430\u0430_\u0411\u044F\u043C".split("_"),weekdaysMin:"\u041D\u044F_\u0414\u0430_\u041C\u044F_\u041B\u0445_\u041F\u04AF_\u0411\u0430_\u0411\u044F".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY \u043E\u043D\u044B MMMM\u044B\u043D D",LLL:"YYYY \u043E\u043D\u044B MMMM\u044B\u043D D HH:mm",LLLL:"dddd, YYYY \u043E\u043D\u044B MMMM\u044B\u043D D HH:mm"},meridiemParse:/ҮӨ|ҮХ/i,isPM:function(s){return s==="\u04AE\u0425"},meridiem:function(s,n,d){return s<12?"\u04AE\u04E8":"\u04AE\u0425"},calendar:{sameDay:"[\u04E8\u043D\u04E9\u04E9\u0434\u04E9\u0440] LT",nextDay:"[\u041C\u0430\u0440\u0433\u0430\u0430\u0448] LT",nextWeek:"[\u0418\u0440\u044D\u0445] dddd LT",lastDay:"[\u04E8\u0447\u0438\u0433\u0434\u04E9\u0440] LT",lastWeek:"[\u04E8\u043D\u0433\u04E9\u0440\u0441\u04E9\u043D] dddd LT",sameElse:"L"},relativeTime:{future:"%s \u0434\u0430\u0440\u0430\u0430",past:"%s \u04E9\u043C\u043D\u04E9",s:a,ss:a,m:a,mm:a,h:a,hh:a,d:a,dd:a,M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2} өдөр/,ordinal:function(s,n){switch(n){case"d":case"D":case"DDD":return s+" \u04E9\u0434\u04E9\u0440";default:return s}}});return r})},86169(L,f,c){//! moment.js locale configuration
//! locale : Marathi [mr]
//! author : Harshad Kale : https://github.com/kalehv
//! author : Vivek Athalye : https://github.com/vnathalye
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0967",2:"\u0968",3:"\u0969",4:"\u096A",5:"\u096B",6:"\u096C",7:"\u096D",8:"\u096E",9:"\u096F",0:"\u0966"},r={"\u0967":"1","\u0968":"2","\u0969":"3","\u096A":"4","\u096B":"5","\u096C":"6","\u096D":"7","\u096E":"8","\u096F":"9","\u0966":"0"};function s(d,u,m,h){var y="";if(u)switch(m){case"s":y="\u0915\u093E\u0939\u0940 \u0938\u0947\u0915\u0902\u0926";break;case"ss":y="%d \u0938\u0947\u0915\u0902\u0926";break;case"m":y="\u090F\u0915 \u092E\u093F\u0928\u093F\u091F";break;case"mm":y="%d \u092E\u093F\u0928\u093F\u091F\u0947";break;case"h":y="\u090F\u0915 \u0924\u093E\u0938";break;case"hh":y="%d \u0924\u093E\u0938";break;case"d":y="\u090F\u0915 \u0926\u093F\u0935\u0938";break;case"dd":y="%d \u0926\u093F\u0935\u0938";break;case"M":y="\u090F\u0915 \u092E\u0939\u093F\u0928\u093E";break;case"MM":y="%d \u092E\u0939\u093F\u0928\u0947";break;case"y":y="\u090F\u0915 \u0935\u0930\u094D\u0937";break;case"yy":y="%d \u0935\u0930\u094D\u0937\u0947";break}else switch(m){case"s":y="\u0915\u093E\u0939\u0940 \u0938\u0947\u0915\u0902\u0926\u093E\u0902";break;case"ss":y="%d \u0938\u0947\u0915\u0902\u0926\u093E\u0902";break;case"m":y="\u090F\u0915\u093E \u092E\u093F\u0928\u093F\u091F\u093E";break;case"mm":y="%d \u092E\u093F\u0928\u093F\u091F\u093E\u0902";break;case"h":y="\u090F\u0915\u093E \u0924\u093E\u0938\u093E";break;case"hh":y="%d \u0924\u093E\u0938\u093E\u0902";break;case"d":y="\u090F\u0915\u093E \u0926\u093F\u0935\u0938\u093E";break;case"dd":y="%d \u0926\u093F\u0935\u0938\u093E\u0902";break;case"M":y="\u090F\u0915\u093E \u092E\u0939\u093F\u0928\u094D\u092F\u093E";break;case"MM":y="%d \u092E\u0939\u093F\u0928\u094D\u092F\u093E\u0902";break;case"y":y="\u090F\u0915\u093E \u0935\u0930\u094D\u0937\u093E";break;case"yy":y="%d \u0935\u0930\u094D\u0937\u093E\u0902";break}return y.replace(/%d/i,d)}var n=o.defineLocale("mr",{months:"\u091C\u093E\u0928\u0947\u0935\u093E\u0930\u0940_\u092B\u0947\u092C\u094D\u0930\u0941\u0935\u093E\u0930\u0940_\u092E\u093E\u0930\u094D\u091A_\u090F\u092A\u094D\u0930\u093F\u0932_\u092E\u0947_\u091C\u0942\u0928_\u091C\u0941\u0932\u0948_\u0911\u0917\u0938\u094D\u091F_\u0938\u092A\u094D\u091F\u0947\u0902\u092C\u0930_\u0911\u0915\u094D\u091F\u094B\u092C\u0930_\u0928\u094B\u0935\u094D\u0939\u0947\u0902\u092C\u0930_\u0921\u093F\u0938\u0947\u0902\u092C\u0930".split("_"),monthsShort:"\u091C\u093E\u0928\u0947._\u092B\u0947\u092C\u094D\u0930\u0941._\u092E\u093E\u0930\u094D\u091A._\u090F\u092A\u094D\u0930\u093F._\u092E\u0947._\u091C\u0942\u0928._\u091C\u0941\u0932\u0948._\u0911\u0917._\u0938\u092A\u094D\u091F\u0947\u0902._\u0911\u0915\u094D\u091F\u094B._\u0928\u094B\u0935\u094D\u0939\u0947\u0902._\u0921\u093F\u0938\u0947\u0902.".split("_"),monthsParseExact:!0,weekdays:"\u0930\u0935\u093F\u0935\u093E\u0930_\u0938\u094B\u092E\u0935\u093E\u0930_\u092E\u0902\u0917\u0933\u0935\u093E\u0930_\u092C\u0941\u0927\u0935\u093E\u0930_\u0917\u0941\u0930\u0942\u0935\u093E\u0930_\u0936\u0941\u0915\u094D\u0930\u0935\u093E\u0930_\u0936\u0928\u093F\u0935\u093E\u0930".split("_"),weekdaysShort:"\u0930\u0935\u093F_\u0938\u094B\u092E_\u092E\u0902\u0917\u0933_\u092C\u0941\u0927_\u0917\u0941\u0930\u0942_\u0936\u0941\u0915\u094D\u0930_\u0936\u0928\u093F".split("_"),weekdaysMin:"\u0930_\u0938\u094B_\u092E\u0902_\u092C\u0941_\u0917\u0941_\u0936\u0941_\u0936".split("_"),longDateFormat:{LT:"A h:mm \u0935\u093E\u091C\u0924\u093E",LTS:"A h:mm:ss \u0935\u093E\u091C\u0924\u093E",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm \u0935\u093E\u091C\u0924\u093E",LLLL:"dddd, D MMMM YYYY, A h:mm \u0935\u093E\u091C\u0924\u093E"},calendar:{sameDay:"[\u0906\u091C] LT",nextDay:"[\u0909\u0926\u094D\u092F\u093E] LT",nextWeek:"dddd, LT",lastDay:"[\u0915\u093E\u0932] LT",lastWeek:"[\u092E\u093E\u0917\u0940\u0932] dddd, LT",sameElse:"L"},relativeTime:{future:"%s\u092E\u0927\u094D\u092F\u0947",past:"%s\u092A\u0942\u0930\u094D\u0935\u0940",s,ss:s,m:s,mm:s,h:s,hh:s,d:s,dd:s,M:s,MM:s,y:s,yy:s},preparse:function(d){return d.replace(/[१२३४५६७८९०]/g,function(u){return r[u]})},postformat:function(d){return d.replace(/\d/g,function(u){return a[u]})},meridiemParse:/पहाटे|सकाळी|दुपारी|सायंकाळी|रात्री/,meridiemHour:function(d,u){if(d===12&&(d=0),u==="\u092A\u0939\u093E\u091F\u0947"||u==="\u0938\u0915\u093E\u0933\u0940")return d;if(u==="\u0926\u0941\u092A\u093E\u0930\u0940"||u==="\u0938\u093E\u092F\u0902\u0915\u093E\u0933\u0940"||u==="\u0930\u093E\u0924\u094D\u0930\u0940")return d>=12?d:d+12},meridiem:function(d,u,m){return d>=0&&d<6?"\u092A\u0939\u093E\u091F\u0947":d<12?"\u0938\u0915\u093E\u0933\u0940":d<17?"\u0926\u0941\u092A\u093E\u0930\u0940":d<20?"\u0938\u093E\u092F\u0902\u0915\u093E\u0933\u0940":"\u0930\u093E\u0924\u094D\u0930\u0940"},week:{dow:0,doy:6}});return n})},92297(L,f,c){//! moment.js locale configuration
//! locale : Malay [ms-my]
//! note : DEPRECATED, the correct one is [ms]
//! author : Weldan Jamili : https://github.com/weldan
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ms-my",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(r,s){if(r===12&&(r=0),s==="pagi")return r;if(s==="tengahari")return r>=11?r:r+12;if(s==="petang"||s==="malam")return r+12},meridiem:function(r,s,n){return r<11?"pagi":r<15?"tengahari":r<19?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",ss:"%d saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,doy:7}});return a})},73386(L,f,c){//! moment.js locale configuration
//! locale : Malay [ms]
//! author : Weldan Jamili : https://github.com/weldan
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ms",{months:"Januari_Februari_Mac_April_Mei_Jun_Julai_Ogos_September_Oktober_November_Disember".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ogs_Sep_Okt_Nov_Dis".split("_"),weekdays:"Ahad_Isnin_Selasa_Rabu_Khamis_Jumaat_Sabtu".split("_"),weekdaysShort:"Ahd_Isn_Sel_Rab_Kha_Jum_Sab".split("_"),weekdaysMin:"Ah_Is_Sl_Rb_Km_Jm_Sb".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [pukul] HH.mm",LLLL:"dddd, D MMMM YYYY [pukul] HH.mm"},meridiemParse:/pagi|tengahari|petang|malam/,meridiemHour:function(r,s){if(r===12&&(r=0),s==="pagi")return r;if(s==="tengahari")return r>=11?r:r+12;if(s==="petang"||s==="malam")return r+12},meridiem:function(r,s,n){return r<11?"pagi":r<15?"tengahari":r<19?"petang":"malam"},calendar:{sameDay:"[Hari ini pukul] LT",nextDay:"[Esok pukul] LT",nextWeek:"dddd [pukul] LT",lastDay:"[Kelmarin pukul] LT",lastWeek:"dddd [lepas pukul] LT",sameElse:"L"},relativeTime:{future:"dalam %s",past:"%s yang lepas",s:"beberapa saat",ss:"%d saat",m:"seminit",mm:"%d minit",h:"sejam",hh:"%d jam",d:"sehari",dd:"%d hari",M:"sebulan",MM:"%d bulan",y:"setahun",yy:"%d tahun"},week:{dow:1,doy:7}});return a})},77075(L,f,c){//! moment.js locale configuration
//! locale : Maltese (Malta) [mt]
//! author : Alessandro Maruccia : https://github.com/alesma
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("mt",{months:"Jannar_Frar_Marzu_April_Mejju_\u0120unju_Lulju_Awwissu_Settembru_Ottubru_Novembru_Di\u010Bembru".split("_"),monthsShort:"Jan_Fra_Mar_Apr_Mej_\u0120un_Lul_Aww_Set_Ott_Nov_Di\u010B".split("_"),weekdays:"Il-\u0126add_It-Tnejn_It-Tlieta_L-Erbg\u0127a_Il-\u0126amis_Il-\u0120img\u0127a_Is-Sibt".split("_"),weekdaysShort:"\u0126ad_Tne_Tli_Erb_\u0126am_\u0120im_Sib".split("_"),weekdaysMin:"\u0126a_Tn_Tl_Er_\u0126a_\u0120i_Si".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Illum fil-]LT",nextDay:"[G\u0127ada fil-]LT",nextWeek:"dddd [fil-]LT",lastDay:"[Il-biera\u0127 fil-]LT",lastWeek:"dddd [li g\u0127adda] [fil-]LT",sameElse:"L"},relativeTime:{future:"f\u2019 %s",past:"%s ilu",s:"ftit sekondi",ss:"%d sekondi",m:"minuta",mm:"%d minuti",h:"sieg\u0127a",hh:"%d sieg\u0127at",d:"\u0121urnata",dd:"%d \u0121ranet",M:"xahar",MM:"%d xhur",y:"sena",yy:"%d sni"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%d\xBA",week:{dow:1,doy:4}});return a})},72264(L,f,c){//! moment.js locale configuration
//! locale : Burmese [my]
//! author : Squar team, mysquar.com
//! author : David Rossellat : https://github.com/gholadr
//! author : Tin Aung Lin : https://github.com/thanyawzinmin
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u1041",2:"\u1042",3:"\u1043",4:"\u1044",5:"\u1045",6:"\u1046",7:"\u1047",8:"\u1048",9:"\u1049",0:"\u1040"},r={"\u1041":"1","\u1042":"2","\u1043":"3","\u1044":"4","\u1045":"5","\u1046":"6","\u1047":"7","\u1048":"8","\u1049":"9","\u1040":"0"},s=o.defineLocale("my",{months:"\u1007\u1014\u103A\u1014\u101D\u102B\u101B\u102E_\u1016\u1031\u1016\u1031\u102C\u103A\u101D\u102B\u101B\u102E_\u1019\u1010\u103A_\u1027\u1015\u103C\u102E_\u1019\u1031_\u1007\u103D\u1014\u103A_\u1007\u1030\u101C\u102D\u102F\u1004\u103A_\u101E\u103C\u1002\u102F\u1010\u103A_\u1005\u1000\u103A\u1010\u1004\u103A\u1018\u102C_\u1021\u1031\u102C\u1000\u103A\u1010\u102D\u102F\u1018\u102C_\u1014\u102D\u102F\u101D\u1004\u103A\u1018\u102C_\u1012\u102E\u1007\u1004\u103A\u1018\u102C".split("_"),monthsShort:"\u1007\u1014\u103A_\u1016\u1031_\u1019\u1010\u103A_\u1015\u103C\u102E_\u1019\u1031_\u1007\u103D\u1014\u103A_\u101C\u102D\u102F\u1004\u103A_\u101E\u103C_\u1005\u1000\u103A_\u1021\u1031\u102C\u1000\u103A_\u1014\u102D\u102F_\u1012\u102E".split("_"),weekdays:"\u1010\u1014\u1004\u103A\u1039\u1002\u1014\u103D\u1031_\u1010\u1014\u1004\u103A\u1039\u101C\u102C_\u1021\u1004\u103A\u1039\u1002\u102B_\u1017\u102F\u1012\u1039\u1013\u101F\u1030\u1038_\u1000\u103C\u102C\u101E\u1015\u1010\u1031\u1038_\u101E\u1031\u102C\u1000\u103C\u102C_\u1005\u1014\u1031".split("_"),weekdaysShort:"\u1014\u103D\u1031_\u101C\u102C_\u1002\u102B_\u101F\u1030\u1038_\u1000\u103C\u102C_\u101E\u1031\u102C_\u1014\u1031".split("_"),weekdaysMin:"\u1014\u103D\u1031_\u101C\u102C_\u1002\u102B_\u101F\u1030\u1038_\u1000\u103C\u102C_\u101E\u1031\u102C_\u1014\u1031".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[\u101A\u1014\u1031.] LT [\u1019\u103E\u102C]",nextDay:"[\u1019\u1014\u1000\u103A\u1016\u103C\u1014\u103A] LT [\u1019\u103E\u102C]",nextWeek:"dddd LT [\u1019\u103E\u102C]",lastDay:"[\u1019\u1014\u1031.\u1000] LT [\u1019\u103E\u102C]",lastWeek:"[\u1015\u103C\u102E\u1038\u1001\u1032\u1037\u101E\u1031\u102C] dddd LT [\u1019\u103E\u102C]",sameElse:"L"},relativeTime:{future:"\u101C\u102C\u1019\u100A\u103A\u1037 %s \u1019\u103E\u102C",past:"\u101C\u103D\u1014\u103A\u1001\u1032\u1037\u101E\u1031\u102C %s \u1000",s:"\u1005\u1000\u1039\u1000\u1014\u103A.\u1021\u1014\u100A\u103A\u1038\u1004\u101A\u103A",ss:"%d \u1005\u1000\u1039\u1000\u1014\u1037\u103A",m:"\u1010\u1005\u103A\u1019\u102D\u1014\u1005\u103A",mm:"%d \u1019\u102D\u1014\u1005\u103A",h:"\u1010\u1005\u103A\u1014\u102C\u101B\u102E",hh:"%d \u1014\u102C\u101B\u102E",d:"\u1010\u1005\u103A\u101B\u1000\u103A",dd:"%d \u101B\u1000\u103A",M:"\u1010\u1005\u103A\u101C",MM:"%d \u101C",y:"\u1010\u1005\u103A\u1014\u103E\u1005\u103A",yy:"%d \u1014\u103E\u1005\u103A"},preparse:function(n){return n.replace(/[၁၂၃၄၅၆၇၈၉၀]/g,function(d){return r[d]})},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]})},week:{dow:1,doy:4}});return s})},22274(L,f,c){//! moment.js locale configuration
//! locale : Norwegian Bokmål [nb]
//! authors : Espen Hovlandsdal : https://github.com/rexxars
//!           Sigurd Gartmann : https://github.com/sigurdga
//!           Stephen Ramthun : https://github.com/stephenramthun
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("nb",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan._feb._mars_apr._mai_juni_juli_aug._sep._okt._nov._des.".split("_"),monthsParseExact:!0,weekdays:"s\xF8ndag_mandag_tirsdag_onsdag_torsdag_fredag_l\xF8rdag".split("_"),weekdaysShort:"s\xF8._ma._ti._on._to._fr._l\xF8.".split("_"),weekdaysMin:"s\xF8_ma_ti_on_to_fr_l\xF8".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] HH:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[i dag kl.] LT",nextDay:"[i morgen kl.] LT",nextWeek:"dddd [kl.] LT",lastDay:"[i g\xE5r kl.] LT",lastWeek:"[forrige] dddd [kl.] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s siden",s:"noen sekunder",ss:"%d sekunder",m:"ett minutt",mm:"%d minutter",h:"\xE9n time",hh:"%d timer",d:"\xE9n dag",dd:"%d dager",w:"\xE9n uke",ww:"%d uker",M:"\xE9n m\xE5ned",MM:"%d m\xE5neder",y:"ett \xE5r",yy:"%d \xE5r"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a})},8235(L,f,c){//! moment.js locale configuration
//! locale : Nepalese [ne]
//! author : suvash : https://github.com/suvash
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0967",2:"\u0968",3:"\u0969",4:"\u096A",5:"\u096B",6:"\u096C",7:"\u096D",8:"\u096E",9:"\u096F",0:"\u0966"},r={"\u0967":"1","\u0968":"2","\u0969":"3","\u096A":"4","\u096B":"5","\u096C":"6","\u096D":"7","\u096E":"8","\u096F":"9","\u0966":"0"},s=o.defineLocale("ne",{months:"\u091C\u0928\u0935\u0930\u0940_\u092B\u0947\u092C\u094D\u0930\u0941\u0935\u0930\u0940_\u092E\u093E\u0930\u094D\u091A_\u0905\u092A\u094D\u0930\u093F\u0932_\u092E\u0908_\u091C\u0941\u0928_\u091C\u0941\u0932\u093E\u0908_\u0905\u0917\u0937\u094D\u091F_\u0938\u0947\u092A\u094D\u091F\u0947\u092E\u094D\u092C\u0930_\u0905\u0915\u094D\u091F\u094B\u092C\u0930_\u0928\u094B\u092D\u0947\u092E\u094D\u092C\u0930_\u0921\u093F\u0938\u0947\u092E\u094D\u092C\u0930".split("_"),monthsShort:"\u091C\u0928._\u092B\u0947\u092C\u094D\u0930\u0941._\u092E\u093E\u0930\u094D\u091A_\u0905\u092A\u094D\u0930\u093F._\u092E\u0908_\u091C\u0941\u0928_\u091C\u0941\u0932\u093E\u0908._\u0905\u0917._\u0938\u0947\u092A\u094D\u091F._\u0905\u0915\u094D\u091F\u094B._\u0928\u094B\u092D\u0947._\u0921\u093F\u0938\u0947.".split("_"),monthsParseExact:!0,weekdays:"\u0906\u0907\u0924\u092C\u093E\u0930_\u0938\u094B\u092E\u092C\u093E\u0930_\u092E\u0919\u094D\u0917\u0932\u092C\u093E\u0930_\u092C\u0941\u0927\u092C\u093E\u0930_\u092C\u093F\u0939\u093F\u092C\u093E\u0930_\u0936\u0941\u0915\u094D\u0930\u092C\u093E\u0930_\u0936\u0928\u093F\u092C\u093E\u0930".split("_"),weekdaysShort:"\u0906\u0907\u0924._\u0938\u094B\u092E._\u092E\u0919\u094D\u0917\u0932._\u092C\u0941\u0927._\u092C\u093F\u0939\u093F._\u0936\u0941\u0915\u094D\u0930._\u0936\u0928\u093F.".split("_"),weekdaysMin:"\u0906._\u0938\u094B._\u092E\u0902._\u092C\u0941._\u092C\u093F._\u0936\u0941._\u0936.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"A\u0915\u094B h:mm \u092C\u091C\u0947",LTS:"A\u0915\u094B h:mm:ss \u092C\u091C\u0947",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A\u0915\u094B h:mm \u092C\u091C\u0947",LLLL:"dddd, D MMMM YYYY, A\u0915\u094B h:mm \u092C\u091C\u0947"},preparse:function(n){return n.replace(/[१२३४५६७८९०]/g,function(d){return r[d]})},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]})},meridiemParse:/राति|बिहान|दिउँसो|साँझ/,meridiemHour:function(n,d){if(n===12&&(n=0),d==="\u0930\u093E\u0924\u093F")return n<4?n:n+12;if(d==="\u092C\u093F\u0939\u093E\u0928")return n;if(d==="\u0926\u093F\u0909\u0901\u0938\u094B")return n>=10?n:n+12;if(d==="\u0938\u093E\u0901\u091D")return n+12},meridiem:function(n,d,u){return n<3?"\u0930\u093E\u0924\u093F":n<12?"\u092C\u093F\u0939\u093E\u0928":n<16?"\u0926\u093F\u0909\u0901\u0938\u094B":n<20?"\u0938\u093E\u0901\u091D":"\u0930\u093E\u0924\u093F"},calendar:{sameDay:"[\u0906\u091C] LT",nextDay:"[\u092D\u094B\u0932\u093F] LT",nextWeek:"[\u0906\u0909\u0901\u0926\u094B] dddd[,] LT",lastDay:"[\u0939\u093F\u091C\u094B] LT",lastWeek:"[\u0917\u090F\u0915\u094B] dddd[,] LT",sameElse:"L"},relativeTime:{future:"%s\u092E\u093E",past:"%s \u0905\u0917\u093E\u0921\u093F",s:"\u0915\u0947\u0939\u0940 \u0915\u094D\u0937\u0923",ss:"%d \u0938\u0947\u0915\u0947\u0923\u094D\u0921",m:"\u090F\u0915 \u092E\u093F\u0928\u0947\u091F",mm:"%d \u092E\u093F\u0928\u0947\u091F",h:"\u090F\u0915 \u0918\u0923\u094D\u091F\u093E",hh:"%d \u0918\u0923\u094D\u091F\u093E",d:"\u090F\u0915 \u0926\u093F\u0928",dd:"%d \u0926\u093F\u0928",M:"\u090F\u0915 \u092E\u0939\u093F\u0928\u093E",MM:"%d \u092E\u0939\u093F\u0928\u093E",y:"\u090F\u0915 \u092C\u0930\u094D\u0937",yy:"%d \u092C\u0930\u094D\u0937"},week:{dow:0,doy:6}});return s})},43784(L,f,c){//! moment.js locale configuration
//! locale : Dutch (Belgium) [nl-be]
//! author : Joris Röling : https://github.com/jorisroling
//! author : Jacob Middag : https://github.com/middagj
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.".split("_"),r="jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec".split("_"),s=[/^jan/i,/^feb/i,/^(maart|mrt\.?)$/i,/^apr/i,/^mei$/i,/^jun[i.]?$/i,/^jul[i.]?$/i,/^aug/i,/^sep/i,/^okt/i,/^nov/i,/^dec/i],n=/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,d=o.defineLocale("nl-be",{months:"januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december".split("_"),monthsShort:function(u,m){return u?/-MMM-/.test(m)?r[u.month()]:a[u.month()]:a},monthsRegex:n,monthsShortRegex:n,monthsStrictRegex:/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,monthsShortStrictRegex:/^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,monthsParse:s,longMonthsParse:s,shortMonthsParse:s,weekdays:"zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag".split("_"),weekdaysShort:"zo._ma._di._wo._do._vr._za.".split("_"),weekdaysMin:"zo_ma_di_wo_do_vr_za".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[vandaag om] LT",nextDay:"[morgen om] LT",nextWeek:"dddd [om] LT",lastDay:"[gisteren om] LT",lastWeek:"[afgelopen] dddd [om] LT",sameElse:"L"},relativeTime:{future:"over %s",past:"%s geleden",s:"een paar seconden",ss:"%d seconden",m:"\xE9\xE9n minuut",mm:"%d minuten",h:"\xE9\xE9n uur",hh:"%d uur",d:"\xE9\xE9n dag",dd:"%d dagen",M:"\xE9\xE9n maand",MM:"%d maanden",y:"\xE9\xE9n jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(u){return u+(u===1||u===8||u>=20?"ste":"de")},week:{dow:1,doy:4}});return d})},92572(L,f,c){//! moment.js locale configuration
//! locale : Dutch [nl]
//! author : Joris Röling : https://github.com/jorisroling
//! author : Jacob Middag : https://github.com/middagj
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="jan._feb._mrt._apr._mei_jun._jul._aug._sep._okt._nov._dec.".split("_"),r="jan_feb_mrt_apr_mei_jun_jul_aug_sep_okt_nov_dec".split("_"),s=[/^jan/i,/^feb/i,/^(maart|mrt\.?)$/i,/^apr/i,/^mei$/i,/^jun[i.]?$/i,/^jul[i.]?$/i,/^aug/i,/^sep/i,/^okt/i,/^nov/i,/^dec/i],n=/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december|jan\.?|feb\.?|mrt\.?|apr\.?|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,d=o.defineLocale("nl",{months:"januari_februari_maart_april_mei_juni_juli_augustus_september_oktober_november_december".split("_"),monthsShort:function(u,m){return u?/-MMM-/.test(m)?r[u.month()]:a[u.month()]:a},monthsRegex:n,monthsShortRegex:n,monthsStrictRegex:/^(januari|februari|maart|april|mei|ju[nl]i|augustus|september|oktober|november|december)/i,monthsShortStrictRegex:/^(jan\.?|feb\.?|mrt\.?|apr\.?|mei|ju[nl]\.?|aug\.?|sep\.?|okt\.?|nov\.?|dec\.?)/i,monthsParse:s,longMonthsParse:s,shortMonthsParse:s,weekdays:"zondag_maandag_dinsdag_woensdag_donderdag_vrijdag_zaterdag".split("_"),weekdaysShort:"zo._ma._di._wo._do._vr._za.".split("_"),weekdaysMin:"zo_ma_di_wo_do_vr_za".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD-MM-YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[vandaag om] LT",nextDay:"[morgen om] LT",nextWeek:"dddd [om] LT",lastDay:"[gisteren om] LT",lastWeek:"[afgelopen] dddd [om] LT",sameElse:"L"},relativeTime:{future:"over %s",past:"%s geleden",s:"een paar seconden",ss:"%d seconden",m:"\xE9\xE9n minuut",mm:"%d minuten",h:"\xE9\xE9n uur",hh:"%d uur",d:"\xE9\xE9n dag",dd:"%d dagen",w:"\xE9\xE9n week",ww:"%d weken",M:"\xE9\xE9n maand",MM:"%d maanden",y:"\xE9\xE9n jaar",yy:"%d jaar"},dayOfMonthOrdinalParse:/\d{1,2}(ste|de)/,ordinal:function(u){return u+(u===1||u===8||u>=20?"ste":"de")},week:{dow:1,doy:4}});return d})},54566(L,f,c){//! moment.js locale configuration
//! locale : Nynorsk [nn]
//! authors : https://github.com/mechuwind
//!           Stephen Ramthun : https://github.com/stephenramthun
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("nn",{months:"januar_februar_mars_april_mai_juni_juli_august_september_oktober_november_desember".split("_"),monthsShort:"jan._feb._mars_apr._mai_juni_juli_aug._sep._okt._nov._des.".split("_"),monthsParseExact:!0,weekdays:"sundag_m\xE5ndag_tysdag_onsdag_torsdag_fredag_laurdag".split("_"),weekdaysShort:"su._m\xE5._ty._on._to._fr._lau.".split("_"),weekdaysMin:"su_m\xE5_ty_on_to_fr_la".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY [kl.] H:mm",LLLL:"dddd D. MMMM YYYY [kl.] HH:mm"},calendar:{sameDay:"[I dag klokka] LT",nextDay:"[I morgon klokka] LT",nextWeek:"dddd [klokka] LT",lastDay:"[I g\xE5r klokka] LT",lastWeek:"[F\xF8reg\xE5ande] dddd [klokka] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"%s sidan",s:"nokre sekund",ss:"%d sekund",m:"eit minutt",mm:"%d minutt",h:"ein time",hh:"%d timar",d:"ein dag",dd:"%d dagar",w:"ei veke",ww:"%d veker",M:"ein m\xE5nad",MM:"%d m\xE5nader",y:"eit \xE5r",yy:"%d \xE5r"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a})},69330(L,f,c){//! moment.js locale configuration
//! locale : Occitan, lengadocian dialecte [oc-lnc]
//! author : Quentin PAGÈS : https://github.com/Quenty31
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("oc-lnc",{months:{standalone:"geni\xE8r_febri\xE8r_mar\xE7_abril_mai_junh_julhet_agost_setembre_oct\xF2bre_novembre_decembre".split("_"),format:"de geni\xE8r_de febri\xE8r_de mar\xE7_d'abril_de mai_de junh_de julhet_d'agost_de setembre_d'oct\xF2bre_de novembre_de decembre".split("_"),isFormat:/D[oD]?(\s)+MMMM/},monthsShort:"gen._febr._mar\xE7_abr._mai_junh_julh._ago._set._oct._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"dimenge_diluns_dimars_dim\xE8cres_dij\xF2us_divendres_dissabte".split("_"),weekdaysShort:"dg._dl._dm._dc._dj._dv._ds.".split("_"),weekdaysMin:"dg_dl_dm_dc_dj_dv_ds".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [de] YYYY",ll:"D MMM YYYY",LLL:"D MMMM [de] YYYY [a] H:mm",lll:"D MMM YYYY, H:mm",LLLL:"dddd D MMMM [de] YYYY [a] H:mm",llll:"ddd D MMM YYYY, H:mm"},calendar:{sameDay:"[u\xE8i a] LT",nextDay:"[deman a] LT",nextWeek:"dddd [a] LT",lastDay:"[i\xE8r a] LT",lastWeek:"dddd [passat a] LT",sameElse:"L"},relativeTime:{future:"d'aqu\xED %s",past:"fa %s",s:"unas segondas",ss:"%d segondas",m:"una minuta",mm:"%d minutas",h:"una ora",hh:"%d oras",d:"un jorn",dd:"%d jorns",M:"un mes",MM:"%d meses",y:"un an",yy:"%d ans"},dayOfMonthOrdinalParse:/\d{1,2}(r|n|t|è|a)/,ordinal:function(r,s){var n=r===1?"r":r===2?"n":r===3?"r":r===4?"t":"\xE8";return(s==="w"||s==="W")&&(n="a"),r+n},week:{dow:1,doy:4}});return a})},29849(L,f,c){//! moment.js locale configuration
//! locale : Punjabi (India) [pa-in]
//! author : Harpreet Singh : https://github.com/harpreetkhalsagtbit
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0A67",2:"\u0A68",3:"\u0A69",4:"\u0A6A",5:"\u0A6B",6:"\u0A6C",7:"\u0A6D",8:"\u0A6E",9:"\u0A6F",0:"\u0A66"},r={"\u0A67":"1","\u0A68":"2","\u0A69":"3","\u0A6A":"4","\u0A6B":"5","\u0A6C":"6","\u0A6D":"7","\u0A6E":"8","\u0A6F":"9","\u0A66":"0"},s=o.defineLocale("pa-in",{months:"\u0A1C\u0A28\u0A35\u0A30\u0A40_\u0A2B\u0A3C\u0A30\u0A35\u0A30\u0A40_\u0A2E\u0A3E\u0A30\u0A1A_\u0A05\u0A2A\u0A4D\u0A30\u0A48\u0A32_\u0A2E\u0A08_\u0A1C\u0A42\u0A28_\u0A1C\u0A41\u0A32\u0A3E\u0A08_\u0A05\u0A17\u0A38\u0A24_\u0A38\u0A24\u0A70\u0A2C\u0A30_\u0A05\u0A15\u0A24\u0A42\u0A2C\u0A30_\u0A28\u0A35\u0A70\u0A2C\u0A30_\u0A26\u0A38\u0A70\u0A2C\u0A30".split("_"),monthsShort:"\u0A1C\u0A28\u0A35\u0A30\u0A40_\u0A2B\u0A3C\u0A30\u0A35\u0A30\u0A40_\u0A2E\u0A3E\u0A30\u0A1A_\u0A05\u0A2A\u0A4D\u0A30\u0A48\u0A32_\u0A2E\u0A08_\u0A1C\u0A42\u0A28_\u0A1C\u0A41\u0A32\u0A3E\u0A08_\u0A05\u0A17\u0A38\u0A24_\u0A38\u0A24\u0A70\u0A2C\u0A30_\u0A05\u0A15\u0A24\u0A42\u0A2C\u0A30_\u0A28\u0A35\u0A70\u0A2C\u0A30_\u0A26\u0A38\u0A70\u0A2C\u0A30".split("_"),weekdays:"\u0A10\u0A24\u0A35\u0A3E\u0A30_\u0A38\u0A4B\u0A2E\u0A35\u0A3E\u0A30_\u0A2E\u0A70\u0A17\u0A32\u0A35\u0A3E\u0A30_\u0A2C\u0A41\u0A27\u0A35\u0A3E\u0A30_\u0A35\u0A40\u0A30\u0A35\u0A3E\u0A30_\u0A38\u0A3C\u0A41\u0A71\u0A15\u0A30\u0A35\u0A3E\u0A30_\u0A38\u0A3C\u0A28\u0A40\u0A1A\u0A30\u0A35\u0A3E\u0A30".split("_"),weekdaysShort:"\u0A10\u0A24_\u0A38\u0A4B\u0A2E_\u0A2E\u0A70\u0A17\u0A32_\u0A2C\u0A41\u0A27_\u0A35\u0A40\u0A30_\u0A38\u0A3C\u0A41\u0A15\u0A30_\u0A38\u0A3C\u0A28\u0A40".split("_"),weekdaysMin:"\u0A10\u0A24_\u0A38\u0A4B\u0A2E_\u0A2E\u0A70\u0A17\u0A32_\u0A2C\u0A41\u0A27_\u0A35\u0A40\u0A30_\u0A38\u0A3C\u0A41\u0A15\u0A30_\u0A38\u0A3C\u0A28\u0A40".split("_"),longDateFormat:{LT:"A h:mm \u0A35\u0A1C\u0A47",LTS:"A h:mm:ss \u0A35\u0A1C\u0A47",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm \u0A35\u0A1C\u0A47",LLLL:"dddd, D MMMM YYYY, A h:mm \u0A35\u0A1C\u0A47"},calendar:{sameDay:"[\u0A05\u0A1C] LT",nextDay:"[\u0A15\u0A32] LT",nextWeek:"[\u0A05\u0A17\u0A32\u0A3E] dddd, LT",lastDay:"[\u0A15\u0A32] LT",lastWeek:"[\u0A2A\u0A3F\u0A1B\u0A32\u0A47] dddd, LT",sameElse:"L"},relativeTime:{future:"%s \u0A35\u0A3F\u0A71\u0A1A",past:"%s \u0A2A\u0A3F\u0A1B\u0A32\u0A47",s:"\u0A15\u0A41\u0A1D \u0A38\u0A15\u0A3F\u0A70\u0A1F",ss:"%d \u0A38\u0A15\u0A3F\u0A70\u0A1F",m:"\u0A07\u0A15 \u0A2E\u0A3F\u0A70\u0A1F",mm:"%d \u0A2E\u0A3F\u0A70\u0A1F",h:"\u0A07\u0A71\u0A15 \u0A18\u0A70\u0A1F\u0A3E",hh:"%d \u0A18\u0A70\u0A1F\u0A47",d:"\u0A07\u0A71\u0A15 \u0A26\u0A3F\u0A28",dd:"%d \u0A26\u0A3F\u0A28",M:"\u0A07\u0A71\u0A15 \u0A2E\u0A39\u0A40\u0A28\u0A3E",MM:"%d \u0A2E\u0A39\u0A40\u0A28\u0A47",y:"\u0A07\u0A71\u0A15 \u0A38\u0A3E\u0A32",yy:"%d \u0A38\u0A3E\u0A32"},preparse:function(n){return n.replace(/[੧੨੩੪੫੬੭੮੯੦]/g,function(d){return r[d]})},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]})},meridiemParse:/ਰਾਤ|ਸਵੇਰ|ਦੁਪਹਿਰ|ਸ਼ਾਮ/,meridiemHour:function(n,d){if(n===12&&(n=0),d==="\u0A30\u0A3E\u0A24")return n<4?n:n+12;if(d==="\u0A38\u0A35\u0A47\u0A30")return n;if(d==="\u0A26\u0A41\u0A2A\u0A39\u0A3F\u0A30")return n>=10?n:n+12;if(d==="\u0A38\u0A3C\u0A3E\u0A2E")return n+12},meridiem:function(n,d,u){return n<4?"\u0A30\u0A3E\u0A24":n<10?"\u0A38\u0A35\u0A47\u0A30":n<17?"\u0A26\u0A41\u0A2A\u0A39\u0A3F\u0A30":n<20?"\u0A38\u0A3C\u0A3E\u0A2E":"\u0A30\u0A3E\u0A24"},week:{dow:0,doy:6}});return s})},94418(L,f,c){//! moment.js locale configuration
//! locale : Polish [pl]
//! author : Rafal Hirsz : https://github.com/evoL
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="stycze\u0144_luty_marzec_kwiecie\u0144_maj_czerwiec_lipiec_sierpie\u0144_wrzesie\u0144_pa\u017Adziernik_listopad_grudzie\u0144".split("_"),r="stycznia_lutego_marca_kwietnia_maja_czerwca_lipca_sierpnia_wrze\u015Bnia_pa\u017Adziernika_listopada_grudnia".split("_"),s=[/^sty/i,/^lut/i,/^mar/i,/^kwi/i,/^maj/i,/^cze/i,/^lip/i,/^sie/i,/^wrz/i,/^paź/i,/^lis/i,/^gru/i];function n(m){return m%10<5&&m%10>1&&~~(m/10)%10!==1}function d(m,h,y){var g=m+" ";switch(y){case"ss":return g+(n(m)?"sekundy":"sekund");case"m":return h?"minuta":"minut\u0119";case"mm":return g+(n(m)?"minuty":"minut");case"h":return h?"godzina":"godzin\u0119";case"hh":return g+(n(m)?"godziny":"godzin");case"ww":return g+(n(m)?"tygodnie":"tygodni");case"MM":return g+(n(m)?"miesi\u0105ce":"miesi\u0119cy");case"yy":return g+(n(m)?"lata":"lat")}}var u=o.defineLocale("pl",{months:function(m,h){return m?/D MMMM/.test(h)?r[m.month()]:a[m.month()]:a},monthsShort:"sty_lut_mar_kwi_maj_cze_lip_sie_wrz_pa\u017A_lis_gru".split("_"),monthsParse:s,longMonthsParse:s,shortMonthsParse:s,weekdays:"niedziela_poniedzia\u0142ek_wtorek_\u015Broda_czwartek_pi\u0105tek_sobota".split("_"),weekdaysShort:"ndz_pon_wt_\u015Br_czw_pt_sob".split("_"),weekdaysMin:"Nd_Pn_Wt_\u015Ar_Cz_Pt_So".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Dzi\u015B o] LT",nextDay:"[Jutro o] LT",nextWeek:function(){switch(this.day()){case 0:return"[W niedziel\u0119 o] LT";case 2:return"[We wtorek o] LT";case 3:return"[W \u015Brod\u0119 o] LT";case 6:return"[W sobot\u0119 o] LT";default:return"[W] dddd [o] LT"}},lastDay:"[Wczoraj o] LT",lastWeek:function(){switch(this.day()){case 0:return"[W zesz\u0142\u0105 niedziel\u0119 o] LT";case 3:return"[W zesz\u0142\u0105 \u015Brod\u0119 o] LT";case 6:return"[W zesz\u0142\u0105 sobot\u0119 o] LT";default:return"[W zesz\u0142y] dddd [o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"%s temu",s:"kilka sekund",ss:d,m:d,mm:d,h:d,hh:d,d:"1 dzie\u0144",dd:"%d dni",w:"tydzie\u0144",ww:d,M:"miesi\u0105c",MM:d,y:"rok",yy:d},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return u})},48303(L,f,c){//! moment.js locale configuration
//! locale : Portuguese (Brazil) [pt-br]
//! author : Caio Ribeiro Pereira : https://github.com/caio-ribeiro-pereira
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("pt-br",{months:"janeiro_fevereiro_mar\xE7o_abril_maio_junho_julho_agosto_setembro_outubro_novembro_dezembro".split("_"),monthsShort:"jan_fev_mar_abr_mai_jun_jul_ago_set_out_nov_dez".split("_"),weekdays:"domingo_segunda-feira_ter\xE7a-feira_quarta-feira_quinta-feira_sexta-feira_s\xE1bado".split("_"),weekdaysShort:"dom_seg_ter_qua_qui_sex_s\xE1b".split("_"),weekdaysMin:"do_2\xAA_3\xAA_4\xAA_5\xAA_6\xAA_s\xE1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY [\xE0s] HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY [\xE0s] HH:mm"},calendar:{sameDay:"[Hoje \xE0s] LT",nextDay:"[Amanh\xE3 \xE0s] LT",nextWeek:"dddd [\xE0s] LT",lastDay:"[Ontem \xE0s] LT",lastWeek:function(){return this.day()===0||this.day()===6?"[\xDAltimo] dddd [\xE0s] LT":"[\xDAltima] dddd [\xE0s] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"h\xE1 %s",s:"poucos segundos",ss:"%d segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",M:"um m\xEAs",MM:"%d meses",y:"um ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%d\xBA",invalidDate:"Data inv\xE1lida"});return a})},79834(L,f,c){//! moment.js locale configuration
//! locale : Portuguese [pt]
//! author : Jefferson : https://github.com/jalex79
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("pt",{months:"janeiro_fevereiro_mar\xE7o_abril_maio_junho_julho_agosto_setembro_outubro_novembro_dezembro".split("_"),monthsShort:"jan_fev_mar_abr_mai_jun_jul_ago_set_out_nov_dez".split("_"),weekdays:"Domingo_Segunda-feira_Ter\xE7a-feira_Quarta-feira_Quinta-feira_Sexta-feira_S\xE1bado".split("_"),weekdaysShort:"Dom_Seg_Ter_Qua_Qui_Sex_S\xE1b".split("_"),weekdaysMin:"Do_2\xAA_3\xAA_4\xAA_5\xAA_6\xAA_S\xE1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D [de] MMMM [de] YYYY",LLL:"D [de] MMMM [de] YYYY HH:mm",LLLL:"dddd, D [de] MMMM [de] YYYY HH:mm"},calendar:{sameDay:"[Hoje \xE0s] LT",nextDay:"[Amanh\xE3 \xE0s] LT",nextWeek:"dddd [\xE0s] LT",lastDay:"[Ontem \xE0s] LT",lastWeek:function(){return this.day()===0||this.day()===6?"[\xDAltimo] dddd [\xE0s] LT":"[\xDAltima] dddd [\xE0s] LT"},sameElse:"L"},relativeTime:{future:"em %s",past:"h\xE1 %s",s:"segundos",ss:"%d segundos",m:"um minuto",mm:"%d minutos",h:"uma hora",hh:"%d horas",d:"um dia",dd:"%d dias",w:"uma semana",ww:"%d semanas",M:"um m\xEAs",MM:"%d meses",y:"um ano",yy:"%d anos"},dayOfMonthOrdinalParse:/\d{1,2}º/,ordinal:"%d\xBA",week:{dow:1,doy:4}});return a})},24457(L,f,c){//! moment.js locale configuration
//! locale : Romanian [ro]
//! author : Vlad Gurdiga : https://github.com/gurdiga
//! author : Valentin Agachi : https://github.com/avaly
//! author : Emanuel Cepoi : https://github.com/cepem
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(s,n,d){var u={ss:"secunde",mm:"minute",hh:"ore",dd:"zile",ww:"s\u0103pt\u0103m\xE2ni",MM:"luni",yy:"ani"},m=" ";return(s%100>=20||s>=100&&s%100===0)&&(m=" de "),s+m+u[d]}var r=o.defineLocale("ro",{months:"ianuarie_februarie_martie_aprilie_mai_iunie_iulie_august_septembrie_octombrie_noiembrie_decembrie".split("_"),monthsShort:"ian._feb._mart._apr._mai_iun._iul._aug._sept._oct._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"duminic\u0103_luni_mar\u021Bi_miercuri_joi_vineri_s\xE2mb\u0103t\u0103".split("_"),weekdaysShort:"Dum_Lun_Mar_Mie_Joi_Vin_S\xE2m".split("_"),weekdaysMin:"Du_Lu_Ma_Mi_Jo_Vi_S\xE2".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY H:mm",LLLL:"dddd, D MMMM YYYY H:mm"},calendar:{sameDay:"[azi la] LT",nextDay:"[m\xE2ine la] LT",nextWeek:"dddd [la] LT",lastDay:"[ieri la] LT",lastWeek:"[fosta] dddd [la] LT",sameElse:"L"},relativeTime:{future:"peste %s",past:"%s \xEEn urm\u0103",s:"c\xE2teva secunde",ss:a,m:"un minut",mm:a,h:"o or\u0103",hh:a,d:"o zi",dd:a,w:"o s\u0103pt\u0103m\xE2n\u0103",ww:a,M:"o lun\u0103",MM:a,y:"un an",yy:a},week:{dow:1,doy:7}});return r})},82271(L,f,c){//! moment.js locale configuration
//! locale : Russian [ru]
//! author : Viktorminator : https://github.com/Viktorminator
//! author : Menelion Elensúle : https://github.com/Oire
//! author : Коренберг Марк : https://github.com/socketpair
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(d,u){var m=d.split("_");return u%10===1&&u%100!==11?m[0]:u%10>=2&&u%10<=4&&(u%100<10||u%100>=20)?m[1]:m[2]}function r(d,u,m){var h={ss:u?"\u0441\u0435\u043A\u0443\u043D\u0434\u0430_\u0441\u0435\u043A\u0443\u043D\u0434\u044B_\u0441\u0435\u043A\u0443\u043D\u0434":"\u0441\u0435\u043A\u0443\u043D\u0434\u0443_\u0441\u0435\u043A\u0443\u043D\u0434\u044B_\u0441\u0435\u043A\u0443\u043D\u0434",mm:u?"\u043C\u0438\u043D\u0443\u0442\u0430_\u043C\u0438\u043D\u0443\u0442\u044B_\u043C\u0438\u043D\u0443\u0442":"\u043C\u0438\u043D\u0443\u0442\u0443_\u043C\u0438\u043D\u0443\u0442\u044B_\u043C\u0438\u043D\u0443\u0442",hh:"\u0447\u0430\u0441_\u0447\u0430\u0441\u0430_\u0447\u0430\u0441\u043E\u0432",dd:"\u0434\u0435\u043D\u044C_\u0434\u043D\u044F_\u0434\u043D\u0435\u0439",ww:"\u043D\u0435\u0434\u0435\u043B\u044F_\u043D\u0435\u0434\u0435\u043B\u0438_\u043D\u0435\u0434\u0435\u043B\u044C",MM:"\u043C\u0435\u0441\u044F\u0446_\u043C\u0435\u0441\u044F\u0446\u0430_\u043C\u0435\u0441\u044F\u0446\u0435\u0432",yy:"\u0433\u043E\u0434_\u0433\u043E\u0434\u0430_\u043B\u0435\u0442"};return m==="m"?u?"\u043C\u0438\u043D\u0443\u0442\u0430":"\u043C\u0438\u043D\u0443\u0442\u0443":d+" "+a(h[m],+d)}var s=[/^янв/i,/^фев/i,/^мар/i,/^апр/i,/^ма[йя]/i,/^июн/i,/^июл/i,/^авг/i,/^сен/i,/^окт/i,/^ноя/i,/^дек/i],n=o.defineLocale("ru",{months:{format:"\u044F\u043D\u0432\u0430\u0440\u044F_\u0444\u0435\u0432\u0440\u0430\u043B\u044F_\u043C\u0430\u0440\u0442\u0430_\u0430\u043F\u0440\u0435\u043B\u044F_\u043C\u0430\u044F_\u0438\u044E\u043D\u044F_\u0438\u044E\u043B\u044F_\u0430\u0432\u0433\u0443\u0441\u0442\u0430_\u0441\u0435\u043D\u0442\u044F\u0431\u0440\u044F_\u043E\u043A\u0442\u044F\u0431\u0440\u044F_\u043D\u043E\u044F\u0431\u0440\u044F_\u0434\u0435\u043A\u0430\u0431\u0440\u044F".split("_"),standalone:"\u044F\u043D\u0432\u0430\u0440\u044C_\u0444\u0435\u0432\u0440\u0430\u043B\u044C_\u043C\u0430\u0440\u0442_\u0430\u043F\u0440\u0435\u043B\u044C_\u043C\u0430\u0439_\u0438\u044E\u043D\u044C_\u0438\u044E\u043B\u044C_\u0430\u0432\u0433\u0443\u0441\u0442_\u0441\u0435\u043D\u0442\u044F\u0431\u0440\u044C_\u043E\u043A\u0442\u044F\u0431\u0440\u044C_\u043D\u043E\u044F\u0431\u0440\u044C_\u0434\u0435\u043A\u0430\u0431\u0440\u044C".split("_")},monthsShort:{format:"\u044F\u043D\u0432._\u0444\u0435\u0432\u0440._\u043C\u0430\u0440._\u0430\u043F\u0440._\u043C\u0430\u044F_\u0438\u044E\u043D\u044F_\u0438\u044E\u043B\u044F_\u0430\u0432\u0433._\u0441\u0435\u043D\u0442._\u043E\u043A\u0442._\u043D\u043E\u044F\u0431._\u0434\u0435\u043A.".split("_"),standalone:"\u044F\u043D\u0432._\u0444\u0435\u0432\u0440._\u043C\u0430\u0440\u0442_\u0430\u043F\u0440._\u043C\u0430\u0439_\u0438\u044E\u043D\u044C_\u0438\u044E\u043B\u044C_\u0430\u0432\u0433._\u0441\u0435\u043D\u0442._\u043E\u043A\u0442._\u043D\u043E\u044F\u0431._\u0434\u0435\u043A.".split("_")},weekdays:{standalone:"\u0432\u043E\u0441\u043A\u0440\u0435\u0441\u0435\u043D\u044C\u0435_\u043F\u043E\u043D\u0435\u0434\u0435\u043B\u044C\u043D\u0438\u043A_\u0432\u0442\u043E\u0440\u043D\u0438\u043A_\u0441\u0440\u0435\u0434\u0430_\u0447\u0435\u0442\u0432\u0435\u0440\u0433_\u043F\u044F\u0442\u043D\u0438\u0446\u0430_\u0441\u0443\u0431\u0431\u043E\u0442\u0430".split("_"),format:"\u0432\u043E\u0441\u043A\u0440\u0435\u0441\u0435\u043D\u044C\u0435_\u043F\u043E\u043D\u0435\u0434\u0435\u043B\u044C\u043D\u0438\u043A_\u0432\u0442\u043E\u0440\u043D\u0438\u043A_\u0441\u0440\u0435\u0434\u0443_\u0447\u0435\u0442\u0432\u0435\u0440\u0433_\u043F\u044F\u0442\u043D\u0438\u0446\u0443_\u0441\u0443\u0431\u0431\u043E\u0442\u0443".split("_"),isFormat:/\[ ?[Вв] ?(?:прошлую|следующую|эту)? ?] ?dddd/},weekdaysShort:"\u0432\u0441_\u043F\u043D_\u0432\u0442_\u0441\u0440_\u0447\u0442_\u043F\u0442_\u0441\u0431".split("_"),weekdaysMin:"\u0432\u0441_\u043F\u043D_\u0432\u0442_\u0441\u0440_\u0447\u0442_\u043F\u0442_\u0441\u0431".split("_"),monthsParse:s,longMonthsParse:s,shortMonthsParse:s,monthsRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,monthsShortRegex:/^(январ[ья]|янв\.?|феврал[ья]|февр?\.?|марта?|мар\.?|апрел[ья]|апр\.?|ма[йя]|июн[ья]|июн\.?|июл[ья]|июл\.?|августа?|авг\.?|сентябр[ья]|сент?\.?|октябр[ья]|окт\.?|ноябр[ья]|нояб?\.?|декабр[ья]|дек\.?)/i,monthsStrictRegex:/^(январ[яь]|феврал[яь]|марта?|апрел[яь]|ма[яй]|июн[яь]|июл[яь]|августа?|сентябр[яь]|октябр[яь]|ноябр[яь]|декабр[яь])/i,monthsShortStrictRegex:/^(янв\.|февр?\.|мар[т.]|апр\.|ма[яй]|июн[ья.]|июл[ья.]|авг\.|сент?\.|окт\.|нояб?\.|дек\.)/i,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY \u0433.",LLL:"D MMMM YYYY \u0433., H:mm",LLLL:"dddd, D MMMM YYYY \u0433., H:mm"},calendar:{sameDay:"[\u0421\u0435\u0433\u043E\u0434\u043D\u044F, \u0432] LT",nextDay:"[\u0417\u0430\u0432\u0442\u0440\u0430, \u0432] LT",lastDay:"[\u0412\u0447\u0435\u0440\u0430, \u0432] LT",nextWeek:function(d){if(d.week()!==this.week())switch(this.day()){case 0:return"[\u0412 \u0441\u043B\u0435\u0434\u0443\u044E\u0449\u0435\u0435] dddd, [\u0432] LT";case 1:case 2:case 4:return"[\u0412 \u0441\u043B\u0435\u0434\u0443\u044E\u0449\u0438\u0439] dddd, [\u0432] LT";case 3:case 5:case 6:return"[\u0412 \u0441\u043B\u0435\u0434\u0443\u044E\u0449\u0443\u044E] dddd, [\u0432] LT"}else return this.day()===2?"[\u0412\u043E] dddd, [\u0432] LT":"[\u0412] dddd, [\u0432] LT"},lastWeek:function(d){if(d.week()!==this.week())switch(this.day()){case 0:return"[\u0412 \u043F\u0440\u043E\u0448\u043B\u043E\u0435] dddd, [\u0432] LT";case 1:case 2:case 4:return"[\u0412 \u043F\u0440\u043E\u0448\u043B\u044B\u0439] dddd, [\u0432] LT";case 3:case 5:case 6:return"[\u0412 \u043F\u0440\u043E\u0448\u043B\u0443\u044E] dddd, [\u0432] LT"}else return this.day()===2?"[\u0412\u043E] dddd, [\u0432] LT":"[\u0412] dddd, [\u0432] LT"},sameElse:"L"},relativeTime:{future:"\u0447\u0435\u0440\u0435\u0437 %s",past:"%s \u043D\u0430\u0437\u0430\u0434",s:"\u043D\u0435\u0441\u043A\u043E\u043B\u044C\u043A\u043E \u0441\u0435\u043A\u0443\u043D\u0434",ss:r,m:r,mm:r,h:"\u0447\u0430\u0441",hh:r,d:"\u0434\u0435\u043D\u044C",dd:r,w:"\u043D\u0435\u0434\u0435\u043B\u044F",ww:r,M:"\u043C\u0435\u0441\u044F\u0446",MM:r,y:"\u0433\u043E\u0434",yy:r},meridiemParse:/ночи|утра|дня|вечера/i,isPM:function(d){return/^(дня|вечера)$/.test(d)},meridiem:function(d,u,m){return d<4?"\u043D\u043E\u0447\u0438":d<12?"\u0443\u0442\u0440\u0430":d<17?"\u0434\u043D\u044F":"\u0432\u0435\u0447\u0435\u0440\u0430"},dayOfMonthOrdinalParse:/\d{1,2}-(й|го|я)/,ordinal:function(d,u){switch(u){case"M":case"d":case"DDD":return d+"-\u0439";case"D":return d+"-\u0433\u043E";case"w":case"W":return d+"-\u044F";default:return d}},week:{dow:1,doy:4}});return n})},1221(L,f,c){//! moment.js locale configuration
//! locale : Sindhi [sd]
//! author : Narain Sagar : https://github.com/narainsagar
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=["\u062C\u0646\u0648\u0631\u064A","\u0641\u064A\u0628\u0631\u0648\u0631\u064A","\u0645\u0627\u0631\u0686","\u0627\u067E\u0631\u064A\u0644","\u0645\u0626\u064A","\u062C\u0648\u0646","\u062C\u0648\u0644\u0627\u0621\u0650","\u0622\u06AF\u0633\u067D","\u0633\u064A\u067E\u067D\u0645\u0628\u0631","\u0622\u06AA\u067D\u0648\u0628\u0631","\u0646\u0648\u0645\u0628\u0631","\u068A\u0633\u0645\u0628\u0631"],r=["\u0622\u0686\u0631","\u0633\u0648\u0645\u0631","\u0627\u06B1\u0627\u0631\u0648","\u0627\u0631\u0628\u0639","\u062E\u0645\u064A\u0633","\u062C\u0645\u0639","\u0687\u0646\u0687\u0631"],s=o.defineLocale("sd",{months:a,monthsShort:a,weekdays:r,weekdaysShort:r,weekdaysMin:r,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd\u060C D MMMM YYYY HH:mm"},meridiemParse:/صبح|شام/,isPM:function(n){return n==="\u0634\u0627\u0645"},meridiem:function(n,d,u){return n<12?"\u0635\u0628\u062D":"\u0634\u0627\u0645"},calendar:{sameDay:"[\u0627\u0684] LT",nextDay:"[\u0633\u0680\u0627\u06BB\u064A] LT",nextWeek:"dddd [\u0627\u06B3\u064A\u0646 \u0647\u0641\u062A\u064A \u062A\u064A] LT",lastDay:"[\u06AA\u0627\u0644\u0647\u0647] LT",lastWeek:"[\u06AF\u0632\u0631\u064A\u0644 \u0647\u0641\u062A\u064A] dddd [\u062A\u064A] LT",sameElse:"L"},relativeTime:{future:"%s \u067E\u0648\u0621",past:"%s \u0627\u06B3",s:"\u0686\u0646\u062F \u0633\u064A\u06AA\u0646\u068A",ss:"%d \u0633\u064A\u06AA\u0646\u068A",m:"\u0647\u06AA \u0645\u0646\u067D",mm:"%d \u0645\u0646\u067D",h:"\u0647\u06AA \u06AA\u0644\u0627\u06AA",hh:"%d \u06AA\u0644\u0627\u06AA",d:"\u0647\u06AA \u068F\u064A\u0646\u0647\u0646",dd:"%d \u068F\u064A\u0646\u0647\u0646",M:"\u0647\u06AA \u0645\u0647\u064A\u0646\u0648",MM:"%d \u0645\u0647\u064A\u0646\u0627",y:"\u0647\u06AA \u0633\u0627\u0644",yy:"%d \u0633\u0627\u0644"},preparse:function(n){return n.replace(/،/g,",")},postformat:function(n){return n.replace(/,/g,"\u060C")},week:{dow:1,doy:4}});return s})},33478(L,f,c){//! moment.js locale configuration
//! locale : Northern Sami [se]
//! authors : Bård Rolstad Henriksen : https://github.com/karamell
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("se",{months:"o\u0111\u0111ajagem\xE1nnu_guovvam\xE1nnu_njuk\u010Dam\xE1nnu_cuo\u014Bom\xE1nnu_miessem\xE1nnu_geassem\xE1nnu_suoidnem\xE1nnu_borgem\xE1nnu_\u010Dak\u010Dam\xE1nnu_golggotm\xE1nnu_sk\xE1bmam\xE1nnu_juovlam\xE1nnu".split("_"),monthsShort:"o\u0111\u0111j_guov_njuk_cuo_mies_geas_suoi_borg_\u010Dak\u010D_golg_sk\xE1b_juov".split("_"),weekdays:"sotnabeaivi_vuoss\xE1rga_ma\u014B\u014Beb\xE1rga_gaskavahkku_duorastat_bearjadat_l\xE1vvardat".split("_"),weekdaysShort:"sotn_vuos_ma\u014B_gask_duor_bear_l\xE1v".split("_"),weekdaysMin:"s_v_m_g_d_b_L".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"MMMM D. [b.] YYYY",LLL:"MMMM D. [b.] YYYY [ti.] HH:mm",LLLL:"dddd, MMMM D. [b.] YYYY [ti.] HH:mm"},calendar:{sameDay:"[otne ti] LT",nextDay:"[ihttin ti] LT",nextWeek:"dddd [ti] LT",lastDay:"[ikte ti] LT",lastWeek:"[ovddit] dddd [ti] LT",sameElse:"L"},relativeTime:{future:"%s gea\u017Ees",past:"ma\u014Bit %s",s:"moadde sekunddat",ss:"%d sekunddat",m:"okta minuhta",mm:"%d minuhtat",h:"okta diimmu",hh:"%d diimmut",d:"okta beaivi",dd:"%d beaivvit",M:"okta m\xE1nnu",MM:"%d m\xE1nut",y:"okta jahki",yy:"%d jagit"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a})},17538(L,f,c){//! moment.js locale configuration
//! locale : Sinhalese [si]
//! author : Sampath Sitinamaluwa : https://github.com/sampathsris
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("si",{months:"\u0DA2\u0DB1\u0DC0\u0DCF\u0DBB\u0DD2_\u0DB4\u0DD9\u0DB6\u0DBB\u0DC0\u0DCF\u0DBB\u0DD2_\u0DB8\u0DCF\u0DBB\u0DCA\u0DAD\u0DD4_\u0D85\u0DB4\u0DCA\u200D\u0DBB\u0DDA\u0DBD\u0DCA_\u0DB8\u0DD0\u0DBA\u0DD2_\u0DA2\u0DD6\u0DB1\u0DD2_\u0DA2\u0DD6\u0DBD\u0DD2_\u0D85\u0D9C\u0DDD\u0DC3\u0DCA\u0DAD\u0DD4_\u0DC3\u0DD0\u0DB4\u0DCA\u0DAD\u0DD0\u0DB8\u0DCA\u0DB6\u0DBB\u0DCA_\u0D94\u0D9A\u0DCA\u0DAD\u0DDD\u0DB6\u0DBB\u0DCA_\u0DB1\u0DDC\u0DC0\u0DD0\u0DB8\u0DCA\u0DB6\u0DBB\u0DCA_\u0DAF\u0DD9\u0DC3\u0DD0\u0DB8\u0DCA\u0DB6\u0DBB\u0DCA".split("_"),monthsShort:"\u0DA2\u0DB1_\u0DB4\u0DD9\u0DB6_\u0DB8\u0DCF\u0DBB\u0DCA_\u0D85\u0DB4\u0DCA_\u0DB8\u0DD0\u0DBA\u0DD2_\u0DA2\u0DD6\u0DB1\u0DD2_\u0DA2\u0DD6\u0DBD\u0DD2_\u0D85\u0D9C\u0DDD_\u0DC3\u0DD0\u0DB4\u0DCA_\u0D94\u0D9A\u0DCA_\u0DB1\u0DDC\u0DC0\u0DD0_\u0DAF\u0DD9\u0DC3\u0DD0".split("_"),weekdays:"\u0D89\u0DBB\u0DD2\u0DAF\u0DCF_\u0DC3\u0DB3\u0DD4\u0DAF\u0DCF_\u0D85\u0D9F\u0DC4\u0DBB\u0DD4\u0DC0\u0DCF\u0DAF\u0DCF_\u0DB6\u0DAF\u0DCF\u0DAF\u0DCF_\u0DB6\u0DCA\u200D\u0DBB\u0DC4\u0DC3\u0DCA\u0DB4\u0DAD\u0DD2\u0DB1\u0DCA\u0DAF\u0DCF_\u0DC3\u0DD2\u0D9A\u0DD4\u0DBB\u0DCF\u0DAF\u0DCF_\u0DC3\u0DD9\u0DB1\u0DC3\u0DD4\u0DBB\u0DCF\u0DAF\u0DCF".split("_"),weekdaysShort:"\u0D89\u0DBB\u0DD2_\u0DC3\u0DB3\u0DD4_\u0D85\u0D9F_\u0DB6\u0DAF\u0DCF_\u0DB6\u0DCA\u200D\u0DBB\u0DC4_\u0DC3\u0DD2\u0D9A\u0DD4_\u0DC3\u0DD9\u0DB1".split("_"),weekdaysMin:"\u0D89_\u0DC3_\u0D85_\u0DB6_\u0DB6\u0DCA\u200D\u0DBB_\u0DC3\u0DD2_\u0DC3\u0DD9".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"a h:mm",LTS:"a h:mm:ss",L:"YYYY/MM/DD",LL:"YYYY MMMM D",LLL:"YYYY MMMM D, a h:mm",LLLL:"YYYY MMMM D [\u0DC0\u0DD0\u0DB1\u0DD2] dddd, a h:mm:ss"},calendar:{sameDay:"[\u0D85\u0DAF] LT[\u0DA7]",nextDay:"[\u0DC4\u0DD9\u0DA7] LT[\u0DA7]",nextWeek:"dddd LT[\u0DA7]",lastDay:"[\u0D8A\u0DBA\u0DDA] LT[\u0DA7]",lastWeek:"[\u0DB4\u0DC3\u0DD4\u0D9C\u0DD2\u0DBA] dddd LT[\u0DA7]",sameElse:"L"},relativeTime:{future:"%s\u0D9A\u0DD2\u0DB1\u0DCA",past:"%s\u0D9A\u0DA7 \u0DB4\u0DD9\u0DBB",s:"\u0DAD\u0DAD\u0DCA\u0DB4\u0DBB \u0D9A\u0DD2\u0DC4\u0DD2\u0DB4\u0DBA",ss:"\u0DAD\u0DAD\u0DCA\u0DB4\u0DBB %d",m:"\u0DB8\u0DD2\u0DB1\u0DD2\u0DAD\u0DCA\u0DAD\u0DD4\u0DC0",mm:"\u0DB8\u0DD2\u0DB1\u0DD2\u0DAD\u0DCA\u0DAD\u0DD4 %d",h:"\u0DB4\u0DD0\u0DBA",hh:"\u0DB4\u0DD0\u0DBA %d",d:"\u0DAF\u0DD2\u0DB1\u0DBA",dd:"\u0DAF\u0DD2\u0DB1 %d",M:"\u0DB8\u0DCF\u0DC3\u0DBA",MM:"\u0DB8\u0DCF\u0DC3 %d",y:"\u0DC0\u0DC3\u0DBB",yy:"\u0DC0\u0DC3\u0DBB %d"},dayOfMonthOrdinalParse:/\d{1,2} වැනි/,ordinal:function(r){return r+" \u0DC0\u0DD0\u0DB1\u0DD2"},meridiemParse:/පෙර වරු|පස් වරු|පෙ.ව|ප.ව./,isPM:function(r){return r==="\u0DB4.\u0DC0."||r==="\u0DB4\u0DC3\u0DCA \u0DC0\u0DBB\u0DD4"},meridiem:function(r,s,n){return r>11?n?"\u0DB4.\u0DC0.":"\u0DB4\u0DC3\u0DCA \u0DC0\u0DBB\u0DD4":n?"\u0DB4\u0DD9.\u0DC0.":"\u0DB4\u0DD9\u0DBB \u0DC0\u0DBB\u0DD4"}});return a})},5784(L,f,c){//! moment.js locale configuration
//! locale : Slovak [sk]
//! author : Martin Minka : https://github.com/k2s
//! based on work of petrbela : https://github.com/petrbela
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="janu\xE1r_febru\xE1r_marec_apr\xEDl_m\xE1j_j\xFAn_j\xFAl_august_september_okt\xF3ber_november_december".split("_"),r="jan_feb_mar_apr_m\xE1j_j\xFAn_j\xFAl_aug_sep_okt_nov_dec".split("_");function s(u){return u>1&&u<5}function n(u,m,h,y){var g=u+" ";switch(h){case"s":return m||y?"p\xE1r sek\xFAnd":"p\xE1r sekundami";case"ss":return m||y?g+(s(u)?"sekundy":"sek\xFAnd"):g+"sekundami";case"m":return m?"min\xFAta":y?"min\xFAtu":"min\xFAtou";case"mm":return m||y?g+(s(u)?"min\xFAty":"min\xFAt"):g+"min\xFAtami";case"h":return m?"hodina":y?"hodinu":"hodinou";case"hh":return m||y?g+(s(u)?"hodiny":"hod\xEDn"):g+"hodinami";case"d":return m||y?"de\u0148":"d\u0148om";case"dd":return m||y?g+(s(u)?"dni":"dn\xED"):g+"d\u0148ami";case"M":return m||y?"mesiac":"mesiacom";case"MM":return m||y?g+(s(u)?"mesiace":"mesiacov"):g+"mesiacmi";case"y":return m||y?"rok":"rokom";case"yy":return m||y?g+(s(u)?"roky":"rokov"):g+"rokmi"}}var d=o.defineLocale("sk",{months:a,monthsShort:r,weekdays:"nede\u013Ea_pondelok_utorok_streda_\u0161tvrtok_piatok_sobota".split("_"),weekdaysShort:"ne_po_ut_st_\u0161t_pi_so".split("_"),weekdaysMin:"ne_po_ut_st_\u0161t_pi_so".split("_"),longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD.MM.YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd D. MMMM YYYY H:mm"},calendar:{sameDay:"[dnes o] LT",nextDay:"[zajtra o] LT",nextWeek:function(){switch(this.day()){case 0:return"[v nede\u013Eu o] LT";case 1:case 2:return"[v] dddd [o] LT";case 3:return"[v stredu o] LT";case 4:return"[vo \u0161tvrtok o] LT";case 5:return"[v piatok o] LT";case 6:return"[v sobotu o] LT"}},lastDay:"[v\u010Dera o] LT",lastWeek:function(){switch(this.day()){case 0:return"[minul\xFA nede\u013Eu o] LT";case 1:case 2:return"[minul\xFD] dddd [o] LT";case 3:return"[minul\xFA stredu o] LT";case 4:case 5:return"[minul\xFD] dddd [o] LT";case 6:return"[minul\xFA sobotu o] LT"}},sameElse:"L"},relativeTime:{future:"za %s",past:"pred %s",s:n,ss:n,m:n,mm:n,h:n,hh:n,d:n,dd:n,M:n,MM:n,y:n,yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return d})},46637(L,f,c){//! moment.js locale configuration
//! locale : Slovenian [sl]
//! author : Robert Sedovšek : https://github.com/sedovsek
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(s,n,d,u){var m=s+" ";switch(d){case"s":return n||u?"nekaj sekund":"nekaj sekundami";case"ss":return s===1?m+=n?"sekundo":"sekundi":s===2?m+=n||u?"sekundi":"sekundah":s<5?m+=n||u?"sekunde":"sekundah":m+="sekund",m;case"m":return n?"ena minuta":"eno minuto";case"mm":return s===1?m+=n?"minuta":"minuto":s===2?m+=n||u?"minuti":"minutama":s<5?m+=n||u?"minute":"minutami":m+=n||u?"minut":"minutami",m;case"h":return n?"ena ura":"eno uro";case"hh":return s===1?m+=n?"ura":"uro":s===2?m+=n||u?"uri":"urama":s<5?m+=n||u?"ure":"urami":m+=n||u?"ur":"urami",m;case"d":return n||u?"en dan":"enim dnem";case"dd":return s===1?m+=n||u?"dan":"dnem":s===2?m+=n||u?"dni":"dnevoma":m+=n||u?"dni":"dnevi",m;case"M":return n||u?"en mesec":"enim mesecem";case"MM":return s===1?m+=n||u?"mesec":"mesecem":s===2?m+=n||u?"meseca":"mesecema":s<5?m+=n||u?"mesece":"meseci":m+=n||u?"mesecev":"meseci",m;case"y":return n||u?"eno leto":"enim letom";case"yy":return s===1?m+=n||u?"leto":"letom":s===2?m+=n||u?"leti":"letoma":s<5?m+=n||u?"leta":"leti":m+=n||u?"let":"leti",m}}var r=o.defineLocale("sl",{months:"januar_februar_marec_april_maj_junij_julij_avgust_september_oktober_november_december".split("_"),monthsShort:"jan._feb._mar._apr._maj._jun._jul._avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljek_torek_sreda_\u010Detrtek_petek_sobota".split("_"),weekdaysShort:"ned._pon._tor._sre._\u010Det._pet._sob.".split("_"),weekdaysMin:"ne_po_to_sr_\u010De_pe_so".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD. MM. YYYY",LL:"D. MMMM YYYY",LLL:"D. MMMM YYYY H:mm",LLLL:"dddd, D. MMMM YYYY H:mm"},calendar:{sameDay:"[danes ob] LT",nextDay:"[jutri ob] LT",nextWeek:function(){switch(this.day()){case 0:return"[v] [nedeljo] [ob] LT";case 3:return"[v] [sredo] [ob] LT";case 6:return"[v] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[v] dddd [ob] LT"}},lastDay:"[v\u010Deraj ob] LT",lastWeek:function(){switch(this.day()){case 0:return"[prej\u0161njo] [nedeljo] [ob] LT";case 3:return"[prej\u0161njo] [sredo] [ob] LT";case 6:return"[prej\u0161njo] [soboto] [ob] LT";case 1:case 2:case 4:case 5:return"[prej\u0161nji] dddd [ob] LT"}},sameElse:"L"},relativeTime:{future:"\u010Dez %s",past:"pred %s",s:a,ss:a,m:a,mm:a,h:a,hh:a,d:a,dd:a,M:a,MM:a,y:a,yy:a},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return r})},86794(L,f,c){//! moment.js locale configuration
//! locale : Albanian [sq]
//! author : Flakërim Ismani : https://github.com/flakerimi
//! author : Menelion Elensúle : https://github.com/Oire
//! author : Oerd Cukalla : https://github.com/oerd
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("sq",{months:"Janar_Shkurt_Mars_Prill_Maj_Qershor_Korrik_Gusht_Shtator_Tetor_N\xEBntor_Dhjetor".split("_"),monthsShort:"Jan_Shk_Mar_Pri_Maj_Qer_Kor_Gus_Sht_Tet_N\xEBn_Dhj".split("_"),weekdays:"E Diel_E H\xEBn\xEB_E Mart\xEB_E M\xEBrkur\xEB_E Enjte_E Premte_E Shtun\xEB".split("_"),weekdaysShort:"Die_H\xEBn_Mar_M\xEBr_Enj_Pre_Sht".split("_"),weekdaysMin:"D_H_Ma_M\xEB_E_P_Sh".split("_"),weekdaysParseExact:!0,meridiemParse:/PD|MD/,isPM:function(r){return r.charAt(0)==="M"},meridiem:function(r,s,n){return r<12?"PD":"MD"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Sot n\xEB] LT",nextDay:"[Nes\xEBr n\xEB] LT",nextWeek:"dddd [n\xEB] LT",lastDay:"[Dje n\xEB] LT",lastWeek:"dddd [e kaluar n\xEB] LT",sameElse:"L"},relativeTime:{future:"n\xEB %s",past:"%s m\xEB par\xEB",s:"disa sekonda",ss:"%d sekonda",m:"nj\xEB minut\xEB",mm:"%d minuta",h:"nj\xEB or\xEB",hh:"%d or\xEB",d:"nj\xEB dit\xEB",dd:"%d dit\xEB",M:"nj\xEB muaj",MM:"%d muaj",y:"nj\xEB vit",yy:"%d vite"},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return a})},3322(L,f,c){//! moment.js locale configuration
//! locale : Serbian Cyrillic [sr-cyrl]
//! author : Milan Janačković<milanjanackovic@gmail.com> : https://github.com/milan-j
//! author : Stefan Crnjaković <stefan@hotmail.rs> : https://github.com/crnjakovic
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={words:{ss:["\u0441\u0435\u043A\u0443\u043D\u0434\u0430","\u0441\u0435\u043A\u0443\u043D\u0434\u0435","\u0441\u0435\u043A\u0443\u043D\u0434\u0438"],m:["\u0458\u0435\u0434\u0430\u043D \u043C\u0438\u043D\u0443\u0442","\u0458\u0435\u0434\u043D\u043E\u0433 \u043C\u0438\u043D\u0443\u0442\u0430"],mm:["\u043C\u0438\u043D\u0443\u0442","\u043C\u0438\u043D\u0443\u0442\u0430","\u043C\u0438\u043D\u0443\u0442\u0430"],h:["\u0458\u0435\u0434\u0430\u043D \u0441\u0430\u0442","\u0458\u0435\u0434\u043D\u043E\u0433 \u0441\u0430\u0442\u0430"],hh:["\u0441\u0430\u0442","\u0441\u0430\u0442\u0430","\u0441\u0430\u0442\u0438"],d:["\u0458\u0435\u0434\u0430\u043D \u0434\u0430\u043D","\u0458\u0435\u0434\u043D\u043E\u0433 \u0434\u0430\u043D\u0430"],dd:["\u0434\u0430\u043D","\u0434\u0430\u043D\u0430","\u0434\u0430\u043D\u0430"],M:["\u0458\u0435\u0434\u0430\u043D \u043C\u0435\u0441\u0435\u0446","\u0458\u0435\u0434\u043D\u043E\u0433 \u043C\u0435\u0441\u0435\u0446\u0430"],MM:["\u043C\u0435\u0441\u0435\u0446","\u043C\u0435\u0441\u0435\u0446\u0430","\u043C\u0435\u0441\u0435\u0446\u0438"],y:["\u0458\u0435\u0434\u043D\u0443 \u0433\u043E\u0434\u0438\u043D\u0443","\u0458\u0435\u0434\u043D\u0435 \u0433\u043E\u0434\u0438\u043D\u0435"],yy:["\u0433\u043E\u0434\u0438\u043D\u0443","\u0433\u043E\u0434\u0438\u043D\u0435","\u0433\u043E\u0434\u0438\u043D\u0430"]},correctGrammaticalCase:function(s,n){return s%10>=1&&s%10<=4&&(s%100<10||s%100>=20)?s%10===1?n[0]:n[1]:n[2]},translate:function(s,n,d,u){var m=a.words[d],h;return d.length===1?d==="y"&&n?"\u0458\u0435\u0434\u043D\u0430 \u0433\u043E\u0434\u0438\u043D\u0430":u||n?m[0]:m[1]:(h=a.correctGrammaticalCase(s,m),d==="yy"&&n&&h==="\u0433\u043E\u0434\u0438\u043D\u0443"?s+" \u0433\u043E\u0434\u0438\u043D\u0430":s+" "+h)}},r=o.defineLocale("sr-cyrl",{months:"\u0458\u0430\u043D\u0443\u0430\u0440_\u0444\u0435\u0431\u0440\u0443\u0430\u0440_\u043C\u0430\u0440\u0442_\u0430\u043F\u0440\u0438\u043B_\u043C\u0430\u0458_\u0458\u0443\u043D_\u0458\u0443\u043B_\u0430\u0432\u0433\u0443\u0441\u0442_\u0441\u0435\u043F\u0442\u0435\u043C\u0431\u0430\u0440_\u043E\u043A\u0442\u043E\u0431\u0430\u0440_\u043D\u043E\u0432\u0435\u043C\u0431\u0430\u0440_\u0434\u0435\u0446\u0435\u043C\u0431\u0430\u0440".split("_"),monthsShort:"\u0458\u0430\u043D._\u0444\u0435\u0431._\u043C\u0430\u0440._\u0430\u043F\u0440._\u043C\u0430\u0458_\u0458\u0443\u043D_\u0458\u0443\u043B_\u0430\u0432\u0433._\u0441\u0435\u043F._\u043E\u043A\u0442._\u043D\u043E\u0432._\u0434\u0435\u0446.".split("_"),monthsParseExact:!0,weekdays:"\u043D\u0435\u0434\u0435\u0459\u0430_\u043F\u043E\u043D\u0435\u0434\u0435\u0459\u0430\u043A_\u0443\u0442\u043E\u0440\u0430\u043A_\u0441\u0440\u0435\u0434\u0430_\u0447\u0435\u0442\u0432\u0440\u0442\u0430\u043A_\u043F\u0435\u0442\u0430\u043A_\u0441\u0443\u0431\u043E\u0442\u0430".split("_"),weekdaysShort:"\u043D\u0435\u0434._\u043F\u043E\u043D._\u0443\u0442\u043E._\u0441\u0440\u0435._\u0447\u0435\u0442._\u043F\u0435\u0442._\u0441\u0443\u0431.".split("_"),weekdaysMin:"\u043D\u0435_\u043F\u043E_\u0443\u0442_\u0441\u0440_\u0447\u0435_\u043F\u0435_\u0441\u0443".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D. M. YYYY.",LL:"D. MMMM YYYY.",LLL:"D. MMMM YYYY. H:mm",LLLL:"dddd, D. MMMM YYYY. H:mm"},calendar:{sameDay:"[\u0434\u0430\u043D\u0430\u0441 \u0443] LT",nextDay:"[\u0441\u0443\u0442\u0440\u0430 \u0443] LT",nextWeek:function(){switch(this.day()){case 0:return"[\u0443] [\u043D\u0435\u0434\u0435\u0459\u0443] [\u0443] LT";case 3:return"[\u0443] [\u0441\u0440\u0435\u0434\u0443] [\u0443] LT";case 6:return"[\u0443] [\u0441\u0443\u0431\u043E\u0442\u0443] [\u0443] LT";case 1:case 2:case 4:case 5:return"[\u0443] dddd [\u0443] LT"}},lastDay:"[\u0458\u0443\u0447\u0435 \u0443] LT",lastWeek:function(){var s=["[\u043F\u0440\u043E\u0448\u043B\u0435] [\u043D\u0435\u0434\u0435\u0459\u0435] [\u0443] LT","[\u043F\u0440\u043E\u0448\u043B\u043E\u0433] [\u043F\u043E\u043D\u0435\u0434\u0435\u0459\u043A\u0430] [\u0443] LT","[\u043F\u0440\u043E\u0448\u043B\u043E\u0433] [\u0443\u0442\u043E\u0440\u043A\u0430] [\u0443] LT","[\u043F\u0440\u043E\u0448\u043B\u0435] [\u0441\u0440\u0435\u0434\u0435] [\u0443] LT","[\u043F\u0440\u043E\u0448\u043B\u043E\u0433] [\u0447\u0435\u0442\u0432\u0440\u0442\u043A\u0430] [\u0443] LT","[\u043F\u0440\u043E\u0448\u043B\u043E\u0433] [\u043F\u0435\u0442\u043A\u0430] [\u0443] LT","[\u043F\u0440\u043E\u0448\u043B\u0435] [\u0441\u0443\u0431\u043E\u0442\u0435] [\u0443] LT"];return s[this.day()]},sameElse:"L"},relativeTime:{future:"\u0437\u0430 %s",past:"\u043F\u0440\u0435 %s",s:"\u043D\u0435\u043A\u043E\u043B\u0438\u043A\u043E \u0441\u0435\u043A\u0443\u043D\u0434\u0438",ss:a.translate,m:a.translate,mm:a.translate,h:a.translate,hh:a.translate,d:a.translate,dd:a.translate,M:a.translate,MM:a.translate,y:a.translate,yy:a.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return r})},45719(L,f,c){//! moment.js locale configuration
//! locale : Serbian [sr]
//! author : Milan Janačković<milanjanackovic@gmail.com> : https://github.com/milan-j
//! author : Stefan Crnjaković <stefan@hotmail.rs> : https://github.com/crnjakovic
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={words:{ss:["sekunda","sekunde","sekundi"],m:["jedan minut","jednog minuta"],mm:["minut","minuta","minuta"],h:["jedan sat","jednog sata"],hh:["sat","sata","sati"],d:["jedan dan","jednog dana"],dd:["dan","dana","dana"],M:["jedan mesec","jednog meseca"],MM:["mesec","meseca","meseci"],y:["jednu godinu","jedne godine"],yy:["godinu","godine","godina"]},correctGrammaticalCase:function(s,n){return s%10>=1&&s%10<=4&&(s%100<10||s%100>=20)?s%10===1?n[0]:n[1]:n[2]},translate:function(s,n,d,u){var m=a.words[d],h;return d.length===1?d==="y"&&n?"jedna godina":u||n?m[0]:m[1]:(h=a.correctGrammaticalCase(s,m),d==="yy"&&n&&h==="godinu"?s+" godina":s+" "+h)}},r=o.defineLocale("sr",{months:"januar_februar_mart_april_maj_jun_jul_avgust_septembar_oktobar_novembar_decembar".split("_"),monthsShort:"jan._feb._mar._apr._maj_jun_jul_avg._sep._okt._nov._dec.".split("_"),monthsParseExact:!0,weekdays:"nedelja_ponedeljak_utorak_sreda_\u010Detvrtak_petak_subota".split("_"),weekdaysShort:"ned._pon._uto._sre._\u010Det._pet._sub.".split("_"),weekdaysMin:"ne_po_ut_sr_\u010De_pe_su".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"D. M. YYYY.",LL:"D. MMMM YYYY.",LLL:"D. MMMM YYYY. H:mm",LLLL:"dddd, D. MMMM YYYY. H:mm"},calendar:{sameDay:"[danas u] LT",nextDay:"[sutra u] LT",nextWeek:function(){switch(this.day()){case 0:return"[u] [nedelju] [u] LT";case 3:return"[u] [sredu] [u] LT";case 6:return"[u] [subotu] [u] LT";case 1:case 2:case 4:case 5:return"[u] dddd [u] LT"}},lastDay:"[ju\u010De u] LT",lastWeek:function(){var s=["[pro\u0161le] [nedelje] [u] LT","[pro\u0161log] [ponedeljka] [u] LT","[pro\u0161log] [utorka] [u] LT","[pro\u0161le] [srede] [u] LT","[pro\u0161log] [\u010Detvrtka] [u] LT","[pro\u0161log] [petka] [u] LT","[pro\u0161le] [subote] [u] LT"];return s[this.day()]},sameElse:"L"},relativeTime:{future:"za %s",past:"pre %s",s:"nekoliko sekundi",ss:a.translate,m:a.translate,mm:a.translate,h:a.translate,hh:a.translate,d:a.translate,dd:a.translate,M:a.translate,MM:a.translate,y:a.translate,yy:a.translate},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:7}});return r})},56e3(L,f,c){//! moment.js locale configuration
//! locale : siSwati [ss]
//! author : Nicolai Davies<mail@nicolai.io> : https://github.com/nicolaidavies
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ss",{months:"Bhimbidvwane_Indlovana_Indlov'lenkhulu_Mabasa_Inkhwekhweti_Inhlaba_Kholwane_Ingci_Inyoni_Imphala_Lweti_Ingongoni".split("_"),monthsShort:"Bhi_Ina_Inu_Mab_Ink_Inh_Kho_Igc_Iny_Imp_Lwe_Igo".split("_"),weekdays:"Lisontfo_Umsombuluko_Lesibili_Lesitsatfu_Lesine_Lesihlanu_Umgcibelo".split("_"),weekdaysShort:"Lis_Umb_Lsb_Les_Lsi_Lsh_Umg".split("_"),weekdaysMin:"Li_Us_Lb_Lt_Ls_Lh_Ug".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[Namuhla nga] LT",nextDay:"[Kusasa nga] LT",nextWeek:"dddd [nga] LT",lastDay:"[Itolo nga] LT",lastWeek:"dddd [leliphelile] [nga] LT",sameElse:"L"},relativeTime:{future:"nga %s",past:"wenteka nga %s",s:"emizuzwana lomcane",ss:"%d mzuzwana",m:"umzuzu",mm:"%d emizuzu",h:"lihora",hh:"%d emahora",d:"lilanga",dd:"%d emalanga",M:"inyanga",MM:"%d tinyanga",y:"umnyaka",yy:"%d iminyaka"},meridiemParse:/ekuseni|emini|entsambama|ebusuku/,meridiem:function(r,s,n){return r<11?"ekuseni":r<15?"emini":r<19?"entsambama":"ebusuku"},meridiemHour:function(r,s){if(r===12&&(r=0),s==="ekuseni")return r;if(s==="emini")return r>=11?r:r+12;if(s==="entsambama"||s==="ebusuku")return r===0?0:r+12},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:"%d",week:{dow:1,doy:4}});return a})},41011(L,f,c){//! moment.js locale configuration
//! locale : Swedish [sv]
//! author : Jens Alm : https://github.com/ulmus
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("sv",{months:"januari_februari_mars_april_maj_juni_juli_augusti_september_oktober_november_december".split("_"),monthsShort:"jan_feb_mar_apr_maj_jun_jul_aug_sep_okt_nov_dec".split("_"),weekdays:"s\xF6ndag_m\xE5ndag_tisdag_onsdag_torsdag_fredag_l\xF6rdag".split("_"),weekdaysShort:"s\xF6n_m\xE5n_tis_ons_tor_fre_l\xF6r".split("_"),weekdaysMin:"s\xF6_m\xE5_ti_on_to_fr_l\xF6".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"D MMMM YYYY",LLL:"D MMMM YYYY [kl.] HH:mm",LLLL:"dddd D MMMM YYYY [kl.] HH:mm",lll:"D MMM YYYY HH:mm",llll:"ddd D MMM YYYY HH:mm"},calendar:{sameDay:"[Idag] LT",nextDay:"[Imorgon] LT",lastDay:"[Ig\xE5r] LT",nextWeek:"[P\xE5] dddd LT",lastWeek:"[I] dddd[s] LT",sameElse:"L"},relativeTime:{future:"om %s",past:"f\xF6r %s sedan",s:"n\xE5gra sekunder",ss:"%d sekunder",m:"en minut",mm:"%d minuter",h:"en timme",hh:"%d timmar",d:"en dag",dd:"%d dagar",M:"en m\xE5nad",MM:"%d m\xE5nader",y:"ett \xE5r",yy:"%d \xE5r"},dayOfMonthOrdinalParse:/\d{1,2}(\:e|\:a)/,ordinal:function(r){var s=r%10,n=~~(r%100/10)===1?":e":s===1||s===2?":a":":e";return r+n},week:{dow:1,doy:4}});return a})},40748(L,f,c){//! moment.js locale configuration
//! locale : Swahili [sw]
//! author : Fahad Kassim : https://github.com/fadsel
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("sw",{months:"Januari_Februari_Machi_Aprili_Mei_Juni_Julai_Agosti_Septemba_Oktoba_Novemba_Desemba".split("_"),monthsShort:"Jan_Feb_Mac_Apr_Mei_Jun_Jul_Ago_Sep_Okt_Nov_Des".split("_"),weekdays:"Jumapili_Jumatatu_Jumanne_Jumatano_Alhamisi_Ijumaa_Jumamosi".split("_"),weekdaysShort:"Jpl_Jtat_Jnne_Jtan_Alh_Ijm_Jmos".split("_"),weekdaysMin:"J2_J3_J4_J5_Al_Ij_J1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"hh:mm A",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[leo saa] LT",nextDay:"[kesho saa] LT",nextWeek:"[wiki ijayo] dddd [saat] LT",lastDay:"[jana] LT",lastWeek:"[wiki iliyopita] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s baadaye",past:"tokea %s",s:"hivi punde",ss:"sekunde %d",m:"dakika moja",mm:"dakika %d",h:"saa limoja",hh:"masaa %d",d:"siku moja",dd:"siku %d",M:"mwezi mmoja",MM:"miezi %d",y:"mwaka mmoja",yy:"miaka %d"},week:{dow:1,doy:7}});return a})},11025(L,f,c){//! moment.js locale configuration
//! locale : Tamil [ta]
//! author : Arjunkumar Krishnamoorthy : https://github.com/tk120404
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"\u0BE7",2:"\u0BE8",3:"\u0BE9",4:"\u0BEA",5:"\u0BEB",6:"\u0BEC",7:"\u0BED",8:"\u0BEE",9:"\u0BEF",0:"\u0BE6"},r={"\u0BE7":"1","\u0BE8":"2","\u0BE9":"3","\u0BEA":"4","\u0BEB":"5","\u0BEC":"6","\u0BED":"7","\u0BEE":"8","\u0BEF":"9","\u0BE6":"0"},s=o.defineLocale("ta",{months:"\u0B9C\u0BA9\u0BB5\u0BB0\u0BBF_\u0BAA\u0BBF\u0BAA\u0BCD\u0BB0\u0BB5\u0BB0\u0BBF_\u0BAE\u0BBE\u0BB0\u0BCD\u0B9A\u0BCD_\u0B8F\u0BAA\u0BCD\u0BB0\u0BB2\u0BCD_\u0BAE\u0BC7_\u0B9C\u0BC2\u0BA9\u0BCD_\u0B9C\u0BC2\u0BB2\u0BC8_\u0B86\u0B95\u0BB8\u0BCD\u0B9F\u0BCD_\u0B9A\u0BC6\u0BAA\u0BCD\u0B9F\u0BC6\u0BAE\u0BCD\u0BAA\u0BB0\u0BCD_\u0B85\u0B95\u0BCD\u0B9F\u0BC7\u0BBE\u0BAA\u0BB0\u0BCD_\u0BA8\u0BB5\u0BAE\u0BCD\u0BAA\u0BB0\u0BCD_\u0B9F\u0BBF\u0B9A\u0BAE\u0BCD\u0BAA\u0BB0\u0BCD".split("_"),monthsShort:"\u0B9C\u0BA9\u0BB5\u0BB0\u0BBF_\u0BAA\u0BBF\u0BAA\u0BCD\u0BB0\u0BB5\u0BB0\u0BBF_\u0BAE\u0BBE\u0BB0\u0BCD\u0B9A\u0BCD_\u0B8F\u0BAA\u0BCD\u0BB0\u0BB2\u0BCD_\u0BAE\u0BC7_\u0B9C\u0BC2\u0BA9\u0BCD_\u0B9C\u0BC2\u0BB2\u0BC8_\u0B86\u0B95\u0BB8\u0BCD\u0B9F\u0BCD_\u0B9A\u0BC6\u0BAA\u0BCD\u0B9F\u0BC6\u0BAE\u0BCD\u0BAA\u0BB0\u0BCD_\u0B85\u0B95\u0BCD\u0B9F\u0BC7\u0BBE\u0BAA\u0BB0\u0BCD_\u0BA8\u0BB5\u0BAE\u0BCD\u0BAA\u0BB0\u0BCD_\u0B9F\u0BBF\u0B9A\u0BAE\u0BCD\u0BAA\u0BB0\u0BCD".split("_"),weekdays:"\u0B9E\u0BBE\u0BAF\u0BBF\u0BB1\u0BCD\u0BB1\u0BC1\u0B95\u0BCD\u0B95\u0BBF\u0BB4\u0BAE\u0BC8_\u0BA4\u0BBF\u0B99\u0BCD\u0B95\u0B9F\u0BCD\u0B95\u0BBF\u0BB4\u0BAE\u0BC8_\u0B9A\u0BC6\u0BB5\u0BCD\u0BB5\u0BBE\u0BAF\u0BCD\u0B95\u0BBF\u0BB4\u0BAE\u0BC8_\u0BAA\u0BC1\u0BA4\u0BA9\u0BCD\u0B95\u0BBF\u0BB4\u0BAE\u0BC8_\u0BB5\u0BBF\u0BAF\u0BBE\u0BB4\u0B95\u0BCD\u0B95\u0BBF\u0BB4\u0BAE\u0BC8_\u0BB5\u0BC6\u0BB3\u0BCD\u0BB3\u0BBF\u0B95\u0BCD\u0B95\u0BBF\u0BB4\u0BAE\u0BC8_\u0B9A\u0BA9\u0BBF\u0B95\u0BCD\u0B95\u0BBF\u0BB4\u0BAE\u0BC8".split("_"),weekdaysShort:"\u0B9E\u0BBE\u0BAF\u0BBF\u0BB1\u0BC1_\u0BA4\u0BBF\u0B99\u0BCD\u0B95\u0BB3\u0BCD_\u0B9A\u0BC6\u0BB5\u0BCD\u0BB5\u0BBE\u0BAF\u0BCD_\u0BAA\u0BC1\u0BA4\u0BA9\u0BCD_\u0BB5\u0BBF\u0BAF\u0BBE\u0BB4\u0BA9\u0BCD_\u0BB5\u0BC6\u0BB3\u0BCD\u0BB3\u0BBF_\u0B9A\u0BA9\u0BBF".split("_"),weekdaysMin:"\u0B9E\u0BBE_\u0BA4\u0BBF_\u0B9A\u0BC6_\u0BAA\u0BC1_\u0BB5\u0BBF_\u0BB5\u0BC6_\u0B9A".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, HH:mm",LLLL:"dddd, D MMMM YYYY, HH:mm"},calendar:{sameDay:"[\u0B87\u0BA9\u0BCD\u0BB1\u0BC1] LT",nextDay:"[\u0BA8\u0BBE\u0BB3\u0BC8] LT",nextWeek:"dddd, LT",lastDay:"[\u0BA8\u0BC7\u0BB1\u0BCD\u0BB1\u0BC1] LT",lastWeek:"[\u0B95\u0B9F\u0BA8\u0BCD\u0BA4 \u0BB5\u0BBE\u0BB0\u0BAE\u0BCD] dddd, LT",sameElse:"L"},relativeTime:{future:"%s \u0B87\u0BB2\u0BCD",past:"%s \u0BAE\u0BC1\u0BA9\u0BCD",s:"\u0B92\u0BB0\u0BC1 \u0B9A\u0BBF\u0BB2 \u0BB5\u0BBF\u0BA8\u0BBE\u0B9F\u0BBF\u0B95\u0BB3\u0BCD",ss:"%d \u0BB5\u0BBF\u0BA8\u0BBE\u0B9F\u0BBF\u0B95\u0BB3\u0BCD",m:"\u0B92\u0BB0\u0BC1 \u0BA8\u0BBF\u0BAE\u0BBF\u0B9F\u0BAE\u0BCD",mm:"%d \u0BA8\u0BBF\u0BAE\u0BBF\u0B9F\u0B99\u0BCD\u0B95\u0BB3\u0BCD",h:"\u0B92\u0BB0\u0BC1 \u0BAE\u0BA3\u0BBF \u0BA8\u0BC7\u0BB0\u0BAE\u0BCD",hh:"%d \u0BAE\u0BA3\u0BBF \u0BA8\u0BC7\u0BB0\u0BAE\u0BCD",d:"\u0B92\u0BB0\u0BC1 \u0BA8\u0BBE\u0BB3\u0BCD",dd:"%d \u0BA8\u0BBE\u0B9F\u0BCD\u0B95\u0BB3\u0BCD",M:"\u0B92\u0BB0\u0BC1 \u0BAE\u0BBE\u0BA4\u0BAE\u0BCD",MM:"%d \u0BAE\u0BBE\u0BA4\u0B99\u0BCD\u0B95\u0BB3\u0BCD",y:"\u0B92\u0BB0\u0BC1 \u0BB5\u0BB0\u0BC1\u0B9F\u0BAE\u0BCD",yy:"%d \u0B86\u0BA3\u0BCD\u0B9F\u0BC1\u0B95\u0BB3\u0BCD"},dayOfMonthOrdinalParse:/\d{1,2}வது/,ordinal:function(n){return n+"\u0BB5\u0BA4\u0BC1"},preparse:function(n){return n.replace(/[௧௨௩௪௫௬௭௮௯௦]/g,function(d){return r[d]})},postformat:function(n){return n.replace(/\d/g,function(d){return a[d]})},meridiemParse:/யாமம்|வைகறை|காலை|நண்பகல்|எற்பாடு|மாலை/,meridiem:function(n,d,u){return n<2?" \u0BAF\u0BBE\u0BAE\u0BAE\u0BCD":n<6?" \u0BB5\u0BC8\u0B95\u0BB1\u0BC8":n<10?" \u0B95\u0BBE\u0BB2\u0BC8":n<14?" \u0BA8\u0BA3\u0BCD\u0BAA\u0B95\u0BB2\u0BCD":n<18?" \u0B8E\u0BB1\u0BCD\u0BAA\u0BBE\u0B9F\u0BC1":n<22?" \u0BAE\u0BBE\u0BB2\u0BC8":" \u0BAF\u0BBE\u0BAE\u0BAE\u0BCD"},meridiemHour:function(n,d){return n===12&&(n=0),d==="\u0BAF\u0BBE\u0BAE\u0BAE\u0BCD"?n<2?n:n+12:d==="\u0BB5\u0BC8\u0B95\u0BB1\u0BC8"||d==="\u0B95\u0BBE\u0BB2\u0BC8"||d==="\u0BA8\u0BA3\u0BCD\u0BAA\u0B95\u0BB2\u0BCD"&&n>=10?n:n+12},week:{dow:0,doy:6}});return s})},11885(L,f,c){//! moment.js locale configuration
//! locale : Telugu [te]
//! author : Krishna Chaitanya Thota : https://github.com/kcthota
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("te",{months:"\u0C1C\u0C28\u0C35\u0C30\u0C3F_\u0C2B\u0C3F\u0C2C\u0C4D\u0C30\u0C35\u0C30\u0C3F_\u0C2E\u0C3E\u0C30\u0C4D\u0C1A\u0C3F_\u0C0F\u0C2A\u0C4D\u0C30\u0C3F\u0C32\u0C4D_\u0C2E\u0C47_\u0C1C\u0C42\u0C28\u0C4D_\u0C1C\u0C41\u0C32\u0C48_\u0C06\u0C17\u0C38\u0C4D\u0C1F\u0C41_\u0C38\u0C46\u0C2A\u0C4D\u0C1F\u0C46\u0C02\u0C2C\u0C30\u0C4D_\u0C05\u0C15\u0C4D\u0C1F\u0C4B\u0C2C\u0C30\u0C4D_\u0C28\u0C35\u0C02\u0C2C\u0C30\u0C4D_\u0C21\u0C3F\u0C38\u0C46\u0C02\u0C2C\u0C30\u0C4D".split("_"),monthsShort:"\u0C1C\u0C28._\u0C2B\u0C3F\u0C2C\u0C4D\u0C30._\u0C2E\u0C3E\u0C30\u0C4D\u0C1A\u0C3F_\u0C0F\u0C2A\u0C4D\u0C30\u0C3F._\u0C2E\u0C47_\u0C1C\u0C42\u0C28\u0C4D_\u0C1C\u0C41\u0C32\u0C48_\u0C06\u0C17._\u0C38\u0C46\u0C2A\u0C4D._\u0C05\u0C15\u0C4D\u0C1F\u0C4B._\u0C28\u0C35._\u0C21\u0C3F\u0C38\u0C46.".split("_"),monthsParseExact:!0,weekdays:"\u0C06\u0C26\u0C3F\u0C35\u0C3E\u0C30\u0C02_\u0C38\u0C4B\u0C2E\u0C35\u0C3E\u0C30\u0C02_\u0C2E\u0C02\u0C17\u0C33\u0C35\u0C3E\u0C30\u0C02_\u0C2C\u0C41\u0C27\u0C35\u0C3E\u0C30\u0C02_\u0C17\u0C41\u0C30\u0C41\u0C35\u0C3E\u0C30\u0C02_\u0C36\u0C41\u0C15\u0C4D\u0C30\u0C35\u0C3E\u0C30\u0C02_\u0C36\u0C28\u0C3F\u0C35\u0C3E\u0C30\u0C02".split("_"),weekdaysShort:"\u0C06\u0C26\u0C3F_\u0C38\u0C4B\u0C2E_\u0C2E\u0C02\u0C17\u0C33_\u0C2C\u0C41\u0C27_\u0C17\u0C41\u0C30\u0C41_\u0C36\u0C41\u0C15\u0C4D\u0C30_\u0C36\u0C28\u0C3F".split("_"),weekdaysMin:"\u0C06_\u0C38\u0C4B_\u0C2E\u0C02_\u0C2C\u0C41_\u0C17\u0C41_\u0C36\u0C41_\u0C36".split("_"),longDateFormat:{LT:"A h:mm",LTS:"A h:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY, A h:mm",LLLL:"dddd, D MMMM YYYY, A h:mm"},calendar:{sameDay:"[\u0C28\u0C47\u0C21\u0C41] LT",nextDay:"[\u0C30\u0C47\u0C2A\u0C41] LT",nextWeek:"dddd, LT",lastDay:"[\u0C28\u0C3F\u0C28\u0C4D\u0C28] LT",lastWeek:"[\u0C17\u0C24] dddd, LT",sameElse:"L"},relativeTime:{future:"%s \u0C32\u0C4B",past:"%s \u0C15\u0C4D\u0C30\u0C3F\u0C24\u0C02",s:"\u0C15\u0C4A\u0C28\u0C4D\u0C28\u0C3F \u0C15\u0C4D\u0C37\u0C23\u0C3E\u0C32\u0C41",ss:"%d \u0C38\u0C46\u0C15\u0C28\u0C4D\u0C32\u0C41",m:"\u0C12\u0C15 \u0C28\u0C3F\u0C2E\u0C3F\u0C37\u0C02",mm:"%d \u0C28\u0C3F\u0C2E\u0C3F\u0C37\u0C3E\u0C32\u0C41",h:"\u0C12\u0C15 \u0C17\u0C02\u0C1F",hh:"%d \u0C17\u0C02\u0C1F\u0C32\u0C41",d:"\u0C12\u0C15 \u0C30\u0C4B\u0C1C\u0C41",dd:"%d \u0C30\u0C4B\u0C1C\u0C41\u0C32\u0C41",M:"\u0C12\u0C15 \u0C28\u0C46\u0C32",MM:"%d \u0C28\u0C46\u0C32\u0C32\u0C41",y:"\u0C12\u0C15 \u0C38\u0C02\u0C35\u0C24\u0C4D\u0C38\u0C30\u0C02",yy:"%d \u0C38\u0C02\u0C35\u0C24\u0C4D\u0C38\u0C30\u0C3E\u0C32\u0C41"},dayOfMonthOrdinalParse:/\d{1,2}వ/,ordinal:"%d\u0C35",meridiemParse:/రాత్రి|ఉదయం|మధ్యాహ్నం|సాయంత్రం/,meridiemHour:function(r,s){if(r===12&&(r=0),s==="\u0C30\u0C3E\u0C24\u0C4D\u0C30\u0C3F")return r<4?r:r+12;if(s==="\u0C09\u0C26\u0C2F\u0C02")return r;if(s==="\u0C2E\u0C27\u0C4D\u0C2F\u0C3E\u0C39\u0C4D\u0C28\u0C02")return r>=10?r:r+12;if(s==="\u0C38\u0C3E\u0C2F\u0C02\u0C24\u0C4D\u0C30\u0C02")return r+12},meridiem:function(r,s,n){return r<4?"\u0C30\u0C3E\u0C24\u0C4D\u0C30\u0C3F":r<10?"\u0C09\u0C26\u0C2F\u0C02":r<17?"\u0C2E\u0C27\u0C4D\u0C2F\u0C3E\u0C39\u0C4D\u0C28\u0C02":r<20?"\u0C38\u0C3E\u0C2F\u0C02\u0C24\u0C4D\u0C30\u0C02":"\u0C30\u0C3E\u0C24\u0C4D\u0C30\u0C3F"},week:{dow:0,doy:6}});return a})},28861(L,f,c){//! moment.js locale configuration
//! locale : Tetun Dili (East Timor) [tet]
//! author : Joshua Brooks : https://github.com/joshbrooks
//! author : Onorio De J. Afonso : https://github.com/marobo
//! author : Sonia Simoes : https://github.com/soniasimoes
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("tet",{months:"Janeiru_Fevereiru_Marsu_Abril_Maiu_Ju\xF1u_Jullu_Agustu_Setembru_Outubru_Novembru_Dezembru".split("_"),monthsShort:"Jan_Fev_Mar_Abr_Mai_Jun_Jul_Ago_Set_Out_Nov_Dez".split("_"),weekdays:"Domingu_Segunda_Tersa_Kuarta_Kinta_Sesta_Sabadu".split("_"),weekdaysShort:"Dom_Seg_Ters_Kua_Kint_Sest_Sab".split("_"),weekdaysMin:"Do_Seg_Te_Ku_Ki_Ses_Sa".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[Ohin iha] LT",nextDay:"[Aban iha] LT",nextWeek:"dddd [iha] LT",lastDay:"[Horiseik iha] LT",lastWeek:"dddd [semana kotuk] [iha] LT",sameElse:"L"},relativeTime:{future:"iha %s",past:"%s liuba",s:"segundu balun",ss:"segundu %d",m:"minutu ida",mm:"minutu %d",h:"oras ida",hh:"oras %d",d:"loron ida",dd:"loron %d",M:"fulan ida",MM:"fulan %d",y:"tinan ida",yy:"tinan %d"},dayOfMonthOrdinalParse:/\d{1,2}(st|nd|rd|th)/,ordinal:function(r){var s=r%10,n=~~(r%100/10)===1?"th":s===1?"st":s===2?"nd":s===3?"rd":"th";return r+n},week:{dow:1,doy:4}});return a})},86571(L,f,c){//! moment.js locale configuration
//! locale : Tajik [tg]
//! author : Orif N. Jr. : https://github.com/orif-jr
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={0:"-\u0443\u043C",1:"-\u0443\u043C",2:"-\u044E\u043C",3:"-\u044E\u043C",4:"-\u0443\u043C",5:"-\u0443\u043C",6:"-\u0443\u043C",7:"-\u0443\u043C",8:"-\u0443\u043C",9:"-\u0443\u043C",10:"-\u0443\u043C",12:"-\u0443\u043C",13:"-\u0443\u043C",20:"-\u0443\u043C",30:"-\u044E\u043C",40:"-\u0443\u043C",50:"-\u0443\u043C",60:"-\u0443\u043C",70:"-\u0443\u043C",80:"-\u0443\u043C",90:"-\u0443\u043C",100:"-\u0443\u043C"},r=o.defineLocale("tg",{months:{format:"\u044F\u043D\u0432\u0430\u0440\u0438_\u0444\u0435\u0432\u0440\u0430\u043B\u0438_\u043C\u0430\u0440\u0442\u0438_\u0430\u043F\u0440\u0435\u043B\u0438_\u043C\u0430\u0439\u0438_\u0438\u044E\u043D\u0438_\u0438\u044E\u043B\u0438_\u0430\u0432\u0433\u0443\u0441\u0442\u0438_\u0441\u0435\u043D\u0442\u044F\u0431\u0440\u0438_\u043E\u043A\u0442\u044F\u0431\u0440\u0438_\u043D\u043E\u044F\u0431\u0440\u0438_\u0434\u0435\u043A\u0430\u0431\u0440\u0438".split("_"),standalone:"\u044F\u043D\u0432\u0430\u0440_\u0444\u0435\u0432\u0440\u0430\u043B_\u043C\u0430\u0440\u0442_\u0430\u043F\u0440\u0435\u043B_\u043C\u0430\u0439_\u0438\u044E\u043D_\u0438\u044E\u043B_\u0430\u0432\u0433\u0443\u0441\u0442_\u0441\u0435\u043D\u0442\u044F\u0431\u0440_\u043E\u043A\u0442\u044F\u0431\u0440_\u043D\u043E\u044F\u0431\u0440_\u0434\u0435\u043A\u0430\u0431\u0440".split("_")},monthsShort:"\u044F\u043D\u0432_\u0444\u0435\u0432_\u043C\u0430\u0440_\u0430\u043F\u0440_\u043C\u0430\u0439_\u0438\u044E\u043D_\u0438\u044E\u043B_\u0430\u0432\u0433_\u0441\u0435\u043D_\u043E\u043A\u0442_\u043D\u043E\u044F_\u0434\u0435\u043A".split("_"),weekdays:"\u044F\u043A\u0448\u0430\u043D\u0431\u0435_\u0434\u0443\u0448\u0430\u043D\u0431\u0435_\u0441\u0435\u0448\u0430\u043D\u0431\u0435_\u0447\u043E\u0440\u0448\u0430\u043D\u0431\u0435_\u043F\u0430\u043D\u04B7\u0448\u0430\u043D\u0431\u0435_\u04B7\u0443\u043C\u044A\u0430_\u0448\u0430\u043D\u0431\u0435".split("_"),weekdaysShort:"\u044F\u0448\u0431_\u0434\u0448\u0431_\u0441\u0448\u0431_\u0447\u0448\u0431_\u043F\u0448\u0431_\u04B7\u0443\u043C_\u0448\u043D\u0431".split("_"),weekdaysMin:"\u044F\u0448_\u0434\u0448_\u0441\u0448_\u0447\u0448_\u043F\u0448_\u04B7\u043C_\u0448\u0431".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[\u0418\u043C\u0440\u04EF\u0437 \u0441\u043E\u0430\u0442\u0438] LT",nextDay:"[\u0424\u0430\u0440\u0434\u043E \u0441\u043E\u0430\u0442\u0438] LT",lastDay:"[\u0414\u0438\u0440\u04EF\u0437 \u0441\u043E\u0430\u0442\u0438] LT",nextWeek:"dddd[\u0438] [\u04B3\u0430\u0444\u0442\u0430\u0438 \u043E\u044F\u043D\u0434\u0430 \u0441\u043E\u0430\u0442\u0438] LT",lastWeek:"dddd[\u0438] [\u04B3\u0430\u0444\u0442\u0430\u0438 \u0433\u0443\u0437\u0430\u0448\u0442\u0430 \u0441\u043E\u0430\u0442\u0438] LT",sameElse:"L"},relativeTime:{future:"\u0431\u0430\u044A\u0434\u0438 %s",past:"%s \u043F\u0435\u0448",s:"\u044F\u043A\u0447\u0430\u043D\u0434 \u0441\u043E\u043D\u0438\u044F",m:"\u044F\u043A \u0434\u0430\u049B\u0438\u049B\u0430",mm:"%d \u0434\u0430\u049B\u0438\u049B\u0430",h:"\u044F\u043A \u0441\u043E\u0430\u0442",hh:"%d \u0441\u043E\u0430\u0442",d:"\u044F\u043A \u0440\u04EF\u0437",dd:"%d \u0440\u04EF\u0437",M:"\u044F\u043A \u043C\u043E\u04B3",MM:"%d \u043C\u043E\u04B3",y:"\u044F\u043A \u0441\u043E\u043B",yy:"%d \u0441\u043E\u043B"},meridiemParse:/шаб|субҳ|рӯз|бегоҳ/,meridiemHour:function(s,n){if(s===12&&(s=0),n==="\u0448\u0430\u0431")return s<4?s:s+12;if(n==="\u0441\u0443\u0431\u04B3")return s;if(n==="\u0440\u04EF\u0437")return s>=11?s:s+12;if(n==="\u0431\u0435\u0433\u043E\u04B3")return s+12},meridiem:function(s,n,d){return s<4?"\u0448\u0430\u0431":s<11?"\u0441\u0443\u0431\u04B3":s<16?"\u0440\u04EF\u0437":s<19?"\u0431\u0435\u0433\u043E\u04B3":"\u0448\u0430\u0431"},dayOfMonthOrdinalParse:/\d{1,2}-(ум|юм)/,ordinal:function(s){var n=s%10,d=s>=100?100:null;return s+(a[s]||a[n]||a[d])},week:{dow:1,doy:7}});return r})},55802(L,f,c){//! moment.js locale configuration
//! locale : Thai [th]
//! author : Kridsada Thanabulpong : https://github.com/sirn
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("th",{months:"\u0E21\u0E01\u0E23\u0E32\u0E04\u0E21_\u0E01\u0E38\u0E21\u0E20\u0E32\u0E1E\u0E31\u0E19\u0E18\u0E4C_\u0E21\u0E35\u0E19\u0E32\u0E04\u0E21_\u0E40\u0E21\u0E29\u0E32\u0E22\u0E19_\u0E1E\u0E24\u0E29\u0E20\u0E32\u0E04\u0E21_\u0E21\u0E34\u0E16\u0E38\u0E19\u0E32\u0E22\u0E19_\u0E01\u0E23\u0E01\u0E0E\u0E32\u0E04\u0E21_\u0E2A\u0E34\u0E07\u0E2B\u0E32\u0E04\u0E21_\u0E01\u0E31\u0E19\u0E22\u0E32\u0E22\u0E19_\u0E15\u0E38\u0E25\u0E32\u0E04\u0E21_\u0E1E\u0E24\u0E28\u0E08\u0E34\u0E01\u0E32\u0E22\u0E19_\u0E18\u0E31\u0E19\u0E27\u0E32\u0E04\u0E21".split("_"),monthsShort:"\u0E21.\u0E04._\u0E01.\u0E1E._\u0E21\u0E35.\u0E04._\u0E40\u0E21.\u0E22._\u0E1E.\u0E04._\u0E21\u0E34.\u0E22._\u0E01.\u0E04._\u0E2A.\u0E04._\u0E01.\u0E22._\u0E15.\u0E04._\u0E1E.\u0E22._\u0E18.\u0E04.".split("_"),monthsParseExact:!0,weekdays:"\u0E2D\u0E32\u0E17\u0E34\u0E15\u0E22\u0E4C_\u0E08\u0E31\u0E19\u0E17\u0E23\u0E4C_\u0E2D\u0E31\u0E07\u0E04\u0E32\u0E23_\u0E1E\u0E38\u0E18_\u0E1E\u0E24\u0E2B\u0E31\u0E2A\u0E1A\u0E14\u0E35_\u0E28\u0E38\u0E01\u0E23\u0E4C_\u0E40\u0E2A\u0E32\u0E23\u0E4C".split("_"),weekdaysShort:"\u0E2D\u0E32\u0E17\u0E34\u0E15\u0E22\u0E4C_\u0E08\u0E31\u0E19\u0E17\u0E23\u0E4C_\u0E2D\u0E31\u0E07\u0E04\u0E32\u0E23_\u0E1E\u0E38\u0E18_\u0E1E\u0E24\u0E2B\u0E31\u0E2A_\u0E28\u0E38\u0E01\u0E23\u0E4C_\u0E40\u0E2A\u0E32\u0E23\u0E4C".split("_"),weekdaysMin:"\u0E2D\u0E32._\u0E08._\u0E2D._\u0E1E._\u0E1E\u0E24._\u0E28._\u0E2A.".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"H:mm",LTS:"H:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY \u0E40\u0E27\u0E25\u0E32 H:mm",LLLL:"\u0E27\u0E31\u0E19dddd\u0E17\u0E35\u0E48 D MMMM YYYY \u0E40\u0E27\u0E25\u0E32 H:mm"},meridiemParse:/ก่อนเที่ยง|หลังเที่ยง/,isPM:function(r){return r==="\u0E2B\u0E25\u0E31\u0E07\u0E40\u0E17\u0E35\u0E48\u0E22\u0E07"},meridiem:function(r,s,n){return r<12?"\u0E01\u0E48\u0E2D\u0E19\u0E40\u0E17\u0E35\u0E48\u0E22\u0E07":"\u0E2B\u0E25\u0E31\u0E07\u0E40\u0E17\u0E35\u0E48\u0E22\u0E07"},calendar:{sameDay:"[\u0E27\u0E31\u0E19\u0E19\u0E35\u0E49 \u0E40\u0E27\u0E25\u0E32] LT",nextDay:"[\u0E1E\u0E23\u0E38\u0E48\u0E07\u0E19\u0E35\u0E49 \u0E40\u0E27\u0E25\u0E32] LT",nextWeek:"dddd[\u0E2B\u0E19\u0E49\u0E32 \u0E40\u0E27\u0E25\u0E32] LT",lastDay:"[\u0E40\u0E21\u0E37\u0E48\u0E2D\u0E27\u0E32\u0E19\u0E19\u0E35\u0E49 \u0E40\u0E27\u0E25\u0E32] LT",lastWeek:"[\u0E27\u0E31\u0E19]dddd[\u0E17\u0E35\u0E48\u0E41\u0E25\u0E49\u0E27 \u0E40\u0E27\u0E25\u0E32] LT",sameElse:"L"},relativeTime:{future:"\u0E2D\u0E35\u0E01 %s",past:"%s\u0E17\u0E35\u0E48\u0E41\u0E25\u0E49\u0E27",s:"\u0E44\u0E21\u0E48\u0E01\u0E35\u0E48\u0E27\u0E34\u0E19\u0E32\u0E17\u0E35",ss:"%d \u0E27\u0E34\u0E19\u0E32\u0E17\u0E35",m:"1 \u0E19\u0E32\u0E17\u0E35",mm:"%d \u0E19\u0E32\u0E17\u0E35",h:"1 \u0E0A\u0E31\u0E48\u0E27\u0E42\u0E21\u0E07",hh:"%d \u0E0A\u0E31\u0E48\u0E27\u0E42\u0E21\u0E07",d:"1 \u0E27\u0E31\u0E19",dd:"%d \u0E27\u0E31\u0E19",w:"1 \u0E2A\u0E31\u0E1B\u0E14\u0E32\u0E2B\u0E4C",ww:"%d \u0E2A\u0E31\u0E1B\u0E14\u0E32\u0E2B\u0E4C",M:"1 \u0E40\u0E14\u0E37\u0E2D\u0E19",MM:"%d \u0E40\u0E14\u0E37\u0E2D\u0E19",y:"1 \u0E1B\u0E35",yy:"%d \u0E1B\u0E35"}});return a})},59527(L,f,c){//! moment.js locale configuration
//! locale : Turkmen [tk]
//! author : Atamyrat Abdyrahmanov : https://github.com/atamyratabdy
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"'inji",5:"'inji",8:"'inji",70:"'inji",80:"'inji",2:"'nji",7:"'nji",20:"'nji",50:"'nji",3:"'\xFCnji",4:"'\xFCnji",100:"'\xFCnji",6:"'njy",9:"'unjy",10:"'unjy",30:"'unjy",60:"'ynjy",90:"'ynjy"},r=o.defineLocale("tk",{months:"\xDDanwar_Fewral_Mart_Aprel_Ma\xFD_I\xFDun_I\xFDul_Awgust_Sent\xFDabr_Okt\xFDabr_No\xFDabr_Dekabr".split("_"),monthsShort:"\xDDan_Few_Mar_Apr_Ma\xFD_I\xFDn_I\xFDl_Awg_Sen_Okt_No\xFD_Dek".split("_"),weekdays:"\xDDek\u015Fenbe_Du\u015Fenbe_Si\u015Fenbe_\xC7ar\u015Fenbe_Pen\u015Fenbe_Anna_\u015Eenbe".split("_"),weekdaysShort:"\xDDek_Du\u015F_Si\u015F_\xC7ar_Pen_Ann_\u015Een".split("_"),weekdaysMin:"\xDDk_D\u015F_S\u015F_\xC7r_Pn_An_\u015En".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bug\xFCn sagat] LT",nextDay:"[ertir sagat] LT",nextWeek:"[indiki] dddd [sagat] LT",lastDay:"[d\xFC\xFDn] LT",lastWeek:"[ge\xE7en] dddd [sagat] LT",sameElse:"L"},relativeTime:{future:"%s so\u0148",past:"%s \xF6\u0148",s:"birn\xE4\xE7e sekunt",m:"bir minut",mm:"%d minut",h:"bir sagat",hh:"%d sagat",d:"bir g\xFCn",dd:"%d g\xFCn",M:"bir a\xFD",MM:"%d a\xFD",y:"bir \xFDyl",yy:"%d \xFDyl"},ordinal:function(s,n){switch(n){case"d":case"D":case"Do":case"DD":return s;default:if(s===0)return s+"'unjy";var d=s%10,u=s%100-d,m=s>=100?100:null;return s+(a[d]||a[u]||a[m])}},week:{dow:1,doy:7}});return r})},29231(L,f,c){//! moment.js locale configuration
//! locale : Tagalog (Philippines) [tl-ph]
//! author : Dan Hagman : https://github.com/hagmandan
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("tl-ph",{months:"Enero_Pebrero_Marso_Abril_Mayo_Hunyo_Hulyo_Agosto_Setyembre_Oktubre_Nobyembre_Disyembre".split("_"),monthsShort:"Ene_Peb_Mar_Abr_May_Hun_Hul_Ago_Set_Okt_Nob_Dis".split("_"),weekdays:"Linggo_Lunes_Martes_Miyerkules_Huwebes_Biyernes_Sabado".split("_"),weekdaysShort:"Lin_Lun_Mar_Miy_Huw_Biy_Sab".split("_"),weekdaysMin:"Li_Lu_Ma_Mi_Hu_Bi_Sab".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"MM/D/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY HH:mm",LLLL:"dddd, MMMM DD, YYYY HH:mm"},calendar:{sameDay:"LT [ngayong araw]",nextDay:"[Bukas ng] LT",nextWeek:"LT [sa susunod na] dddd",lastDay:"LT [kahapon]",lastWeek:"LT [noong nakaraang] dddd",sameElse:"L"},relativeTime:{future:"sa loob ng %s",past:"%s ang nakalipas",s:"ilang segundo",ss:"%d segundo",m:"isang minuto",mm:"%d minuto",h:"isang oras",hh:"%d oras",d:"isang araw",dd:"%d araw",M:"isang buwan",MM:"%d buwan",y:"isang taon",yy:"%d taon"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(r){return r},week:{dow:1,doy:4}});return a})},31052(L,f,c){//! moment.js locale configuration
//! locale : Klingon [tlh]
//! author : Dominika Kruk : https://github.com/amaranthrose
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a="pagh_wa\u2019_cha\u2019_wej_loS_vagh_jav_Soch_chorgh_Hut".split("_");function r(m){var h=m;return h=m.indexOf("jaj")!==-1?h.slice(0,-3)+"leS":m.indexOf("jar")!==-1?h.slice(0,-3)+"waQ":m.indexOf("DIS")!==-1?h.slice(0,-3)+"nem":h+" pIq",h}function s(m){var h=m;return h=m.indexOf("jaj")!==-1?h.slice(0,-3)+"Hu\u2019":m.indexOf("jar")!==-1?h.slice(0,-3)+"wen":m.indexOf("DIS")!==-1?h.slice(0,-3)+"ben":h+" ret",h}function n(m,h,y,g){var v=d(m);switch(y){case"ss":return v+" lup";case"mm":return v+" tup";case"hh":return v+" rep";case"dd":return v+" jaj";case"MM":return v+" jar";case"yy":return v+" DIS"}}function d(m){var h=Math.floor(m%1e3/100),y=Math.floor(m%100/10),g=m%10,v="";return h>0&&(v+=a[h]+"vatlh"),y>0&&(v+=(v!==""?" ":"")+a[y]+"maH"),g>0&&(v+=(v!==""?" ":"")+a[g]),v===""?"pagh":v}var u=o.defineLocale("tlh",{months:"tera\u2019 jar wa\u2019_tera\u2019 jar cha\u2019_tera\u2019 jar wej_tera\u2019 jar loS_tera\u2019 jar vagh_tera\u2019 jar jav_tera\u2019 jar Soch_tera\u2019 jar chorgh_tera\u2019 jar Hut_tera\u2019 jar wa\u2019maH_tera\u2019 jar wa\u2019maH wa\u2019_tera\u2019 jar wa\u2019maH cha\u2019".split("_"),monthsShort:"jar wa\u2019_jar cha\u2019_jar wej_jar loS_jar vagh_jar jav_jar Soch_jar chorgh_jar Hut_jar wa\u2019maH_jar wa\u2019maH wa\u2019_jar wa\u2019maH cha\u2019".split("_"),monthsParseExact:!0,weekdays:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysShort:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),weekdaysMin:"lojmItjaj_DaSjaj_povjaj_ghItlhjaj_loghjaj_buqjaj_ghInjaj".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[DaHjaj] LT",nextDay:"[wa\u2019leS] LT",nextWeek:"LLL",lastDay:"[wa\u2019Hu\u2019] LT",lastWeek:"LLL",sameElse:"L"},relativeTime:{future:r,past:s,s:"puS lup",ss:n,m:"wa\u2019 tup",mm:n,h:"wa\u2019 rep",hh:n,d:"wa\u2019 jaj",dd:n,M:"wa\u2019 jar",MM:n,y:"wa\u2019 DIS",yy:n},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});return u})},85096(L,f,c){//! moment.js locale configuration
//! locale : Turkish [tr]
//! authors : Erhan Gundogan : https://github.com/erhangundogan,
//!           Burak Yiğit Kaya: https://github.com/BYK
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a={1:"'inci",5:"'inci",8:"'inci",70:"'inci",80:"'inci",2:"'nci",7:"'nci",20:"'nci",50:"'nci",3:"'\xFCnc\xFC",4:"'\xFCnc\xFC",100:"'\xFCnc\xFC",6:"'nc\u0131",9:"'uncu",10:"'uncu",30:"'uncu",60:"'\u0131nc\u0131",90:"'\u0131nc\u0131"},r=o.defineLocale("tr",{months:"Ocak_\u015Eubat_Mart_Nisan_May\u0131s_Haziran_Temmuz_A\u011Fustos_Eyl\xFCl_Ekim_Kas\u0131m_Aral\u0131k".split("_"),monthsShort:"Oca_\u015Eub_Mar_Nis_May_Haz_Tem_A\u011Fu_Eyl_Eki_Kas_Ara".split("_"),weekdays:"Pazar_Pazartesi_Sal\u0131_\xC7ar\u015Famba_Per\u015Fembe_Cuma_Cumartesi".split("_"),weekdaysShort:"Paz_Pzt_Sal_\xC7ar_Per_Cum_Cmt".split("_"),weekdaysMin:"Pz_Pt_Sa_\xC7a_Pe_Cu_Ct".split("_"),meridiem:function(s,n,d){return s<12?d?"\xF6\xF6":"\xD6\xD6":d?"\xF6s":"\xD6S"},meridiemParse:/öö|ÖÖ|ös|ÖS/,isPM:function(s){return s==="\xF6s"||s==="\xD6S"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[bug\xFCn saat] LT",nextDay:"[yar\u0131n saat] LT",nextWeek:"[gelecek] dddd [saat] LT",lastDay:"[d\xFCn] LT",lastWeek:"[ge\xE7en] dddd [saat] LT",sameElse:"L"},relativeTime:{future:"%s sonra",past:"%s \xF6nce",s:"birka\xE7 saniye",ss:"%d saniye",m:"bir dakika",mm:"%d dakika",h:"bir saat",hh:"%d saat",d:"bir g\xFCn",dd:"%d g\xFCn",w:"bir hafta",ww:"%d hafta",M:"bir ay",MM:"%d ay",y:"bir y\u0131l",yy:"%d y\u0131l"},ordinal:function(s,n){switch(n){case"d":case"D":case"Do":case"DD":return s;default:if(s===0)return s+"'\u0131nc\u0131";var d=s%10,u=s%100-d,m=s>=100?100:null;return s+(a[d]||a[u]||a[m])}},week:{dow:1,doy:7}});return r})},79846(L,f,c){//! moment.js locale configuration
//! locale : Talossan [tzl]
//! author : Robin van der Vliet : https://github.com/robin0van0der0v
//! author : Iustì Canun
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("tzl",{months:"Januar_Fevraglh_Mar\xE7_Avr\xEFu_Mai_G\xFCn_Julia_Guscht_Setemvar_Listop\xE4ts_Noemvar_Zecemvar".split("_"),monthsShort:"Jan_Fev_Mar_Avr_Mai_G\xFCn_Jul_Gus_Set_Lis_Noe_Zec".split("_"),weekdays:"S\xFAladi_L\xFAne\xE7i_Maitzi_M\xE1rcuri_Xh\xFAadi_Vi\xE9ner\xE7i_S\xE1turi".split("_"),weekdaysShort:"S\xFAl_L\xFAn_Mai_M\xE1r_Xh\xFA_Vi\xE9_S\xE1t".split("_"),weekdaysMin:"S\xFA_L\xFA_Ma_M\xE1_Xh_Vi_S\xE1".split("_"),longDateFormat:{LT:"HH.mm",LTS:"HH.mm.ss",L:"DD.MM.YYYY",LL:"D. MMMM [dallas] YYYY",LLL:"D. MMMM [dallas] YYYY HH.mm",LLLL:"dddd, [li] D. MMMM [dallas] YYYY HH.mm"},meridiemParse:/d\'o|d\'a/i,isPM:function(s){return s.toLowerCase()==="d'o"},meridiem:function(s,n,d){return s>11?d?"d'o":"D'O":d?"d'a":"D'A"},calendar:{sameDay:"[oxhi \xE0] LT",nextDay:"[dem\xE0 \xE0] LT",nextWeek:"dddd [\xE0] LT",lastDay:"[ieiri \xE0] LT",lastWeek:"[s\xFCr el] dddd [lasteu \xE0] LT",sameElse:"L"},relativeTime:{future:"osprei %s",past:"ja%s",s:r,ss:r,m:r,mm:r,h:r,hh:r,d:r,dd:r,M:r,MM:r,y:r,yy:r},dayOfMonthOrdinalParse:/\d{1,2}\./,ordinal:"%d.",week:{dow:1,doy:4}});function r(s,n,d,u){var m={s:["viensas secunds","'iensas secunds"],ss:[s+" secunds",""+s+" secunds"],m:["'n m\xEDut","'iens m\xEDut"],mm:[s+" m\xEDuts",""+s+" m\xEDuts"],h:["'n \xFEora","'iensa \xFEora"],hh:[s+" \xFEoras",""+s+" \xFEoras"],d:["'n ziua","'iensa ziua"],dd:[s+" ziuas",""+s+" ziuas"],M:["'n mes","'iens mes"],MM:[s+" mesen",""+s+" mesen"],y:["'n ar","'iens ar"],yy:[s+" ars",""+s+" ars"]};return u||n?m[d][0]:m[d][1]}return a})},97711(L,f,c){//! moment.js locale configuration
//! locale : Central Atlas Tamazight Latin [tzm-latn]
//! author : Abdel Said : https://github.com/abdelsaid
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("tzm-latn",{months:"innayr_br\u02E4ayr\u02E4_mar\u02E4s\u02E4_ibrir_mayyw_ywnyw_ywlywz_\u0263w\u0161t_\u0161wtanbir_kt\u02E4wbr\u02E4_nwwanbir_dwjnbir".split("_"),monthsShort:"innayr_br\u02E4ayr\u02E4_mar\u02E4s\u02E4_ibrir_mayyw_ywnyw_ywlywz_\u0263w\u0161t_\u0161wtanbir_kt\u02E4wbr\u02E4_nwwanbir_dwjnbir".split("_"),weekdays:"asamas_aynas_asinas_akras_akwas_asimwas_asi\u1E0Dyas".split("_"),weekdaysShort:"asamas_aynas_asinas_akras_akwas_asimwas_asi\u1E0Dyas".split("_"),weekdaysMin:"asamas_aynas_asinas_akras_akwas_asimwas_asi\u1E0Dyas".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[asdkh g] LT",nextDay:"[aska g] LT",nextWeek:"dddd [g] LT",lastDay:"[assant g] LT",lastWeek:"dddd [g] LT",sameElse:"L"},relativeTime:{future:"dadkh s yan %s",past:"yan %s",s:"imik",ss:"%d imik",m:"minu\u1E0D",mm:"%d minu\u1E0D",h:"sa\u025Ba",hh:"%d tassa\u025Bin",d:"ass",dd:"%d ossan",M:"ayowr",MM:"%d iyyirn",y:"asgas",yy:"%d isgasn"},week:{dow:6,doy:12}});return a})},81765(L,f,c){//! moment.js locale configuration
//! locale : Central Atlas Tamazight [tzm]
//! author : Abdel Said : https://github.com/abdelsaid
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("tzm",{months:"\u2D49\u2D4F\u2D4F\u2D30\u2D62\u2D54_\u2D31\u2D55\u2D30\u2D62\u2D55_\u2D4E\u2D30\u2D55\u2D5A_\u2D49\u2D31\u2D54\u2D49\u2D54_\u2D4E\u2D30\u2D62\u2D62\u2D53_\u2D62\u2D53\u2D4F\u2D62\u2D53_\u2D62\u2D53\u2D4D\u2D62\u2D53\u2D63_\u2D56\u2D53\u2D5B\u2D5C_\u2D5B\u2D53\u2D5C\u2D30\u2D4F\u2D31\u2D49\u2D54_\u2D3D\u2D5F\u2D53\u2D31\u2D55_\u2D4F\u2D53\u2D61\u2D30\u2D4F\u2D31\u2D49\u2D54_\u2D37\u2D53\u2D4A\u2D4F\u2D31\u2D49\u2D54".split("_"),monthsShort:"\u2D49\u2D4F\u2D4F\u2D30\u2D62\u2D54_\u2D31\u2D55\u2D30\u2D62\u2D55_\u2D4E\u2D30\u2D55\u2D5A_\u2D49\u2D31\u2D54\u2D49\u2D54_\u2D4E\u2D30\u2D62\u2D62\u2D53_\u2D62\u2D53\u2D4F\u2D62\u2D53_\u2D62\u2D53\u2D4D\u2D62\u2D53\u2D63_\u2D56\u2D53\u2D5B\u2D5C_\u2D5B\u2D53\u2D5C\u2D30\u2D4F\u2D31\u2D49\u2D54_\u2D3D\u2D5F\u2D53\u2D31\u2D55_\u2D4F\u2D53\u2D61\u2D30\u2D4F\u2D31\u2D49\u2D54_\u2D37\u2D53\u2D4A\u2D4F\u2D31\u2D49\u2D54".split("_"),weekdays:"\u2D30\u2D59\u2D30\u2D4E\u2D30\u2D59_\u2D30\u2D62\u2D4F\u2D30\u2D59_\u2D30\u2D59\u2D49\u2D4F\u2D30\u2D59_\u2D30\u2D3D\u2D54\u2D30\u2D59_\u2D30\u2D3D\u2D61\u2D30\u2D59_\u2D30\u2D59\u2D49\u2D4E\u2D61\u2D30\u2D59_\u2D30\u2D59\u2D49\u2D39\u2D62\u2D30\u2D59".split("_"),weekdaysShort:"\u2D30\u2D59\u2D30\u2D4E\u2D30\u2D59_\u2D30\u2D62\u2D4F\u2D30\u2D59_\u2D30\u2D59\u2D49\u2D4F\u2D30\u2D59_\u2D30\u2D3D\u2D54\u2D30\u2D59_\u2D30\u2D3D\u2D61\u2D30\u2D59_\u2D30\u2D59\u2D49\u2D4E\u2D61\u2D30\u2D59_\u2D30\u2D59\u2D49\u2D39\u2D62\u2D30\u2D59".split("_"),weekdaysMin:"\u2D30\u2D59\u2D30\u2D4E\u2D30\u2D59_\u2D30\u2D62\u2D4F\u2D30\u2D59_\u2D30\u2D59\u2D49\u2D4F\u2D30\u2D59_\u2D30\u2D3D\u2D54\u2D30\u2D59_\u2D30\u2D3D\u2D61\u2D30\u2D59_\u2D30\u2D59\u2D49\u2D4E\u2D61\u2D30\u2D59_\u2D30\u2D59\u2D49\u2D39\u2D62\u2D30\u2D59".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd D MMMM YYYY HH:mm"},calendar:{sameDay:"[\u2D30\u2D59\u2D37\u2D45 \u2D34] LT",nextDay:"[\u2D30\u2D59\u2D3D\u2D30 \u2D34] LT",nextWeek:"dddd [\u2D34] LT",lastDay:"[\u2D30\u2D5A\u2D30\u2D4F\u2D5C \u2D34] LT",lastWeek:"dddd [\u2D34] LT",sameElse:"L"},relativeTime:{future:"\u2D37\u2D30\u2D37\u2D45 \u2D59 \u2D62\u2D30\u2D4F %s",past:"\u2D62\u2D30\u2D4F %s",s:"\u2D49\u2D4E\u2D49\u2D3D",ss:"%d \u2D49\u2D4E\u2D49\u2D3D",m:"\u2D4E\u2D49\u2D4F\u2D53\u2D3A",mm:"%d \u2D4E\u2D49\u2D4F\u2D53\u2D3A",h:"\u2D59\u2D30\u2D44\u2D30",hh:"%d \u2D5C\u2D30\u2D59\u2D59\u2D30\u2D44\u2D49\u2D4F",d:"\u2D30\u2D59\u2D59",dd:"%d o\u2D59\u2D59\u2D30\u2D4F",M:"\u2D30\u2D62o\u2D53\u2D54",MM:"%d \u2D49\u2D62\u2D62\u2D49\u2D54\u2D4F",y:"\u2D30\u2D59\u2D33\u2D30\u2D59",yy:"%d \u2D49\u2D59\u2D33\u2D30\u2D59\u2D4F"},week:{dow:6,doy:12}});return a})},48414(L,f,c){//! moment.js locale configuration
//! locale : Uyghur (China) [ug-cn]
//! author: boyaq : https://github.com/boyaq
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("ug-cn",{months:"\u064A\u0627\u0646\u06CB\u0627\u0631_\u0641\u06D0\u06CB\u0631\u0627\u0644_\u0645\u0627\u0631\u062A_\u0626\u0627\u067E\u0631\u06D0\u0644_\u0645\u0627\u064A_\u0626\u0649\u064A\u06C7\u0646_\u0626\u0649\u064A\u06C7\u0644_\u0626\u0627\u06CB\u063A\u06C7\u0633\u062A_\u0633\u06D0\u0646\u062A\u06D5\u0628\u0649\u0631_\u0626\u06C6\u0643\u062A\u06D5\u0628\u0649\u0631_\u0646\u0648\u064A\u0627\u0628\u0649\u0631_\u062F\u06D0\u0643\u0627\u0628\u0649\u0631".split("_"),monthsShort:"\u064A\u0627\u0646\u06CB\u0627\u0631_\u0641\u06D0\u06CB\u0631\u0627\u0644_\u0645\u0627\u0631\u062A_\u0626\u0627\u067E\u0631\u06D0\u0644_\u0645\u0627\u064A_\u0626\u0649\u064A\u06C7\u0646_\u0626\u0649\u064A\u06C7\u0644_\u0626\u0627\u06CB\u063A\u06C7\u0633\u062A_\u0633\u06D0\u0646\u062A\u06D5\u0628\u0649\u0631_\u0626\u06C6\u0643\u062A\u06D5\u0628\u0649\u0631_\u0646\u0648\u064A\u0627\u0628\u0649\u0631_\u062F\u06D0\u0643\u0627\u0628\u0649\u0631".split("_"),weekdays:"\u064A\u06D5\u0643\u0634\u06D5\u0646\u0628\u06D5_\u062F\u06C8\u0634\u06D5\u0646\u0628\u06D5_\u0633\u06D5\u064A\u0634\u06D5\u0646\u0628\u06D5_\u0686\u0627\u0631\u0634\u06D5\u0646\u0628\u06D5_\u067E\u06D5\u064A\u0634\u06D5\u0646\u0628\u06D5_\u062C\u06C8\u0645\u06D5_\u0634\u06D5\u0646\u0628\u06D5".split("_"),weekdaysShort:"\u064A\u06D5_\u062F\u06C8_\u0633\u06D5_\u0686\u0627_\u067E\u06D5_\u062C\u06C8_\u0634\u06D5".split("_"),weekdaysMin:"\u064A\u06D5_\u062F\u06C8_\u0633\u06D5_\u0686\u0627_\u067E\u06D5_\u062C\u06C8_\u0634\u06D5".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY-MM-DD",LL:"YYYY-\u064A\u0649\u0644\u0649M-\u0626\u0627\u064A\u0646\u0649\u06ADD-\u0643\u06C8\u0646\u0649",LLL:"YYYY-\u064A\u0649\u0644\u0649M-\u0626\u0627\u064A\u0646\u0649\u06ADD-\u0643\u06C8\u0646\u0649\u060C HH:mm",LLLL:"dddd\u060C YYYY-\u064A\u0649\u0644\u0649M-\u0626\u0627\u064A\u0646\u0649\u06ADD-\u0643\u06C8\u0646\u0649\u060C HH:mm"},meridiemParse:/يېرىم كېچە|سەھەر|چۈشتىن بۇرۇن|چۈش|چۈشتىن كېيىن|كەچ/,meridiemHour:function(r,s){return r===12&&(r=0),s==="\u064A\u06D0\u0631\u0649\u0645 \u0643\u06D0\u0686\u06D5"||s==="\u0633\u06D5\u06BE\u06D5\u0631"||s==="\u0686\u06C8\u0634\u062A\u0649\u0646 \u0628\u06C7\u0631\u06C7\u0646"?r:s==="\u0686\u06C8\u0634\u062A\u0649\u0646 \u0643\u06D0\u064A\u0649\u0646"||s==="\u0643\u06D5\u0686"?r+12:r>=11?r:r+12},meridiem:function(r,s,n){var d=r*100+s;return d<600?"\u064A\u06D0\u0631\u0649\u0645 \u0643\u06D0\u0686\u06D5":d<900?"\u0633\u06D5\u06BE\u06D5\u0631":d<1130?"\u0686\u06C8\u0634\u062A\u0649\u0646 \u0628\u06C7\u0631\u06C7\u0646":d<1230?"\u0686\u06C8\u0634":d<1800?"\u0686\u06C8\u0634\u062A\u0649\u0646 \u0643\u06D0\u064A\u0649\u0646":"\u0643\u06D5\u0686"},calendar:{sameDay:"[\u0628\u06C8\u06AF\u06C8\u0646 \u0633\u0627\u0626\u06D5\u062A] LT",nextDay:"[\u0626\u06D5\u062A\u06D5 \u0633\u0627\u0626\u06D5\u062A] LT",nextWeek:"[\u0643\u06D0\u0644\u06D5\u0631\u0643\u0649] dddd [\u0633\u0627\u0626\u06D5\u062A] LT",lastDay:"[\u062A\u06C6\u0646\u06C8\u06AF\u06C8\u0646] LT",lastWeek:"[\u0626\u0627\u0644\u062F\u0649\u0646\u0642\u0649] dddd [\u0633\u0627\u0626\u06D5\u062A] LT",sameElse:"L"},relativeTime:{future:"%s \u0643\u06D0\u064A\u0649\u0646",past:"%s \u0628\u06C7\u0631\u06C7\u0646",s:"\u0646\u06D5\u0686\u0686\u06D5 \u0633\u06D0\u0643\u0648\u0646\u062A",ss:"%d \u0633\u06D0\u0643\u0648\u0646\u062A",m:"\u0628\u0649\u0631 \u0645\u0649\u0646\u06C7\u062A",mm:"%d \u0645\u0649\u0646\u06C7\u062A",h:"\u0628\u0649\u0631 \u0633\u0627\u0626\u06D5\u062A",hh:"%d \u0633\u0627\u0626\u06D5\u062A",d:"\u0628\u0649\u0631 \u0643\u06C8\u0646",dd:"%d \u0643\u06C8\u0646",M:"\u0628\u0649\u0631 \u0626\u0627\u064A",MM:"%d \u0626\u0627\u064A",y:"\u0628\u0649\u0631 \u064A\u0649\u0644",yy:"%d \u064A\u0649\u0644"},dayOfMonthOrdinalParse:/\d{1,2}(-كۈنى|-ئاي|-ھەپتە)/,ordinal:function(r,s){switch(s){case"d":case"D":case"DDD":return r+"-\u0643\u06C8\u0646\u0649";case"w":case"W":return r+"-\u06BE\u06D5\u067E\u062A\u06D5";default:return r}},preparse:function(r){return r.replace(/،/g,",")},postformat:function(r){return r.replace(/,/g,"\u060C")},week:{dow:1,doy:7}});return a})},16618(L,f,c){//! moment.js locale configuration
//! locale : Ukrainian [uk]
//! author : zemlanin : https://github.com/zemlanin
//! Author : Menelion Elensúle : https://github.com/Oire
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
function a(u,m){var h=u.split("_");return m%10===1&&m%100!==11?h[0]:m%10>=2&&m%10<=4&&(m%100<10||m%100>=20)?h[1]:h[2]}function r(u,m,h){var y={ss:m?"\u0441\u0435\u043A\u0443\u043D\u0434\u0430_\u0441\u0435\u043A\u0443\u043D\u0434\u0438_\u0441\u0435\u043A\u0443\u043D\u0434":"\u0441\u0435\u043A\u0443\u043D\u0434\u0443_\u0441\u0435\u043A\u0443\u043D\u0434\u0438_\u0441\u0435\u043A\u0443\u043D\u0434",mm:m?"\u0445\u0432\u0438\u043B\u0438\u043D\u0430_\u0445\u0432\u0438\u043B\u0438\u043D\u0438_\u0445\u0432\u0438\u043B\u0438\u043D":"\u0445\u0432\u0438\u043B\u0438\u043D\u0443_\u0445\u0432\u0438\u043B\u0438\u043D\u0438_\u0445\u0432\u0438\u043B\u0438\u043D",hh:m?"\u0433\u043E\u0434\u0438\u043D\u0430_\u0433\u043E\u0434\u0438\u043D\u0438_\u0433\u043E\u0434\u0438\u043D":"\u0433\u043E\u0434\u0438\u043D\u0443_\u0433\u043E\u0434\u0438\u043D\u0438_\u0433\u043E\u0434\u0438\u043D",dd:"\u0434\u0435\u043D\u044C_\u0434\u043D\u0456_\u0434\u043D\u0456\u0432",MM:"\u043C\u0456\u0441\u044F\u0446\u044C_\u043C\u0456\u0441\u044F\u0446\u0456_\u043C\u0456\u0441\u044F\u0446\u0456\u0432",yy:"\u0440\u0456\u043A_\u0440\u043E\u043A\u0438_\u0440\u043E\u043A\u0456\u0432"};return h==="m"?m?"\u0445\u0432\u0438\u043B\u0438\u043D\u0430":"\u0445\u0432\u0438\u043B\u0438\u043D\u0443":h==="h"?m?"\u0433\u043E\u0434\u0438\u043D\u0430":"\u0433\u043E\u0434\u0438\u043D\u0443":u+" "+a(y[h],+u)}function s(u,m){var h={nominative:"\u043D\u0435\u0434\u0456\u043B\u044F_\u043F\u043E\u043D\u0435\u0434\u0456\u043B\u043E\u043A_\u0432\u0456\u0432\u0442\u043E\u0440\u043E\u043A_\u0441\u0435\u0440\u0435\u0434\u0430_\u0447\u0435\u0442\u0432\u0435\u0440_\u043F\u2019\u044F\u0442\u043D\u0438\u0446\u044F_\u0441\u0443\u0431\u043E\u0442\u0430".split("_"),accusative:"\u043D\u0435\u0434\u0456\u043B\u044E_\u043F\u043E\u043D\u0435\u0434\u0456\u043B\u043E\u043A_\u0432\u0456\u0432\u0442\u043E\u0440\u043E\u043A_\u0441\u0435\u0440\u0435\u0434\u0443_\u0447\u0435\u0442\u0432\u0435\u0440_\u043F\u2019\u044F\u0442\u043D\u0438\u0446\u044E_\u0441\u0443\u0431\u043E\u0442\u0443".split("_"),genitive:"\u043D\u0435\u0434\u0456\u043B\u0456_\u043F\u043E\u043D\u0435\u0434\u0456\u043B\u043A\u0430_\u0432\u0456\u0432\u0442\u043E\u0440\u043A\u0430_\u0441\u0435\u0440\u0435\u0434\u0438_\u0447\u0435\u0442\u0432\u0435\u0440\u0433\u0430_\u043F\u2019\u044F\u0442\u043D\u0438\u0446\u0456_\u0441\u0443\u0431\u043E\u0442\u0438".split("_")},y;return u===!0?h.nominative.slice(1,7).concat(h.nominative.slice(0,1)):u?(y=/(\[[ВвУу]\]) ?dddd/.test(m)?"accusative":/\[?(?:минулої|наступної)? ?\] ?dddd/.test(m)?"genitive":"nominative",h[y][u.day()]):h.nominative}function n(u){return function(){return u+"\u043E"+(this.hours()===11?"\u0431":"")+"] LT"}}var d=o.defineLocale("uk",{months:{format:"\u0441\u0456\u0447\u043D\u044F_\u043B\u044E\u0442\u043E\u0433\u043E_\u0431\u0435\u0440\u0435\u0437\u043D\u044F_\u043A\u0432\u0456\u0442\u043D\u044F_\u0442\u0440\u0430\u0432\u043D\u044F_\u0447\u0435\u0440\u0432\u043D\u044F_\u043B\u0438\u043F\u043D\u044F_\u0441\u0435\u0440\u043F\u043D\u044F_\u0432\u0435\u0440\u0435\u0441\u043D\u044F_\u0436\u043E\u0432\u0442\u043D\u044F_\u043B\u0438\u0441\u0442\u043E\u043F\u0430\u0434\u0430_\u0433\u0440\u0443\u0434\u043D\u044F".split("_"),standalone:"\u0441\u0456\u0447\u0435\u043D\u044C_\u043B\u044E\u0442\u0438\u0439_\u0431\u0435\u0440\u0435\u0437\u0435\u043D\u044C_\u043A\u0432\u0456\u0442\u0435\u043D\u044C_\u0442\u0440\u0430\u0432\u0435\u043D\u044C_\u0447\u0435\u0440\u0432\u0435\u043D\u044C_\u043B\u0438\u043F\u0435\u043D\u044C_\u0441\u0435\u0440\u043F\u0435\u043D\u044C_\u0432\u0435\u0440\u0435\u0441\u0435\u043D\u044C_\u0436\u043E\u0432\u0442\u0435\u043D\u044C_\u043B\u0438\u0441\u0442\u043E\u043F\u0430\u0434_\u0433\u0440\u0443\u0434\u0435\u043D\u044C".split("_")},monthsShort:"\u0441\u0456\u0447_\u043B\u044E\u0442_\u0431\u0435\u0440_\u043A\u0432\u0456\u0442_\u0442\u0440\u0430\u0432_\u0447\u0435\u0440\u0432_\u043B\u0438\u043F_\u0441\u0435\u0440\u043F_\u0432\u0435\u0440_\u0436\u043E\u0432\u0442_\u043B\u0438\u0441\u0442_\u0433\u0440\u0443\u0434".split("_"),weekdays:s,weekdaysShort:"\u043D\u0434_\u043F\u043D_\u0432\u0442_\u0441\u0440_\u0447\u0442_\u043F\u0442_\u0441\u0431".split("_"),weekdaysMin:"\u043D\u0434_\u043F\u043D_\u0432\u0442_\u0441\u0440_\u0447\u0442_\u043F\u0442_\u0441\u0431".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD.MM.YYYY",LL:"D MMMM YYYY \u0440.",LLL:"D MMMM YYYY \u0440., HH:mm",LLLL:"dddd, D MMMM YYYY \u0440., HH:mm"},calendar:{sameDay:n("[\u0421\u044C\u043E\u0433\u043E\u0434\u043D\u0456 "),nextDay:n("[\u0417\u0430\u0432\u0442\u0440\u0430 "),lastDay:n("[\u0412\u0447\u043E\u0440\u0430 "),nextWeek:n("[\u0423] dddd ["),lastWeek:function(){switch(this.day()){case 0:case 3:case 5:case 6:return n("[\u041C\u0438\u043D\u0443\u043B\u043E\u0457] dddd [").call(this);case 1:case 2:case 4:return n("[\u041C\u0438\u043D\u0443\u043B\u043E\u0433\u043E] dddd [").call(this)}},sameElse:"L"},relativeTime:{future:"\u0437\u0430 %s",past:"%s \u0442\u043E\u043C\u0443",s:"\u0434\u0435\u043A\u0456\u043B\u044C\u043A\u0430 \u0441\u0435\u043A\u0443\u043D\u0434",ss:r,m:r,mm:r,h:"\u0433\u043E\u0434\u0438\u043D\u0443",hh:r,d:"\u0434\u0435\u043D\u044C",dd:r,M:"\u043C\u0456\u0441\u044F\u0446\u044C",MM:r,y:"\u0440\u0456\u043A",yy:r},meridiemParse:/ночі|ранку|дня|вечора/,isPM:function(u){return/^(дня|вечора)$/.test(u)},meridiem:function(u,m,h){return u<4?"\u043D\u043E\u0447\u0456":u<12?"\u0440\u0430\u043D\u043A\u0443":u<17?"\u0434\u043D\u044F":"\u0432\u0435\u0447\u043E\u0440\u0430"},dayOfMonthOrdinalParse:/\d{1,2}-(й|го)/,ordinal:function(u,m){switch(m){case"M":case"d":case"DDD":case"w":case"W":return u+"-\u0439";case"D":return u+"-\u0433\u043E";default:return u}},week:{dow:1,doy:7}});return d})},57777(L,f,c){//! moment.js locale configuration
//! locale : Urdu [ur]
//! author : Sawood Alam : https://github.com/ibnesayeed
//! author : Zack : https://github.com/ZackVision
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=["\u062C\u0646\u0648\u0631\u06CC","\u0641\u0631\u0648\u0631\u06CC","\u0645\u0627\u0631\u0686","\u0627\u067E\u0631\u06CC\u0644","\u0645\u0626\u06CC","\u062C\u0648\u0646","\u062C\u0648\u0644\u0627\u0626\u06CC","\u0627\u06AF\u0633\u062A","\u0633\u062A\u0645\u0628\u0631","\u0627\u06A9\u062A\u0648\u0628\u0631","\u0646\u0648\u0645\u0628\u0631","\u062F\u0633\u0645\u0628\u0631"],r=["\u0627\u062A\u0648\u0627\u0631","\u067E\u06CC\u0631","\u0645\u0646\u06AF\u0644","\u0628\u062F\u06BE","\u062C\u0645\u0639\u0631\u0627\u062A","\u062C\u0645\u0639\u06C1","\u06C1\u0641\u062A\u06C1"],s=o.defineLocale("ur",{months:a,monthsShort:a,weekdays:r,weekdaysShort:r,weekdaysMin:r,longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd\u060C D MMMM YYYY HH:mm"},meridiemParse:/صبح|شام/,isPM:function(n){return n==="\u0634\u0627\u0645"},meridiem:function(n,d,u){return n<12?"\u0635\u0628\u062D":"\u0634\u0627\u0645"},calendar:{sameDay:"[\u0622\u062C \u0628\u0648\u0642\u062A] LT",nextDay:"[\u06A9\u0644 \u0628\u0648\u0642\u062A] LT",nextWeek:"dddd [\u0628\u0648\u0642\u062A] LT",lastDay:"[\u06AF\u0630\u0634\u062A\u06C1 \u0631\u0648\u0632 \u0628\u0648\u0642\u062A] LT",lastWeek:"[\u06AF\u0630\u0634\u062A\u06C1] dddd [\u0628\u0648\u0642\u062A] LT",sameElse:"L"},relativeTime:{future:"%s \u0628\u0639\u062F",past:"%s \u0642\u0628\u0644",s:"\u0686\u0646\u062F \u0633\u06CC\u06A9\u0646\u0688",ss:"%d \u0633\u06CC\u06A9\u0646\u0688",m:"\u0627\u06CC\u06A9 \u0645\u0646\u0679",mm:"%d \u0645\u0646\u0679",h:"\u0627\u06CC\u06A9 \u06AF\u06BE\u0646\u0679\u06C1",hh:"%d \u06AF\u06BE\u0646\u0679\u06D2",d:"\u0627\u06CC\u06A9 \u062F\u0646",dd:"%d \u062F\u0646",M:"\u0627\u06CC\u06A9 \u0645\u0627\u06C1",MM:"%d \u0645\u0627\u06C1",y:"\u0627\u06CC\u06A9 \u0633\u0627\u0644",yy:"%d \u0633\u0627\u0644"},preparse:function(n){return n.replace(/،/g,",")},postformat:function(n){return n.replace(/,/g,"\u060C")},week:{dow:1,doy:4}});return s})},72475(L,f,c){//! moment.js locale configuration
//! locale : Uzbek Latin [uz-latn]
//! author : Rasulbek Mirzayev : github.com/Rasulbeeek
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("uz-latn",{months:"Yanvar_Fevral_Mart_Aprel_May_Iyun_Iyul_Avgust_Sentabr_Oktabr_Noyabr_Dekabr".split("_"),monthsShort:"Yan_Fev_Mar_Apr_May_Iyun_Iyul_Avg_Sen_Okt_Noy_Dek".split("_"),weekdays:"Yakshanba_Dushanba_Seshanba_Chorshanba_Payshanba_Juma_Shanba".split("_"),weekdaysShort:"Yak_Dush_Sesh_Chor_Pay_Jum_Shan".split("_"),weekdaysMin:"Ya_Du_Se_Cho_Pa_Ju_Sha".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"D MMMM YYYY, dddd HH:mm"},calendar:{sameDay:"[Bugun soat] LT [da]",nextDay:"[Ertaga] LT [da]",nextWeek:"dddd [kuni soat] LT [da]",lastDay:"[Kecha soat] LT [da]",lastWeek:"[O'tgan] dddd [kuni soat] LT [da]",sameElse:"L"},relativeTime:{future:"Yaqin %s ichida",past:"Bir necha %s oldin",s:"soniya",ss:"%d soniya",m:"bir daqiqa",mm:"%d daqiqa",h:"bir soat",hh:"%d soat",d:"bir kun",dd:"%d kun",M:"bir oy",MM:"%d oy",y:"bir yil",yy:"%d yil"},week:{dow:1,doy:7}});return a})},57609(L,f,c){//! moment.js locale configuration
//! locale : Uzbek [uz]
//! author : Sardor Muminov : https://github.com/muminoff
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("uz",{months:"\u044F\u043D\u0432\u0430\u0440_\u0444\u0435\u0432\u0440\u0430\u043B_\u043C\u0430\u0440\u0442_\u0430\u043F\u0440\u0435\u043B_\u043C\u0430\u0439_\u0438\u044E\u043D_\u0438\u044E\u043B_\u0430\u0432\u0433\u0443\u0441\u0442_\u0441\u0435\u043D\u0442\u044F\u0431\u0440_\u043E\u043A\u0442\u044F\u0431\u0440_\u043D\u043E\u044F\u0431\u0440_\u0434\u0435\u043A\u0430\u0431\u0440".split("_"),monthsShort:"\u044F\u043D\u0432_\u0444\u0435\u0432_\u043C\u0430\u0440_\u0430\u043F\u0440_\u043C\u0430\u0439_\u0438\u044E\u043D_\u0438\u044E\u043B_\u0430\u0432\u0433_\u0441\u0435\u043D_\u043E\u043A\u0442_\u043D\u043E\u044F_\u0434\u0435\u043A".split("_"),weekdays:"\u042F\u043A\u0448\u0430\u043D\u0431\u0430_\u0414\u0443\u0448\u0430\u043D\u0431\u0430_\u0421\u0435\u0448\u0430\u043D\u0431\u0430_\u0427\u043E\u0440\u0448\u0430\u043D\u0431\u0430_\u041F\u0430\u0439\u0448\u0430\u043D\u0431\u0430_\u0416\u0443\u043C\u0430_\u0428\u0430\u043D\u0431\u0430".split("_"),weekdaysShort:"\u042F\u043A\u0448_\u0414\u0443\u0448_\u0421\u0435\u0448_\u0427\u043E\u0440_\u041F\u0430\u0439_\u0416\u0443\u043C_\u0428\u0430\u043D".split("_"),weekdaysMin:"\u042F\u043A_\u0414\u0443_\u0421\u0435_\u0427\u043E_\u041F\u0430_\u0416\u0443_\u0428\u0430".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"D MMMM YYYY, dddd HH:mm"},calendar:{sameDay:"[\u0411\u0443\u0433\u0443\u043D \u0441\u043E\u0430\u0442] LT [\u0434\u0430]",nextDay:"[\u042D\u0440\u0442\u0430\u0433\u0430] LT [\u0434\u0430]",nextWeek:"dddd [\u043A\u0443\u043D\u0438 \u0441\u043E\u0430\u0442] LT [\u0434\u0430]",lastDay:"[\u041A\u0435\u0447\u0430 \u0441\u043E\u0430\u0442] LT [\u0434\u0430]",lastWeek:"[\u0423\u0442\u0433\u0430\u043D] dddd [\u043A\u0443\u043D\u0438 \u0441\u043E\u0430\u0442] LT [\u0434\u0430]",sameElse:"L"},relativeTime:{future:"\u042F\u043A\u0438\u043D %s \u0438\u0447\u0438\u0434\u0430",past:"\u0411\u0438\u0440 \u043D\u0435\u0447\u0430 %s \u043E\u043B\u0434\u0438\u043D",s:"\u0444\u0443\u0440\u0441\u0430\u0442",ss:"%d \u0444\u0443\u0440\u0441\u0430\u0442",m:"\u0431\u0438\u0440 \u0434\u0430\u043A\u0438\u043A\u0430",mm:"%d \u0434\u0430\u043A\u0438\u043A\u0430",h:"\u0431\u0438\u0440 \u0441\u043E\u0430\u0442",hh:"%d \u0441\u043E\u0430\u0442",d:"\u0431\u0438\u0440 \u043A\u0443\u043D",dd:"%d \u043A\u0443\u043D",M:"\u0431\u0438\u0440 \u043E\u0439",MM:"%d \u043E\u0439",y:"\u0431\u0438\u0440 \u0439\u0438\u043B",yy:"%d \u0439\u0438\u043B"},week:{dow:1,doy:7}});return a})},21135(L,f,c){//! moment.js locale configuration
//! locale : Vietnamese [vi]
//! author : Bang Nguyen : https://github.com/bangnk
//! author : Chien Kira : https://github.com/chienkira
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("vi",{months:"th\xE1ng 1_th\xE1ng 2_th\xE1ng 3_th\xE1ng 4_th\xE1ng 5_th\xE1ng 6_th\xE1ng 7_th\xE1ng 8_th\xE1ng 9_th\xE1ng 10_th\xE1ng 11_th\xE1ng 12".split("_"),monthsShort:"Thg 01_Thg 02_Thg 03_Thg 04_Thg 05_Thg 06_Thg 07_Thg 08_Thg 09_Thg 10_Thg 11_Thg 12".split("_"),monthsParseExact:!0,weekdays:"ch\u1EE7 nh\u1EADt_th\u1EE9 hai_th\u1EE9 ba_th\u1EE9 t\u01B0_th\u1EE9 n\u0103m_th\u1EE9 s\xE1u_th\u1EE9 b\u1EA3y".split("_"),weekdaysShort:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysMin:"CN_T2_T3_T4_T5_T6_T7".split("_"),weekdaysParseExact:!0,meridiemParse:/sa|ch/i,isPM:function(r){return/^ch$/i.test(r)},meridiem:function(r,s,n){return r<12?n?"sa":"SA":n?"ch":"CH"},longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"D MMMM [n\u0103m] YYYY",LLL:"D MMMM [n\u0103m] YYYY HH:mm",LLLL:"dddd, D MMMM [n\u0103m] YYYY HH:mm",l:"DD/M/YYYY",ll:"D MMM YYYY",lll:"D MMM YYYY HH:mm",llll:"ddd, D MMM YYYY HH:mm"},calendar:{sameDay:"[H\xF4m nay l\xFAc] LT",nextDay:"[Ng\xE0y mai l\xFAc] LT",nextWeek:"dddd [tu\u1EA7n t\u1EDBi l\xFAc] LT",lastDay:"[H\xF4m qua l\xFAc] LT",lastWeek:"dddd [tu\u1EA7n tr\u01B0\u1EDBc l\xFAc] LT",sameElse:"L"},relativeTime:{future:"%s t\u1EDBi",past:"%s tr\u01B0\u1EDBc",s:"v\xE0i gi\xE2y",ss:"%d gi\xE2y",m:"m\u1ED9t ph\xFAt",mm:"%d ph\xFAt",h:"m\u1ED9t gi\u1EDD",hh:"%d gi\u1EDD",d:"m\u1ED9t ng\xE0y",dd:"%d ng\xE0y",w:"m\u1ED9t tu\u1EA7n",ww:"%d tu\u1EA7n",M:"m\u1ED9t th\xE1ng",MM:"%d th\xE1ng",y:"m\u1ED9t n\u0103m",yy:"%d n\u0103m"},dayOfMonthOrdinalParse:/\d{1,2}/,ordinal:function(r){return r},week:{dow:1,doy:4}});return a})},64051(L,f,c){//! moment.js locale configuration
//! locale : Pseudo [x-pseudo]
//! author : Andrew Hood : https://github.com/andrewhood125
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("x-pseudo",{months:"J~\xE1\xF1\xFA\xE1~r\xFD_F~\xE9br\xFA~\xE1r\xFD_~M\xE1rc~h_\xC1p~r\xEDl_~M\xE1\xFD_~J\xFA\xF1\xE9~_J\xFAl~\xFD_\xC1\xFA~g\xFAst~_S\xE9p~t\xE9mb~\xE9r_\xD3~ct\xF3b~\xE9r_\xD1~\xF3v\xE9m~b\xE9r_~D\xE9c\xE9~mb\xE9r".split("_"),monthsShort:"J~\xE1\xF1_~F\xE9b_~M\xE1r_~\xC1pr_~M\xE1\xFD_~J\xFA\xF1_~J\xFAl_~\xC1\xFAg_~S\xE9p_~\xD3ct_~\xD1\xF3v_~D\xE9c".split("_"),monthsParseExact:!0,weekdays:"S~\xFA\xF1d\xE1~\xFD_M\xF3~\xF1d\xE1\xFD~_T\xFA\xE9~sd\xE1\xFD~_W\xE9d~\xF1\xE9sd~\xE1\xFD_T~h\xFArs~d\xE1\xFD_~Fr\xEDd~\xE1\xFD_S~\xE1t\xFAr~d\xE1\xFD".split("_"),weekdaysShort:"S~\xFA\xF1_~M\xF3\xF1_~T\xFA\xE9_~W\xE9d_~Th\xFA_~Fr\xED_~S\xE1t".split("_"),weekdaysMin:"S~\xFA_M\xF3~_T\xFA_~W\xE9_T~h_Fr~_S\xE1".split("_"),weekdaysParseExact:!0,longDateFormat:{LT:"HH:mm",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY HH:mm",LLLL:"dddd, D MMMM YYYY HH:mm"},calendar:{sameDay:"[T~\xF3d\xE1~\xFD \xE1t] LT",nextDay:"[T~\xF3m\xF3~rr\xF3~w \xE1t] LT",nextWeek:"dddd [\xE1t] LT",lastDay:"[\xDD~\xE9st~\xE9rd\xE1~\xFD \xE1t] LT",lastWeek:"[L~\xE1st] dddd [\xE1t] LT",sameElse:"L"},relativeTime:{future:"\xED~\xF1 %s",past:"%s \xE1~g\xF3",s:"\xE1 ~f\xE9w ~s\xE9c\xF3~\xF1ds",ss:"%d s~\xE9c\xF3\xF1~ds",m:"\xE1 ~m\xED\xF1~\xFAt\xE9",mm:"%d m~\xED\xF1\xFA~t\xE9s",h:"\xE1~\xF1 h\xF3~\xFAr",hh:"%d h~\xF3\xFArs",d:"\xE1 ~d\xE1\xFD",dd:"%d d~\xE1\xFDs",M:"\xE1 ~m\xF3\xF1~th",MM:"%d m~\xF3\xF1t~hs",y:"\xE1 ~\xFD\xE9\xE1r",yy:"%d \xFD~\xE9\xE1rs"},dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(r){var s=r%10,n=~~(r%100/10)===1?"th":s===1?"st":s===2?"nd":s===3?"rd":"th";return r+n},week:{dow:1,doy:4}});return a})},82218(L,f,c){//! moment.js locale configuration
//! locale : Yoruba Nigeria [yo]
//! author : Atolagbe Abisoye : https://github.com/andela-batolagbe
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("yo",{months:"S\u1EB9\u0301r\u1EB9\u0301_E\u0300re\u0300le\u0300_\u1EB8r\u1EB9\u0300na\u0300_I\u0300gbe\u0301_E\u0300bibi_O\u0300ku\u0300du_Ag\u1EB9mo_O\u0300gu\u0301n_Owewe_\u1ECC\u0300wa\u0300ra\u0300_Be\u0301lu\u0301_\u1ECC\u0300p\u1EB9\u0300\u0300".split("_"),monthsShort:"S\u1EB9\u0301r_E\u0300rl_\u1EB8rn_I\u0300gb_E\u0300bi_O\u0300ku\u0300_Ag\u1EB9_O\u0300gu\u0301_Owe_\u1ECC\u0300wa\u0300_Be\u0301l_\u1ECC\u0300p\u1EB9\u0300\u0300".split("_"),weekdays:"A\u0300i\u0300ku\u0301_Aje\u0301_I\u0300s\u1EB9\u0301gun_\u1ECCj\u1ECD\u0301ru\u0301_\u1ECCj\u1ECD\u0301b\u1ECD_\u1EB8ti\u0300_A\u0300ba\u0301m\u1EB9\u0301ta".split("_"),weekdaysShort:"A\u0300i\u0300k_Aje\u0301_I\u0300s\u1EB9\u0301_\u1ECCjr_\u1ECCjb_\u1EB8ti\u0300_A\u0300ba\u0301".split("_"),weekdaysMin:"A\u0300i\u0300_Aj_I\u0300s_\u1ECCr_\u1ECCb_\u1EB8t_A\u0300b".split("_"),longDateFormat:{LT:"h:mm A",LTS:"h:mm:ss A",L:"DD/MM/YYYY",LL:"D MMMM YYYY",LLL:"D MMMM YYYY h:mm A",LLLL:"dddd, D MMMM YYYY h:mm A"},calendar:{sameDay:"[O\u0300ni\u0300 ni] LT",nextDay:"[\u1ECC\u0300la ni] LT",nextWeek:"dddd [\u1ECCs\u1EB9\u0300 to\u0301n'b\u1ECD] [ni] LT",lastDay:"[A\u0300na ni] LT",lastWeek:"dddd [\u1ECCs\u1EB9\u0300 to\u0301l\u1ECD\u0301] [ni] LT",sameElse:"L"},relativeTime:{future:"ni\u0301 %s",past:"%s k\u1ECDja\u0301",s:"i\u0300s\u1EB9ju\u0301 aaya\u0301 die",ss:"aaya\u0301 %d",m:"i\u0300s\u1EB9ju\u0301 kan",mm:"i\u0300s\u1EB9ju\u0301 %d",h:"wa\u0301kati kan",hh:"wa\u0301kati %d",d:"\u1ECDj\u1ECD\u0301 kan",dd:"\u1ECDj\u1ECD\u0301 %d",M:"osu\u0300 kan",MM:"osu\u0300 %d",y:"\u1ECDdu\u0301n kan",yy:"\u1ECDdu\u0301n %d"},dayOfMonthOrdinalParse:/ọjọ́\s\d{1,2}/,ordinal:"\u1ECDj\u1ECD\u0301 %d",week:{dow:1,doy:4}});return a})},52648(L,f,c){//! moment.js locale configuration
//! locale : Chinese (China) [zh-cn]
//! author : suupic : https://github.com/suupic
//! author : Zeno Zeng : https://github.com/zenozeng
//! author : uu109 : https://github.com/uu109
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("zh-cn",{months:"\u4E00\u6708_\u4E8C\u6708_\u4E09\u6708_\u56DB\u6708_\u4E94\u6708_\u516D\u6708_\u4E03\u6708_\u516B\u6708_\u4E5D\u6708_\u5341\u6708_\u5341\u4E00\u6708_\u5341\u4E8C\u6708".split("_"),monthsShort:"1\u6708_2\u6708_3\u6708_4\u6708_5\u6708_6\u6708_7\u6708_8\u6708_9\u6708_10\u6708_11\u6708_12\u6708".split("_"),weekdays:"\u661F\u671F\u65E5_\u661F\u671F\u4E00_\u661F\u671F\u4E8C_\u661F\u671F\u4E09_\u661F\u671F\u56DB_\u661F\u671F\u4E94_\u661F\u671F\u516D".split("_"),weekdaysShort:"\u5468\u65E5_\u5468\u4E00_\u5468\u4E8C_\u5468\u4E09_\u5468\u56DB_\u5468\u4E94_\u5468\u516D".split("_"),weekdaysMin:"\u65E5_\u4E00_\u4E8C_\u4E09_\u56DB_\u4E94_\u516D".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY\u5E74M\u6708D\u65E5",LLL:"YYYY\u5E74M\u6708D\u65E5Ah\u70B9mm\u5206",LLLL:"YYYY\u5E74M\u6708D\u65E5ddddAh\u70B9mm\u5206",l:"YYYY/M/D",ll:"YYYY\u5E74M\u6708D\u65E5",lll:"YYYY\u5E74M\u6708D\u65E5 HH:mm",llll:"YYYY\u5E74M\u6708D\u65E5dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(r,s){return r===12&&(r=0),s==="\u51CC\u6668"||s==="\u65E9\u4E0A"||s==="\u4E0A\u5348"?r:s==="\u4E0B\u5348"||s==="\u665A\u4E0A"?r+12:r>=11?r:r+12},meridiem:function(r,s,n){var d=r*100+s;return d<600?"\u51CC\u6668":d<900?"\u65E9\u4E0A":d<1130?"\u4E0A\u5348":d<1230?"\u4E2D\u5348":d<1800?"\u4E0B\u5348":"\u665A\u4E0A"},calendar:{sameDay:"[\u4ECA\u5929]LT",nextDay:"[\u660E\u5929]LT",nextWeek:function(r){return r.week()!==this.week()?"[\u4E0B]dddLT":"[\u672C]dddLT"},lastDay:"[\u6628\u5929]LT",lastWeek:function(r){return this.week()!==r.week()?"[\u4E0A]dddLT":"[\u672C]dddLT"},sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|周)/,ordinal:function(r,s){switch(s){case"d":case"D":case"DDD":return r+"\u65E5";case"M":return r+"\u6708";case"w":case"W":return r+"\u5468";default:return r}},relativeTime:{future:"%s\u540E",past:"%s\u524D",s:"\u51E0\u79D2",ss:"%d \u79D2",m:"1 \u5206\u949F",mm:"%d \u5206\u949F",h:"1 \u5C0F\u65F6",hh:"%d \u5C0F\u65F6",d:"1 \u5929",dd:"%d \u5929",w:"1 \u5468",ww:"%d \u5468",M:"1 \u4E2A\u6708",MM:"%d \u4E2A\u6708",y:"1 \u5E74",yy:"%d \u5E74"},week:{dow:1,doy:4}});return a})},1632(L,f,c){//! moment.js locale configuration
//! locale : Chinese (Hong Kong) [zh-hk]
//! author : Ben : https://github.com/ben-lin
//! author : Chris Lam : https://github.com/hehachris
//! author : Konstantin : https://github.com/skfd
//! author : Anthony : https://github.com/anthonylau
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("zh-hk",{months:"\u4E00\u6708_\u4E8C\u6708_\u4E09\u6708_\u56DB\u6708_\u4E94\u6708_\u516D\u6708_\u4E03\u6708_\u516B\u6708_\u4E5D\u6708_\u5341\u6708_\u5341\u4E00\u6708_\u5341\u4E8C\u6708".split("_"),monthsShort:"1\u6708_2\u6708_3\u6708_4\u6708_5\u6708_6\u6708_7\u6708_8\u6708_9\u6708_10\u6708_11\u6708_12\u6708".split("_"),weekdays:"\u661F\u671F\u65E5_\u661F\u671F\u4E00_\u661F\u671F\u4E8C_\u661F\u671F\u4E09_\u661F\u671F\u56DB_\u661F\u671F\u4E94_\u661F\u671F\u516D".split("_"),weekdaysShort:"\u9031\u65E5_\u9031\u4E00_\u9031\u4E8C_\u9031\u4E09_\u9031\u56DB_\u9031\u4E94_\u9031\u516D".split("_"),weekdaysMin:"\u65E5_\u4E00_\u4E8C_\u4E09_\u56DB_\u4E94_\u516D".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY\u5E74M\u6708D\u65E5",LLL:"YYYY\u5E74M\u6708D\u65E5 HH:mm",LLLL:"YYYY\u5E74M\u6708D\u65E5dddd HH:mm",l:"YYYY/M/D",ll:"YYYY\u5E74M\u6708D\u65E5",lll:"YYYY\u5E74M\u6708D\u65E5 HH:mm",llll:"YYYY\u5E74M\u6708D\u65E5dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(r,s){if(r===12&&(r=0),s==="\u51CC\u6668"||s==="\u65E9\u4E0A"||s==="\u4E0A\u5348")return r;if(s==="\u4E2D\u5348")return r>=11?r:r+12;if(s==="\u4E0B\u5348"||s==="\u665A\u4E0A")return r+12},meridiem:function(r,s,n){var d=r*100+s;return d<600?"\u51CC\u6668":d<900?"\u65E9\u4E0A":d<1200?"\u4E0A\u5348":d===1200?"\u4E2D\u5348":d<1800?"\u4E0B\u5348":"\u665A\u4E0A"},calendar:{sameDay:"[\u4ECA\u5929]LT",nextDay:"[\u660E\u5929]LT",nextWeek:"[\u4E0B]ddddLT",lastDay:"[\u6628\u5929]LT",lastWeek:"[\u4E0A]ddddLT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(r,s){switch(s){case"d":case"D":case"DDD":return r+"\u65E5";case"M":return r+"\u6708";case"w":case"W":return r+"\u9031";default:return r}},relativeTime:{future:"%s\u5F8C",past:"%s\u524D",s:"\u5E7E\u79D2",ss:"%d \u79D2",m:"1 \u5206\u9418",mm:"%d \u5206\u9418",h:"1 \u5C0F\u6642",hh:"%d \u5C0F\u6642",d:"1 \u5929",dd:"%d \u5929",M:"1 \u500B\u6708",MM:"%d \u500B\u6708",y:"1 \u5E74",yy:"%d \u5E74"}});return a})},31541(L,f,c){//! moment.js locale configuration
//! locale : Chinese (Macau) [zh-mo]
//! author : Ben : https://github.com/ben-lin
//! author : Chris Lam : https://github.com/hehachris
//! author : Tan Yuanhong : https://github.com/le0tan
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("zh-mo",{months:"\u4E00\u6708_\u4E8C\u6708_\u4E09\u6708_\u56DB\u6708_\u4E94\u6708_\u516D\u6708_\u4E03\u6708_\u516B\u6708_\u4E5D\u6708_\u5341\u6708_\u5341\u4E00\u6708_\u5341\u4E8C\u6708".split("_"),monthsShort:"1\u6708_2\u6708_3\u6708_4\u6708_5\u6708_6\u6708_7\u6708_8\u6708_9\u6708_10\u6708_11\u6708_12\u6708".split("_"),weekdays:"\u661F\u671F\u65E5_\u661F\u671F\u4E00_\u661F\u671F\u4E8C_\u661F\u671F\u4E09_\u661F\u671F\u56DB_\u661F\u671F\u4E94_\u661F\u671F\u516D".split("_"),weekdaysShort:"\u9031\u65E5_\u9031\u4E00_\u9031\u4E8C_\u9031\u4E09_\u9031\u56DB_\u9031\u4E94_\u9031\u516D".split("_"),weekdaysMin:"\u65E5_\u4E00_\u4E8C_\u4E09_\u56DB_\u4E94_\u516D".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"DD/MM/YYYY",LL:"YYYY\u5E74M\u6708D\u65E5",LLL:"YYYY\u5E74M\u6708D\u65E5 HH:mm",LLLL:"YYYY\u5E74M\u6708D\u65E5dddd HH:mm",l:"D/M/YYYY",ll:"YYYY\u5E74M\u6708D\u65E5",lll:"YYYY\u5E74M\u6708D\u65E5 HH:mm",llll:"YYYY\u5E74M\u6708D\u65E5dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(r,s){if(r===12&&(r=0),s==="\u51CC\u6668"||s==="\u65E9\u4E0A"||s==="\u4E0A\u5348")return r;if(s==="\u4E2D\u5348")return r>=11?r:r+12;if(s==="\u4E0B\u5348"||s==="\u665A\u4E0A")return r+12},meridiem:function(r,s,n){var d=r*100+s;return d<600?"\u51CC\u6668":d<900?"\u65E9\u4E0A":d<1130?"\u4E0A\u5348":d<1230?"\u4E2D\u5348":d<1800?"\u4E0B\u5348":"\u665A\u4E0A"},calendar:{sameDay:"[\u4ECA\u5929] LT",nextDay:"[\u660E\u5929] LT",nextWeek:"[\u4E0B]dddd LT",lastDay:"[\u6628\u5929] LT",lastWeek:"[\u4E0A]dddd LT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(r,s){switch(s){case"d":case"D":case"DDD":return r+"\u65E5";case"M":return r+"\u6708";case"w":case"W":return r+"\u9031";default:return r}},relativeTime:{future:"%s\u5167",past:"%s\u524D",s:"\u5E7E\u79D2",ss:"%d \u79D2",m:"1 \u5206\u9418",mm:"%d \u5206\u9418",h:"1 \u5C0F\u6642",hh:"%d \u5C0F\u6642",d:"1 \u5929",dd:"%d \u5929",M:"1 \u500B\u6708",MM:"%d \u500B\u6708",y:"1 \u5E74",yy:"%d \u5E74"}});return a})},50304(L,f,c){//! moment.js locale configuration
//! locale : Chinese (Taiwan) [zh-tw]
//! author : Ben : https://github.com/ben-lin
//! author : Chris Lam : https://github.com/hehachris
(function(o,a){a(c(95093))})(this,function(o){"use strict";//! moment.js locale configuration
var a=o.defineLocale("zh-tw",{months:"\u4E00\u6708_\u4E8C\u6708_\u4E09\u6708_\u56DB\u6708_\u4E94\u6708_\u516D\u6708_\u4E03\u6708_\u516B\u6708_\u4E5D\u6708_\u5341\u6708_\u5341\u4E00\u6708_\u5341\u4E8C\u6708".split("_"),monthsShort:"1\u6708_2\u6708_3\u6708_4\u6708_5\u6708_6\u6708_7\u6708_8\u6708_9\u6708_10\u6708_11\u6708_12\u6708".split("_"),weekdays:"\u661F\u671F\u65E5_\u661F\u671F\u4E00_\u661F\u671F\u4E8C_\u661F\u671F\u4E09_\u661F\u671F\u56DB_\u661F\u671F\u4E94_\u661F\u671F\u516D".split("_"),weekdaysShort:"\u9031\u65E5_\u9031\u4E00_\u9031\u4E8C_\u9031\u4E09_\u9031\u56DB_\u9031\u4E94_\u9031\u516D".split("_"),weekdaysMin:"\u65E5_\u4E00_\u4E8C_\u4E09_\u56DB_\u4E94_\u516D".split("_"),longDateFormat:{LT:"HH:mm",LTS:"HH:mm:ss",L:"YYYY/MM/DD",LL:"YYYY\u5E74M\u6708D\u65E5",LLL:"YYYY\u5E74M\u6708D\u65E5 HH:mm",LLLL:"YYYY\u5E74M\u6708D\u65E5dddd HH:mm",l:"YYYY/M/D",ll:"YYYY\u5E74M\u6708D\u65E5",lll:"YYYY\u5E74M\u6708D\u65E5 HH:mm",llll:"YYYY\u5E74M\u6708D\u65E5dddd HH:mm"},meridiemParse:/凌晨|早上|上午|中午|下午|晚上/,meridiemHour:function(r,s){if(r===12&&(r=0),s==="\u51CC\u6668"||s==="\u65E9\u4E0A"||s==="\u4E0A\u5348")return r;if(s==="\u4E2D\u5348")return r>=11?r:r+12;if(s==="\u4E0B\u5348"||s==="\u665A\u4E0A")return r+12},meridiem:function(r,s,n){var d=r*100+s;return d<600?"\u51CC\u6668":d<900?"\u65E9\u4E0A":d<1130?"\u4E0A\u5348":d<1230?"\u4E2D\u5348":d<1800?"\u4E0B\u5348":"\u665A\u4E0A"},calendar:{sameDay:"[\u4ECA\u5929] LT",nextDay:"[\u660E\u5929] LT",nextWeek:"[\u4E0B]dddd LT",lastDay:"[\u6628\u5929] LT",lastWeek:"[\u4E0A]dddd LT",sameElse:"L"},dayOfMonthOrdinalParse:/\d{1,2}(日|月|週)/,ordinal:function(r,s){switch(s){case"d":case"D":case"DDD":return r+"\u65E5";case"M":return r+"\u6708";case"w":case"W":return r+"\u9031";default:return r}},relativeTime:{future:"%s\u5F8C",past:"%s\u524D",s:"\u5E7E\u79D2",ss:"%d \u79D2",m:"1 \u5206\u9418",mm:"%d \u5206\u9418",h:"1 \u5C0F\u6642",hh:"%d \u5C0F\u6642",d:"1 \u5929",dd:"%d \u5929",M:"1 \u500B\u6708",MM:"%d \u500B\u6708",y:"1 \u5E74",yy:"%d \u5E74"}});return a})},95093(L,f,c){L=c.nmd(L);//! moment.js
//! version : 2.30.1
//! authors : Tim Wood, Iskren Chernev, Moment.js contributors
//! license : MIT
//! momentjs.com
(function(o,a){L.exports=a()})(this,function(){"use strict";var o;function a(){return o.apply(null,arguments)}function r(e){o=e}function s(e){return e instanceof Array||Object.prototype.toString.call(e)==="[object Array]"}function n(e){return e!=null&&Object.prototype.toString.call(e)==="[object Object]"}function d(e,t){return Object.prototype.hasOwnProperty.call(e,t)}function u(e){if(Object.getOwnPropertyNames)return Object.getOwnPropertyNames(e).length===0;var t;for(t in e)if(d(e,t))return!1;return!0}function m(e){return e===void 0}function h(e){return typeof e=="number"||Object.prototype.toString.call(e)==="[object Number]"}function y(e){return e instanceof Date||Object.prototype.toString.call(e)==="[object Date]"}function g(e,t){var _=[],i,l=e.length;for(i=0;i<l;++i)_.push(t(e[i],i));return _}function v(e,t){for(var _ in t)d(t,_)&&(e[_]=t[_]);return d(t,"toString")&&(e.toString=t.toString),d(t,"valueOf")&&(e.valueOf=t.valueOf),e}function H(e,t,_,i){return sa(e,t,_,i,!0).utc()}function Q(){return{empty:!1,unusedTokens:[],unusedInput:[],overflow:-2,charsLeftOver:0,nullInput:!1,invalidEra:null,invalidMonth:null,invalidFormat:!1,userInvalidated:!1,iso:!1,parsedDateParts:[],era:null,meridiem:null,rfc2822:!1,weekdayMismatch:!1}}function k(e){return e._pf==null&&(e._pf=Q()),e._pf}var I;Array.prototype.some?I=Array.prototype.some:I=function(e){var t=Object(this),_=t.length>>>0,i;for(i=0;i<_;i++)if(i in t&&e.call(this,t[i],i,t))return!0;return!1};function Me(e){var t=null,_=!1,i=e._d&&!isNaN(e._d.getTime());if(i&&(t=k(e),_=I.call(t.parsedDateParts,function(l){return l!=null}),i=t.overflow<0&&!t.empty&&!t.invalidEra&&!t.invalidMonth&&!t.invalidWeekday&&!t.weekdayMismatch&&!t.nullInput&&!t.invalidFormat&&!t.userInvalidated&&(!t.meridiem||t.meridiem&&_),e._strict&&(i=i&&t.charsLeftOver===0&&t.unusedTokens.length===0&&t.bigHour===void 0)),Object.isFrozen==null||!Object.isFrozen(e))e._isValid=i;else return i;return e._isValid}function he(e){var t=H(NaN);return e!=null?v(k(t),e):k(t).userInvalidated=!0,t}var xt=a.momentProperties=[],tt=!1;function at(e,t){var _,i,l,M=xt.length;if(m(t._isAMomentObject)||(e._isAMomentObject=t._isAMomentObject),m(t._i)||(e._i=t._i),m(t._f)||(e._f=t._f),m(t._l)||(e._l=t._l),m(t._strict)||(e._strict=t._strict),m(t._tzm)||(e._tzm=t._tzm),m(t._isUTC)||(e._isUTC=t._isUTC),m(t._offset)||(e._offset=t._offset),m(t._pf)||(e._pf=k(t)),m(t._locale)||(e._locale=t._locale),M>0)for(_=0;_<M;_++)i=xt[_],l=t[i],m(l)||(e[i]=l);return e}function Te(e){at(this,e),this._d=new Date(e._d!=null?e._d.getTime():NaN),this.isValid()||(this._d=new Date(NaN)),tt===!1&&(tt=!0,a.updateOffset(this),tt=!1)}function V(e){return e instanceof Te||e!=null&&e._isAMomentObject!=null}function Pt(e){a.suppressDeprecationWarnings===!1&&typeof console<"u"&&console.warn&&console.warn("Deprecation warning: "+e)}function C(e,t){var _=!0;return v(function(){if(a.deprecationHandler!=null&&a.deprecationHandler(null,e),_){var i=[],l,M,Y,D=arguments.length;for(M=0;M<D;M++){if(l="",typeof arguments[M]=="object"){l+=`
[`+M+"] ";for(Y in arguments[0])d(arguments[0],Y)&&(l+=Y+": "+arguments[0][Y]+", ");l=l.slice(0,-2)}else l=arguments[M];i.push(l)}Pt(e+`
Arguments: `+Array.prototype.slice.call(i).join("")+`
`+new Error().stack),_=!1}return t.apply(this,arguments)},t)}var Ot={};function Wt(e,t){a.deprecationHandler!=null&&a.deprecationHandler(e,t),Ot[e]||(Pt(t),Ot[e]=!0)}a.suppressDeprecationWarnings=!1,a.deprecationHandler=null;function q(e){return typeof Function<"u"&&e instanceof Function||Object.prototype.toString.call(e)==="[object Function]"}function va(e){var t,_;for(_ in e)d(e,_)&&(t=e[_],q(t)?this[_]=t:this["_"+_]=t);this._config=e,this._dayOfMonthOrdinalParseLenient=new RegExp((this._dayOfMonthOrdinalParse.source||this._ordinalParse.source)+"|"+/\d{1,2}/.source)}function st(e,t){var _=v({},e),i;for(i in t)d(t,i)&&(n(e[i])&&n(t[i])?(_[i]={},v(_[i],e[i]),v(_[i],t[i])):t[i]!=null?_[i]=t[i]:delete _[i]);for(i in e)d(e,i)&&!d(t,i)&&n(e[i])&&(_[i]=v({},_[i]));return _}function nt(e){e!=null&&this.set(e)}var rt;Object.keys?rt=Object.keys:rt=function(e){var t,_=[];for(t in e)d(e,t)&&_.push(t);return _};var ba={sameDay:"[Today at] LT",nextDay:"[Tomorrow at] LT",nextWeek:"dddd [at] LT",lastDay:"[Yesterday at] LT",lastWeek:"[Last] dddd [at] LT",sameElse:"L"};function Ha(e,t,_){var i=this._calendar[e]||this._calendar.sameElse;return q(i)?i.call(t,_):i}function Z(e,t,_){var i=""+Math.abs(e),l=t-i.length,M=e>=0;return(M?_?"+":"":"-")+Math.pow(10,Math.max(0,l)).toString().substr(1)+i}var _t=/(\[[^\[]*\])|(\\)?([Hh]mm(ss)?|Mo|MM?M?M?|Do|DDDo|DD?D?D?|ddd?d?|do?|w[o|w]?|W[o|W]?|Qo?|N{1,5}|YYYYYY|YYYYY|YYYY|YY|y{2,4}|yo?|gg(ggg?)?|GG(GGG?)?|e|E|a|A|hh?|HH?|kk?|mm?|ss?|S{1,9}|x|X|zz?|ZZ?|.)/g,Ae=/(\[[^\[]*\])|(\\)?(LTS|LT|LL?L?L?|l{1,4})/g,it={},Le={};function T(e,t,_,i){var l=i;typeof i=="string"&&(l=function(){return this[i]()}),e&&(Le[e]=l),t&&(Le[t[0]]=function(){return Z(l.apply(this,arguments),t[1],t[2])}),_&&(Le[_]=function(){return this.localeData().ordinal(l.apply(this,arguments),e)})}function Sa(e){return e.match(/\[[\s\S]/)?e.replace(/^\[|\]$/g,""):e.replace(/\\/g,"")}function ja(e){var t=e.match(_t),_,i;for(_=0,i=t.length;_<i;_++)Le[t[_]]?t[_]=Le[t[_]]:t[_]=Sa(t[_]);return function(l){var M="",Y;for(Y=0;Y<i;Y++)M+=q(t[Y])?t[Y].call(l,e):t[Y];return M}}function Ee(e,t){return e.isValid()?(t=At(t,e.localeData()),it[t]=it[t]||ja(t),it[t](e)):e.localeData().invalidDate()}function At(e,t){var _=5;function i(l){return t.longDateFormat(l)||l}for(Ae.lastIndex=0;_>=0&&Ae.test(e);)e=e.replace(Ae,i),Ae.lastIndex=0,_-=1;return e}var xa={LTS:"h:mm:ss A",LT:"h:mm A",L:"MM/DD/YYYY",LL:"MMMM D, YYYY",LLL:"MMMM D, YYYY h:mm A",LLLL:"dddd, MMMM D, YYYY h:mm A"};function Pa(e){var t=this._longDateFormat[e],_=this._longDateFormat[e.toUpperCase()];return t||!_?t:(this._longDateFormat[e]=_.match(_t).map(function(i){return i==="MMMM"||i==="MM"||i==="DD"||i==="dddd"?i.slice(1):i}).join(""),this._longDateFormat[e])}var Oa="Invalid date";function Wa(){return this._invalidDate}var Aa="%d",Ea=/\d{1,2}/;function Fa(e){return this._ordinal.replace("%d",e)}var za={future:"in %s",past:"%s ago",s:"a few seconds",ss:"%d seconds",m:"a minute",mm:"%d minutes",h:"an hour",hh:"%d hours",d:"a day",dd:"%d days",w:"a week",ww:"%d weeks",M:"a month",MM:"%d months",y:"a year",yy:"%d years"};function Na(e,t,_,i){var l=this._relativeTime[_];return q(l)?l(e,t,_,i):l.replace(/%d/i,e)}function Ra(e,t){var _=this._relativeTime[e>0?"future":"past"];return q(_)?_(t):_.replace(/%s/i,t)}var Et={D:"date",dates:"date",date:"date",d:"day",days:"day",day:"day",e:"weekday",weekdays:"weekday",weekday:"weekday",E:"isoWeekday",isoweekdays:"isoWeekday",isoweekday:"isoWeekday",DDD:"dayOfYear",dayofyears:"dayOfYear",dayofyear:"dayOfYear",h:"hour",hours:"hour",hour:"hour",ms:"millisecond",milliseconds:"millisecond",millisecond:"millisecond",m:"minute",minutes:"minute",minute:"minute",M:"month",months:"month",month:"month",Q:"quarter",quarters:"quarter",quarter:"quarter",s:"second",seconds:"second",second:"second",gg:"weekYear",weekyears:"weekYear",weekyear:"weekYear",GG:"isoWeekYear",isoweekyears:"isoWeekYear",isoweekyear:"isoWeekYear",w:"week",weeks:"week",week:"week",W:"isoWeek",isoweeks:"isoWeek",isoweek:"isoWeek",y:"year",years:"year",year:"year"};function U(e){return typeof e=="string"?Et[e]||Et[e.toLowerCase()]:void 0}function dt(e){var t={},_,i;for(i in e)d(e,i)&&(_=U(i),_&&(t[_]=e[i]));return t}var Ja={date:9,day:11,weekday:11,isoWeekday:11,dayOfYear:4,hour:13,millisecond:16,minute:14,month:8,quarter:7,second:15,weekYear:1,isoWeekYear:1,week:5,isoWeek:5,year:1};function Ia(e){var t=[],_;for(_ in e)d(e,_)&&t.push({unit:_,priority:Ja[_]});return t.sort(function(i,l){return i.priority-l.priority}),t}var Ft=/\d/,J=/\d\d/,zt=/\d{3}/,ot=/\d{4}/,Fe=/[+-]?\d{6}/,W=/\d\d?/,Nt=/\d\d\d\d?/,Rt=/\d\d\d\d\d\d?/,ze=/\d{1,3}/,ut=/\d{1,4}/,Ne=/[+-]?\d{1,6}/,fe=/\d+/,Re=/[+-]?\d+/,Ca=/Z|[+-]\d\d:?\d\d/gi,Je=/Z|[+-]\d\d(?::?\d\d)?/gi,Ua=/[+-]?\d+(\.\d{1,3})?/,ve=/[0-9]{0,256}['a-z\u00A0-\u05FF\u0700-\uD7FF\uF900-\uFDCF\uFDF0-\uFF07\uFF10-\uFFEF]{1,256}|[\u0600-\u06FF\/]{1,256}(\s*?[\u0600-\u06FF]{1,256}){1,2}/i,Ye=/^[1-9]\d?/,lt=/^([1-9]\d|\d)/,Ie;Ie={};function w(e,t,_){Ie[e]=q(t)?t:function(i,l){return i&&_?_:t}}function Ga(e,t){return d(Ie,e)?Ie[e](t._strict,t._locale):new RegExp(Va(e))}function Va(e){return X(e.replace("\\","").replace(/\\(\[)|\\(\])|\[([^\]\[]*)\]|\\(.)/g,function(t,_,i,l,M){return _||i||l||M}))}function X(e){return e.replace(/[-\/\\^$*+?.()|[\]{}]/g,"\\$&")}function G(e){return e<0?Math.ceil(e)||0:Math.floor(e)}function S(e){var t=+e,_=0;return t!==0&&isFinite(t)&&(_=G(t)),_}var mt={};function O(e,t){var _,i=t,l;for(typeof e=="string"&&(e=[e]),h(t)&&(i=function(M,Y){Y[t]=S(M)}),l=e.length,_=0;_<l;_++)mt[e[_]]=i}function be(e,t){O(e,function(_,i,l,M){l._w=l._w||{},t(_,l._w,l,M)})}function Ba(e,t,_){t!=null&&d(mt,e)&&mt[e](t,_._a,_,e)}function Ce(e){return e%4===0&&e%100!==0||e%400===0}var N=0,ee=1,K=2,z=3,B=4,te=5,le=6,$a=7,qa=8;T("Y",0,0,function(){var e=this.year();return e<=9999?Z(e,4):"+"+e}),T(0,["YY",2],0,function(){return this.year()%100}),T(0,["YYYY",4],0,"year"),T(0,["YYYYY",5],0,"year"),T(0,["YYYYYY",6,!0],0,"year"),w("Y",Re),w("YY",W,J),w("YYYY",ut,ot),w("YYYYY",Ne,Fe),w("YYYYYY",Ne,Fe),O(["YYYYY","YYYYYY"],N),O("YYYY",function(e,t){t[N]=e.length===2?a.parseTwoDigitYear(e):S(e)}),O("YY",function(e,t){t[N]=a.parseTwoDigitYear(e)}),O("Y",function(e,t){t[N]=parseInt(e,10)});function He(e){return Ce(e)?366:365}a.parseTwoDigitYear=function(e){return S(e)+(S(e)>68?1900:2e3)};var Jt=ye("FullYear",!0);function Za(){return Ce(this.year())}function ye(e,t){return function(_){return _!=null?(It(this,e,_),a.updateOffset(this,t),this):Se(this,e)}}function Se(e,t){if(!e.isValid())return NaN;var _=e._d,i=e._isUTC;switch(t){case"Milliseconds":return i?_.getUTCMilliseconds():_.getMilliseconds();case"Seconds":return i?_.getUTCSeconds():_.getSeconds();case"Minutes":return i?_.getUTCMinutes():_.getMinutes();case"Hours":return i?_.getUTCHours():_.getHours();case"Date":return i?_.getUTCDate():_.getDate();case"Day":return i?_.getUTCDay():_.getDay();case"Month":return i?_.getUTCMonth():_.getMonth();case"FullYear":return i?_.getUTCFullYear():_.getFullYear();default:return NaN}}function It(e,t,_){var i,l,M,Y,D;if(!(!e.isValid()||isNaN(_))){switch(i=e._d,l=e._isUTC,t){case"Milliseconds":return void(l?i.setUTCMilliseconds(_):i.setMilliseconds(_));case"Seconds":return void(l?i.setUTCSeconds(_):i.setSeconds(_));case"Minutes":return void(l?i.setUTCMinutes(_):i.setMinutes(_));case"Hours":return void(l?i.setUTCHours(_):i.setHours(_));case"Date":return void(l?i.setUTCDate(_):i.setDate(_));case"FullYear":break;default:return}M=_,Y=e.month(),D=e.date(),D=D===29&&Y===1&&!Ce(M)?28:D,l?i.setUTCFullYear(M,Y,D):i.setFullYear(M,Y,D)}}function Ka(e){return e=U(e),q(this[e])?this[e]():this}function Qa(e,t){if(typeof e=="object"){e=dt(e);var _=Ia(e),i,l=_.length;for(i=0;i<l;i++)this[_[i].unit](e[_[i].unit])}else if(e=U(e),q(this[e]))return this[e](t);return this}function Xa(e,t){return(e%t+t)%t}var F;Array.prototype.indexOf?F=Array.prototype.indexOf:F=function(e){var t;for(t=0;t<this.length;++t)if(this[t]===e)return t;return-1};function ct(e,t){if(isNaN(e)||isNaN(t))return NaN;var _=Xa(t,12);return e+=(t-_)/12,_===1?Ce(e)?29:28:31-_%7%2}T("M",["MM",2],"Mo",function(){return this.month()+1}),T("MMM",0,0,function(e){return this.localeData().monthsShort(this,e)}),T("MMMM",0,0,function(e){return this.localeData().months(this,e)}),w("M",W,Ye),w("MM",W,J),w("MMM",function(e,t){return t.monthsShortRegex(e)}),w("MMMM",function(e,t){return t.monthsRegex(e)}),O(["M","MM"],function(e,t){t[ee]=S(e)-1}),O(["MMM","MMMM"],function(e,t,_,i){var l=_._locale.monthsParse(e,i,_._strict);l!=null?t[ee]=l:k(_).invalidMonth=e});var es="January_February_March_April_May_June_July_August_September_October_November_December".split("_"),Ct="Jan_Feb_Mar_Apr_May_Jun_Jul_Aug_Sep_Oct_Nov_Dec".split("_"),Ut=/D[oD]?(\[[^\[\]]*\]|\s)+MMMM?/,ts=ve,as=ve;function ss(e,t){return e?s(this._months)?this._months[e.month()]:this._months[(this._months.isFormat||Ut).test(t)?"format":"standalone"][e.month()]:s(this._months)?this._months:this._months.standalone}function ns(e,t){return e?s(this._monthsShort)?this._monthsShort[e.month()]:this._monthsShort[Ut.test(t)?"format":"standalone"][e.month()]:s(this._monthsShort)?this._monthsShort:this._monthsShort.standalone}function rs(e,t,_){var i,l,M,Y=e.toLocaleLowerCase();if(!this._monthsParse)for(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[],i=0;i<12;++i)M=H([2e3,i]),this._shortMonthsParse[i]=this.monthsShort(M,"").toLocaleLowerCase(),this._longMonthsParse[i]=this.months(M,"").toLocaleLowerCase();return _?t==="MMM"?(l=F.call(this._shortMonthsParse,Y),l!==-1?l:null):(l=F.call(this._longMonthsParse,Y),l!==-1?l:null):t==="MMM"?(l=F.call(this._shortMonthsParse,Y),l!==-1?l:(l=F.call(this._longMonthsParse,Y),l!==-1?l:null)):(l=F.call(this._longMonthsParse,Y),l!==-1?l:(l=F.call(this._shortMonthsParse,Y),l!==-1?l:null))}function _s(e,t,_){var i,l,M;if(this._monthsParseExact)return rs.call(this,e,t,_);for(this._monthsParse||(this._monthsParse=[],this._longMonthsParse=[],this._shortMonthsParse=[]),i=0;i<12;i++){if(l=H([2e3,i]),_&&!this._longMonthsParse[i]&&(this._longMonthsParse[i]=new RegExp("^"+this.months(l,"").replace(".","")+"$","i"),this._shortMonthsParse[i]=new RegExp("^"+this.monthsShort(l,"").replace(".","")+"$","i")),!_&&!this._monthsParse[i]&&(M="^"+this.months(l,"")+"|^"+this.monthsShort(l,""),this._monthsParse[i]=new RegExp(M.replace(".",""),"i")),_&&t==="MMMM"&&this._longMonthsParse[i].test(e))return i;if(_&&t==="MMM"&&this._shortMonthsParse[i].test(e))return i;if(!_&&this._monthsParse[i].test(e))return i}}function Gt(e,t){if(!e.isValid())return e;if(typeof t=="string"){if(/^\d+$/.test(t))t=S(t);else if(t=e.localeData().monthsParse(t),!h(t))return e}var _=t,i=e.date();return i=i<29?i:Math.min(i,ct(e.year(),_)),e._isUTC?e._d.setUTCMonth(_,i):e._d.setMonth(_,i),e}function Vt(e){return e!=null?(Gt(this,e),a.updateOffset(this,!0),this):Se(this,"Month")}function is(){return ct(this.year(),this.month())}function ds(e){return this._monthsParseExact?(d(this,"_monthsRegex")||Bt.call(this),e?this._monthsShortStrictRegex:this._monthsShortRegex):(d(this,"_monthsShortRegex")||(this._monthsShortRegex=ts),this._monthsShortStrictRegex&&e?this._monthsShortStrictRegex:this._monthsShortRegex)}function os(e){return this._monthsParseExact?(d(this,"_monthsRegex")||Bt.call(this),e?this._monthsStrictRegex:this._monthsRegex):(d(this,"_monthsRegex")||(this._monthsRegex=as),this._monthsStrictRegex&&e?this._monthsStrictRegex:this._monthsRegex)}function Bt(){function e(b,j){return j.length-b.length}var t=[],_=[],i=[],l,M,Y,D;for(l=0;l<12;l++)M=H([2e3,l]),Y=X(this.monthsShort(M,"")),D=X(this.months(M,"")),t.push(Y),_.push(D),i.push(D),i.push(Y);t.sort(e),_.sort(e),i.sort(e),this._monthsRegex=new RegExp("^("+i.join("|")+")","i"),this._monthsShortRegex=this._monthsRegex,this._monthsStrictRegex=new RegExp("^("+_.join("|")+")","i"),this._monthsShortStrictRegex=new RegExp("^("+t.join("|")+")","i")}function us(e,t,_,i,l,M,Y){var D;return e<100&&e>=0?(D=new Date(e+400,t,_,i,l,M,Y),isFinite(D.getFullYear())&&D.setFullYear(e)):D=new Date(e,t,_,i,l,M,Y),D}function je(e){var t,_;return e<100&&e>=0?(_=Array.prototype.slice.call(arguments),_[0]=e+400,t=new Date(Date.UTC.apply(null,_)),isFinite(t.getUTCFullYear())&&t.setUTCFullYear(e)):t=new Date(Date.UTC.apply(null,arguments)),t}function Ue(e,t,_){var i=7+t-_,l=(7+je(e,0,i).getUTCDay()-t)%7;return-l+i-1}function $t(e,t,_,i,l){var M=(7+_-i)%7,Y=Ue(e,i,l),D=1+7*(t-1)+M+Y,b,j;return D<=0?(b=e-1,j=He(b)+D):D>He(e)?(b=e+1,j=D-He(e)):(b=e,j=D),{year:b,dayOfYear:j}}function xe(e,t,_){var i=Ue(e.year(),t,_),l=Math.floor((e.dayOfYear()-i-1)/7)+1,M,Y;return l<1?(Y=e.year()-1,M=l+ae(Y,t,_)):l>ae(e.year(),t,_)?(M=l-ae(e.year(),t,_),Y=e.year()+1):(Y=e.year(),M=l),{week:M,year:Y}}function ae(e,t,_){var i=Ue(e,t,_),l=Ue(e+1,t,_);return(He(e)-i+l)/7}T("w",["ww",2],"wo","week"),T("W",["WW",2],"Wo","isoWeek"),w("w",W,Ye),w("ww",W,J),w("W",W,Ye),w("WW",W,J),be(["w","ww","W","WW"],function(e,t,_,i){t[i.substr(0,1)]=S(e)});function ls(e){return xe(e,this._week.dow,this._week.doy).week}var ms={dow:0,doy:6};function cs(){return this._week.dow}function Ms(){return this._week.doy}function hs(e){var t=this.localeData().week(this);return e==null?t:this.add((e-t)*7,"d")}function Ls(e){var t=xe(this,1,4).week;return e==null?t:this.add((e-t)*7,"d")}T("d",0,"do","day"),T("dd",0,0,function(e){return this.localeData().weekdaysMin(this,e)}),T("ddd",0,0,function(e){return this.localeData().weekdaysShort(this,e)}),T("dddd",0,0,function(e){return this.localeData().weekdays(this,e)}),T("e",0,0,"weekday"),T("E",0,0,"isoWeekday"),w("d",W),w("e",W),w("E",W),w("dd",function(e,t){return t.weekdaysMinRegex(e)}),w("ddd",function(e,t){return t.weekdaysShortRegex(e)}),w("dddd",function(e,t){return t.weekdaysRegex(e)}),be(["dd","ddd","dddd"],function(e,t,_,i){var l=_._locale.weekdaysParse(e,i,_._strict);l!=null?t.d=l:k(_).invalidWeekday=e}),be(["d","e","E"],function(e,t,_,i){t[i]=S(e)});function fs(e,t){return typeof e!="string"?e:isNaN(e)?(e=t.weekdaysParse(e),typeof e=="number"?e:null):parseInt(e,10)}function Ys(e,t){return typeof e=="string"?t.weekdaysParse(e)%7||7:isNaN(e)?null:e}function Mt(e,t){return e.slice(t,7).concat(e.slice(0,t))}var ys="Sunday_Monday_Tuesday_Wednesday_Thursday_Friday_Saturday".split("_"),qt="Sun_Mon_Tue_Wed_Thu_Fri_Sat".split("_"),ps="Su_Mo_Tu_We_Th_Fr_Sa".split("_"),ks=ve,Ds=ve,ws=ve;function gs(e,t){var _=s(this._weekdays)?this._weekdays:this._weekdays[e&&e!==!0&&this._weekdays.isFormat.test(t)?"format":"standalone"];return e===!0?Mt(_,this._week.dow):e?_[e.day()]:_}function Ts(e){return e===!0?Mt(this._weekdaysShort,this._week.dow):e?this._weekdaysShort[e.day()]:this._weekdaysShort}function vs(e){return e===!0?Mt(this._weekdaysMin,this._week.dow):e?this._weekdaysMin[e.day()]:this._weekdaysMin}function bs(e,t,_){var i,l,M,Y=e.toLocaleLowerCase();if(!this._weekdaysParse)for(this._weekdaysParse=[],this._shortWeekdaysParse=[],this._minWeekdaysParse=[],i=0;i<7;++i)M=H([2e3,1]).day(i),this._minWeekdaysParse[i]=this.weekdaysMin(M,"").toLocaleLowerCase(),this._shortWeekdaysParse[i]=this.weekdaysShort(M,"").toLocaleLowerCase(),this._weekdaysParse[i]=this.weekdays(M,"").toLocaleLowerCase();return _?t==="dddd"?(l=F.call(this._weekdaysParse,Y),l!==-1?l:null):t==="ddd"?(l=F.call(this._shortWeekdaysParse,Y),l!==-1?l:null):(l=F.call(this._minWeekdaysParse,Y),l!==-1?l:null):t==="dddd"?(l=F.call(this._weekdaysParse,Y),l!==-1||(l=F.call(this._shortWeekdaysParse,Y),l!==-1)?l:(l=F.call(this._minWeekdaysParse,Y),l!==-1?l:null)):t==="ddd"?(l=F.call(this._shortWeekdaysParse,Y),l!==-1||(l=F.call(this._weekdaysParse,Y),l!==-1)?l:(l=F.call(this._minWeekdaysParse,Y),l!==-1?l:null)):(l=F.call(this._minWeekdaysParse,Y),l!==-1||(l=F.call(this._weekdaysParse,Y),l!==-1)?l:(l=F.call(this._shortWeekdaysParse,Y),l!==-1?l:null))}function Hs(e,t,_){var i,l,M;if(this._weekdaysParseExact)return bs.call(this,e,t,_);for(this._weekdaysParse||(this._weekdaysParse=[],this._minWeekdaysParse=[],this._shortWeekdaysParse=[],this._fullWeekdaysParse=[]),i=0;i<7;i++){if(l=H([2e3,1]).day(i),_&&!this._fullWeekdaysParse[i]&&(this._fullWeekdaysParse[i]=new RegExp("^"+this.weekdays(l,"").replace(".","\\.?")+"$","i"),this._shortWeekdaysParse[i]=new RegExp("^"+this.weekdaysShort(l,"").replace(".","\\.?")+"$","i"),this._minWeekdaysParse[i]=new RegExp("^"+this.weekdaysMin(l,"").replace(".","\\.?")+"$","i")),this._weekdaysParse[i]||(M="^"+this.weekdays(l,"")+"|^"+this.weekdaysShort(l,"")+"|^"+this.weekdaysMin(l,""),this._weekdaysParse[i]=new RegExp(M.replace(".",""),"i")),_&&t==="dddd"&&this._fullWeekdaysParse[i].test(e))return i;if(_&&t==="ddd"&&this._shortWeekdaysParse[i].test(e))return i;if(_&&t==="dd"&&this._minWeekdaysParse[i].test(e))return i;if(!_&&this._weekdaysParse[i].test(e))return i}}function Ss(e){if(!this.isValid())return e!=null?this:NaN;var t=Se(this,"Day");return e!=null?(e=fs(e,this.localeData()),this.add(e-t,"d")):t}function js(e){if(!this.isValid())return e!=null?this:NaN;var t=(this.day()+7-this.localeData()._week.dow)%7;return e==null?t:this.add(e-t,"d")}function xs(e){if(!this.isValid())return e!=null?this:NaN;if(e!=null){var t=Ys(e,this.localeData());return this.day(this.day()%7?t:t-7)}else return this.day()||7}function Ps(e){return this._weekdaysParseExact?(d(this,"_weekdaysRegex")||ht.call(this),e?this._weekdaysStrictRegex:this._weekdaysRegex):(d(this,"_weekdaysRegex")||(this._weekdaysRegex=ks),this._weekdaysStrictRegex&&e?this._weekdaysStrictRegex:this._weekdaysRegex)}function Os(e){return this._weekdaysParseExact?(d(this,"_weekdaysRegex")||ht.call(this),e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex):(d(this,"_weekdaysShortRegex")||(this._weekdaysShortRegex=Ds),this._weekdaysShortStrictRegex&&e?this._weekdaysShortStrictRegex:this._weekdaysShortRegex)}function Ws(e){return this._weekdaysParseExact?(d(this,"_weekdaysRegex")||ht.call(this),e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex):(d(this,"_weekdaysMinRegex")||(this._weekdaysMinRegex=ws),this._weekdaysMinStrictRegex&&e?this._weekdaysMinStrictRegex:this._weekdaysMinRegex)}function ht(){function e(R,ie){return ie.length-R.length}var t=[],_=[],i=[],l=[],M,Y,D,b,j;for(M=0;M<7;M++)Y=H([2e3,1]).day(M),D=X(this.weekdaysMin(Y,"")),b=X(this.weekdaysShort(Y,"")),j=X(this.weekdays(Y,"")),t.push(D),_.push(b),i.push(j),l.push(D),l.push(b),l.push(j);t.sort(e),_.sort(e),i.sort(e),l.sort(e),this._weekdaysRegex=new RegExp("^("+l.join("|")+")","i"),this._weekdaysShortRegex=this._weekdaysRegex,this._weekdaysMinRegex=this._weekdaysRegex,this._weekdaysStrictRegex=new RegExp("^("+i.join("|")+")","i"),this._weekdaysShortStrictRegex=new RegExp("^("+_.join("|")+")","i"),this._weekdaysMinStrictRegex=new RegExp("^("+t.join("|")+")","i")}function Lt(){return this.hours()%12||12}function As(){return this.hours()||24}T("H",["HH",2],0,"hour"),T("h",["hh",2],0,Lt),T("k",["kk",2],0,As),T("hmm",0,0,function(){return""+Lt.apply(this)+Z(this.minutes(),2)}),T("hmmss",0,0,function(){return""+Lt.apply(this)+Z(this.minutes(),2)+Z(this.seconds(),2)}),T("Hmm",0,0,function(){return""+this.hours()+Z(this.minutes(),2)}),T("Hmmss",0,0,function(){return""+this.hours()+Z(this.minutes(),2)+Z(this.seconds(),2)});function Zt(e,t){T(e,0,0,function(){return this.localeData().meridiem(this.hours(),this.minutes(),t)})}Zt("a",!0),Zt("A",!1);function Kt(e,t){return t._meridiemParse}w("a",Kt),w("A",Kt),w("H",W,lt),w("h",W,Ye),w("k",W,Ye),w("HH",W,J),w("hh",W,J),w("kk",W,J),w("hmm",Nt),w("hmmss",Rt),w("Hmm",Nt),w("Hmmss",Rt),O(["H","HH"],z),O(["k","kk"],function(e,t,_){var i=S(e);t[z]=i===24?0:i}),O(["a","A"],function(e,t,_){_._isPm=_._locale.isPM(e),_._meridiem=e}),O(["h","hh"],function(e,t,_){t[z]=S(e),k(_).bigHour=!0}),O("hmm",function(e,t,_){var i=e.length-2;t[z]=S(e.substr(0,i)),t[B]=S(e.substr(i)),k(_).bigHour=!0}),O("hmmss",function(e,t,_){var i=e.length-4,l=e.length-2;t[z]=S(e.substr(0,i)),t[B]=S(e.substr(i,2)),t[te]=S(e.substr(l)),k(_).bigHour=!0}),O("Hmm",function(e,t,_){var i=e.length-2;t[z]=S(e.substr(0,i)),t[B]=S(e.substr(i))}),O("Hmmss",function(e,t,_){var i=e.length-4,l=e.length-2;t[z]=S(e.substr(0,i)),t[B]=S(e.substr(i,2)),t[te]=S(e.substr(l))});function Es(e){return(e+"").toLowerCase().charAt(0)==="p"}var Fs=/[ap]\.?m?\.?/i,zs=ye("Hours",!0);function Ns(e,t,_){return e>11?_?"pm":"PM":_?"am":"AM"}var Qt={calendar:ba,longDateFormat:xa,invalidDate:Oa,ordinal:Aa,dayOfMonthOrdinalParse:Ea,relativeTime:za,months:es,monthsShort:Ct,week:ms,weekdays:ys,weekdaysMin:ps,weekdaysShort:qt,meridiemParse:Fs},E={},Pe={},Oe;function Rs(e,t){var _,i=Math.min(e.length,t.length);for(_=0;_<i;_+=1)if(e[_]!==t[_])return _;return i}function Xt(e){return e&&e.toLowerCase().replace("_","-")}function Js(e){for(var t=0,_,i,l,M;t<e.length;){for(M=Xt(e[t]).split("-"),_=M.length,i=Xt(e[t+1]),i=i?i.split("-"):null;_>0;){if(l=Ge(M.slice(0,_).join("-")),l)return l;if(i&&i.length>=_&&Rs(M,i)>=_-1)break;_--}t++}return Oe}function Is(e){return!!(e&&e.match("^[^/\\\\]*$"))}function Ge(e){var t=null,_;if(E[e]===void 0&&L&&L.exports&&Is(e))try{t=Oe._abbr,_=void 0,c(35358)("./"+e),de(t)}catch{E[e]=null}return E[e]}function de(e,t){var _;return e&&(m(t)?_=se(e):_=ft(e,t),_?Oe=_:typeof console<"u"&&console.warn&&console.warn("Locale "+e+" not found. Did you forget to load it?")),Oe._abbr}function ft(e,t){if(t!==null){var _,i=Qt;if(t.abbr=e,E[e]!=null)Wt("defineLocaleOverride","use moment.updateLocale(localeName, config) to change an existing locale. moment.defineLocale(localeName, config) should only be used for creating a new locale See http://momentjs.com/guides/#/warnings/define-locale/ for more info."),i=E[e]._config;else if(t.parentLocale!=null)if(E[t.parentLocale]!=null)i=E[t.parentLocale]._config;else if(_=Ge(t.parentLocale),_!=null)i=_._config;else return Pe[t.parentLocale]||(Pe[t.parentLocale]=[]),Pe[t.parentLocale].push({name:e,config:t}),null;return E[e]=new nt(st(i,t)),Pe[e]&&Pe[e].forEach(function(l){ft(l.name,l.config)}),de(e),E[e]}else return delete E[e],null}function Cs(e,t){if(t!=null){var _,i,l=Qt;E[e]!=null&&E[e].parentLocale!=null?E[e].set(st(E[e]._config,t)):(i=Ge(e),i!=null&&(l=i._config),t=st(l,t),i==null&&(t.abbr=e),_=new nt(t),_.parentLocale=E[e],E[e]=_),de(e)}else E[e]!=null&&(E[e].parentLocale!=null?(E[e]=E[e].parentLocale,e===de()&&de(e)):E[e]!=null&&delete E[e]);return E[e]}function se(e){var t;if(e&&e._locale&&e._locale._abbr&&(e=e._locale._abbr),!e)return Oe;if(!s(e)){if(t=Ge(e),t)return t;e=[e]}return Js(e)}function Us(){return rt(E)}function Yt(e){var t,_=e._a;return _&&k(e).overflow===-2&&(t=_[ee]<0||_[ee]>11?ee:_[K]<1||_[K]>ct(_[N],_[ee])?K:_[z]<0||_[z]>24||_[z]===24&&(_[B]!==0||_[te]!==0||_[le]!==0)?z:_[B]<0||_[B]>59?B:_[te]<0||_[te]>59?te:_[le]<0||_[le]>999?le:-1,k(e)._overflowDayOfYear&&(t<N||t>K)&&(t=K),k(e)._overflowWeeks&&t===-1&&(t=$a),k(e)._overflowWeekday&&t===-1&&(t=qa),k(e).overflow=t),e}var Gs=/^\s*((?:[+-]\d{6}|\d{4})-(?:\d\d-\d\d|W\d\d-\d|W\d\d|\d\d\d|\d\d))(?:(T| )(\d\d(?::\d\d(?::\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,Vs=/^\s*((?:[+-]\d{6}|\d{4})(?:\d\d\d\d|W\d\d\d|W\d\d|\d\d\d|\d\d|))(?:(T| )(\d\d(?:\d\d(?:\d\d(?:[.,]\d+)?)?)?)([+-]\d\d(?::?\d\d)?|\s*Z)?)?$/,Bs=/Z|[+-]\d\d(?::?\d\d)?/,Ve=[["YYYYYY-MM-DD",/[+-]\d{6}-\d\d-\d\d/],["YYYY-MM-DD",/\d{4}-\d\d-\d\d/],["GGGG-[W]WW-E",/\d{4}-W\d\d-\d/],["GGGG-[W]WW",/\d{4}-W\d\d/,!1],["YYYY-DDD",/\d{4}-\d{3}/],["YYYY-MM",/\d{4}-\d\d/,!1],["YYYYYYMMDD",/[+-]\d{10}/],["YYYYMMDD",/\d{8}/],["GGGG[W]WWE",/\d{4}W\d{3}/],["GGGG[W]WW",/\d{4}W\d{2}/,!1],["YYYYDDD",/\d{7}/],["YYYYMM",/\d{6}/,!1],["YYYY",/\d{4}/,!1]],yt=[["HH:mm:ss.SSSS",/\d\d:\d\d:\d\d\.\d+/],["HH:mm:ss,SSSS",/\d\d:\d\d:\d\d,\d+/],["HH:mm:ss",/\d\d:\d\d:\d\d/],["HH:mm",/\d\d:\d\d/],["HHmmss.SSSS",/\d\d\d\d\d\d\.\d+/],["HHmmss,SSSS",/\d\d\d\d\d\d,\d+/],["HHmmss",/\d\d\d\d\d\d/],["HHmm",/\d\d\d\d/],["HH",/\d\d/]],$s=/^\/?Date\((-?\d+)/i,qs=/^(?:(Mon|Tue|Wed|Thu|Fri|Sat|Sun),?\s)?(\d{1,2})\s(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\s(\d{2,4})\s(\d\d):(\d\d)(?::(\d\d))?\s(?:(UT|GMT|[ECMP][SD]T)|([Zz])|([+-]\d{4}))$/,Zs={UT:0,GMT:0,EDT:-240,EST:-300,CDT:-300,CST:-360,MDT:-360,MST:-420,PDT:-420,PST:-480};function ea(e){var t,_,i=e._i,l=Gs.exec(i)||Vs.exec(i),M,Y,D,b,j=Ve.length,R=yt.length;if(l){for(k(e).iso=!0,t=0,_=j;t<_;t++)if(Ve[t][1].exec(l[1])){Y=Ve[t][0],M=Ve[t][2]!==!1;break}if(Y==null){e._isValid=!1;return}if(l[3]){for(t=0,_=R;t<_;t++)if(yt[t][1].exec(l[3])){D=(l[2]||" ")+yt[t][0];break}if(D==null){e._isValid=!1;return}}if(!M&&D!=null){e._isValid=!1;return}if(l[4])if(Bs.exec(l[4]))b="Z";else{e._isValid=!1;return}e._f=Y+(D||"")+(b||""),kt(e)}else e._isValid=!1}function Ks(e,t,_,i,l,M){var Y=[Qs(e),Ct.indexOf(t),parseInt(_,10),parseInt(i,10),parseInt(l,10)];return M&&Y.push(parseInt(M,10)),Y}function Qs(e){var t=parseInt(e,10);return t<=49?2e3+t:t<=999?1900+t:t}function Xs(e){return e.replace(/\([^()]*\)|[\n\t]/g," ").replace(/(\s\s+)/g," ").replace(/^\s\s*/,"").replace(/\s\s*$/,"")}function en(e,t,_){if(e){var i=qt.indexOf(e),l=new Date(t[0],t[1],t[2]).getDay();if(i!==l)return k(_).weekdayMismatch=!0,_._isValid=!1,!1}return!0}function tn(e,t,_){if(e)return Zs[e];if(t)return 0;var i=parseInt(_,10),l=i%100,M=(i-l)/100;return M*60+l}function ta(e){var t=qs.exec(Xs(e._i)),_;if(t){if(_=Ks(t[4],t[3],t[2],t[5],t[6],t[7]),!en(t[1],_,e))return;e._a=_,e._tzm=tn(t[8],t[9],t[10]),e._d=je.apply(null,e._a),e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),k(e).rfc2822=!0}else e._isValid=!1}function an(e){var t=$s.exec(e._i);if(t!==null){e._d=new Date(+t[1]);return}if(ea(e),e._isValid===!1)delete e._isValid;else return;if(ta(e),e._isValid===!1)delete e._isValid;else return;e._strict?e._isValid=!1:a.createFromInputFallback(e)}a.createFromInputFallback=C("value provided is not in a recognized RFC2822 or ISO format. moment construction falls back to js Date(), which is not reliable across all browsers and versions. Non RFC2822/ISO date formats are discouraged. Please refer to http://momentjs.com/guides/#/warnings/js-date/ for more info.",function(e){e._d=new Date(e._i+(e._useUTC?" UTC":""))});function pe(e,t,_){return e??t??_}function sn(e){var t=new Date(a.now());return e._useUTC?[t.getUTCFullYear(),t.getUTCMonth(),t.getUTCDate()]:[t.getFullYear(),t.getMonth(),t.getDate()]}function pt(e){var t,_,i=[],l,M,Y;if(!e._d){for(l=sn(e),e._w&&e._a[K]==null&&e._a[ee]==null&&nn(e),e._dayOfYear!=null&&(Y=pe(e._a[N],l[N]),(e._dayOfYear>He(Y)||e._dayOfYear===0)&&(k(e)._overflowDayOfYear=!0),_=je(Y,0,e._dayOfYear),e._a[ee]=_.getUTCMonth(),e._a[K]=_.getUTCDate()),t=0;t<3&&e._a[t]==null;++t)e._a[t]=i[t]=l[t];for(;t<7;t++)e._a[t]=i[t]=e._a[t]==null?t===2?1:0:e._a[t];e._a[z]===24&&e._a[B]===0&&e._a[te]===0&&e._a[le]===0&&(e._nextDay=!0,e._a[z]=0),e._d=(e._useUTC?je:us).apply(null,i),M=e._useUTC?e._d.getUTCDay():e._d.getDay(),e._tzm!=null&&e._d.setUTCMinutes(e._d.getUTCMinutes()-e._tzm),e._nextDay&&(e._a[z]=24),e._w&&typeof e._w.d<"u"&&e._w.d!==M&&(k(e).weekdayMismatch=!0)}}function nn(e){var t,_,i,l,M,Y,D,b,j;t=e._w,t.GG!=null||t.W!=null||t.E!=null?(M=1,Y=4,_=pe(t.GG,e._a[N],xe(A(),1,4).year),i=pe(t.W,1),l=pe(t.E,1),(l<1||l>7)&&(b=!0)):(M=e._locale._week.dow,Y=e._locale._week.doy,j=xe(A(),M,Y),_=pe(t.gg,e._a[N],j.year),i=pe(t.w,j.week),t.d!=null?(l=t.d,(l<0||l>6)&&(b=!0)):t.e!=null?(l=t.e+M,(t.e<0||t.e>6)&&(b=!0)):l=M),i<1||i>ae(_,M,Y)?k(e)._overflowWeeks=!0:b!=null?k(e)._overflowWeekday=!0:(D=$t(_,i,l,M,Y),e._a[N]=D.year,e._dayOfYear=D.dayOfYear)}a.ISO_8601=function(){},a.RFC_2822=function(){};function kt(e){if(e._f===a.ISO_8601){ea(e);return}if(e._f===a.RFC_2822){ta(e);return}e._a=[],k(e).empty=!0;var t=""+e._i,_,i,l,M,Y,D=t.length,b=0,j,R;for(l=At(e._f,e._locale).match(_t)||[],R=l.length,_=0;_<R;_++)M=l[_],i=(t.match(Ga(M,e))||[])[0],i&&(Y=t.substr(0,t.indexOf(i)),Y.length>0&&k(e).unusedInput.push(Y),t=t.slice(t.indexOf(i)+i.length),b+=i.length),Le[M]?(i?k(e).empty=!1:k(e).unusedTokens.push(M),Ba(M,i,e)):e._strict&&!i&&k(e).unusedTokens.push(M);k(e).charsLeftOver=D-b,t.length>0&&k(e).unusedInput.push(t),e._a[z]<=12&&k(e).bigHour===!0&&e._a[z]>0&&(k(e).bigHour=void 0),k(e).parsedDateParts=e._a.slice(0),k(e).meridiem=e._meridiem,e._a[z]=rn(e._locale,e._a[z],e._meridiem),j=k(e).era,j!==null&&(e._a[N]=e._locale.erasConvertYear(j,e._a[N])),pt(e),Yt(e)}function rn(e,t,_){var i;return _==null?t:e.meridiemHour!=null?e.meridiemHour(t,_):(e.isPM!=null&&(i=e.isPM(_),i&&t<12&&(t+=12),!i&&t===12&&(t=0)),t)}function _n(e){var t,_,i,l,M,Y,D=!1,b=e._f.length;if(b===0){k(e).invalidFormat=!0,e._d=new Date(NaN);return}for(l=0;l<b;l++)M=0,Y=!1,t=at({},e),e._useUTC!=null&&(t._useUTC=e._useUTC),t._f=e._f[l],kt(t),Me(t)&&(Y=!0),M+=k(t).charsLeftOver,M+=k(t).unusedTokens.length*10,k(t).score=M,D?M<i&&(i=M,_=t):(i==null||M<i||Y)&&(i=M,_=t,Y&&(D=!0));v(e,_||t)}function dn(e){if(!e._d){var t=dt(e._i),_=t.day===void 0?t.date:t.day;e._a=g([t.year,t.month,_,t.hour,t.minute,t.second,t.millisecond],function(i){return i&&parseInt(i,10)}),pt(e)}}function on(e){var t=new Te(Yt(aa(e)));return t._nextDay&&(t.add(1,"d"),t._nextDay=void 0),t}function aa(e){var t=e._i,_=e._f;return e._locale=e._locale||se(e._l),t===null||_===void 0&&t===""?he({nullInput:!0}):(typeof t=="string"&&(e._i=t=e._locale.preparse(t)),V(t)?new Te(Yt(t)):(y(t)?e._d=t:s(_)?_n(e):_?kt(e):un(e),Me(e)||(e._d=null),e))}function un(e){var t=e._i;m(t)?e._d=new Date(a.now()):y(t)?e._d=new Date(t.valueOf()):typeof t=="string"?an(e):s(t)?(e._a=g(t.slice(0),function(_){return parseInt(_,10)}),pt(e)):n(t)?dn(e):h(t)?e._d=new Date(t):a.createFromInputFallback(e)}function sa(e,t,_,i,l){var M={};return(t===!0||t===!1)&&(i=t,t=void 0),(_===!0||_===!1)&&(i=_,_=void 0),(n(e)&&u(e)||s(e)&&e.length===0)&&(e=void 0),M._isAMomentObject=!0,M._useUTC=M._isUTC=l,M._l=_,M._i=e,M._f=t,M._strict=i,on(M)}function A(e,t,_,i){return sa(e,t,_,i,!1)}var ln=C("moment().min is deprecated, use moment.max instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var e=A.apply(null,arguments);return this.isValid()&&e.isValid()?e<this?this:e:he()}),mn=C("moment().max is deprecated, use moment.min instead. http://momentjs.com/guides/#/warnings/min-max/",function(){var e=A.apply(null,arguments);return this.isValid()&&e.isValid()?e>this?this:e:he()});function na(e,t){var _,i;if(t.length===1&&s(t[0])&&(t=t[0]),!t.length)return A();for(_=t[0],i=1;i<t.length;++i)(!t[i].isValid()||t[i][e](_))&&(_=t[i]);return _}function cn(){var e=[].slice.call(arguments,0);return na("isBefore",e)}function Mn(){var e=[].slice.call(arguments,0);return na("isAfter",e)}var hn=function(){return Date.now?Date.now():+new Date},We=["year","quarter","month","week","day","hour","minute","second","millisecond"];function Ln(e){var t,_=!1,i,l=We.length;for(t in e)if(d(e,t)&&!(F.call(We,t)!==-1&&(e[t]==null||!isNaN(e[t]))))return!1;for(i=0;i<l;++i)if(e[We[i]]){if(_)return!1;parseFloat(e[We[i]])!==S(e[We[i]])&&(_=!0)}return!0}function fn(){return this._isValid}function Yn(){return $(NaN)}function Be(e){var t=dt(e),_=t.year||0,i=t.quarter||0,l=t.month||0,M=t.week||t.isoWeek||0,Y=t.day||0,D=t.hour||0,b=t.minute||0,j=t.second||0,R=t.millisecond||0;this._isValid=Ln(t),this._milliseconds=+R+j*1e3+b*6e4+D*1e3*60*60,this._days=+Y+M*7,this._months=+l+i*3+_*12,this._data={},this._locale=se(),this._bubble()}function $e(e){return e instanceof Be}function Dt(e){return e<0?Math.round(-1*e)*-1:Math.round(e)}function yn(e,t,_){var i=Math.min(e.length,t.length),l=Math.abs(e.length-t.length),M=0,Y;for(Y=0;Y<i;Y++)(_&&e[Y]!==t[Y]||!_&&S(e[Y])!==S(t[Y]))&&M++;return M+l}function ra(e,t){T(e,0,0,function(){var _=this.utcOffset(),i="+";return _<0&&(_=-_,i="-"),i+Z(~~(_/60),2)+t+Z(~~_%60,2)})}ra("Z",":"),ra("ZZ",""),w("Z",Je),w("ZZ",Je),O(["Z","ZZ"],function(e,t,_){_._useUTC=!0,_._tzm=wt(Je,e)});var pn=/([\+\-]|\d\d)/gi;function wt(e,t){var _=(t||"").match(e),i,l,M;return _===null?null:(i=_[_.length-1]||[],l=(i+"").match(pn)||["-",0,0],M=+(l[1]*60)+S(l[2]),M===0?0:l[0]==="+"?M:-M)}function gt(e,t){var _,i;return t._isUTC?(_=t.clone(),i=(V(e)||y(e)?e.valueOf():A(e).valueOf())-_.valueOf(),_._d.setTime(_._d.valueOf()+i),a.updateOffset(_,!1),_):A(e).local()}function Tt(e){return-Math.round(e._d.getTimezoneOffset())}a.updateOffset=function(){};function kn(e,t,_){var i=this._offset||0,l;if(!this.isValid())return e!=null?this:NaN;if(e!=null){if(typeof e=="string"){if(e=wt(Je,e),e===null)return this}else Math.abs(e)<16&&!_&&(e=e*60);return!this._isUTC&&t&&(l=Tt(this)),this._offset=e,this._isUTC=!0,l!=null&&this.add(l,"m"),i!==e&&(!t||this._changeInProgress?oa(this,$(e-i,"m"),1,!1):this._changeInProgress||(this._changeInProgress=!0,a.updateOffset(this,!0),this._changeInProgress=null)),this}else return this._isUTC?i:Tt(this)}function Dn(e,t){return e!=null?(typeof e!="string"&&(e=-e),this.utcOffset(e,t),this):-this.utcOffset()}function wn(e){return this.utcOffset(0,e)}function gn(e){return this._isUTC&&(this.utcOffset(0,e),this._isUTC=!1,e&&this.subtract(Tt(this),"m")),this}function Tn(){if(this._tzm!=null)this.utcOffset(this._tzm,!1,!0);else if(typeof this._i=="string"){var e=wt(Ca,this._i);e!=null?this.utcOffset(e):this.utcOffset(0,!0)}return this}function vn(e){return this.isValid()?(e=e?A(e).utcOffset():0,(this.utcOffset()-e)%60===0):!1}function bn(){return this.utcOffset()>this.clone().month(0).utcOffset()||this.utcOffset()>this.clone().month(5).utcOffset()}function Hn(){if(!m(this._isDSTShifted))return this._isDSTShifted;var e={},t;return at(e,this),e=aa(e),e._a?(t=e._isUTC?H(e._a):A(e._a),this._isDSTShifted=this.isValid()&&yn(e._a,t.toArray())>0):this._isDSTShifted=!1,this._isDSTShifted}function Sn(){return this.isValid()?!this._isUTC:!1}function jn(){return this.isValid()?this._isUTC:!1}function _a(){return this.isValid()?this._isUTC&&this._offset===0:!1}var xn=/^(-|\+)?(?:(\d*)[. ])?(\d+):(\d+)(?::(\d+)(\.\d*)?)?$/,Pn=/^(-|\+)?P(?:([-+]?[0-9,.]*)Y)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)W)?(?:([-+]?[0-9,.]*)D)?(?:T(?:([-+]?[0-9,.]*)H)?(?:([-+]?[0-9,.]*)M)?(?:([-+]?[0-9,.]*)S)?)?$/;function $(e,t){var _=e,i=null,l,M,Y;return $e(e)?_={ms:e._milliseconds,d:e._days,M:e._months}:h(e)||!isNaN(+e)?(_={},t?_[t]=+e:_.milliseconds=+e):(i=xn.exec(e))?(l=i[1]==="-"?-1:1,_={y:0,d:S(i[K])*l,h:S(i[z])*l,m:S(i[B])*l,s:S(i[te])*l,ms:S(Dt(i[le]*1e3))*l}):(i=Pn.exec(e))?(l=i[1]==="-"?-1:1,_={y:me(i[2],l),M:me(i[3],l),w:me(i[4],l),d:me(i[5],l),h:me(i[6],l),m:me(i[7],l),s:me(i[8],l)}):_==null?_={}:typeof _=="object"&&("from"in _||"to"in _)&&(Y=On(A(_.from),A(_.to)),_={},_.ms=Y.milliseconds,_.M=Y.months),M=new Be(_),$e(e)&&d(e,"_locale")&&(M._locale=e._locale),$e(e)&&d(e,"_isValid")&&(M._isValid=e._isValid),M}$.fn=Be.prototype,$.invalid=Yn;function me(e,t){var _=e&&parseFloat(e.replace(",","."));return(isNaN(_)?0:_)*t}function ia(e,t){var _={};return _.months=t.month()-e.month()+(t.year()-e.year())*12,e.clone().add(_.months,"M").isAfter(t)&&--_.months,_.milliseconds=+t-+e.clone().add(_.months,"M"),_}function On(e,t){var _;return e.isValid()&&t.isValid()?(t=gt(t,e),e.isBefore(t)?_=ia(e,t):(_=ia(t,e),_.milliseconds=-_.milliseconds,_.months=-_.months),_):{milliseconds:0,months:0}}function da(e,t){return function(_,i){var l,M;return i!==null&&!isNaN(+i)&&(Wt(t,"moment()."+t+"(period, number) is deprecated. Please use moment()."+t+"(number, period). See http://momentjs.com/guides/#/warnings/add-inverted-param/ for more info."),M=_,_=i,i=M),l=$(_,i),oa(this,l,e),this}}function oa(e,t,_,i){var l=t._milliseconds,M=Dt(t._days),Y=Dt(t._months);e.isValid()&&(i=i??!0,Y&&Gt(e,Se(e,"Month")+Y*_),M&&It(e,"Date",Se(e,"Date")+M*_),l&&e._d.setTime(e._d.valueOf()+l*_),i&&a.updateOffset(e,M||Y))}var Wn=da(1,"add"),An=da(-1,"subtract");function ua(e){return typeof e=="string"||e instanceof String}function En(e){return V(e)||y(e)||ua(e)||h(e)||zn(e)||Fn(e)||e===null||e===void 0}function Fn(e){var t=n(e)&&!u(e),_=!1,i=["years","year","y","months","month","M","days","day","d","dates","date","D","hours","hour","h","minutes","minute","m","seconds","second","s","milliseconds","millisecond","ms"],l,M,Y=i.length;for(l=0;l<Y;l+=1)M=i[l],_=_||d(e,M);return t&&_}function zn(e){var t=s(e),_=!1;return t&&(_=e.filter(function(i){return!h(i)&&ua(e)}).length===0),t&&_}function Nn(e){var t=n(e)&&!u(e),_=!1,i=["sameDay","nextDay","lastDay","nextWeek","lastWeek","sameElse"],l,M;for(l=0;l<i.length;l+=1)M=i[l],_=_||d(e,M);return t&&_}function Rn(e,t){var _=e.diff(t,"days",!0);return _<-6?"sameElse":_<-1?"lastWeek":_<0?"lastDay":_<1?"sameDay":_<2?"nextDay":_<7?"nextWeek":"sameElse"}function Jn(e,t){arguments.length===1&&(arguments[0]?En(arguments[0])?(e=arguments[0],t=void 0):Nn(arguments[0])&&(t=arguments[0],e=void 0):(e=void 0,t=void 0));var _=e||A(),i=gt(_,this).startOf("day"),l=a.calendarFormat(this,i)||"sameElse",M=t&&(q(t[l])?t[l].call(this,_):t[l]);return this.format(M||this.localeData().calendar(l,this,A(_)))}function In(){return new Te(this)}function Cn(e,t){var _=V(e)?e:A(e);return this.isValid()&&_.isValid()?(t=U(t)||"millisecond",t==="millisecond"?this.valueOf()>_.valueOf():_.valueOf()<this.clone().startOf(t).valueOf()):!1}function Un(e,t){var _=V(e)?e:A(e);return this.isValid()&&_.isValid()?(t=U(t)||"millisecond",t==="millisecond"?this.valueOf()<_.valueOf():this.clone().endOf(t).valueOf()<_.valueOf()):!1}function Gn(e,t,_,i){var l=V(e)?e:A(e),M=V(t)?t:A(t);return this.isValid()&&l.isValid()&&M.isValid()?(i=i||"()",(i[0]==="("?this.isAfter(l,_):!this.isBefore(l,_))&&(i[1]===")"?this.isBefore(M,_):!this.isAfter(M,_))):!1}function Vn(e,t){var _=V(e)?e:A(e),i;return this.isValid()&&_.isValid()?(t=U(t)||"millisecond",t==="millisecond"?this.valueOf()===_.valueOf():(i=_.valueOf(),this.clone().startOf(t).valueOf()<=i&&i<=this.clone().endOf(t).valueOf())):!1}function Bn(e,t){return this.isSame(e,t)||this.isAfter(e,t)}function $n(e,t){return this.isSame(e,t)||this.isBefore(e,t)}function qn(e,t,_){var i,l,M;if(!this.isValid())return NaN;if(i=gt(e,this),!i.isValid())return NaN;switch(l=(i.utcOffset()-this.utcOffset())*6e4,t=U(t),t){case"year":M=qe(this,i)/12;break;case"month":M=qe(this,i);break;case"quarter":M=qe(this,i)/3;break;case"second":M=(this-i)/1e3;break;case"minute":M=(this-i)/6e4;break;case"hour":M=(this-i)/36e5;break;case"day":M=(this-i-l)/864e5;break;case"week":M=(this-i-l)/6048e5;break;default:M=this-i}return _?M:G(M)}function qe(e,t){if(e.date()<t.date())return-qe(t,e);var _=(t.year()-e.year())*12+(t.month()-e.month()),i=e.clone().add(_,"months"),l,M;return t-i<0?(l=e.clone().add(_-1,"months"),M=(t-i)/(i-l)):(l=e.clone().add(_+1,"months"),M=(t-i)/(l-i)),-(_+M)||0}a.defaultFormat="YYYY-MM-DDTHH:mm:ssZ",a.defaultFormatUtc="YYYY-MM-DDTHH:mm:ss[Z]";function Zn(){return this.clone().locale("en").format("ddd MMM DD YYYY HH:mm:ss [GMT]ZZ")}function Kn(e){if(!this.isValid())return null;var t=e!==!0,_=t?this.clone().utc():this;return _.year()<0||_.year()>9999?Ee(_,t?"YYYYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYYYY-MM-DD[T]HH:mm:ss.SSSZ"):q(Date.prototype.toISOString)?t?this.toDate().toISOString():new Date(this.valueOf()+this.utcOffset()*60*1e3).toISOString().replace("Z",Ee(_,"Z")):Ee(_,t?"YYYY-MM-DD[T]HH:mm:ss.SSS[Z]":"YYYY-MM-DD[T]HH:mm:ss.SSSZ")}function Qn(){if(!this.isValid())return"moment.invalid(/* "+this._i+" */)";var e="moment",t="",_,i,l,M;return this.isLocal()||(e=this.utcOffset()===0?"moment.utc":"moment.parseZone",t="Z"),_="["+e+'("]',i=0<=this.year()&&this.year()<=9999?"YYYY":"YYYYYY",l="-MM-DD[T]HH:mm:ss.SSS",M=t+'[")]',this.format(_+i+l+M)}function Xn(e){e||(e=this.isUtc()?a.defaultFormatUtc:a.defaultFormat);var t=Ee(this,e);return this.localeData().postformat(t)}function er(e,t){return this.isValid()&&(V(e)&&e.isValid()||A(e).isValid())?$({to:this,from:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()}function tr(e){return this.from(A(),e)}function ar(e,t){return this.isValid()&&(V(e)&&e.isValid()||A(e).isValid())?$({from:this,to:e}).locale(this.locale()).humanize(!t):this.localeData().invalidDate()}function sr(e){return this.to(A(),e)}function la(e){var t;return e===void 0?this._locale._abbr:(t=se(e),t!=null&&(this._locale=t),this)}var ma=C("moment().lang() is deprecated. Instead, use moment().localeData() to get the language configuration. Use moment().locale() to change languages.",function(e){return e===void 0?this.localeData():this.locale(e)});function ca(){return this._locale}var Ze=1e3,ke=60*Ze,Ke=60*ke,Ma=(365*400+97)*24*Ke;function De(e,t){return(e%t+t)%t}function ha(e,t,_){return e<100&&e>=0?new Date(e+400,t,_)-Ma:new Date(e,t,_).valueOf()}function La(e,t,_){return e<100&&e>=0?Date.UTC(e+400,t,_)-Ma:Date.UTC(e,t,_)}function nr(e){var t,_;if(e=U(e),e===void 0||e==="millisecond"||!this.isValid())return this;switch(_=this._isUTC?La:ha,e){case"year":t=_(this.year(),0,1);break;case"quarter":t=_(this.year(),this.month()-this.month()%3,1);break;case"month":t=_(this.year(),this.month(),1);break;case"week":t=_(this.year(),this.month(),this.date()-this.weekday());break;case"isoWeek":t=_(this.year(),this.month(),this.date()-(this.isoWeekday()-1));break;case"day":case"date":t=_(this.year(),this.month(),this.date());break;case"hour":t=this._d.valueOf(),t-=De(t+(this._isUTC?0:this.utcOffset()*ke),Ke);break;case"minute":t=this._d.valueOf(),t-=De(t,ke);break;case"second":t=this._d.valueOf(),t-=De(t,Ze);break}return this._d.setTime(t),a.updateOffset(this,!0),this}function rr(e){var t,_;if(e=U(e),e===void 0||e==="millisecond"||!this.isValid())return this;switch(_=this._isUTC?La:ha,e){case"year":t=_(this.year()+1,0,1)-1;break;case"quarter":t=_(this.year(),this.month()-this.month()%3+3,1)-1;break;case"month":t=_(this.year(),this.month()+1,1)-1;break;case"week":t=_(this.year(),this.month(),this.date()-this.weekday()+7)-1;break;case"isoWeek":t=_(this.year(),this.month(),this.date()-(this.isoWeekday()-1)+7)-1;break;case"day":case"date":t=_(this.year(),this.month(),this.date()+1)-1;break;case"hour":t=this._d.valueOf(),t+=Ke-De(t+(this._isUTC?0:this.utcOffset()*ke),Ke)-1;break;case"minute":t=this._d.valueOf(),t+=ke-De(t,ke)-1;break;case"second":t=this._d.valueOf(),t+=Ze-De(t,Ze)-1;break}return this._d.setTime(t),a.updateOffset(this,!0),this}function _r(){return this._d.valueOf()-(this._offset||0)*6e4}function ir(){return Math.floor(this.valueOf()/1e3)}function dr(){return new Date(this.valueOf())}function or(){var e=this;return[e.year(),e.month(),e.date(),e.hour(),e.minute(),e.second(),e.millisecond()]}function ur(){var e=this;return{years:e.year(),months:e.month(),date:e.date(),hours:e.hours(),minutes:e.minutes(),seconds:e.seconds(),milliseconds:e.milliseconds()}}function lr(){return this.isValid()?this.toISOString():null}function mr(){return Me(this)}function cr(){return v({},k(this))}function Mr(){return k(this).overflow}function hr(){return{input:this._i,format:this._f,locale:this._locale,isUTC:this._isUTC,strict:this._strict}}T("N",0,0,"eraAbbr"),T("NN",0,0,"eraAbbr"),T("NNN",0,0,"eraAbbr"),T("NNNN",0,0,"eraName"),T("NNNNN",0,0,"eraNarrow"),T("y",["y",1],"yo","eraYear"),T("y",["yy",2],0,"eraYear"),T("y",["yyy",3],0,"eraYear"),T("y",["yyyy",4],0,"eraYear"),w("N",vt),w("NN",vt),w("NNN",vt),w("NNNN",vr),w("NNNNN",br),O(["N","NN","NNN","NNNN","NNNNN"],function(e,t,_,i){var l=_._locale.erasParse(e,i,_._strict);l?k(_).era=l:k(_).invalidEra=e}),w("y",fe),w("yy",fe),w("yyy",fe),w("yyyy",fe),w("yo",Hr),O(["y","yy","yyy","yyyy"],N),O(["yo"],function(e,t,_,i){var l;_._locale._eraYearOrdinalRegex&&(l=e.match(_._locale._eraYearOrdinalRegex)),_._locale.eraYearOrdinalParse?t[N]=_._locale.eraYearOrdinalParse(e,l):t[N]=parseInt(e,10)});function Lr(e,t){var _,i,l,M=this._eras||se("en")._eras;for(_=0,i=M.length;_<i;++_){switch(typeof M[_].since){case"string":l=a(M[_].since).startOf("day"),M[_].since=l.valueOf();break}switch(typeof M[_].until){case"undefined":M[_].until=1/0;break;case"string":l=a(M[_].until).startOf("day").valueOf(),M[_].until=l.valueOf();break}}return M}function fr(e,t,_){var i,l,M=this.eras(),Y,D,b;for(e=e.toUpperCase(),i=0,l=M.length;i<l;++i)if(Y=M[i].name.toUpperCase(),D=M[i].abbr.toUpperCase(),b=M[i].narrow.toUpperCase(),_)switch(t){case"N":case"NN":case"NNN":if(D===e)return M[i];break;case"NNNN":if(Y===e)return M[i];break;case"NNNNN":if(b===e)return M[i];break}else if([Y,D,b].indexOf(e)>=0)return M[i]}function Yr(e,t){var _=e.since<=e.until?1:-1;return t===void 0?a(e.since).year():a(e.since).year()+(t-e.offset)*_}function yr(){var e,t,_,i=this.localeData().eras();for(e=0,t=i.length;e<t;++e)if(_=this.clone().startOf("day").valueOf(),i[e].since<=_&&_<=i[e].until||i[e].until<=_&&_<=i[e].since)return i[e].name;return""}function pr(){var e,t,_,i=this.localeData().eras();for(e=0,t=i.length;e<t;++e)if(_=this.clone().startOf("day").valueOf(),i[e].since<=_&&_<=i[e].until||i[e].until<=_&&_<=i[e].since)return i[e].narrow;return""}function kr(){var e,t,_,i=this.localeData().eras();for(e=0,t=i.length;e<t;++e)if(_=this.clone().startOf("day").valueOf(),i[e].since<=_&&_<=i[e].until||i[e].until<=_&&_<=i[e].since)return i[e].abbr;return""}function Dr(){var e,t,_,i,l=this.localeData().eras();for(e=0,t=l.length;e<t;++e)if(_=l[e].since<=l[e].until?1:-1,i=this.clone().startOf("day").valueOf(),l[e].since<=i&&i<=l[e].until||l[e].until<=i&&i<=l[e].since)return(this.year()-a(l[e].since).year())*_+l[e].offset;return this.year()}function wr(e){return d(this,"_erasNameRegex")||bt.call(this),e?this._erasNameRegex:this._erasRegex}function gr(e){return d(this,"_erasAbbrRegex")||bt.call(this),e?this._erasAbbrRegex:this._erasRegex}function Tr(e){return d(this,"_erasNarrowRegex")||bt.call(this),e?this._erasNarrowRegex:this._erasRegex}function vt(e,t){return t.erasAbbrRegex(e)}function vr(e,t){return t.erasNameRegex(e)}function br(e,t){return t.erasNarrowRegex(e)}function Hr(e,t){return t._eraYearOrdinalRegex||fe}function bt(){var e=[],t=[],_=[],i=[],l,M,Y,D,b,j=this.eras();for(l=0,M=j.length;l<M;++l)Y=X(j[l].name),D=X(j[l].abbr),b=X(j[l].narrow),t.push(Y),e.push(D),_.push(b),i.push(Y),i.push(D),i.push(b);this._erasRegex=new RegExp("^("+i.join("|")+")","i"),this._erasNameRegex=new RegExp("^("+t.join("|")+")","i"),this._erasAbbrRegex=new RegExp("^("+e.join("|")+")","i"),this._erasNarrowRegex=new RegExp("^("+_.join("|")+")","i")}T(0,["gg",2],0,function(){return this.weekYear()%100}),T(0,["GG",2],0,function(){return this.isoWeekYear()%100});function Qe(e,t){T(0,[e,e.length],0,t)}Qe("gggg","weekYear"),Qe("ggggg","weekYear"),Qe("GGGG","isoWeekYear"),Qe("GGGGG","isoWeekYear"),w("G",Re),w("g",Re),w("GG",W,J),w("gg",W,J),w("GGGG",ut,ot),w("gggg",ut,ot),w("GGGGG",Ne,Fe),w("ggggg",Ne,Fe),be(["gggg","ggggg","GGGG","GGGGG"],function(e,t,_,i){t[i.substr(0,2)]=S(e)}),be(["gg","GG"],function(e,t,_,i){t[i]=a.parseTwoDigitYear(e)});function Sr(e){return fa.call(this,e,this.week(),this.weekday()+this.localeData()._week.dow,this.localeData()._week.dow,this.localeData()._week.doy)}function jr(e){return fa.call(this,e,this.isoWeek(),this.isoWeekday(),1,4)}function xr(){return ae(this.year(),1,4)}function Pr(){return ae(this.isoWeekYear(),1,4)}function Or(){var e=this.localeData()._week;return ae(this.year(),e.dow,e.doy)}function Wr(){var e=this.localeData()._week;return ae(this.weekYear(),e.dow,e.doy)}function fa(e,t,_,i,l){var M;return e==null?xe(this,i,l).year:(M=ae(e,i,l),t>M&&(t=M),Ar.call(this,e,t,_,i,l))}function Ar(e,t,_,i,l){var M=$t(e,t,_,i,l),Y=je(M.year,0,M.dayOfYear);return this.year(Y.getUTCFullYear()),this.month(Y.getUTCMonth()),this.date(Y.getUTCDate()),this}T("Q",0,"Qo","quarter"),w("Q",Ft),O("Q",function(e,t){t[ee]=(S(e)-1)*3});function Er(e){return e==null?Math.ceil((this.month()+1)/3):this.month((e-1)*3+this.month()%3)}T("D",["DD",2],"Do","date"),w("D",W,Ye),w("DD",W,J),w("Do",function(e,t){return e?t._dayOfMonthOrdinalParse||t._ordinalParse:t._dayOfMonthOrdinalParseLenient}),O(["D","DD"],K),O("Do",function(e,t){t[K]=S(e.match(W)[0])});var Ya=ye("Date",!0);T("DDD",["DDDD",3],"DDDo","dayOfYear"),w("DDD",ze),w("DDDD",zt),O(["DDD","DDDD"],function(e,t,_){_._dayOfYear=S(e)});function Fr(e){var t=Math.round((this.clone().startOf("day")-this.clone().startOf("year"))/864e5)+1;return e==null?t:this.add(e-t,"d")}T("m",["mm",2],0,"minute"),w("m",W,lt),w("mm",W,J),O(["m","mm"],B);var zr=ye("Minutes",!1);T("s",["ss",2],0,"second"),w("s",W,lt),w("ss",W,J),O(["s","ss"],te);var Nr=ye("Seconds",!1);T("S",0,0,function(){return~~(this.millisecond()/100)}),T(0,["SS",2],0,function(){return~~(this.millisecond()/10)}),T(0,["SSS",3],0,"millisecond"),T(0,["SSSS",4],0,function(){return this.millisecond()*10}),T(0,["SSSSS",5],0,function(){return this.millisecond()*100}),T(0,["SSSSSS",6],0,function(){return this.millisecond()*1e3}),T(0,["SSSSSSS",7],0,function(){return this.millisecond()*1e4}),T(0,["SSSSSSSS",8],0,function(){return this.millisecond()*1e5}),T(0,["SSSSSSSSS",9],0,function(){return this.millisecond()*1e6}),w("S",ze,Ft),w("SS",ze,J),w("SSS",ze,zt);var oe,ya;for(oe="SSSS";oe.length<=9;oe+="S")w(oe,fe);function Rr(e,t){t[le]=S(("0."+e)*1e3)}for(oe="S";oe.length<=9;oe+="S")O(oe,Rr);ya=ye("Milliseconds",!1),T("z",0,0,"zoneAbbr"),T("zz",0,0,"zoneName");function Jr(){return this._isUTC?"UTC":""}function Ir(){return this._isUTC?"Coordinated Universal Time":""}var p=Te.prototype;p.add=Wn,p.calendar=Jn,p.clone=In,p.diff=qn,p.endOf=rr,p.format=Xn,p.from=er,p.fromNow=tr,p.to=ar,p.toNow=sr,p.get=Ka,p.invalidAt=Mr,p.isAfter=Cn,p.isBefore=Un,p.isBetween=Gn,p.isSame=Vn,p.isSameOrAfter=Bn,p.isSameOrBefore=$n,p.isValid=mr,p.lang=ma,p.locale=la,p.localeData=ca,p.max=mn,p.min=ln,p.parsingFlags=cr,p.set=Qa,p.startOf=nr,p.subtract=An,p.toArray=or,p.toObject=ur,p.toDate=dr,p.toISOString=Kn,p.inspect=Qn,typeof Symbol<"u"&&Symbol.for!=null&&(p[Symbol.for("nodejs.util.inspect.custom")]=function(){return"Moment<"+this.format()+">"}),p.toJSON=lr,p.toString=Zn,p.unix=ir,p.valueOf=_r,p.creationData=hr,p.eraName=yr,p.eraNarrow=pr,p.eraAbbr=kr,p.eraYear=Dr,p.year=Jt,p.isLeapYear=Za,p.weekYear=Sr,p.isoWeekYear=jr,p.quarter=p.quarters=Er,p.month=Vt,p.daysInMonth=is,p.week=p.weeks=hs,p.isoWeek=p.isoWeeks=Ls,p.weeksInYear=Or,p.weeksInWeekYear=Wr,p.isoWeeksInYear=xr,p.isoWeeksInISOWeekYear=Pr,p.date=Ya,p.day=p.days=Ss,p.weekday=js,p.isoWeekday=xs,p.dayOfYear=Fr,p.hour=p.hours=zs,p.minute=p.minutes=zr,p.second=p.seconds=Nr,p.millisecond=p.milliseconds=ya,p.utcOffset=kn,p.utc=wn,p.local=gn,p.parseZone=Tn,p.hasAlignedHourOffset=vn,p.isDST=bn,p.isLocal=Sn,p.isUtcOffset=jn,p.isUtc=_a,p.isUTC=_a,p.zoneAbbr=Jr,p.zoneName=Ir,p.dates=C("dates accessor is deprecated. Use date instead.",Ya),p.months=C("months accessor is deprecated. Use month instead",Vt),p.years=C("years accessor is deprecated. Use year instead",Jt),p.zone=C("moment().zone is deprecated, use moment().utcOffset instead. http://momentjs.com/guides/#/warnings/zone/",Dn),p.isDSTShifted=C("isDSTShifted is deprecated. See http://momentjs.com/guides/#/warnings/dst-shifted/ for more information",Hn);function Cr(e){return A(e*1e3)}function Ur(){return A.apply(null,arguments).parseZone()}function pa(e){return e}var P=nt.prototype;P.calendar=Ha,P.longDateFormat=Pa,P.invalidDate=Wa,P.ordinal=Fa,P.preparse=pa,P.postformat=pa,P.relativeTime=Na,P.pastFuture=Ra,P.set=va,P.eras=Lr,P.erasParse=fr,P.erasConvertYear=Yr,P.erasAbbrRegex=gr,P.erasNameRegex=wr,P.erasNarrowRegex=Tr,P.months=ss,P.monthsShort=ns,P.monthsParse=_s,P.monthsRegex=os,P.monthsShortRegex=ds,P.week=ls,P.firstDayOfYear=Ms,P.firstDayOfWeek=cs,P.weekdays=gs,P.weekdaysMin=vs,P.weekdaysShort=Ts,P.weekdaysParse=Hs,P.weekdaysRegex=Ps,P.weekdaysShortRegex=Os,P.weekdaysMinRegex=Ws,P.isPM=Es,P.meridiem=Ns;function Xe(e,t,_,i){var l=se(),M=H().set(i,t);return l[_](M,e)}function ka(e,t,_){if(h(e)&&(t=e,e=void 0),e=e||"",t!=null)return Xe(e,t,_,"month");var i,l=[];for(i=0;i<12;i++)l[i]=Xe(e,i,_,"month");return l}function Ht(e,t,_,i){typeof e=="boolean"?(h(t)&&(_=t,t=void 0),t=t||""):(t=e,_=t,e=!1,h(t)&&(_=t,t=void 0),t=t||"");var l=se(),M=e?l._week.dow:0,Y,D=[];if(_!=null)return Xe(t,(_+M)%7,i,"day");for(Y=0;Y<7;Y++)D[Y]=Xe(t,(Y+M)%7,i,"day");return D}function Gr(e,t){return ka(e,t,"months")}function Vr(e,t){return ka(e,t,"monthsShort")}function Br(e,t,_){return Ht(e,t,_,"weekdays")}function $r(e,t,_){return Ht(e,t,_,"weekdaysShort")}function qr(e,t,_){return Ht(e,t,_,"weekdaysMin")}de("en",{eras:[{since:"0001-01-01",until:1/0,offset:1,name:"Anno Domini",narrow:"AD",abbr:"AD"},{since:"0000-12-31",until:-1/0,offset:1,name:"Before Christ",narrow:"BC",abbr:"BC"}],dayOfMonthOrdinalParse:/\d{1,2}(th|st|nd|rd)/,ordinal:function(e){var t=e%10,_=S(e%100/10)===1?"th":t===1?"st":t===2?"nd":t===3?"rd":"th";return e+_}}),a.lang=C("moment.lang is deprecated. Use moment.locale instead.",de),a.langData=C("moment.langData is deprecated. Use moment.localeData instead.",se);var ne=Math.abs;function Zr(){var e=this._data;return this._milliseconds=ne(this._milliseconds),this._days=ne(this._days),this._months=ne(this._months),e.milliseconds=ne(e.milliseconds),e.seconds=ne(e.seconds),e.minutes=ne(e.minutes),e.hours=ne(e.hours),e.months=ne(e.months),e.years=ne(e.years),this}function Da(e,t,_,i){var l=$(t,_);return e._milliseconds+=i*l._milliseconds,e._days+=i*l._days,e._months+=i*l._months,e._bubble()}function Kr(e,t){return Da(this,e,t,1)}function Qr(e,t){return Da(this,e,t,-1)}function wa(e){return e<0?Math.floor(e):Math.ceil(e)}function Xr(){var e=this._milliseconds,t=this._days,_=this._months,i=this._data,l,M,Y,D,b;return e>=0&&t>=0&&_>=0||e<=0&&t<=0&&_<=0||(e+=wa(St(_)+t)*864e5,t=0,_=0),i.milliseconds=e%1e3,l=G(e/1e3),i.seconds=l%60,M=G(l/60),i.minutes=M%60,Y=G(M/60),i.hours=Y%24,t+=G(Y/24),b=G(ga(t)),_+=b,t-=wa(St(b)),D=G(_/12),_%=12,i.days=t,i.months=_,i.years=D,this}function ga(e){return e*4800/146097}function St(e){return e*146097/4800}function e_(e){if(!this.isValid())return NaN;var t,_,i=this._milliseconds;if(e=U(e),e==="month"||e==="quarter"||e==="year")switch(t=this._days+i/864e5,_=this._months+ga(t),e){case"month":return _;case"quarter":return _/3;case"year":return _/12}else switch(t=this._days+Math.round(St(this._months)),e){case"week":return t/7+i/6048e5;case"day":return t+i/864e5;case"hour":return t*24+i/36e5;case"minute":return t*1440+i/6e4;case"second":return t*86400+i/1e3;case"millisecond":return Math.floor(t*864e5)+i;default:throw new Error("Unknown unit "+e)}}function re(e){return function(){return this.as(e)}}var Ta=re("ms"),t_=re("s"),a_=re("m"),s_=re("h"),n_=re("d"),r_=re("w"),__=re("M"),i_=re("Q"),d_=re("y"),o_=Ta;function u_(){return $(this)}function l_(e){return e=U(e),this.isValid()?this[e+"s"]():NaN}function ce(e){return function(){return this.isValid()?this._data[e]:NaN}}var m_=ce("milliseconds"),c_=ce("seconds"),M_=ce("minutes"),h_=ce("hours"),L_=ce("days"),f_=ce("months"),Y_=ce("years");function y_(){return G(this.days()/7)}var _e=Math.round,we={ss:44,s:45,m:45,h:22,d:26,w:null,M:11};function p_(e,t,_,i,l){return l.relativeTime(t||1,!!_,e,i)}function k_(e,t,_,i){var l=$(e).abs(),M=_e(l.as("s")),Y=_e(l.as("m")),D=_e(l.as("h")),b=_e(l.as("d")),j=_e(l.as("M")),R=_e(l.as("w")),ie=_e(l.as("y")),ue=M<=_.ss&&["s",M]||M<_.s&&["ss",M]||Y<=1&&["m"]||Y<_.m&&["mm",Y]||D<=1&&["h"]||D<_.h&&["hh",D]||b<=1&&["d"]||b<_.d&&["dd",b];return _.w!=null&&(ue=ue||R<=1&&["w"]||R<_.w&&["ww",R]),ue=ue||j<=1&&["M"]||j<_.M&&["MM",j]||ie<=1&&["y"]||["yy",ie],ue[2]=t,ue[3]=+e>0,ue[4]=i,p_.apply(null,ue)}function D_(e){return e===void 0?_e:typeof e=="function"?(_e=e,!0):!1}function w_(e,t){return we[e]===void 0?!1:t===void 0?we[e]:(we[e]=t,e==="s"&&(we.ss=t-1),!0)}function g_(e,t){if(!this.isValid())return this.localeData().invalidDate();var _=!1,i=we,l,M;return typeof e=="object"&&(t=e,e=!1),typeof e=="boolean"&&(_=e),typeof t=="object"&&(i=Object.assign({},we,t),t.s!=null&&t.ss==null&&(i.ss=t.s-1)),l=this.localeData(),M=k_(this,!_,i,l),_&&(M=l.pastFuture(+this,M)),l.postformat(M)}var jt=Math.abs;function ge(e){return(e>0)-(e<0)||+e}function et(){if(!this.isValid())return this.localeData().invalidDate();var e=jt(this._milliseconds)/1e3,t=jt(this._days),_=jt(this._months),i,l,M,Y,D=this.asSeconds(),b,j,R,ie;return D?(i=G(e/60),l=G(i/60),e%=60,i%=60,M=G(_/12),_%=12,Y=e?e.toFixed(3).replace(/\.?0+$/,""):"",b=D<0?"-":"",j=ge(this._months)!==ge(D)?"-":"",R=ge(this._days)!==ge(D)?"-":"",ie=ge(this._milliseconds)!==ge(D)?"-":"",b+"P"+(M?j+M+"Y":"")+(_?j+_+"M":"")+(t?R+t+"D":"")+(l||i||e?"T":"")+(l?ie+l+"H":"")+(i?ie+i+"M":"")+(e?ie+Y+"S":"")):"P0D"}var x=Be.prototype;x.isValid=fn,x.abs=Zr,x.add=Kr,x.subtract=Qr,x.as=e_,x.asMilliseconds=Ta,x.asSeconds=t_,x.asMinutes=a_,x.asHours=s_,x.asDays=n_,x.asWeeks=r_,x.asMonths=__,x.asQuarters=i_,x.asYears=d_,x.valueOf=o_,x._bubble=Xr,x.clone=u_,x.get=l_,x.milliseconds=m_,x.seconds=c_,x.minutes=M_,x.hours=h_,x.days=L_,x.weeks=y_,x.months=f_,x.years=Y_,x.humanize=g_,x.toISOString=et,x.toString=et,x.toJSON=et,x.locale=la,x.localeData=ca,x.toIsoString=C("toIsoString() is deprecated. Please use toISOString() instead (notice the capitals)",et),x.lang=ma,T("X",0,0,"unix"),T("x",0,0,"valueOf"),w("x",Re),w("X",Ua),O("X",function(e,t,_){_._d=new Date(parseFloat(e)*1e3)}),O("x",function(e,t,_){_._d=new Date(S(e))});//! moment.js
return a.version="2.30.1",r(A),a.fn=p,a.min=cn,a.max=Mn,a.now=hn,a.utc=H,a.unix=Cr,a.months=Gr,a.isDate=y,a.locale=de,a.invalid=he,a.duration=$,a.isMoment=V,a.weekdays=Br,a.parseZone=Ur,a.localeData=se,a.isDuration=$e,a.monthsShort=Vr,a.weekdaysMin=qr,a.defineLocale=ft,a.updateLocale=Cs,a.locales=Us,a.weekdaysShort=$r,a.normalizeUnits=U,a.relativeTimeRounding=D_,a.relativeTimeThreshold=w_,a.calendarFormat=Rn,a.prototype=p,a.HTML5_FMT={DATETIME_LOCAL:"YYYY-MM-DDTHH:mm",DATETIME_LOCAL_SECONDS:"YYYY-MM-DDTHH:mm:ss",DATETIME_LOCAL_MS:"YYYY-MM-DDTHH:mm:ss.SSS",DATE:"YYYY-MM-DD",TIME:"HH:mm",TIME_SECONDS:"HH:mm:ss",TIME_MS:"HH:mm:ss.SSS",WEEK:"GGGG-[W]WW",MONTH:"YYYY-MM"},a})}}]);

//# sourceMappingURL=6029-react19.bd349141416938549393.js.map